<?php
defined('BASEPATH') OR exit('No direct script access allowed');

 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

class Express_checkout extends MY_Controller
{
	function __construct()
	{
		parent::__construct();

		// Load helpers
		$this->load->helper('url');

		// Load session library
		$this->load->library('session');

// 		$this->load->database();
// 		$this->load->model('model_parando');
		$this->load->model('accounting_parando');

		// Load PayPal library config
		
		$sandbox = $this->session->userdata('paypal_sandbox');
		if(isset($sandbox) && $sandbox==true){
			$this->config->load('paypal_sandbox');
		}else{
			$this->config->load('paypal_live');
		}


		$config = array(
			'Sandbox' => $this->config->item('Sandbox'),            // Sandbox / testing mode option.
			'APIUsername' => $this->config->item('APIUsername'),    // PayPal API username of the API caller
			'APIPassword' => $this->config->item('APIPassword'),    // PayPal API password of the API caller
			'APISignature' => $this->config->item('APISignature'),    // PayPal API signature of the API caller
			'APISubject' => '',                                    // PayPal API subject (email address of 3rd party user that has granted API permission for your app)
			'APIVersion' => $this->config->item('APIVersion'),        // API version you'd like to use for your call.  You can set a default version in the class and leave this blank if you want.
			'DeviceID' => $this->config->item('DeviceID'),
			'ApplicationID' => $this->config->item('ApplicationID'),
			'DeveloperEmailAccount' => $this->config->item('DeveloperEmailAccount')
		);

		// Show Errors
		if ($config['Sandbox']) {
			error_reporting(E_ALL);
			ini_set('display_errors', '1');
		}

		// Load PayPal library
		$this->load->library('paypal/paypal_pro', $config);
	}

// 	private function die_error($heading,$message){
// 		$this->load->view('errors/html/error_general',array('title'=>$heading, 'heading'=>$heading,'message'=> $message));
// 		$this->output->_display();
// 		exit();
// 	}
	
	private function load_paypal_error_view($errors){
		$text="";
		foreach($errors as $error)
		{
			$text.= '<p>';
			$text.= '<strong>Error Code:</strong> ' . $error[0]['L_ERRORCODE'];
			$text.= '<br /><strong>Error Message:</strong> ' . $error[0]['L_LONGMESSAGE'];
			$text.= '</p>';
		}
		$heading="PayPal Error";
		$this->load->view('errors/html/error_general',['title'=>$heading, 'heading'=>$heading,'message'=> $text]);
	}


	private function invalid_cart_q($cart=null){
		return 	(empty($cart) 
			|| empty($cart['shopping_cart']['grand_total'])
			|| empty($cart['member_first_name'])
			|| empty($cart['member_last_name'])
			|| empty($cart['contact_email'])
			|| empty($cart['invoice_number'])
			|| empty($cart['order_token'])
			|| empty($cart['fulfillment_url'])
			);
	}

	/**
	 * SetExpressCheckout
	 */
	function SetExpressCheckout()
	{
	
		// Clear PayPalResult from session userdata
		$this->session->unset_userdata('PayPalResult');

		// Get cart data from session userdata
		$cart = $this->session->userdata('shopping_cart');

		if($this->invalid_cart_q($cart)){
			$this->die_error(__METHOD__,'Missing parameters.');
		}
		
		
        // TODO SET CLUB CONTEXT HERE

		/**
		 * Here we are setting up the parameters for a basic Express Checkout flow.
		 *
		 * The template provided at /vendor/angelleye/paypal-php-library/templates/SetExpressCheckout.php
		 * contains a lot more parameters that we aren't using here, so I've removed them to keep this clean.
		 *
		 * $domain used here is set in the config file.
		 */
		$SECFields = array(
			'maxamt' => round($cart['shopping_cart']['grand_total'] * 2,2), 					// The expected maximum total amount the order will be, including S&H and sales tax.
			'returnurl' => site_url('express_checkout/GetExpressCheckoutDetails'), 							    // Required.  URL to which the customer will be returned after returning from PayPal.  2048 char max.
			'cancelurl' => site_url('express_checkout/OrderCancelled'), 							    // Required.  URL to which the customer will be returned if they cancel payment on PayPal's site.
			'hdrimg' => $this->model_parando->hdrimg, 		
			'logoimg' => $this->model_parando->logoimg, 		
			'brandname' => $this->model_parando->organization, 		
			'surveyquestion' => '',					                                
			'customerservicenumber' => $this->model_parando->info_phone_number,				
			'noshipping' => 1, // Maybe someday we'll ship things -- till then... 
			'solutiontype' => 'Sole',
			'email' => $cart['contact_email']                      
		);

		/**
		 * Now we begin setting up our payment(s).
		 *
		 * Express Checkout includes the ability to setup parallel payments,
		 * so we have to populate our $Payments array here accordingly.
		 *
		 * For this sample (and in most use cases) we only need a single payment,
		 * but we still have to populate $Payments with a single $Payment array.
		 *
		 * Once again, the template file includes a lot more available parameters,
		 * but for this basic sample we've removed everything that we're not using,
		 * so all we have is an amount.
		 */
		$Payments = array();
		$Payment = array(
			'amt' => $cart['shopping_cart']['grand_total'], 	// Required.  The total cost of the transaction to the customer.  If shipping cost and tax charges are known, include them in this value.  If not, this value should be the current sub-total of the order.
			'itemamt' => $cart['shopping_cart']['subtotal'], 	// Required if you specify itemized L_AMT fields. Sum of cost of all items in this order.
		);

		/**
		 * Here we'll begin creating our order items that belong to this $Payment in the request.
		 * We will loop through the items in our shopping cart to add them each into our
		 * $Payment.
		 */
		$PaymentOrderItems = array();
		foreach($cart['shopping_cart']['items'] as $cart_item)
		{
			$Item = array(
				'name' => $cart_item['name'], 							// Item name. 127 char max.
				'amt' => $cart_item['price'], 							// Cost of item.
				'desc' => $cart_item['description'], 							// Value
				'qty' => $cart_item['qty'], 							// Item qty on order.  Any positive integer.
			);
			array_push($PaymentOrderItems, $Item);
		}
		/**
		 * Now that $PaymentOrderItems is filled with all of our shopping cart items,
		 * we'll add that to our $Payment array.
		 */
		$Payment['order_items'] = $PaymentOrderItems;

		/**
		 * Here we push our single $Payment into our $Payments array.
		 */
		array_push($Payments, $Payment);

		/**
		 * Now we gather all of the arrays above into a single array.
		 */
		$PayPalRequestData = array(
			'SECFields' => $SECFields,
			'Payments' => $Payments,
		);

		/**
		 * Here we are making the call to the SetExpressCheckout function in the library,
		 * and we're passing in our $PayPalRequestData that we just set above.
		 */
		 
// $this->die_error(__METHOD__,print_r($PayPalRequestData, true));
		 
		$PayPalResult = $this->paypal_pro->SetExpressCheckout($PayPalRequestData);

		/**
		 * Now we'll check for any errors returned by PayPal, and if we get an error,
		 * we'll save the error details to a session and redirect the user to an
		 * error page to display it accordingly.
		 *
		 * If all goes well, we save our token in a session variable so that it's
		 * readily available for us later, and then redirect the user to PayPal
		 * using the REDIRECTURL returned by the SetExpressCheckout() function.
		 */
		if(!$this->paypal_pro->APICallSuccessful($PayPalResult['ACK']))
		{
			$errors = array('Errors'=>$PayPalResult['ERRORS']);
			$this->load_paypal_error_view($errors);
		}
		else
		{
			// Successful call.

// $this->die_error(__METHOD__,print_r($PayPalResult, true));

			// Set PayPalResult into session userdata (so we can grab data from it later on a 'payment complete' page)
			$this->session->set_userdata('PayPalResult', $PayPalResult);

			// In most cases you would automatically redirect to the returned 'RedirectURL' by using: redirect($PayPalResult['REDIRECTURL'],'Location');
			// Move to PayPal checkout
			redirect($PayPalResult['REDIRECTURL'], 'Location');
		}
	}

	/**
	 * GetExpressCheckoutDetails
	 */
	function GetExpressCheckoutDetails()
	{
		// Get cart data from session userdata
		$cart = $this->session->userdata('shopping_cart');
		if($this->invalid_cart_q($cart)){
			$this->die_error(__METHOD__,'Missing parameters.');
		}

		// Get PayPal data from session userdata
		$SetExpressCheckoutPayPalResult = $this->session->userdata('PayPalResult');
		$PayPal_Token = $SetExpressCheckoutPayPalResult['TOKEN'];

		/**
		 * Now we pass the PayPal token that we saved to a session variable
		 * in the SetExpressCheckout.php file into the GetExpressCheckoutDetails
		 * request.
		 */
		$PayPalResult = $this->paypal_pro->GetExpressCheckoutDetails($PayPal_Token);


		/**
		 * Now we'll check for any errors returned by PayPal, and if we get an error,
		 * we'll save the error details to a session and redirect the user to an
		 * error page to display it accordingly.
		 *
		 * If the call is successful, we'll save some data we might want to use
		 * later into session variables.
		 */
		if(!$this->paypal_pro->APICallSuccessful($PayPalResult['ACK']))
		{
			$errors = array('Errors'=>$PayPalResult['ERRORS']);
			$this->load_paypal_error_view($errors);
		}
		else
		{
			// Successful call.

			/**
			 * Here we'll pull out data from the PayPal response.
			 * Refer to the PayPal API Reference for all of the variables available
			 * in $PayPalResult['variablename']
			 *
			 * https://developer.paypal.com/docs/classic/api/merchant/GetExpressCheckoutDetails_API_Operation_NVP/
			 *
			 * Again, Express Checkout allows for parallel payments, so what we're doing here
			 * is usually the library to parse out the individual payments using the GetPayments()
			 * method so that we can easily access the data.
			 *
			 * We only have a single payment here, which will be the case with most checkouts,
			 * but we will still loop through the $Payments array returned by the library
			 * to grab our data accordingly.
			 */
			$cart['paypal_payer_id'] = isset($PayPalResult['PAYERID']) ? $PayPalResult['PAYERID'] : '';
			$cart['phone_number'] = isset($PayPalResult['PHONENUM']) ? $PayPalResult['PHONENUM'] : '';
			$cart['email'] = isset($PayPalResult['EMAIL']) ? $PayPalResult['EMAIL'] : '';
			$cart['first_name'] = isset($PayPalResult['FIRSTNAME']) ? $PayPalResult['FIRSTNAME'] : '';
			$cart['last_name'] = isset($PayPalResult['LASTNAME']) ? $PayPalResult['LASTNAME'] : '';

			foreach($PayPalResult['PAYMENTS'] as $payment) {
				$cart['shipping_name'] = isset($payment['SHIPTONAME']) ? $payment['SHIPTONAME'] : '';
				$cart['shipping_street'] = isset($payment['SHIPTOSTREET']) ? $payment['SHIPTOSTREET'] : '';
				$cart['shipping_city'] = isset($payment['SHIPTOCITY']) ? $payment['SHIPTOCITY'] : '';
				$cart['shipping_state'] = isset($payment['SHIPTOSTATE']) ? $payment['SHIPTOSTATE'] : '';
				$cart['shipping_zip'] = isset($payment['SHIPTOZIP']) ? $payment['SHIPTOZIP'] : '';
				$cart['shipping_country_code'] = isset($payment['SHIPTOCOUNTRYCODE']) ? $payment['SHIPTOCOUNTRYCODE'] : '';
				$cart['shipping_country_name'] = isset($payment['SHIPTOCOUNTRYNAME']) ? $payment['SHIPTOCOUNTRYNAME'] : '';
			}

			/**
			 * At this point, we now have the buyer's shipping address available in our app.
			 * We could now run the data through a shipping calculator to retrieve rate
			 * information for this particular order.
			 *
			 * This would also be the time to calculate any sales tax you may need to
			 * add to the order, as well as handling fees.
			 *
			 * We're going to set static values for these things in our static
			 * shopping cart, and then re-calculate our grand total.
			 */
			$cart['shopping_cart']['shipping'] = 0.00;
			$cart['shopping_cart']['handling'] = 0.00;
			$cart['shopping_cart']['tax'] = 0.00;

			$cart['shopping_cart']['grand_total'] = number_format(
				$cart['shopping_cart']['subtotal']
				+ $cart['shopping_cart']['shipping']
				+ $cart['shopping_cart']['handling']
				+ $cart['shopping_cart']['tax'],2, '.', '');

			/**
			 * Now we will redirect the user to a final review
			 * page so they can see the shipping/handling/tax
			 * that has been added to the order.
			 */
			// Set example cart data into session
			$this->session->set_userdata('shopping_cart', $cart);

			// Load example cart data to variable
			$this->load->vars('cart', $cart);

			// Example - Load Review Page - PA Rando Specific
			$this_organization = $this->model_parando->organization;
			$base_url = base_url();
			$contact_url = $this->model_parando->contact_url;

			$purchase=$this->accounting_parando->get_purchase($cart['invoice_number']);
			$pitb=$this->accounting_parando->purchase_item_table_body($purchase);

		$member_first_name=$cart['member_first_name'];
		$member_last_name=$cart['member_last_name'];
		$invoice_number=$cart['invoice_number'];
		$amount_due = $cart['shopping_cart']['grand_total'];
		$shipping_method = $purchase['shipping_method'];

		$subject='Review Order';
		$form_body=<<<EOT
<p>Please review your order and click 'Complete Order' to finalize payment.</p>
<p>
<div class='fee'>
<table>
<tr><td>Name:</td><td>{$member_first_name} {$member_last_name}</td></tr>
<tr><td>Amount Due:</td><td>\${$amount_due}</td></tr>
<tr><td>Order ID:</td><td>{$invoice_number}</td></tr>
<tr><td>Shipping Method:</td><td>{$shipping_method}</td></tr>
</table>
</div>
<h2>Items</h2>
<br>
<div class='product-table'>
<TABLE width=100%>
<TR><TH>Quantity</TH><TH>Description</TH><TH>Value</TH><TH>Unit Price</TH><TH>Ext Price</TH></TR>
$pitb
</TABLE>
</div>
</p>
EOT;
		$form_submit="<button type='submit' name='submit' value='confirm'>Complete Order</button>";
		


		$this->load->view('form_header',['form_title'=>$subject,'form_subject'=>$subject, 'page_styles'=>['form','product_table']]);
		$this->load->view('form_simple',['form_action'=>site_url('express_checkout/DoExpressCheckoutPayment'),'form_body'=>$form_body,'form_submit'=>$form_submit]);
		$this->load->view('form_footer',[]);

		}
	}

	/**
	 * DoExpressCheckoutPayment
	 */
	function DoExpressCheckoutPayment()
	{
		/**
		 * Now we'll setup the request params for the final call in the Express Checkout flow.
		 * This is very similar to SetExpressCheckout except that now we can include values
		 * for the shipping, handling, and tax amounts, as well as the buyer's name and
		 * shipping address that we obtained in the GetExpressCheckoutDetails step.
		 *
		 * If this information is not included in this final call, it will not be
		 * available in PayPal's transaction details data.
		 *
		 * Once again, the template for DoExpressCheckoutPayment provides
		 * many more params that are available, but we've stripped everything
		 * we are not using in this basic demo out.
		 */

		// Get cart data from session userdata
		$cart = $this->session->userdata('shopping_cart');
		if($this->invalid_cart_q($cart)){
			$this->die_error(__METHOD__,'Missing parameters.');
		}

		// Get cart data from session userdata
		$SetExpressCheckoutPayPalResult = $this->session->userdata('PayPalResult');
		$PayPal_Token = $SetExpressCheckoutPayPalResult['TOKEN'];

		$DECPFields = array(
			'token' => $PayPal_Token, 								// Required.  A timestamped token, the value of which was returned by a previous SetExpressCheckout call.
			'payerid' => $cart['paypal_payer_id'], 							// Required.  Unique PayPal customer id of the payer.  Returned by GetExpressCheckoutDetails, or if you used SKIPDETAILS it's returned in the URL back to your RETURNURL.
		);

		/**
		 * Just like with SetExpressCheckout, we need to gather our $Payment
		 * data to pass into our $Payments array.  This time we can include
		 * the shipping, handling, tax, and shipping address details that we
		 * now have.
		 */
		$Payments = array();
		
		
		$Payment = array(
			'amt' => number_format($cart['shopping_cart']['grand_total'],2, '.', ''), 	    // Required.  The total cost of the transaction to the customer.  If shipping cost and tax charges are known, include them in this value.  If not, this value should be the current sub-total of the order.
			'itemamt' => number_format($cart['shopping_cart']['subtotal'],2, '.', ''),       // Subtotal of items only.
			'currencycode' => 'USD', 					                                // A three-character currency code.  Default is USD.
			'shippingamt' => number_format($cart['shopping_cart']['shipping'],2, '.', ''), 	// Total shipping costs for this order.  If you specify SHIPPINGAMT you mut also specify a value for ITEMAMT.
			'handlingamt' => number_format($cart['shopping_cart']['handling'],2, '.', ''), 	// Total handling costs for this order.  If you specify HANDLINGAMT you mut also specify a value for ITEMAMT.
			'taxamt' => number_format($cart['shopping_cart']['tax'],2, '.', ''), 			// Required if you specify itemized L_TAXAMT fields.  Sum of all tax items in this order.
			'shiptoname' => $cart['shipping_name'], 					            // Required if shipping is included.  Person's name associated with this address.  32 char max.
			'shiptostreet' => $cart['shipping_street'], 					        // Required if shipping is included.  First street address.  100 char max.
			'shiptocity' => $cart['shipping_city'], 					            // Required if shipping is included.  Name of city.  40 char max.
			'shiptostate' => $cart['shipping_state'], 					            // Required if shipping is included.  Name of state or province.  40 char max.
			'shiptozip' => $cart['shipping_zip'], 						            // Required if shipping is included.  Postal code of shipping address.  20 char max.
			'shiptocountrycode' => $cart['shipping_country_code'], 				    // Required if shipping is included.  Country code of shipping address.  2 char max.
			'shiptophonenum' => $cart['phone_number'],  				            // Phone number for shipping address.  20 char max.
			'paymentaction' => 'Sale', 					                                // How you want to obtain the payment.  When implementing parallel payments, this field is required and must be set to Order.
		);


		/**
		 * Here we'll begin creating our order items that belong to this $Payment in the request.
		 * We will loop through the items in our shopping cart to add them each into our
		 * $Payment.
		 */
		$PaymentOrderItems = array();
		foreach($cart['shopping_cart']['items'] as $cart_item)
		{
			$Item = array(
				'name' => $cart_item['name'], 							// Item name. 127 char max.
				'amt' => $cart_item['price'], 							// Cost of item.
				'desc' => $cart_item['description'], 							// Value
				'qty' => $cart_item['qty'], 							// Item qty on order.  Any positive integer.
			);
			array_push($PaymentOrderItems, $Item);
		}

		/**
		 * Now that $PaymentOrderItems is filled with all of our shopping cart items,
		 * we'll add that to our $Payment array.
		 */
		$Payment['order_items'] = $PaymentOrderItems;

		/**
		 * Here we push our single $Payment into our $Payments array.
		 */
		array_push($Payments, $Payment);

		/**
		 * Now we gather all of the arrays above into a single array.
		 */
		$PayPalRequestData = array(
			'DECPFields' => $DECPFields,
			'Payments' => $Payments,
		);


		/**
		 * Here we are making the call to the DoExpressCheckoutPayment function in the library,
		 * and we're passing in our $PayPalRequestData that we just set above.
		 */
		$PayPalResult = $this->paypal_pro->DoExpressCheckoutPayment($PayPalRequestData);

		/**
		 * Now we'll check for any errors returned by PayPal, and if we get an error,
		 * we'll save the error details to a session and redirect the user to an
		 * error page to display it accordingly.
		 *
		 * If the call is successful, we'll save some data we might want to use
		 * later into session variables, and then redirect to our final
		 * thank you / receipt page.
		 */
		if(!$this->paypal_pro->APICallSuccessful($PayPalResult['ACK']))
		{
			$errors = array('Errors'=>$PayPalResult['ERRORS']);
			$this->load_paypal_error_view($errors);
		}
		else
		{
			// Successful call.
			/**
			 * Once again, since Express Checkout allows for multiple payments in a single transaction,
			 * the DoExpressCheckoutPayment response is setup to provide data for each potential payment.
			 * As such, we need to loop through all the payment info in the response.
			 *
			 * The library helps us do this using the GetExpressCheckoutPaymentInfo() method.  We'll
			 * load our $payments_info using that method, and then loop through the results to pull
			 * out our details for the transaction.
			 *
			 * Again, in this case we are you only working with a single payment, but we'll still
			 * loop through the results accordingly.
			 *
			 * Here, we're only pulling out the PayPal transaction ID and fee amount, but you may
			 * refer to the API reference for all the additional parameters you have available at
			 * this point.
			 *
			 * https://developer.paypal.com/docs/classic/api/merchant/DoExpressCheckoutPayment_API_Operation_NVP/
			 */
			foreach($PayPalResult['PAYMENTS'] as $payment)
			{
				$cart['paypal_transaction_id'] = isset($payment['TRANSACTIONID']) ? $payment['TRANSACTIONID'] : '';
				$cart['paypal_fee'] = isset($payment['FEEAMT']) ? $payment['FEEAMT'] : '';
			}

			// Set example cart data into session
			$this->session->set_userdata('shopping_cart', $cart);

			// Successful Order
			redirect($cart['fulfillment_url']);
		}
	}


// 	/**
// 	 * Order Complete - Pay Return Url
// 	 */
// 	function OrderComplete()
// 	{
// 		// Get cart from session userdata
// 		$cart = $this->session->userdata('shopping_cart');
// 		if($this->invalid_cart_q($cart)){
// 			$this->die_error(__METHOD__,'Missing parameters.');
// 		}
// 
// 		if(empty($cart)) redirect('paypal/demos/express_checkout');
// 
// 		// Set cart data into session userdata
// 		$this->load->vars('cart', $cart);
// 
// 		// Successful call.  Load view or whatever you need to do here.
// 		$this->load->view('paypal/demos/express_checkout/payment_complete');
// 	}

	/**
	 * Order Cancelled - Pay Cancel Url
	 */
	function OrderCancelled()
	{
		$this->session->unset_userdata('PayPalResult');

		// Get cart data from session userdata, then delete it
		$cart = $this->session->userdata('shopping_cart');
		$this->session->unset_userdata('shopping_cart'); 

		if(! $this->invalid_cart_q($cart)){
			$invoice_number=$cart['invoice_number'];
			$this->session->set_userdata('cancelled_invoice_number',$invoice_number);
			$cancel_url=(!empty($cart['cancel_url']))?$cart['cancel_url']:null;
		}

		if(!empty($cancel_url)){ 
			// will assume the cancel URL will handle deleting the purchase and whatever
			redirect($cancel_url);
		}else{
			// otherwise we do it now
			if(!empty($invoice_number)){$this->accounting_parando->delete_purchase($invoice_number);}

			$this_organization = $this->model_parando->organization;
			$base_url = base_url();
			$contact_url = $this->model_parando->contact_url;

			$form_body=<<<EOT
<p>Your order has been canceled. If you have any questions, please <A HREF='$contact_url'>contact us</A>.</p>
<p>Or return to <A HREF='$base_url'>$this_organization web site</a>.</p> 
EOT;
			$this->load->view('form_header',['form_title'=>'Order Canceled','form_subject'=>'Order Canceled']);
			$this->load->view('form_simple',['form_action'=>'','form_body'=>$form_body,'form_submit'=>'']);
			$this->load->view('form_footer',[]);
		}
	}

}
