<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed'); 

 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

class Login extends MY_Controller {

	function __construct(){		
		parent::__construct();	 		

		$this->load->library('form_validation');

		$params = ['tag' => 'login_session_data'];
		$this->load->library('randosession',$params);
	} 	

	private function login_view(){

		$newpass_url = site_url('member/newpass');

		$form_subject = $this->model_parando->club_name . " Login";
		$form_title = $form_subject;
		$form_action = site_url("login/login_user");
		$rusa_id=set_value('rusa_id');
 		$form_body = <<<EOT
<div class=form-group>
<label for="rusa_id"><b>RUSA Number (or Last Name)</b></label>
<input type="text" placeholder="Enter your RUSA number, or Last Name" name="rusa_id" value="$rusa_id" required>
</div>
<div class=form-group>
<label for="psw"><b>Password</b></label>
<input type="password" placeholder="Enter Password" name="psw" required>
</div>
EOT;

		$form_body .= validation_errors('<p class="error">');
		$form_submit = '<button type="submit">Login</button>';
		$form_fineprint = '<p><center>Need a new password? <A HREF="' . $newpass_url . '">Click Here.</a></center></p>';
	
		$data=compact(
			'form_title',
			'form_subject',
			'form_fineprint',
			'form_action',
			'form_body',
			'form_submit',
			'form_fineprint'
		);
	
		$this->load->view('form_header',array_merge($data,['page_styles'=>['form']]));
		$this->load->view('form_simple',$data);
		$this->load->view('form_footer',$data);

	}


	private function init(){
			$d=array(
				'login_member_id'	=> null,
				'login_is_admin'	=> null,
				'login_rusa_id'		=> null,
				'login_first_name'	=> null,
				'login_last_name'	=> null,
				'login_expires'		=> null,
				'login_logged_in'	=> false
			);
			$this->randosession->init($d);  	  
	}

	public function index()	{		
		$sd = $this->randosession->get_data_all();
		if(isset($sd['login_logged_in']) && true==$sd['login_logged_in']){
			$member_id=$sd['login_member_id'];
			if($sd['login_is_admin']){
				redirect(site_url("/admin/event/future"));			
			}else{
				$status='login_user';
				redirect(site_url("/admin/profile/edit/$member_id"));			
			}
		}else{
			$this->login_view();	
		}
	}	

	public function logout_user(){		
		$this->init();			
		$this->login_view();	
	}		

	public function login_user(){		  
	

			$this->init();
			$this->form_validation->set_rules('rusa_id','RUSA Number', 'trim|required');
			$this->form_validation->set_rules('psw','Password', 'trim|required');
			if($this->form_validation->run()==FALSE){		
				$this->login_view();    	 
			}else{	  
				$rusa_id=$this->input->post('rusa_id');	  
				$password=$this->input->post('psw');	
	
				$result = $this->model_parando->check_credentials(trim($rusa_id), $password);	
				$status = $result['status'];
	
				if($status == 'login_admin' || $status == 'login_user'){
						$row = $result['data'];
						$member_id = $row->id;
					$session_data=array(
						'login_member_id'	=> $row->id,
						'login_is_admin'	=> $row->administrator,
						'login_rusa_id'		=> $row->rusa_id,
						'login_first_name'	=> $row->first_name,
						'login_last_name'	=> $row->last_name,
						'login_expires'		=> $row->expires,
						'login_logged_in'	=> true
					);
					$this->randosession->set_data_array($session_data);
				}
			
	
			switch($status){		  
				case 'login_admin':			
					redirect(site_url("/admin/event/future"));			
					break;		  
				case 'login_user':			
					// redirect(site_url("/admin/profile/edit/$member_id"));			
					redirect(site_url("/admin/event/future"));			
					break;		  
				default:			
					$this->login_view();			
					break;	
			}	
		}	  
	}		
	
	public function member($token){
		$result = $this->model_parando->check_member_code($token);
		if(is_array($result)){
			$this->init();
			list($member_id, $e, $hash)=$result;
			$m=$this->model_parando->get_member($member_id);
			
			$session_data=array(
				'login_member_id'	=> $m['id'],
				'login_is_admin'	=> $m['administrator'],
				'login_rusa_id'		=> $m['rusa_id'],
				'login_first_name'	=> $m['first_name'],
				'login_last_name'	=> $m['last_name'],
				'login_expires'		=> $m['expires'],
				'login_logged_in'	=> true
			);
			$this->randosession->set_data_array($session_data);
				
					redirect(site_url("/admin/profile/edit/$member_id"));			
			
		}else{
			$this->die_error('Invalid credentials: ', $result);
		}
	}

}
