<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
 
 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

class Member extends MY_Controller {

	
  // CONSTRUCTEUR

	public function __construct() {
		parent::__construct();
// 		$this->load->database();
// 		$this->load->model('model_parando');
// 		$this->load->helper('url');
		$this->load->model('accounting_parando');
		$this->load->helper('form');
		$this->load->library('email');
		$this->load->library('form_validation');

		$this->load->library('multipage');	
		$this->multipage->define_pages($this->page);  // SLOW -- Library parameter?
		$this->multipage->set_view('form_simple','form_header','form_footer',['form']);
//		$this->multipage->set_subject('Password Reset');
		$this->multipage->set_controller(strtolower(get_class($this)));

		$this->session->set_userdata('paypal_sandbox', false);

	}
	
	private function member_session_init(){
		$d=[
			'this_organization' => $this->model_parando->organization,
			'base_url' => base_url(),
			'contact_url' => $this->model_parando->contact_url,
			'webmaster_contact_url' => $this->model_parando->webmaster_contact_url,
			'admin_login_url' => $this->model_parando->admin_login_url
		];
		$this->multipage->init($d);
	}
	

// ERROR VIEW LOADER
// 	private function die_error($heading,$message){
// 		$this->model_parando->die_error($heading, $message);
// 	}

	// PAGE LIST DEFINITION
	

	public function _get_country_options(){
		$country_fullname = $this->model_parando->get_countries();
		return $country_fullname;
	}
	
	public function _get_state_options(){ 
		$state_fullname = $this->model_parando->get_states();
		return $state_fullname;
	}

	public function _phone_check() {

		// Validation callback to verify that at least one phone number was entered

		if($this->input->post('contact_phone') || $this->input->post('contact_phone_alt')){
				return TRUE;
		}else{
				$this->form_validation->set_message('_phone_check', 'Please enter atleast one phone number.');
		return FALSE;
		}
	}

	public $page = [
				'pick_member' =>
						[
							'title' => 'Select Rider',
							'form_body' => 'pick_member_form_body'
						],
				'new_member' =>
						[
							'title' => 'Welcome New Rider',
							'next_page' => 'get_new_member_data',
							'fields' => [],
							'comment' => <<<'EOT'
<h2>Welcome new rider!</h2>
<p>You were not found in the {$this_organization} rider database. </p>
<p>Please click "continue" and enter your contact information. </p>
EOT
						],
				'get_new_member_data' =>
						[
							'title' => 'New Rider Info',
							'next_page' => 'pay_for_membership',
							'comment' => '<p>As a new rider with  {$this_organization} we need to obtain your contact information. Starred items are required. Please enter your information below:</p>',
							'form_fineprint' => '<p>Privacy Policy: This information is used by {$this_organization} <em>only</em> to contact you with respect to this event, to notify you and simplify your 
										registration with respect to future events, and in case of emergency. For more details see full <A HREF="{$base_url}privacy.html">privacy policy</a>.</p>',
							'fields' => [
								['contact_street','Street Address','Enter Street Address',true, 'trim|required|max_length[250]'],
								['contact_city','City','Enter City',true, 'trim|required|max_length[250]'],
								['contact_state','State','Select State or N/A for Non-US',true,'required','select','get_state_options'],
								['contact_zip','Postal Code','Enter Postal Code',true, 'trim|required|alpha_dash|min_length[5]|max_length[10]'],
								['contact_country','Nationality','Select your nationality',true,'required','select','get_country_options'],
								['contact_tracker','Tracker URL','Enter Tracker URL',false, 'trim|valid_url|min_length[5]|max_length[250]'],
								['contact_email','Email','Enter Email',true, 'trim|required|valid_email|max_length[250]'],
								['contact_phone','Cell Phone','Your phone number on the bike',true, 'trim|min_length[10]|max_length[25]|callback__phone_check'],
								['contact_phone_alt','Phone (alternative)','Enter alternative phone (optional)',false, 'trim|min_length[10]|max_length[25]|callback__phone_check'],
								['contact_emergency_name','Emergency contact name','Enter Emergency contact name',true, 'trim|required|max_length[250]'],
								['contact_emergency_phone','Emergency contact phone','Enter Emergency contact phone',true, 'trim|required|min_length[10]|max_length[25]'],
								['contact_emergency_phone_alt','Emergency contact phone (alternative)','Enter alternative emergency contact phone (optional)',false, 'trim|min_length[10]|max_length[25]'],
								['password','Password','Enter a new password for access to your member profile',false, 'trim|min_length[8]|max_length[25]','password']
							]
						],
				'pw_reset' =>
						[
							'title' => 'Password Reset',
							'comment' => <<<'EOT'
<p>You may use this form to reset your password. Enter the basic information below. 
A password reset link will be sent to  your email address of record. </p>
EOT
,
							'form_fineprint' => <<<'EOT'
<p>If you 
believe your email address may not be correct in our records, or if 
you have any other questions, please contact the <A HREF='{$webmaster_contact_url}'>{$this_organization} Webmaster</a>.</p>
EOT
,
							'fields' => [
								['first_name','First Name','Enter first name', true, 'trim|required|regex_match[/^([a-zA-Z0-9]+ ?)+?$/]|max_length[50]'], 
								['last_name','Last Name','Enter last name',true, 'trim|required|regex_match[/^([a-zA-Z0-9]+ ?)+?$/]|max_length[50]'],
								['rusa_id','RUSA Number','Enter RUSA number',false,'trim|is_natural_no_zero|less_than[1000000]']
							]
						],
				'new_password' =>
						[
							'title' => 'Enter New Password',
							'next_page' => 'done_reset',
							'fields' => [
								['pw','New Password','Enter new password', true, 'trim|required|min_length[8]|max_length[50]|callback__passwords_match','password'], 
								['pw_again','New Password (repeat)','Enter password again',true, 'trim|required|min_length[8]|max_length[50]|callback__passwords_match','password']
							]
						],
				'pwreset_email_sent' =>
						[
							'title' => 'Email Sent',
							'submit' => [],
							'comment' => <<<'EOT'
<p>An email with a link for resetting your password has been sent to  your email address 
of record. If you believe your email address may not be correct in our records, or if 
you have any other questions, please <A HREF='{$webmaster_contact_url}'>contact the {$this_organization} Webmaster</a>.</p>

<hr width=61.8%>

<p>Return to the <A HREF='{$base_url}'>{$this_organization} Home Page</a></p>							

EOT
						],
				'done_reset' =>
						[
							'title' => 'Password Changed',
							'submit' => [],
							'comment' => <<<'EOT'
<p>Your password has been reset. You can log in now with your new password now by <A HREF='{$admin_login_url}'>clicking here</a>.  If 
you have any other questions, please <A HREF='{$webmaster_contact_url}'>contact the {$this_organization} Webmaster</a>.</p>

<hr width=61.8%>

<p>Return to the <A HREF='{$base_url}'>{$this_organization} Home Page</a></p>							

EOT
						],

				'join_renew' =>
						[
							'title' => 'Join or Renew Membership',
							'next_page' => 'undefined',
							'comment' => <<<'EOT'
<p>To join or renew your membership in {$this_organization} enter your basic info below.</p>
EOT
,
							'fields' => [
								['first_name','First Name','Enter first name', true, 'trim|required|regex_match[/^([a-zA-Z0-9]+ ?)+?$/]|max_length[50]'], 
								['last_name','Last Name','Enter last name',true, 'trim|required|regex_match[/^([a-zA-Z0-9]+ ?)+?$/]|max_length[50]'],
								['rusa_id','RUSA Number','Enter RUSA number (required)',true,'required|trim|is_natural_no_zero|less_than[1000000]']
							]
						],

				'pay_for_membership' =>
						[
							'title' => 'Purchase Membership',
							'submit' => [['tag'=>'pay','text'=>'Pay Now'],['tag'=>'cancel','text'=>'Cancel']],
//							'next_page' => 'paypal_button',
							'form_body' => 'pay_form_body',
							'form_fineprint' => '<div class=centered>If for some reason you don\'t want to pay online, you can mail a check or money order along with 
																		<A HREF={$base_url}/ClubApplication.pdf>this form completed.</a></div>'
						],
//				'paypal_button' =>
//						[
//							'title' => 'Payment',
//							'submit' => [['tag'=>'pay','text'=>'Pay Now'],['tag'=>'cancel','text'=>'Cancel']],
//							'view_body' => 'paypal_button'
//						],
				'payment_credited' =>
						[
							'title' => 'Payment Credited',
							'submit' => [],
							'comment' => <<<'EOT'
<p>Your payment of ${$amount_paid} has been credited. Thank You! Payment credit
is contingent on receipt of funds through the payment service. A description
of your order is as follows:</p>

<p>
<div class='transaction'>
<table>
<tr><td>Member Name</td><td>{$first_name} {$last_name}</td></tr>
<tr><td>Amount Paid</td><td>${$amount_paid}</td></tr>
<tr><td>Order ID</td><td>{$invoice_number}</td></tr>
<tr><td>Entered On</td><td>{$entered_on}</td></tr>
</table>
<h2>Items</h2>
<br>
<TABLE>
<TR><TH>Quantity</TH><TH>Description</TH><TH>Value</TH><TH>Unit Price</TH><TH>Ext Price</TH></TR>
{$purchase_items_table_body}
</TABLE>
</div>
</p>


<p>A summary of this order has been emailed to our address of record. 
If you have any questions about this order, please <A HREF='{$contact_url}'>contact {$this_organization}</a>.</p><br>

<hr width=61.8%>

<p>Return to the <A HREF='{$base_url}'>{$this_organization} Home Page</a></p>							

EOT
						],

				'final' =>
						[
							
						]
				];
				
	
	
	// GENERIC PAGE PROCESSING

	public function index() {
		$this->multipage->process_page();
	}




// JOIN or RENEW CONTROLLER METHOD

	public function Xjoin(){
		$this->join_renew();		
	}

	public function Xrenew(){
		$this->join_renew();		
	}
		
	private function join_renew(){
		$this->member_session_init();
		$this->multipage->set_current_page('join_renew');
		$this->multipage->view_current_page();
	}
	
	
// DEPRECIATED -- use load_memdata.. version in model_parando

	public function _join_renew(){
		$this->member_session_init();
		$this->find_and_process_member('load_memdata_into_session', 'pay_for_membership', 'new_member', 'join or renew membership');
	}

	public function _pay_for_membership(){
		$submit=$this->input->post('submit');
		if( $submit == 'cancel' ){
			$this->member_session_init();
			redirect(base_url());
		}elseif($submit=='pay'){
			$point_of_sale = __METHOD__;
			extract($this->multipage->get_data_array(['sku','new_expires','contact_email','first_name','last_name','member_id']));

			$purchase = $this->accounting_parando->create_purchase_order($member_id,
				[['sku'=>$sku, 'quantity'=>1, 'value'=>'expires '.$new_expires]], $point_of_sale, null, "Membership");
			if(is_string($purchase)){$this->die_error(__METHOD__, "Error creating purchase order: $purchase");}
			
			$invoice_number=$purchase['invoice_number'];
			$order_token=$this->accounting_parando->make_order_token($invoice_number);

			// PAYPAL Express Checkout Controller Setup

			// Clear cart from session userdata
			$this->session->unset_userdata('shopping_cart');

			// Set cart data into session
			$cart=$this->accounting_parando->purchase_item_cart($purchase);
			$cart['member_first_name'] = $first_name;
			$cart['member_last_name'] = $last_name;
			$cart['contact_email'] = $contact_email;
			$cart['invoice_number'] = $invoice_number;
			$cart['order_token'] = $order_token;
			$cart['fulfillment_url'] = site_url(strtolower( get_class($this)) . "/credit_paypal_payment/$order_token");

			$this->session->set_userdata('shopping_cart', $cart);
			redirect(site_url('express_checkout/SetExpressCheckout'));

		}	
	}	
	
	public function _pay_form_body(){
		extract($this->multipage->get_data_all());
		
		$body="";
		
		$now = new DateTime();
		$nmonth = $now->format('m');
		$nyear = $now->format('Y');
		$october = $this->model_parando->membership_grace_month;
		$new_xyear = ($nmonth >= $october) ? $nyear+1 : $nyear;
		$new_expires = $new_xyear . '-12-31';
		$new_xdate = new DateTime($new_expires);
		
		
					
		if($expires!="0000-00-00"){ // current or former club member
			$xdate = new DateTime($expires);
			if($xdate >= $new_xdate){
				$this->die_error('Membership Term', "Your membership term ends $expires and cannot be extended at this time.");
			}
				
			if($xdate > $now){
				$sku="current_renew";
				$product=$this->accounting_parando->get_product($sku);
				$membership_price=$product['price'];
				$body=<<<EOT
<p>You are a member in good standing with $this_organization. Thank you for your support. 
Your membership is active now, and will expire $expires. 
You can extend your membership term to $new_expires by paying \$$membership_price now.</p>
<p>If you would like to pay online to extend your membership term to $new_expires, please click the "Pay Now" button below.</p>
EOT;
			}else{
				$sku="expired_renew";
				$product=$this->accounting_parando->get_product($sku);
				$membership_price=$product['price'];
				$body=<<<EOT
<p>You were once a member of $this_organization. Thank you for your support in the past. 
Your membership term expired on $expires. You can reactivate
your membership with a term ending $new_expires by paying \$$membership_price now.</p>
<p>If you would like to pay online  to activate your membership till $new_expires, please click the "Pay Now" button below.</p>
EOT;
			}
		}else{
				$sku="new_member_join";
				$product=$this->accounting_parando->get_product($sku);
				$membership_price=$product['price'];
				$body=<<<EOT
<p>You are not currently a member of $this_organization. Please join us! Membership in $this_organization
has many benefits. You can join our club with a membership term ending $new_expires by paying \$$membership_price now.</p>
<p>If you would like to pay online  to activate your membership till $new_expires, please click the "Pay Now" button below.</p>
EOT;
		}

		$this->multipage->set_data_array(['sku'=>$sku, 'new_expires'=>$new_expires, 'amount_due'=>$membership_price]);

$body .= <<<EOT
<br><br>
<div class='centered'><img src="https://www.paypalobjects.com/webstatic/en_US/i/buttons/cc-badges-ppmcvdam.png" width=70% alt="Pay now with PayPal" /></div>
EOT;

		return $body;
	}


	
	//////////////////////////////////////////////////////////////////////
	// Public controller generic credit/fulfillment
	
	
	// DEPRECIATED -- use version in accounting_parando
	
	public function credit_paypal_payment($token){
	
		$data=[];
		
		if(empty($token)){
			$this->die_error(__METHOD__, 'Missing order token.');
		}else{
			$result = $this->accounting_parando->check_order_token($token);
			if(is_array($result)){
				list($member_id, $token_time, $hash)=$result;
				$invoice_number="$member_id-$token_time";
				
				$member=$this->model_parando->get_member($member_id);
// DEPRECIATED -- use version in model_parando
				$this->load_memdata_into_session($member);	

				$purchase=$this->accounting_parando->get_purchase($invoice_number);
				if(empty($purchase)){$this->die_error(__METHOD__, 'No such purchase.');}

				$payment=$this->accounting_parando->get_payment($purchase['id']);

				if(empty($payment)){ // not credited or fulfilled yet, so do it
				
					$rc=$this->fulfill_all($purchase);  
					
					if('OK'!=$rc){$this->die_error(__METHOD__,"Could not fulfill purchase: $rc");}

					$point_of_sale = __METHOD__;
					$cart = $this->session->userdata('shopping_cart');
					if(empty($cart)){$this->die_error(__METHOD__,'No shopping cart in session.');}
					if(empty($cart['shopping_cart']['grand_total'])){$this->die_error(__METHOD__,'Nothing paid.');}
					if(empty($cart['paypal_transaction_id'])){$this->die_error(__METHOD__,'Missing transaction ID.');}
					$paypal_transaction_id=$cart['paypal_transaction_id'];
					$transaction_info = print_r($cart,true);
					$amount_paid=$cart['shopping_cart']['grand_total'];
					$this->accounting_parando->post_payment($purchase, $point_of_sale, $amount_paid, $transaction_info);
					$payment=$this->accounting_parando->get_payment($purchase['id']);
					if(empty($payment)){$this->die_error(__METHOD__,"Failed transaction readback");}
				}
				
				$odata=array_merge($payment,
					[  
						'amount_paid'=>$amount_paid,
						'order_token'=>$token, 
						'invoice_number'=>$invoice_number, 
						'paypal_transaction_id'=>$paypal_transaction_id,
						'purchase_items_text'=>$this->accounting_parando->purchase_item_text($purchase),
						'purchase_items_table_body'=>$this->accounting_parando->purchase_item_table_body($purchase)  ]
					);
				
				$data=array_merge($member,$odata);
				$this->multipage->set_data_array($odata);	
				$this->send_payment_confirmation_email($data);

				$this->multipage->set_current_page('payment_credited');
				$this->multipage->view_current_page();
			}else{
				$this->die_error(__METHOD__, $result);
			}
		}
		
		return $data;
		
	}

	private function fulfill_all($purchase){
		$purchase_id=$purchase['id'];
		$items=$this->accounting_parando->get_purchase_items($purchase_id);
		if(!empty($items)){
			foreach($items as $i){
				$sku =  $i['sku'];
				$fulfillment_method = 'fulfill_' . $sku;
				if(method_exists($this,$fulfillment_method)){
					$rc = $this->{$fulfillment_method}($purchase, $i);
					if('OK'!=$rc){return $rc;}
				}else{
					return "No method to fulfill sku: $sku";
				}
			}
		}else{
			return "No items in purchase ID $purchase_id";
		}
		return "OK";
	}
	
	private function fulfill_current_renew($purchase, $item){
		return $this->fulfill_join_renew($purchase, $item);
	}

	private function fulfill_expired_renew($purchase, $item){
		return $this->fulfill_join_renew($purchase, $item);
	}

	private function fulfill_new_member_join($purchase, $item){
		return $this->fulfill_join_renew($purchase, $item);
	}

	private function fulfill_join_renew($purchase, $item){
	
		$value = $item['value'];
		$member_id = $purchase['member_id'];
			
		if(!preg_match("/(\d{4})-(\d{2})-(\d{2})/", $value,$a)){return "Misformatted item value: '$value'";}
		list($new_expires, $xyear, $xmonth, $xdate)=$a;				
								
		$this->db->where('id',$member_id);
		$rc=$this->db->update('member',['expires'=>$new_expires]);
		if(FALSE==$rc){$this->die_error(__METHOD__,"Failed to update membership term expiry date");}
		
		return 'OK';

	}


	private function send_payment_confirmation_email($data){
		
		extract($data);
		$rba=$this->model_parando->get_rba();
		$rba_name=$rba['first_name'] . ' ' . $rba['last_name'];
		$rba_email=$rba['m_email'];
		$this_organization = $this->model_parando->organization;
		$this_controller=strtoupper(get_class($this));
		
		$order_info=<<<EOT
Member Name:	  {$first_name} {$last_name}
Amount Paid:      \${$amount_paid}
Order ID:         $invoice_number
Entered On:       {$entered_on}

Items: 

$purchase_items_text

EOT;

		$rider_message=<<<EOT
Your payment of \$$amount_paid has been credited. Thank You! Payment credit
is contingent on receipt of funds through the payment service. A description
of your order is as follows:

{$order_info}

EOT;

		$rba_message=<<<EOT
A payment of \$$amount_paid has been credited. Payment credit
is contingent on receipt of funds through the payment service. 
A description of the order is as follows:

{$order_info}

EOT;

		$contact_rba=<<<EOT

If you have any questions or concerns, contact $this_organization
at $rba_email by simply replying to this email. 
EOT;

		$contact_rider=<<<EOT

You can contact the rider $first_name $last_name at $m_email
by simply replying to this email. 
EOT;

		$this->email->from($rba_email, $rba_name, $this->model_parando->postmaster_email);
		$this->email->to($m_email);
		$this->email->subject($this_organization . " Payment Confirmation");
		$this->email->message($rider_message.$contact_rba);
		$this->email->send();

		$this->email->from($m_email, "$first_name $last_name", $this->model_parando->postmaster_email);
		$this->email->to($rba_email);
		$this->email->subject("[$this_controller] " . $this_organization . " Payment received");
		$this->email->message($rba_message.$contact_rider);
		$rc = $this->email->send();
		return $rc;
	}
	




// PASSWORD RESET CONTROLLER METHOD

	public function newpass($token=null){

		$this->member_session_init();
		
		if(empty($token)){
			$this->multipage->set_current_page('pw_reset');
			$this->multipage->view_current_page();
		}else{
			$result = $this->model_parando->check_member_code($token);
			if(is_array($result)){
						$this->multipage->set_data('token',$token);
						$this->multipage->set_current_page('new_password');
						$this->multipage->view_current_page();
			}else{
				$this->die_error('Password Reset Error', $result);
			}
		}
	}
	

	

	public function _pw_reset(){
		$this->find_and_process_member('pwreset_known', 'pwreset_email_sent', null, 'reset your password');
	}

	public function _new_password(){
		$pw=$this->input->post('pw');
		$token=$this->multipage->get_data('token');
		$result=$this->model_parando->check_member_code($token);
		if(is_string($result)){$this->die_error('new_password',"Bad token: $result");}
		list($m, $e, $hash)=$result;
		$pw_hash=password_hash($pw,PASSWORD_DEFAULT);
		$this->db->where('id',$m);
		$rc=$this->db->update('member',['password'=>$pw_hash]);
		if(FALSE==$rc){$this->die_error('new_password',"Failed to update password");}
	}
			
	public function _passwords_match() {

		// form validation callback to make sure both entered passwords match

		if($this->input->post('pw') == $this->input->post('pw_again')){
				return TRUE;
		}else{
				$this->form_validation->set_message('_passwords_match', 'Passwords do not match.');
		return FALSE;
		}
	}


	private function pwreset_known($mem){

		// $cto=$this->model_parando->contact_this_organization_link();

		// Saves data into the session for a matched member.

		$rusa_id=$mem['rusa_id'];
		if(!empty($rusa_id)){
			$rms="Yes (". $rusa_id . ")";
		}else{
			$rms="No";
		}

		if($mem['expires']!="0000-00-00"){
			$delta=(time() - strtotime($mem['expires']));
			if($delta <= 0){
				$membership="current";
			}elseif($delta > 0 && $delta < 24*3600*60 ){
				$membership="renewing";
			}else{
				$membership="terminated";
			}
		}else{
			$membership="guest";
		}

		$memdata=array(
			'membership'	=> $membership,
			'rms'		=> $rms,
			'rusa_id'		=> $mem['rusa_id'],
			'first_name'	=> $mem['first_name'],
			'last_name'		=> $mem['last_name'],
			'member_id'		=> $mem['id'],
			'administrator'	=> $mem['administrator'],
			'expires'		=> $mem['expires'],
			'contact_street'	=> $mem['m_street'],
			'contact_city'	=> $mem['m_city'],
			'contact_state'	=> $mem['m_state'],
			'contact_zip'	=> $mem['m_zip'],
			'contact_tracker'	=> $mem['tracker'],
			'contact_phone'	=> $mem['m_phone'],
			'contact_phone_alt'	=> $mem['m_phone_alt'],
			'contact_email'	=> $mem['m_email'],
			'contact_emergency_name'	=> $mem['emergency_name'],
			'contact_emergency_phone'	=> $mem['emergency_phone'],
			'contact_emergency_phone_alt'	=> $mem['emergency_phone_alt']
		);

	
		return $this->send_pwreset_email($memdata);

	}

	private function send_pwreset_email($memdata){
		
		extract($memdata);
		$webmaster=$this->model_parando->get_webmaster();
		extract($webmaster);
		$this_organization=$this->model_parando->organization;

		$member_code=$this->model_parando->make_member_code($member_id);
		$newpass_url=site_url(strtolower(get_class($this)) . "/newpass/$member_code");
		$this->multipage->set_data('member_code',$member_code);
		$this->multipage->set_data('newpass_url',$newpass_url);

		$rider_message=<<<EOT
Someone, possibly you, has asked to reset your access password
to the $this_organization web site. Your information is as follows:

Name:  $first_name $last_name
RUSA Member: $rms
Club Status: $membership
Club Expires: $expires

If you would like to reset your password, please use the following link 
within the next few minutes:

  $newpass_url

EOT;


		$webmaster_message=<<<EOT
Someone, possibly the user, has asked to reset their access password
to the $this_organization web site.


Name:  $first_name $last_name
RUSA Member: $rms
Club Status: $membership
Club Expires: $expires

EOT;

		$contact_webmaster=<<<EOT

If you have any questions or concerns, contact Webmaster $webmaster_name
at $webmaster_email by simply replying to this email. 
EOT;

		$contact_rider=<<<EOT

You can contact the rider $first_name $last_name at $contact_email
by simply replying to this email. 
EOT;

		$this->email->from($webmaster_email, $webmaster_name, $this->model_parando->postmaster_email);
		$this->email->to($contact_email);
		$this->email->subject("$this_organization Web Site");
		$this->email->message($rider_message.$contact_webmaster);
		$this->email->send();

		$this->email->from($contact_email, "$first_name $last_name", $this->model_parando->postmaster_email);
		$this->email->to($webmaster_email);
		$this->email->subject("$this_organization Web Site");
		$this->email->message($webmaster_message.$contact_rider);
		$rc = $this->email->send();
		return $rc;
	}
	


// GENERIC DECISION LOGIC FOR KNOWN/UNKNOWN MEMBER

	private function find_and_process_member($action_callback, $action_page, $unknown_page=null, $activity="continue"){
		$cto=$this->model_parando->contact_this_organization_link();
		$error_comment=" You cannot $activity until this is fixed. If you have questions about this error, $cto.";

		$rusa_id=$this->input->post('rusa_id');
		$first_name=$this->input->post('first_name');
		$last_name=$this->input->post('last_name');

		$this->load_unknown($first_name,$last_name,$rusa_id);

		$found=$this->model_parando->find_member($first_name,$last_name,$rusa_id);	
		
		switch($found['result']){
		  case 'full_match_rusa':
			$q=$found['query'];
			$member=$q->row_array();
			$rc = $this->{$action_callback}($member);
			if($rc){
				$this->multipage->set_current_page($action_page);
			}else{
				$this->die_error('Database Error', "Failed to $activity." . $error_comment);
			}
			break;
		  case 'duplicate_entry':
			$this->die_error('Database Error','Your info matches two entries in our database.'.$error_comment);
			break;
		  case 'unknown':
			if(!empty($unknown_page)){
				$this->multipage->set_current_page($unknown_page);
			}else{
				$this->die_error('Database Error','Your info is not in our database.'.$error_comment);
			}
			break;
		  default:
			$this->multipage->set_current_page('pick_member'); 
			$q=$found['query'];
			$found_multiple = $q->result_array();
			$this->multipage->set_data_array([
				'found_multiple' => $found_multiple,
				'find_and_process_parameters' => [
					'action_callback' => $action_callback,
					'action_page'=>$action_page,
					'unknown_page'=>$unknown_page,
					'activity'=>$activity 
				]
			]);
			break;
		}
	}
	
  private function load_unknown($first_name,$last_name,$rusa_id){

			// loads session with what we do know about an un-matched member.
			// Called prior to the member picker or unknown member data entry

			if(!empty($rusa_id)){
				$rms="Yes (". $rusa_id . ")";
			}else{
				$rms="No";
			}

			$session_data=array(
				'membership'	=> 'unknown',
				'rms'		=> $rms,
				'rusa_id'	=> $rusa_id,
				'first_name'	=> $first_name,
				'last_name'	=> $last_name,
				'expired'	=> null,
				'member_id'	=> null,
				'administrator'	=> null,
				'expires'	=> null
			);
			$this->multipage->set_data_array($session_data);
	  
	}


// PICK MEMBER 

	public function _pick_member(){

		// Processing method for the register_pick_member page 
		// Requires Find and Process parms in session
		// will branch to $unknown_page if no existing member of the same name/rusa
		// or do the action callback (and branch to action page) if there's now a full member match

		$fpp=$this->multipage->get_data('find_and_process_parameters');
		if(empty($fpp)){$this->die_error('_pick_member', 'Required parameters NOT in session.');}
		extract($fpp);

 		$this->form_validation->set_rules('member_id','Member Selection', 'trim|required', array('required'=>'You must either select a member above that matches you, or select below that none are you.') );

		$cto=$this->model_parando->contact_this_organization_link();
		$error_comment=" You cannot $activity untill this is fixed. If you have questions about this error, $cto.";
		
	  	$member_id=$this->input->post('member_id');
		$rusa_id= $this->multipage->get_data('rusa_id');
		$first_name=$this->multipage->get_data('first_name');
		$last_name= $this->multipage->get_data('last_name');
	  
	  if($this->form_validation->run()!=FALSE){
			if($member_id == "not_me"){
				$check=$this->model_parando->check_unknown($first_name,$last_name,$rusa_id);
				switch($check){
					case "existing_first_last":
						$this->die_error('Existing Member',
							"There is already a rider in our database with exactly the same first and last name as you have entered, yet you say this is not you. 
							$error_comment.");   
						break;
					case "existing_rusa":
						$this->die_error('Existing Member',
							"There is already a rider in our database with exactly the same RUSA ID as you have entered, yet you say this is not you. 
							$error_comment.");   
						break;
					case "ok":
						if(!empty($unknown_page)){
							$this->multipage->set_current_page($unknown_page);
						}else{
							$this->die_error('Database Error','Your info is not in our database.'.$error_comment);
						}
						break;
				}
			}else{
				$mem=$this->model_parando->get_member($member_id);
				if(empty($mem)){
					$this->die_error('_pick_member',"No such member ID ($member_id)");
				}elseif(!empty($action_callback) && method_exists($this,$action_callback)){
					$this->{$action_callback}($mem);
					$this->multipage->set_current_page($action_page);
				}elseif(!empty($action_callback)){
					$this->die_error('_pick_member',"No such action callback ($action_callback)");
				}
			}
		}
	}


	public function _pick_member_form_body(){

		// Callback that generates a radio button form for picking 
		// from multiple matched members

		$found=$this->multipage->get_data('found_multiple');

		$num_rows=sizeof($found);
		if($num_rows>1){
			$entries="one or more entries";
			$the_entry_that="the entry that";
			$not_me="None of those are me";
		}else{
			$entries="one entry";
			$the_entry_that="it if it";
			$not_me="That is not me.";
		}

		$body = <<<EOT
<p>Your info partially matches $entries in our rider database. Please select $the_entry_that exactly matches you, or select <em>not me</em></p><br>
<div class=roster>
<table width=100%>

EOT;

		foreach ($found as $row)
		{
			$member_id=$row['id'];
			$rusa_id=(!empty($row['rusa_id']) && $row['rusa_id']>0)?("RUSA " . $row['rusa_id']):"";
			if($row['m_country']=='US'){
				$state = $row['m_state'];
			}else{
				$country_fullname=$this->model_parando->get_countries();
				$state = $country_fullname[$row['m_country']];
			}
	    $body .= "<tr>";
	    $body .= "<td><input type='radio' name='member_id' value=$member_id></td>";
	    $body .= "<td>" . $row['first_name'] . "</td>";
	    $body .= "<td>" . $row['last_name'] . "</td>";
	    $body .= "<td>" . $row['m_city'] . "</td>";
	    $body .= "<td>" . $state . "</td>";
	    $body .= "<td>" . $rusa_id . "</td>";
	    $body .= "</tr>";
		}

		$body .= "</table></div>";

	if(!empty(form_error('member_id'))){
	  $body .= "<div class=verror>" . form_error('member_id') . "</div>";
	}

	$body .= "<p><input type='radio' name='member_id' value='not_me'>$not_me</p>";


		return $body;
	}

	
	public function _get_new_member_data(){
	
	// Processing method for get_new_member_data page
	// Stores aquired data into database and proceeds with known member
	
		// Stuff already in session
		$sdata = $this->multipage->get_data_array(['rusa_id','first_name','last_name']);
		
		$pw = $this->input->post('password');
		$pw_hash = (empty($pw))?"":password_hash($pw,PASSWORD_DEFAULT);
		
		// Stuff added by contact form
		$fdata = array(
			'm_street'		=> $this->input->post('contact_street'),
			'm_city'		=> $this->input->post('contact_city'),
			'm_state'		=> $this->input->post('contact_state'),
			'm_zip'			=> $this->input->post('contact_zip'),
			'm_country'			=> $this->input->post('contact_country'),
			'tracker'			=> $this->input->post('contact_tracker'),
			'm_phone'		=> $this->input->post('contact_phone'),
			'm_phone_alt'		=> $this->input->post('contact_phone_alt'),
			'm_email'		=> $this->input->post('contact_email'),
			'emergency_name'	=> $this->input->post('contact_emergency_name'),
			'emergency_phone'	=> $this->input->post('contact_emergency_phone'),
			'emergency_phone_alt'	=> $this->input->post('contact_emergency_phone_alt'),
			'password' => $pw_hash
		);
		$this->db->insert('member', array_merge($sdata,$fdata));	// Add new member to database

		$member = $this->model_parando->get_member_firstlast($sdata['first_name'],$sdata['last_name']);
		if(empty($member)){$this->die_error('get_new_member_data',"This can't happen -- failed member readback.");}

		$this->load_memdata_into_session($member);  // DEPRECIATED -- use version in model_parando

	}

    // DEPRECIATED -- use version in model_parando

	private function load_memdata_into_session($mem){

		// $cto=$this->model_parando->contact_this_organization_link();

		$rusa_id=$mem['rusa_id'];
		if(!empty($rusa_id)){
			$rms="Yes (". $rusa_id . ")";
		}else{
			$rms="No";
		}

		if($mem['expires']!="0000-00-00"){
			$delta=(time() - strtotime($mem['expires']));
			if($delta <= 0){
				$membership="current";
			}elseif($delta > 0 && $delta < 24*3600*60 ){
				$membership="renewing";
			}else{
				$membership="terminated";
			}
		}else{
			$membership="guest";
		}

		$memdata=array(
			'membership'	=> $membership,
			'rms'		=> $rms,
			'rusa_id'		=> $mem['rusa_id'],
			'first_name'	=> $mem['first_name'],
			'last_name'		=> $mem['last_name'],
			'member_id'		=> $mem['id'],
			'administrator'	=> $mem['administrator'],
			'expires'		=> $mem['expires'],
			'contact_street'	=> $mem['m_street'],
			'contact_city'	=> $mem['m_city'],
			'contact_state'	=> $mem['m_state'],
			'contact_zip'	=> $mem['m_zip'],
			'contact_tracker'	=> $mem['tracker'],
			'contact_phone'	=> $mem['m_phone'],
			'contact_phone_alt'	=> $mem['m_phone_alt'],
			'contact_email'	=> $mem['m_email'],
			'contact_emergency_name'	=> $mem['emergency_name'],
			'contact_emergency_phone'	=> $mem['emergency_phone'],
			'contact_emergency_phone_alt'	=> $mem['emergency_phone_alt']
		);

		$this->multipage->set_data_array($memdata);
	
		return true;

	}


}


?>
