<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.


class Qroute extends MY_Controller {

	function __construct(){
		parent::__construct();
		$this->load->library('rwgps');
	}
	
	// PUBLIC CONTROLLER FUNCTIONS
	
	public $required_event_attributes=[
		
	];
	
	public function info($route_id=0){
	
		$warnings=[];
	
		// The Route
		if($route_id+0 <= 0) $this->die_error(__METHOD__, "Missing or invalid RWGPS route ID");
		$raw_route=$this->rwgps->get_route($route_id);
		if(empty($raw_route)) $this->die_error(__METHOD__, "Unknown or invalid RWGPS route");
		$route=$this->munge_route($raw_route);
		$warnings=array_merge($warnings, $route['route_warnings']);

		// The Controls
		
		list($controles, $ec_warnings) = $this->rwgps->extract_controles($raw_route);
		$route['ncontroles']=$ncontroles=count($controles);
		$route['ec_warnings']=$ec_warnings;
		
		//$route=$this->munge_controls($route,$controles);

		// The Event
		
		$route_event=$route; // TODO

		// Control calculations

		// $this->controletimes->compute_open_close($controles,$route_event);
		// $this->controlecode->compute_info_code($controles,$route_event);

		// View
		$view=$this->input->post('view');
		if(empty($view)){
			$this->view_html($route_event, $controles, $warnings);
		}elseif(method_exists($this,'view_'.$view)){
			$this->{'view_'.$view}($route_event, $controles, $warnings);
		} else {
			$this->die_error(__METHOD__, "No view defined for '$view'");
		}
	}

	private function munge_controls($route, $controles){
	
		extract($route);

		$route=$this->rwgps->get_route($route_id);

		$controles_table_body="";
		$controles_table_body.= "<TR><TH>Controle</TH><TH>Distance</TH><TH>Open/Close</TH><TH>Note</TH><TH>Description (attributes)</TH><TH>Info Code</TH></TR>";
		for($i=0; $i<$ncontroles; $i++){
			$cd_mi=round($controles[$i]['d']/(rwgps::m_per_km*rwgps::km_per_mi),1);
			$cd_km=round($controles[$i]['d']/(rwgps::m_per_km),1);
			$is_start=(isset($controles[$i]['start']))?" [START]":"";
			$is_finish=(isset($controles[$i]['finish']))?" [FINISH]":"";
			$controle_num=$i+1;
			$open_str=$controles[$i]['open']->format('m-d H:i');
			$close_str=$controles[$i]['close']->format('m-d H:i');
			// $ca=print_r($controles[$i]['attributes'], true);
			$ca=$this->format_attributes($controles[$i]['attributes']); 
			$cn=(isset($controles[$i]['n']))?$controles[$i]['n']:"";
			$info_code=$controles[$i]['info_code']; 
			$qrcode_basefile="$info_code.png";
			$qrcode_file=self::QRIMG_PATH . '/' . $qrcode_basefile;
			$this->load->library('ciqrcode');
			if(true || !file_exists($qrcode_file)){
				$params['data'] = "$checkin_url/$event_date/$event_distance/$info_code";
				$params['level'] = 'H';
				$params['size'] = 8;
				$params['savename'] =$qrcode_file;
				if(!$this->ciqrcode->generate($params))
					$this->die_error(__METHOD__, "Could not generate QR code in $info_code_file");
			}
			$qrcode_link="<A HREF=" . self::QRIMG_URL . "/$qrcode_basefile>$info_code</A>";
			$controles_table_body.= "<TR><TD>$controle_num$is_start$is_finish</TD><TD>$cd_mi mi<br>$cd_km km</TD><TD>$open_str<br>$close_str</TD><TD>$cn</TD><TD>$ca</TD><TD>$qrcode_link</TD></TR>";
		}
	
	}
	
	private function munge_route($route){
		$warnings=[];
		$re=[];
		
		// Put all route strings and numbers into route_event
		foreach($route as $key=>$val){
			if(is_string($val) || is_numeric($val)) $re[$key]=$val;
		}

		// These should be available in all cases
		$re['distance_km']=$distance_km=round($route['distance']/rwgps::m_per_km,1);
		$re['distance_mi']=$distance_mi=round($route['distance']/rwgps::m_per_km/rwgps::km_per_mi,1);
		$re['climbing_ft']=$climbing_ft=round($route['elevation_gain']*rwgps::ft_per_m,1);
		$re['route_id']=$route_id=$route['id'];
		$re['route_url']=$route_url=$this->rwgps->make_route_url($route['id']);
		$re['route_filename']=$route_filename=$this->rwgps->make_route_filename($route['id']);
		$re['route_file_basename']=basename($route_filename);
		$re['download_note']=$download_note=$route['download_note'];
		$re['route_name']=$route_name=$route['name'];

		// Parse the description and put it in the $re

		if(empty($route['description'])){
			$warnings[]="No attributes in route description. Please add required attributes to the RWGPS route description.";
			$route['description']='';
		}

		$ed=$this->rwgps->parse_description($route['description'],$this->rwgps->valid_event_description_keys);
		$re['ed_str']=$ed_str=$this->format_attributes($ed);
		if(!empty($ed['unrecognized'])){
				$warnings[]="Unrecognized attributes in RWGPS route description: " . implode(', ',$ed['unrecognized']); 
				$warnings[]="Valid route description attribute keys are: " . implode(', ',$this->rwgps->valid_event_description_keys);
		}

		foreach($ed as $key=>$val){
			if(is_string($val) || is_numeric($val)) $re[$key]=$val;
		}


		
		$re['this_organization']=$this_organization=(isset($ed['organization']))?$ed['organization']:'';
		$re['event_name_dist']=(isset($ed['name'], $ed['distance']))?$event_name_dist=$ed['name'] . ' ' . $ed['distance'] . 'K':$route['name'];

		$re['organizer_name']=$organizer_name=(isset($ed['organizer_name']))?$ed['organizer_name']:null;
		$re['organizer_phone']=$organizer_phone=(isset($ed['organizer_phone']))?$ed['organizer_phone']:null;
		$re['organizer_rusa_id']=$organizer_rusa_id=(isset($ed['organizer_rusa_id']))?$ed['organizer_rusa_id']:null;
		$re['rusa_route_id']=$rusa_route_id=(isset($ed['rusa_route_id']))?$ed['rusa_route_id']:null;
		$re['event_name']=$event_name=(isset($ed['name']))?$ed['name']:null;
		$re['event_type']=$event_type=(isset($ed['type']))?strtolower($ed['type']):null;
		$re['event_style']=$event_style=(isset($ed['style']))?strtolower($ed['style']):null;
		$re['event_date']=$event_date=(isset($ed['date']))?$ed['date']:null;
		$re['event_time']=$event_time=(isset($ed['time']))?$ed['time']:null;
		$re['pavement_type']=$pavement_type=(isset($ed['pavement_type']))?$ed['pavement_type']:'Unknown';

		// Data Files
		$df_links=[];
		foreach($route['route_datafile'] as $ext=>$fn){
			$uc_ext=strtoupper($ext);
			$base_fn=basename($fn);
			$df_links[]="<A TITLE='$uc_ext file download' HREF='" . $route['saved_route_url'][$ext] . "'>$uc_ext</A>"; 
		}
		$re['df_links']=$df_links=implode(', ', $df_links);

		// Validate required route attributes

		// Location
		if(isset($ed['location'])){
			$re['event_location']=$event_location=$ed['location'];
		}else{
			$re['event_location']=$event_location="Unknown";
			$warnings[]="Start location not set. Set #location to the 'City, State' of event start.";
		}

		// Distance
		if(isset($ed['distance'])){
			$re['event_distance']=$event_distance=$ed['distance'];
		}else{
			$re['event_distance']=$event_distance='Unknown';
			$warnings[]="Official distance not set. Set #distance to the official distance of the route in whole kilometers.";
		}
		if($event_distance!=floor($event_distance)){
			$re['event_distance']=$event_distance=floor($event_distance);
			$warnings[]="Fractional part of event official distance truncated. Assuming $event_distance K. Must be a whole number.";
		}
		if($event_distance>$distance_km){
			$warnings[]="Route is too short. Route distance $distance_km km < Official distance $event_distance km.";
		}
		
		// Time Zone
		if(isset($ed['tzname'])){
			$re['event_tzname']=$event_tzname=$ed['tzname'];
		}else{
			$re['event_tzname']=$event_tzname=$this->model_parando->tzname;;
			$warnings[]="Time zone not set. Use tag #tzname to set time zone. Assuming $event_tzname<br>
For complete list of valid timezones, see <A HREF=https://www.php.net/manual/en/timezones.php>https://www.php.net/manual/en/timezones.php</a>";
		}
		$re['event_tz']=$event_tz=new DateTimeZone( $event_tzname );
		date_default_timezone_set($event_tzname);
		$re['last_update']=$last_update=date("Y-m-j H:i:s T",$re['updated_at']); 
		$re['last_download']=$last_download=date("Y-m-j H:i:s T",$re['downloaded_at']); 
		
		// HTML Table of Information about route
		$re['route_info_table']=<<<EOT
<TR><TD><b>Route Name</b></TD><TD>$route_name</TD></TR>
<TR><TD><b>Last Download</b></TD><TD>$last_download</TD></TR>
<TR><TD><b>Modified</b></TD><TD>$last_update</TD></TR>
<TR><TD><b>RWGPS URL</b></TD><TD><A TITLE='View Route in RWGPS' HREF='$route_url'>$route_url</a></TD></TR>
<TR><TD><b>Route File Download</b></TD><TD>$df_links</TD></TR
<TR><TD><b>Start Location</b></TD><TD>$event_location</TD></TR>
<TR><TD><b>Offical Distance</b></TD><TD>$event_distance</TD></TR>
<TR><TD><b>Riding Distance</b></TD><TD>$distance_mi mi / $distance_km km</TD></TR>
<TR><TD><b>Climbing</b></TD><TD>$climbing_ft ft</TD></TR>
<TR><TD><b>Pavement Type</b></TD><TD>$pavement_type</TD></TR>
EOT;
		$re['route_warnings']=$warnings;
		return $re;
	}
	

	private function view_html($re, $controles, $warnings){

		$this_organization=$this->model_parando->organization;
		$this_url=site_url($this->class_name); // site_url(strtolower(get_class($this)));
		$download_url="$this_url/recache";
		$checkin_url="$this_url/checkin";

		extract($re);
		$page_text='';

		// ROUTE INFO
		$page_text.='<H4>ROUTE INFO</H4>';
		$page_text.="<div class='half-width bordered text-center italic sans'>$download_note</div>";
		$page_text.="<div class='narrower ftable'><TABLE WIDTH=100%>$route_info_table</TABLE></div>";
		if(!empty($ed_str)){
			$page_text.=<<<EOT
<H4>ATTRIBUTES</H4>
<p class=nopadding>The following event attribute tags were set in the RWGPS route description:</p>
<div class='narrower bordered tt'>
$ed_str
</div>
EOT;
		}
		$page_text.=$this->warning_message($route_warnings,"There are errors in the route description.");
		$page_text.="<div class='narrower text-center button-container'><A HREF=$download_url/$route_id>Re-download this route from RWGPS</A></div>";

		// CONTROL INFO
		$page_text.='<H4>CONTROL INFO</H4>';
		$page_text.="<div class='narrower ftable'><TABLE WIDTH=100%>";
		$page_text.="<TR><TD><b>Controls</b></TD><TD>$ncontroles</TD></TR>";
		//$page_text.=$control_info_table;
		$page_text.="</TABLE></div>";
		$page_text.=$this->warning_message($ec_warnings,"There are errors in the controls.");

		// GENERATE OPTIONS
		$page_text.='<H4>GENERATE PAPERWORK</H4>';

		// Fine Print
		$page_text.=$this->model_parando->cue_wizard_disclaimer_html;


		$data=['title'=>$re['name'], 'left_column'=>$page_text];
		$this->simple_view('info_one_column',['table','buttonlink','form'],$data);
	
	}
	
	private function warning_message($warnings, $message="There are errors in the route definition"){
		if(!empty($warnings)){
			$cw=implode('</li><li>',$warnings);
			$page_text=<<<EOT
<div class="bordered narrower red">
<div class="vspace"></div>
<div class="large-print sans">ERRORS:</div>
<ul><li>$cw</li></ul>
<p>$message Fix these Errors by editing the route in RWGPS, saving in RWGPS adding appropriate Description #tags, and then finally re-downloading here.</p>
</div><div class="vspace"></div>
EOT;
		}else $page_text='';
		
		return $page_text;
	}

	public function id($route_id=0, $view=null){
		if($route_id+0 == 0) $this->die_error(__METHOD__, "Missing or invalid RWGPS route ID");
		$event=null;
		$view=$this->input->post('view');
		$this->process_route($route_id,$view);
	}
	
	private  $datetime_attributes=[
			['event_date', 'Event Start Date (YYYY-MM-DD)', ''],
			['event_time', 'Event Start Time (HH:MM)', '']
		];
		
	private function process_route($route_id, $view=null){
		$route=$this->rwgps->get_route($route_id);
		list($controles, $controle_warnings) = $this->rwgps->extract_controles($route);
		list($cues, $cue_warnings) = $this->rwgps->extract_cues($route);
		list($route_event, $re_warnings) = $this->merge_route_event_data($route);
		$warnings=array_merge($controle_warnings, $re_warnings, $cue_warnings);

		// Handle Form Submission, if any
		if('POST' == $this->input->method(TRUE)){
			// DateTime Input Field overrides
			foreach($this->datetime_attributes as $a){
				$k=$a[0];
				if(!empty($_POST[$k])) $route_event[$k]=${$k}=$this->input->post($k);
			}
		}
		
		// Date/Time 
		
//$this->die_error(__METHOD__, "Event date " . $route_event['event_date']); 


		if(empty($route_event['event_date']) || $route_event['event_date']==$this->default_event_date){
			$route_event['event_date']=$event_date=$this->default_event_date;
			$warnings[]="Event date not set. Use tag #date=YYYY-MM-DD to set the start date. Default is " . $this->default_event_date;
		}
		
		if(empty($route_event['event_time']) || $route_event['event_time']==$this->default_event_time){
			$route_event['event_time']=$event_time=$this->default_event_time;
			$warnings[]="Event time not set. Use tag #time=HH:MM to set the start time.";
		}


		$route_event['event_datetime'] = $event_datetime = @date_create($route_event['event_date'].' '.$route_event['event_time'], $route_event['event_tz']);
		if(false == $event_datetime) $this->die_error(__METHOD__,"Invalid date or time. Please use the format YYYY-MM-DD for dates, and HH:MM:SS for time.");
		
		$route_event['event_datetime_str'] = $event_datetime_str = $event_datetime->format($this->event_datetime_format);
		$route_event['event_datetime_str_verbose'] = $event_datetime_str_verbose = $event_datetime->format($this->event_datetime_format_verbose);

		// Control calculations

		$this->controletimes->compute_open_close($controles,$route_event);
		$this->controletimes->compute_info_code($controles,$route_event);

		// View

		if(empty($view)){
			$this->view_html($route_event, $controles, $warnings);
		}elseif(method_exists($this,'view_'.$view)){
			$this->{'view_'.$view}($route_event, $controles, $warnings);
		} else {
			$this->die_error(__METHOD__, "No view defined for '$view'");
		}
	}

	private $event_datetime_format='Y-m-d H:i T';
	private $event_datetime_format_verbose='l j F Y, H:i T';

	private $parando_logo_url="https://parando.org/images/parando3D.png";
	private $valid_event_types = ['acp'=>true, 'rusa'=>true, 'permanent'=>true];

	private $default_event_date="2000-01-01";
	private $default_event_time="00:00";

	private 		$required_tags=[
//			'this_organization'=>'organization',
			'organizer_name'=>true,
			'organizer_phone'=>true,
			'event_distance'=>'distance',
			'event_type'=>'type',
			'event_location'=>'location',
			'event_distance'=>'distance',
			'event_name'=>'name'
		];

	private function merge_route_event_data($route){		

		$warnings=[];
		$re=[];
		
		// Put all route strings and numbers into route_event
		foreach($route as $key=>$val){
			if(is_string($val) || is_numeric($val)) $re[$key]=$val;
		}

		$re['distance_km']=$distance_km=round($route['distance']/rwgps::m_per_km,1);
		$re['distance_mi']=$distance_mi=round($route['distance']/rwgps::m_per_km/rwgps::km_per_mi,1);
		$re['climbing_ft']=$climbing_ft=round($route['elevation_gain']*rwgps::ft_per_m,1);
		$re['route_id']=$route_id=$route['id'];
		
		// Some data about the RWGPS route itself in all cases
		$re['route_url']=$this->rwgps->make_route_url($route['id']);

		$re['route_filename']=$route_filename=$this->rwgps->make_route_filename($route['id']);
		$re['route_file_basename']=basename($route_filename);


		$re['download_note']=$route['download_note'];
		$re['route_name']=$route['name'];

		// If there's nothing to go on, complain and create some basic info

		if(empty($route['description']) && empty($event)){
			$warnings[]="No event specified in database, and no RWGPS route description. Please add a full description to the RWGPS route, or add this event to the database.";
			$route['description']='';
		}

		// Put any route description attributes into route_event

		$ed=$this->rwgps->parse_description($route['description'],$this->rwgps->valid_event_description_keys);
		if(!empty($ed['unrecognized'])){
				$warnings[]="Unrecognized attributes in RWGPS route description: " . implode(', ',$ed['unrecognized']); 
				$warnings[]="Valid route description attribute keys are: " . implode(', ',$this->rwgps->valid_event_description_keys);
		}

		foreach($ed as $key=>$val){
			if(is_string($val) || is_numeric($val)) $re[$key]=$val;
		}

		$re['this_organization']=$this_organization=(isset($ed['organization']))?$ed['organization']:'';
		$re['event_name_dist']=(isset($ed['name'], $ed['distance']))?$event_name_dist=$ed['name'] . ' ' . $ed['distance'] . 'K':$route['name'];

		$re['organizer_name']=$organizer_name=(isset($ed['organizer_name']))?$ed['organizer_name']:null;
		$re['organizer_phone']=$organizer_phone=(isset($ed['organizer_phone']))?$ed['organizer_phone']:null;
		$re['organizer_rusa_id']=$organizer_rusa_id=(isset($ed['organizer_rusa_id']))?$ed['organizer_rusa_id']:null;
		$re['rusa_route_id']=$rusa_route_id=(isset($ed['rusa_route_id']))?$ed['rusa_route_id']:null;
		$re['event_distance']=$event_distance=(isset($ed['distance']))?$ed['distance']:0;
		$re['event_name']=$event_name=(isset($ed['name']))?$ed['name']:null;
		$re['event_type']=$event_type=(isset($ed['type']))?strtolower($ed['type']):null;
		$re['event_style']=$event_style=(isset($ed['style']))?strtolower($ed['style']):null;
		$re['event_date']=$event_date=(isset($ed['date']))?$ed['date']:null;
		$re['event_time']=$event_time=(isset($ed['time']))?$ed['time']:null;
		$re['event_location']=$event_location=(isset($ed['location']))?$ed['location']:null;
		$re['event_tzname']=$event_tzname=(isset($ed['tzname']))?$ed['tzname']:null;
		$re['event_tz']=$event_tz=(isset($ed['tzname']))?new DateTimeZone( $event_tzname ):null;
		$re['event_id']=$event_id=null;
		
		
		// Now that all info is in, some calculations are possible

		// Generate warnings/defaults for missing data
		
		$missing_tags=[];
		foreach($this->required_tags as $k=>$v){
    		if(!isset(${$k})){
					if(is_string($v))
						$missing_tags[]=$v;
					else
						$missing_tags[]=$k;
				}
		}

		foreach($missing_tags as $t) 
			$warnings[]="Please provide missing tag in event description: $t";
		
		if(empty($event_location)){
			$re['event_location']=$event_location=" ";
			$warnings[]="Event location not set. Set #location equal to 'City, State' of event start.";
		}

		if($event_distance==0){
			$warnings[]="Official distance not set. Set #distance to the official distance of the route in kilometers.";
			$re['event_distance']=$event_distance="unknown";
		}
		
		if($event_distance>$distance_km){
			$warnings[]="Route is too short. Route distance $distance_km km < Official distance $event_distance km.";
		}
		
		
		if(empty($event_tzname)){
			$re['event_tzname']=$event_tzname=$this->model_parando->tzname;
			$re['event_tz']=$event_tz=new DateTimeZone( $event_tzname );
			$warnings[]="Time zone not set. Use tag #tzname to set time zone. Assuming $event_tzname<br>
For complete list of valid timezones, see <A HREF=https://www.php.net/manual/en/timezones.php>https://www.php.net/manual/en/timezones.php</a>";
		}
		
		
		date_default_timezone_set($event_tzname);
		$re['last_update']=date("Y-m-j H:i:s T",$re['updated_at']); 
		$re['last_download']=date("Y-m-j H:i:s T",$re['downloaded_at']); 

		if(!empty($event_type) && !isset($this->valid_event_types[$event_type])) 
			$this->die_error(__METHOD__, "Unsupported event type: $event_type. Valid types are: " . 
				implode(', ', array_keys($this->valid_event_types)));

		if(empty($event_type)){
			$re['event_type']=$event_type='unknown';
			$warnings[]="Event type not set. Set #type equal to one of the following:" . implode(', ', array_keys($this->valid_event_types));
		}

		if($event_type=='permanent' && !isset($re['organizer_rusa_id'])) 
			$warnings[]="Event is permanent. Must set: #organizer_rusa_id";

		if($event_type=='permanent' && !isset($re['rusa_route_id'])) 
			$warnings[]="Event is permanent. Must set: #rusa_route_id";
		
		return [$re, $warnings];
	}

    // Re-copy route to local cache
	public function recache($route_id){
		if($this->rwgps->is_good_route_id($route_id)){
			$result=$this->rwgps->download_route_data($route_id);
			if($result!==true)
				$this->die_error(__METHOD__, $result);
			redirect(site_url($this->class_name . "/info/$route_id"));
		}else{
			$this->die_error(__METHOD__,"Invalid parameters.");
		}
	}

 	private function make_event_tagname($re){ // adapter for route_event rather than event
    $d=(empty($re['event_datetime']))?"":$re['event_datetime']->format('Y-m-d');
 		$event=[
 			'sanction'=>$re['event_type'], 
 			'type'=>$re['event_style'],
 			'distance'=>$re['event_distance'], 
 			'date'=>$d
 		];
		if(isset($re['rusa_route_id'])) $event['rusa_route_id']=$re['rusa_route_id'];
 		$tagname=$this->model_parando->make_event_tagname($event);
 		return $tagname; 
 	}

	
	const QRIMG_PATH="/home/tomr/public_html/assets/uploads/files";
	const QRIMG_URL="https://parando.org/assets/uploads/files";
	const QRIMG_EXT="png";

	private function view_qrpages($route_event, $controles, $warnings){
	
		$params=[
			'event'=>$route_event, 
			'orientation'=>'P', 
			'size'=>'letter',
			'qrimg_path'=>self::QRIMG_PATH,
			'qrimg_url'=>self::QRIMG_URL,
			'qrimg_ext'=>self::QRIMG_EXT
		];
		$this->load->library('controlqr', $params);
		$this->controlqr->draw_qr_pages($route_event, $controles);
		$event_tagname=$this->make_event_tagname($route_event);
		$this->controlqr->Output("I", "$event_tagname-QRCodes.pdf");

	}

	private function xview_html($re, $controles, $warnings){

		$this_organization=$this->model_parando->organization;
		$this_url=site_url(strtolower(get_class($this)));
		$download_url="$this_url/recache";
		$checkin_url="$this_url/checkin";

		$ncontroles=count($controles);
		extract($re);

		$route=$this->rwgps->get_route($route_id);
		$ed=$this->rwgps->parse_description($route['description']);		
		$ed_str=$this->format_attributes($ed);

		$method='id';
		$route_event_id=$id=$route_id;
		$cue_version_str="";

		$controles_table_body="";
		$controles_table_body.= "<TR><TH>Controle</TH><TH>Distance</TH><TH>Open/Close</TH><TH>Note</TH><TH>Description (attributes)</TH><TH>Info Code</TH></TR>";
		for($i=0; $i<$ncontroles; $i++){
			$cd_mi=round($controles[$i]['d']/(rwgps::m_per_km*rwgps::km_per_mi),1);
			$cd_km=round($controles[$i]['d']/(rwgps::m_per_km),1);
			$is_start=(isset($controles[$i]['start']))?" [START]":"";
			$is_finish=(isset($controles[$i]['finish']))?" [FINISH]":"";
			$controle_num=$i+1;
			$open_str=$controles[$i]['open']->format('m-d H:i');
			$close_str=$controles[$i]['close']->format('m-d H:i');
			// $ca=print_r($controles[$i]['attributes'], true);
			$ca=$this->format_attributes($controles[$i]['attributes']); 
			$cn=(isset($controles[$i]['n']))?$controles[$i]['n']:"";
			$info_code=$controles[$i]['info_code']; 
			$qrcode_basefile="$info_code.png";
			$qrcode_file=self::QRIMG_PATH . '/' . $qrcode_basefile;
			$this->load->library('ciqrcode');
			if(true || !file_exists($qrcode_file)){
				$params['data'] = "$checkin_url/$event_date/$event_distance/$info_code";
				$params['level'] = 'H';
				$params['size'] = 8;
				$params['savename'] =$qrcode_file;
				if(!$this->ciqrcode->generate($params))
					$this->die_error(__METHOD__, "Could not generate QR code in $info_code_file");
			}
			$qrcode_link="<A HREF=" . self::QRIMG_URL . "/$qrcode_basefile>$info_code</A>";
			$controles_table_body.= "<TR><TD>$controle_num$is_start$is_finish</TD><TD>$cd_mi mi<br>$cd_km km</TD><TD>$open_str<br>$close_str</TD><TD>$cn</TD><TD>$ca</TD><TD>$qrcode_link</TD></TR>";
		}
		
			$fineprint=$this->model_parando->cue_wizard_disclaimer_html;
			
		if(!empty($rusa_route_id))
			$rusa_route_id_table_row="<TR><TD><b>RUSA Route ID</b></TD><TD>#$rusa_route_id</TD></TR>";
		else
			$rusa_route_id_table_row="";

		if(!empty($event_id)){
			$attrib_edit_controls="";
			$event_datetime_table_row="<TR><TD><b>Event Date/Time</b></TD><TD>$event_datetime_str</TD></TR>";
		}else{
			$attrib_edit_controls="<div class=vspace></div><p class=nopadding>The following event attributes can be customized now.</p><div>";
			$event_datetime_table_row="";
		  foreach($this->datetime_attributes as $a){
				$k=$a[0]; $v=$a[1];
				$value=${$k};
				//$value=set_value($k,${$k});

				//if(!empty(form_error($k))){
				//	$attrib_edit_controls .= "<div class=verror>" . form_error($k) . "</div>";
				//}
				$attrib_edit_controls.=<<<EOT
<div class=form-group>
	<label for="$k">$v</label>    
		<div class='half-width-indented'><input type='text'  name='$k' value='$value'></div>
EOT;
			}

			$attrib_edit_controls.=<<<EOT
</div><div class="vspace"></div>
<div class="narrower button-stack">
<BUTTON TYPE=SUBMIT NAME='set' value='attributes'>Set Attributes</BUTTON>
</div>
EOT;

		}

		if(!empty($ed_str))
			$rwgps_attributes=<<<EOT
<H4>ATTRIBUTES</H4>
<p class=nopadding>The following event attribute tags were set in the RWGPS route description:</p>
<div class='narrower bordered tt'>
$ed_str
</div>
EOT;
		else
			$rwgps_attributes="";

		if(!empty($event_id) && !empty($ed_str)) 
			$rwgps_attributes.=<<<EOT
<div class='narrower text-center'>
  <p class=nopadding><em>NB: Some of these event attribute tags in RWGPS route description have been supersceded 
  by attributes set in $this_organization event database for Event $event_id: $event_name_dist. </em></p>
</div>
EOT;

		if(!empty($warnings)){
			$cw=implode('</li><li>',$warnings);
			$generate_buttons =<<<EOT
<div class="bordered narrower red">
<div class="vspace"></div>
<div class="large-print sans">ERRORS:</div>
<ul><li>$cw</li></ul>
<p>There are errors in the route. Fix these Errors by editing the RWGPS route, saving in RWGPS, and re-downloading here.</p>
</div><div class="vspace"></div>
EOT;
		}
		
		$generate_buttons.=<<<EOT
<div class="narrower button-stack">

<BUTTON TYPE=SUBMIT NAME='view' value='qrpages'>Generate PDF of Control QR Codes</BUTTON>
</div><div class="vspace"></div>
EOT;

		$left_column="";
		$right_column="";

//$this->die_error(__METHOD__,print_r($re,true));

if(!empty($pavement_type))
	$pt="<TR><TD><b>Pavement Type</b></TD><TD>$pavement_type</TD></TR>";
else
	$pt="";
	
			$df_links=[];
			foreach($route['route_datafile'] as $ext=>$fn){
				$uc_ext=strtoupper($ext);
				$base_fn=basename($fn);
				$df_links[]="<A TITLE='$uc_ext file download' HREF='" . $route['saved_route_url'][$ext] . "'>$uc_ext</A>"; 
			}
			$data_file_row="<TR><TD><b>Route File Download</b></TD><TD>" . implode(', ', $df_links) . "</TD></TR>";


		$left_column.=<<<EOT
<H3>$event_name_dist</H3>
<div class="bordered narrower text-center">$download_note<br>
<div class="button-container"><A HREF=$download_url/$route_event_id>Re-download route from RWGPS</A>
</div></div>
<H4>ROUTE INFO</h4>
<div class='narrower ftable'><TABLE WIDTH=100%>
<TR><TD><b>Route Name</b></TD><TD>$route_name</TD></TR>
<TR><TD><b>Last Download</b></TD><TD>$last_download</TD></TR>
<TR><TD><b>Modified</b></TD><TD>$last_update</TD></TR>
<TR><TD><b>RWGPS URL</b></TD><TD><A HREF=$route_url>$route_url</a></TD></TR>
$data_file_row
<TR><TD><b>Distance</b></TD><TD>$distance_mi mi / $distance_km km</TD></TR>
<TR><TD><b>Climbing</b></TD><TD>$climbing_ft ft</TD></TR>
$pt
<TR><TD><b>Event Type</b></TD><TD>$event_type</TD></TR>
$rusa_route_id_table_row
<TR><TD><b>Official Distance</b></TD><TD>$event_distance km</TD></TR>
$event_datetime_table_row
<TR><TD><b>Controles Found</b></TD><TD>$ncontroles</TD></TR>
$cue_version_str
</TABLE></DIV>

$rwgps_attributes
<FORM ACTION=$this_url/$method/$id METHOD=POST  enctype="multipart/form-data">
$attrib_edit_controls
<div class=vspace></div>
<H4>GENERATE</H4>
$generate_buttons
</FORM>

<H4>CONTROLES</H4>
<div class='narrower ftable'><TABLE WIDTH=100%>
$controles_table_body
</TABLE></DIV>
$fineprint
EOT;
		$data=['title'=>$event_name_dist, 'left_column'=>$left_column, 'right_column'=>$right_column];
		$this->simple_view('info_one_column',['randomap','table','buttonlink','form'],$data);
	
	}
		
	private function format_attributes($alist){
		$ca="";
		foreach($alist as $k=>$v){
				if(is_string($v))
					$ca.="<div class=attribute>#$k=$v</div>";
		}
		return $ca;
	}
	
	
}

