<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

require_once '/home/tomr/public_html/vendor/autoload.php';

use Smartwaiver\Smartwaiver;

 
class Register extends MY_Controller {

  // CONSTRUCTOR

	public $datetime_now;
	public $all_email_to_webmaster;

	public function __construct() {
		parent::__construct();

		$this->load->model('accounting_parando');
		$this->load->helper('form');
		$this->load->library('email');
		$this->load->library('form_validation');
		$this->load->library('multipage');
		$this->load->library('registerpages');
		$this->load->library('controletimes');
		
		$this->multipage->define_pages($this->registerpages->page);  // SLOW -- Library parameter?
		$this->multipage->set_view('form_twocolumn','form_header','form_footer',['form']);
		$this->multipage->set_controller(strtolower(get_class($this)));
		$tz=$this->model_parando->get_local_time_zone();
		$this->datetime_now = new DateTime("now",$tz);   // SHOULD BE BASED ON EVENT
		
		// set this true for debugging
		$debug=false;
		$this->session->set_userdata('paypal_sandbox', $debug);
		$this->all_email_to_webmaster=$debug;
	}
	
	private function event_reg_session_init(){
		$date_stamp=$this->datetime_now->format('j F Y');
		$this->session->unset_userdata('PayPalResult');
		$this->session->unset_userdata('shopping_cart'); 
		
        $club=$this->model_parando->default_club;
		
		$d=[
            'club'=>$club,
			'this_organization' => $club['name'],
			'this_region' => $club['region'],
			'admin_login_url' => $this->model_parando->admin_login_url,
			'base_url' => base_url(),
			'date_stamp' => $date_stamp
		];
		$this->multipage->init($d);
	}

	// GENERIC PAGE PROCESSING

	public function index() {
		$this->multipage->process_page();
	}


    // EVENT REGISTRATION CONTROLLER METHOD  --- START HERE!
    // EVENT REGISTRATION CONTROLLER METHOD  --- START HERE!
    // EVENT REGISTRATION CONTROLLER METHOD  --- START HERE!
    // EVENT REGISTRATION CONTROLLER METHOD  --- START HERE!

	public function event($event_id=-1,$coupon=null){

		$this->event_reg_session_init();

		if($event_id==-1){
			$event_id=$this->model_parando->next_event_id();
		}
		
		$event = $this->model_parando->get_event($event_id);
		if(empty($event)){
			$this->die_error('Event Error','Event (' . $event_id . ') not found.');
		}
		$schedule=$this->model_parando->event_schedule($event);
		
// 		if($schedule['start_datetime']<$this->datetime_now){
// 			$this->die_error('Registration Closed',
// 			  'It is too late to register for this event. Registration closed '. $schedule['reg_cutoff_datetime_str2'].'.');
// 		}

		if($schedule['registration_closed']){
			$this->die_error('Registration Closed',
			  'It is too late to register for this event. Registration closed '. $schedule['reg_cutoff_datetime_str2'].'.');
		}

		$organizer_data=$this->model_parando->get_organizer($event);	
		$event_data=$this->make_event_data($event,$schedule);
		$this->multipage->set_data_array(array_merge($organizer_data,$event_data));

		if($coupon!==null){
					$this->load->model('coupon');
					$coupon_data=$this->coupon->get($coupon);
					if($coupon_data!==false && $this->coupon->is_valid_for_event($coupon_data,$event_id))
							$this->multipage->set_data_array(compact('coupon','coupon_data'));
					else 
						if($this->coupon->valid_syntax($coupon)) $this->die_error(__METHOD__,"Invalid Coupon Code: $coupon");
		}
		
		$this->multipage->set_current_page('find_member');   // ALWAYS GOES TO FIND MEMBER
		$this->multipage->view_current_page();
	}

	private function make_event_data($event,$schedule){

		$event_id=$event['event_id'];

		$event_data=array(
			    'event_id'		=> $event_id,
			    'event_contact_url'	=> site_url("info/contact/$event_id"),
			    'event_start_city'	=> $event['start_city'],
			    'event_start_state'	=> $event['start_state'],
			    'event_region' => $this->model_parando->region,
			    'event_name'	=> $event['name'],
			    'event_type'	=> $event['type'],
			    'event_distance'	=> $this->model_parando->make_distance($event),
			    'event_date'	=> $schedule['start_date_str'],
			    'event_DateTime' => $schedule['start_datetime'],
			    'event_start_datetime' => $schedule['start_datetime_str'],
			    'event_cutoff_datetime' => $schedule['cutoff_datetime'],
			    'event_time_zone' => $schedule['start_time_zone'],
			    'event_reg_cutoff'	=> $schedule['reg_cutoff_datetime_str2'],
			    'event_included'	=> $event['included'],
			    'event_member_price'	=> $this->model_parando->member_price_now($event),
			    'event_earlybird_discounts'	=> $event['earlybird_discounts'],
			    'event_waive_guest_fee'	=> $event['waive_guest_fee'],
				'event_lodging_options' => $event['lodging_options']
			);
		return $event_data;
	}
	
// FIND_MEMBER
// FIND_MEMBER
// FIND_MEMBER
// FIND_MEMBER
// FIND_MEMBER

	public function _find_member(){

		// Processing method for the find_member page
		
		// Decide what sort of match the first/last/rusa id makes with the database. 
		// Calls the method register_known($member) for a full match, otherwise it branches to two 
		// possible forms register_unknown for no match, or register_pick_member for multi match. 
	  
		$contact_url=$this->multipage->get_data('event_contact_url');
		$cto="<A HREF=$contact_url TITLE='Contact the Organizer'>contact the organizer</a>";
		$error_comment=" Registration can not proceed till this is fixed. If you have questions about this error, $cto.";

		$rusa_id=$this->input->post('rusa_id');
		$first_name=$this->input->post('first_name');
		$last_name=$this->input->post('last_name');

        if($this->model_parando->require_rusa_membership){		// Check for RUSA Membership through event date

            $this->load->library('rusa');
            $event_cutoff_datetime=$this->multipage->get_data('event_cutoff_datetime');
            $this->rusa->check_rusa_membership_at_date($rusa_id,$last_name,$event_cutoff_datetime);
    
        }

		$found=$this->model_parando->find_member($first_name,$last_name,$rusa_id);	
		
		switch($found['result']){
		  case 'full_match_rusa':
			$q=$found['query'];
			$member=$q->row_array();
			$this->register_known($member);
			break;
		  case 'duplicate_entry':
			$this->die_error('Database Error','Your info matches two entries in our database.'.$error_comment);
			break;
		  case 'unknown':
			$this->load_unknown($first_name,$last_name,$rusa_id);
			$this->multipage->set_current_page('register_unknown');  // BRANCH REGISTER_UNKNOWN then GET_NEW_MEMBER_DATA
			break;
		  default:
			$this->load_unknown($first_name,$last_name,$rusa_id);
			$this->multipage->set_current_page('register_pick_member'); // BRANCH PICK_MEMBER
			$q=$found['query'];
			$found_multiple = $q->result_array();
			$this->multipage->set_data('found_multiple',$found_multiple);
			break;
		}
	}

	// Set the session with what we do know about the "unknown" rider.
    private function load_unknown($first_name,$last_name,$rusa_id){
			$session_data=array(
				'rusa_id'	=> $rusa_id,
				'first_name'	=> $first_name,
				'last_name'	=> $last_name,
				'given_id' => ['first_name'=>$first_name, 'last_name'=>$last_name, 'rusa_id'=>$rusa_id]
			);
			$this->multipage->set_data_array($session_data);
	}
	
	
	public function rusa_check($rusa_id){
            $this->load->library('rusa');
            $result = $this->rusa->query_rusa_id($rusa_id);
            $this->die_error(__METHOD__, print_r($result,true));
		
		}


	

	// REGISTER_UNKNOWN  (static page, no functions needed, always goes to GET_NEW_MEMBER_DATA) 
	
    // GET_NEW_MEMBER_DATA
    // GET_NEW_MEMBER_DATA
    // GET_NEW_MEMBER_DATA

	public function _get_new_member_data(){
	
	// Processing method for get_new_member_data page
	// Stores aquired data into database and proceeds with known member
	
		// Stuff already in session
		$sdata = $this->multipage->get_data_array(['rusa_id','first_name','last_name']);

		if(empty($sdata['first_name']) || empty($sdata['last_name'])) $this->die_error(__METHOD__, "No member name in session!");
		
		$pw = $this->input->post('password');
		$pw_hash = (empty($pw))?"":password_hash($pw,PASSWORD_DEFAULT);
		
		// Stuff added by contact form
		$fdata = array(
			'm_street'		=> $this->input->post('contact_street'),
			'm_city'		=> $this->input->post('contact_city'),
			'm_state'		=> $this->input->post('contact_state'),
			'm_zip'			=> $this->input->post('contact_zip'),
			'm_country'			=> $this->input->post('contact_country'),
			'tracker'			=> $this->input->post('contact_tracker'),
			'm_phone'		=> $this->input->post('contact_phone'),
			'm_phone_alt'		=> $this->input->post('contact_phone_alt'),
			'm_email'		=> $this->input->post('contact_email'),
			'emergency_name'	=> $this->input->post('contact_emergency_name'),
			'emergency_phone'	=> $this->input->post('contact_emergency_phone'),
			'emergency_phone_alt'	=> $this->input->post('contact_emergency_phone_alt'),
			'password' => $pw_hash

		);

		$mdata=array_merge($sdata,$fdata);

		// Totally new member?
		$member = $this->model_parando->get_member_firstlast($sdata['first_name'],$sdata['last_name']);
		if(empty($member)){ // No existing record, do an insert creating a new member ID
			$rc=$this->db->insert('member', $mdata);	// Add new member to database
			if($rc!==true) $this->die_error(__METHOD__, "Could not insert member record: " . print_r($mdata,true));
		}else{ // Existing member updating information
			if(empty($member['id'])) $this->die_error(__METHOD__, "Empty member ID for member query: " . $sdata['first_name'] . ' ' . $sdata['last_name']);
			$member_id=$member['id'];
			$mdata['id']=$member_id;
			$rc=$this->db->replace('member',$mdata);
			if($rc!==true) $this->die_error(__METHOD__, "Could not replace member record: " . print_r($mdata,true));
		}


		$member = $this->model_parando->get_member_firstlast($sdata['first_name'],$sdata['last_name']);
		if(empty($member)){$this->die_error('get_new_member_data',"This can't happen -- failed member readback.");}

		$this->register_known($member);  // ALWAYS calls register_known()

	}


	public function _get_country_options(){
		$country_fullname = $this->model_parando->get_countries();
		return $country_fullname;
	}
	
	public function _get_state_options(){ 
		$state_fullname = $this->model_parando->get_states();
		return $state_fullname;
	}

	public function _phone_check() {

		// Validation callback to verify that at least one phone number was entered

		if($this->input->post('contact_phone') || $this->input->post('contact_phone_alt')){
				return TRUE;
		}else{
				$this->form_validation->set_message('_phone_check', 'Please enter atleast one phone number.');
		return FALSE;
		}
	}
	
// REGISTER_PICK_MEMBER PAGE

	public function _pick_member_form_body(){

		// Callback that generates a radio button form for picking 
		// from multiple matched members

		$found=$this->multipage->get_data('found_multiple');

		$num_rows=sizeof($found);
		if($num_rows>1){
			$entries="one or more entries";
			$the_entry_that="the entry that";
			$not_me="None of those are me";
		}else{
			$entries="one entry";
			$the_entry_that="it if it";
			$not_me="That is not me.";
		}

		$body = <<<EOT
<p>Your info partially matches $entries in our rider database. Please select $the_entry_that exactly matches you, or select <em>not me</em></p><br>
<div class=roster>
<table width=100%>

EOT;

		foreach ($found as $row)
		{
			$member_id=$row['id'];
			$rusa_id=(!empty($row['rusa_id']) && $row['rusa_id']>0)?("RUSA " . $row['rusa_id']):"";
			if($row['m_country']=='US'){
				$state = $row['m_state'];
			}else{
				$country_fullname=$this->model_parando->get_countries();
				$state = $country_fullname[$row['m_country']];
			}
	    $body .= "<tr>";
	    $body .= "<td>";
			$body .= "<div class=image-radio><input type='radio' name='member_id' id='member_id$member_id' value=$member_id>";
	    $body .= "<label for='member_id$member_id'>" . $row['first_name'] . '&nbsp;' . $row['last_name'] . "</label></div>";
	    $body .= "</td>";
	    $body .= "<td>" . $row['m_city'] . "</td>";
	    $body .= "<td>" . $state . "</td>";
	    $body .= "<td>" . $rusa_id . "</td>";
	    $body .= "</tr>";
		}

		$body .= "</table></div>";

	if(!empty(form_error('member_id'))){
	  $body .= "<div class=verror>" . form_error('member_id') . "</div>";
	}

	$body .= "<div class=image-radio><input type='radio' name='member_id' id='member_id_unknown' value='not_me'><label for='member_id_unknown'>$not_me</label></div>";


		return $body;
	}
	
	// Pick from existing and call register_known(), or branch to REGISTER_UNKNOWN, or die with an inconsistency

    public function _register_pick_member(){

		// Processing method for the register_pick_member page 
		// will branch to register_unknown if no existing member of the same name/rusa
		// or branch to register_known($member) if there's now a full member match

 		$this->form_validation->set_rules('member_id','Member Selection', 'trim|required', array('required'=>'You must either select a member above that matches you, or select below that none are you.') );

		$contact_url=$this->multipage->get_data('event_contact_url');
		$cto="<A HREF=$contact_url TITLE='Contact the Organizer'>contact the organizer</a>";
		$error_comment=" Registration can not proceed till this is fixed. If you have questions about this error, $cto.";

	  $member_id=$this->input->post('member_id');
		$rusa_id= $this->multipage->get_data('rusa_id');
		$first_name=$this->multipage->get_data('first_name');
		$last_name= $this->multipage->get_data('last_name');
		$given_id= $this->multipage->get_data('given_id');
	  
	  if($this->form_validation->run()!=FALSE){
			if($member_id == "not_me"){
				$check=$this->model_parando->check_unknown($first_name,$last_name,$rusa_id);
				switch($check){
					case "existing_first_last":
						$this->die_error('Existing Member',
							"There is already a rider in our database with exactly the same first and last name as you have entered, yet you say this is not you. 
							I'm afraid you cannot register with a duplicate name. Use a different name or $cto.");   
						break;
					case "existing_rusa":
						$this->die_error('Existing Member',
							"There is already a rider in our database with exactly the same RUSA ID as you have entered, yet you say this is not you. 
							I'm afraid you cannot register with a duplicate RUSA ID. Use a different ID or $cto.");   
						break;
					case "ok":
						$this->multipage->set_current_page('register_unknown');   // If truly unknown,  branch to REGISTER_UNKNOWN
						break;
				}
			}else{
				$mem=$this->model_parando->get_member($member_id);
				if(empty($mem)){
					$this->die_error('Member not found','No such member ID.');
				}else{

				                    // TODO! Maybe this shoud DO A RUSA LOOKUP and automatically update? Use this function:
				                    // $this->rusa->get_member_by_first_last($fn,$ln);

				
					if(empty($mem['rusa_id'])) $this->die_error('Previously No RUSA ID',
						"There is already a rider in our database with your name, 
						but that rider is listed in our database as not a RUSA member. Have you recently joined? ". $error_comment);

					if($rusa_id!=$mem['rusa_id']) $this->die_error('RUSA ID Change',
						"There is already a rider in our database with your name, 
						but that rider is listed in our database with a different RUSA ID than you. 
						Did you leave your RUSA number blank? Did you enter your RUSA ID incorrectly now, or in the past? ". $error_comment);

					$this->register_known($mem); // ALWAYS calls register_known() if a good member was picked

				}
			}
		}
	}
	
	// This function is called from either FIND_MEMBER (if the member was found right away), PICK_MEMBER, if the member was picked, or GET_NEW_MEMBER_DATA 
	// if a new member was created with full data
	
	// If all is well, this function will create the item list with the rider's registration as the only item, and push into the page call stack appropriately
	// to pages for "extras" like membership and lodging and the fleche registration thing
	
	// After this stack is all called, we end up at REVIEW_CHARGES

	private function register_known($member){

			// Begins finalizing the registration of a known member

			if(empty($member['id'])) $this->die_error(__METHOD__, "Empty member ID");
			$member_id=$member['id'];

			// We should know this much for sure now. Put it in session
			$session_data=array(
				'member_id' => $member_id,
				'rusa_id'	=> $member['rusa_id'],
				'first_name'	=> $member['first_name'],
				'last_name'	=> $member['last_name']
			);
			$this->multipage->set_data_array($session_data);

			// Check that we have an event to register for
			extract($this->multipage->get_data_array(['event_id','event_type','organizer_id','event_cutoff_datetime']));
			if(empty($event_id)){
				$this->die_error(__METHOD__,'Event not yet defined in session. Can\'t register.');
			}
	
			// Is this rider already registered?
			$reg_status=$this->model_parando->is_registered($event_id,$member_id);			
			if(!empty($reg_status)){
				$contact_url=$this->multipage->get_data('event_contact_url');
				$cto="<A HREF=$contact_url TITLE='Contact the Organizer'>contact the organizer</a>";
				if('' . $reg_status == 'DNS'){
					$emessage = <<<EOT
Our records indicate you have been withdrawn from this event ($reg_status). You can't re-register
after withdrawing. Please $cto if you have any questions.
EOT;
				}else{
					$emessage = <<<EOT
You are already registered for this event. Use the link in your 
confirmation email, or $cto, if you wish to cancel your registration.
EOT;
				}
				$this->die_error('Already Registered',$emessage);
			}

			// Is this the organizer? 
			$this_is_event_organizer=($member_id==$organizer_id);
			$this->multipage->set_data('this_is_event_organizer',$this_is_event_organizer);
			
			if($this->has_full_member_record($member)){
				
 				$this->push_registration_item($member);  // If data is complete, push this registration onto item list
                                                            // SIDE EFFECT -- saves memdata and other stuff to session
                                                            
                extract($this->multipage->get_data_array(['membership','membership_new_expires','expires','event_lodging_options']));

				$extra_pages=[];    // And then figure out what extra pages
                                    // are needed
                                    
				if($event_type=='fleche') $extra_pages[]='register_fleche';
				
				switch($membership){
                    case "guest":
                        $extra_pages[]='choose_join';
                        break;
                    case "terminated":
                        $extra_pages[]='choose_rejoin';
                        break;
                    case "current":
                        $new_xdate= new DateTime($membership_new_expires);
                        $old_xdate= new DateTime($expires);
                        if($new_xdate > $old_xdate) $extra_pages[]='choose_renew';
                        break;
                    default:
                        $this->die_error(__METHOD__, "Unknown or missing membership status: '$membership'");
                        break;
                }

                if(!empty($event_lodging_options)) $extra_pages[]='select_lodging';
                
                if(empty($extra_pages)) 
                    $this->multipage->set_current_page('review_charges');
                else
                    $this->multipage->call_page($extra_pages, 'review_charges'); 
 				
			}else{
                $this->multipage->set_current_page('get_new_member_data');  // data was incomplete (legacy record), branch back to GET_NEW_MEMBER_DATA
			}
			
	}

	public function has_full_member_record($mem){
		$missing_field =
				empty($mem['m_street']) || 
				empty($mem['m_city']) || 
				empty($mem['m_state']) || 
				empty($mem['m_zip']) || 
				empty($mem['m_country']) || 
				empty($mem['m_phone']) || 
				empty($mem['m_email']) || 
				empty($mem['emergency_name']) || 
				empty($mem['emergency_phone']);
		return !$missing_field;
	}
	

    // Create item list and push registration onto it, and then computes possible fees (but don't push them yet)  WHY THE SECOND FUNCTION HERE?

    private function push_registration_item($mem){

		
		$memdata=$this->model_parando->compute_memdata($mem);  // REFACTOR? Maybe this should have been done by get_member?
		extract($memdata);	

        if(empty($rusa_id) && $this->model_parando->require_rusa_membership){
            $this->die_error(__METHOD__,'Not a RUSA member. Can\'t register.');
        }
		
		$items=[];		// start the list of items

		extract($this->multipage->get_data_array(['event_name','event_distance','event_id','event_member_price','event_waive_guest_fee',
			'this_is_event_organizer','coupon','coupon_data']));
		$event_distance=strtr($event_distance,['&egrave;'=>'e']);
		$event_value="$event_name $event_distance (Event: $event_id)";
		$registration_fee_item=$this->accounting_parando->get_product_item('registration_fee');
		if(!empty($coupon_data)) $this->load->model('coupon');
		if(!empty($coupon_data) && $this->coupon->is_valid_for_event($coupon_data, $event_id)){
			$coupon_id=$coupon_data['id'];
			$registration_fee_item['unit_price']=0;
			$registration_fee_item['value']=" Valid Coupon ($coupon_id), fee waived";
			$this->multipage->set_data_array(['spend_coupon'=>$coupon_data['code'],'free_event_comment'=>"Valid Coupon ($coupon_id)."]);
			//$this->die_error(__METHOD__,"Has valid coupon:  $coupon_id");
		}else if($mem['administrator']==1 || $mem['waive_fees']==1 || $this_is_event_organizer){
			$registration_fee_item['unit_price']=0;
			$registration_fee_item['value']=$event_value . " Volunteer";
			$this->multipage->set_data('free_event_comment',"Event Volunteer.");
		}else{
			$registration_fee_item['unit_price']=$event_member_price;
			$registration_fee_item['value']=$event_value;
		}
		$items[]=$registration_fee_item;   // push the registration fee onto the item list
		


// DO WE NEED ALL OF THESE HERE NOW?

// we are getting the prices of the guest fee and the join/renew prices, expiration grace dates, WHY NOW??!!
// Can't add gest fee to item list before giving the rider a chance to join

		$new_member_join=$this->accounting_parando->get_product_price('new_member_join');
		$current_renew=$this->accounting_parando->get_product_price('current_renew');
		$expired_renew=$this->accounting_parando->get_product_price('expired_renew');
		$possible_late_fee=$this->accounting_parando->get_product_price('late_fee');
		if($event_waive_guest_fee){
			$guest_fee_normal=0;
			$guest_fee_special=0;
		}else{
			$guest_fee_normal=$this->accounting_parando->get_product_price('guest_fee');
			$guest_fee_special=$this->accounting_parando->get_product_price('guest_fee_special');
		}

		$nmonth = $this->datetime_now->format('m');
		$nyear = $this->datetime_now->format('Y');
		$october = $this->model_parando->membership_grace_month;
		$membership_grace_period=($nmonth >= $october);
		$new_xyear = ($membership_grace_period) ? $nyear+1 : $nyear;
		$membership_new_expires = $new_xyear . '-12-31';

		$session_data=compact(
			'membership_grace_period',
			'membership_new_expires',
			'new_member_join',
			'expired_renew',
			'current_renew',
			'guest_fee_normal',
			'guest_fee_special',
			'items',
			'possible_late_fee'
		);

		$this->multipage->set_data_array(array_merge($memdata,$session_data));

	}


   // Finally there can be no further charges, commit the purchase order
    // Since we can't fulfill yet (need payment confirmation) we assume any pre-existing
    // PO was generated by a passive payment cancellation, so we delete it silently. 
    
    // After reviewing charges and committing the PO we go on to SMARTWAIVER
    
    public function _review_charges_body(){
        extract($this->multipage->get_data_array(['items','free_event_comment']));
        $total_fee = $this->accounting_parando->items_total($items);
        $this->multipage->set_data('total_fee',$total_fee);
        
        if($total_fee==0)
            $body="<div class='vspace'></div><div class='text-center bigger sans'>There is no fee for this event. $free_event_comment</div>";
        else
            $body='';
        
        return $body;
    }
    
    public function _review_charges(){

		extract($this->multipage->get_data_array(['member_id','items','event_id','invoice_number','spend_coupon']));

        $total_fee = $this->accounting_parando->items_total($items);
        if($total_fee==0){
            $payment_method='free_event';
            $description='Event Registration (free)';
        }else{
            $payment_method='pay_online'; // Hardcoded -- no support for any other method
            $description='Event Registration';
		}
		
		$deleted=$this->accounting_parando->delete_purchase_by_event($event_id,$member_id); // don't want to accidentally make two POs!

		$purchase = $this->accounting_parando->create_purchase_order($member_id, $items, get_class($this) . ':' . $payment_method, $event_id, $description);
		if(is_string($purchase)){$this->die_error(__METHOD__, "Error creating purchase order: $purchase");}
		$invoice_number=$purchase['invoice_number'];
		$total_fee=$purchase['amount_due'];
		
		if(!empty($spend_coupon)){
			$this->load->model('coupon');
			$rc=$this->coupon->spend($spend_coupon,$invoice_number,$member_id);		
			if($rc!==true) $this->die_error(__METHOD__,"Couldn't spend coupon: $rc");
		}		
		
		$this->multipage->set_data_array(compact('items','invoice_number','total_fee','payment_method'));
		

    }

	// SMARTWAIVER
	
	
	
	public function _smartwaiver_form_body(){

		$intro=<<<EOT
<div class="vspace"></div>
<div class="text-center bigger sans">All riders who start 
	this event must agree to a waiver of liability.</div>
<div class="narrower bordered italic"><p>These events are not for beginners.  
The courses are hilly. You may face extremes of weather: heat, cold, wind, rain. 
The roads are unmarked and open to automobile traffic. There will be night riding in 
the longer events.  If you understand these risks, and still want to ride, you will 
need to sign a RUSA waiver to that effect.</P></div>
EOT;

// rider_id  rusa_id  member_id

		$member_id = $this->multipage->get_data('member_id');
		if(empty($member_id)) $this->die_error(__METHOD__, "This can't happen: no member_id in session");
		$birthday = $this->model_parando->get_member_birthday($member_id);

		if(empty($birthday)){


			$birthday_prompt =<<<EOT
<div class="text-center bigger sans">
	<P>Enter your <B>date of birth</B></p>',
</div>
EOT;

    		$field_list=[
['birth_year','Year','Select birthday year',false,'required|callback__birthday_check','select','get_year_options'],
['birth_month','Month','Select birthday month',false,'required','select','get_month_options'],
['birth_day','Day','Select birthday day',false,'required','select','get_day_options']
                           ];
                                               
			foreach ($field_list as $f){
		  		$birthday_prompt .= $this->multipage->form_group(...$f);
			}
	}else{
		extract($birthday);
		$birthday_prompt = <<<EOT
<input type="hidden" id="birth_year" name="birth_year" value="$year">
<input type="hidden" id="birth_month" name="birth_month" value="$month">
<input type="hidden" id="birth_day" name="birth_day" value="$day">
EOT;
	
	}
		      	
                                      
	$redirect_prompt=<<<EOT
<div class="text-center bigger sans">
	Press the button below to be redirected to the RUSA "Smart" Waiver System.
</div>
EOT;
	
	
	
	'';	
	$buttons=[
		['tag'=>'yes','text'=>'Sign the Liability Waiver'],
		['tag'=>'cancel','text'=>'Cancel']
	];
	
		$form_submit = "";
		foreach ($buttons as $button){
			$tag=$button['tag'];
			$text=$button['text'];
			$form_submit .=  "<button type='submit' name='submit' value='$tag'>$text</button>";
		}


	return ($intro . $birthday_prompt . $redirect_prompt. $form_submit);
	}
	
	// private $smartwaiver_url="https://waiver.smartwaiver.com/w/61e6d4476b814/web/"; // TESTWAIVER
//	private $smartwaiver_url="https://waiver.smartwaiver.com/w/61ec4a88504eb/web"; // PAR-EVENT-WAIVER
	private $smartwaiver_url="https://waiver.smartwaiver.com/w/61ec4a88504eb/kiosk"; // PAR-EVENT-WAIVER
	
	private $new_smartwaiver_url="https://waiver.smartwaiver.com/w/u27ctgkovnad1jocxuysoe/kiosk";
	
	
	private $smartwaiver_api_key="f4fdd727ca03e96ec60fd3dcd954d527";
	
	public function _smartwaiver(){  

		if( $this->input->post('submit') == 'cancel' ){
				$this->clean_up_before_cancel(); 
				$this->multipage->set_current_page('register_cancel');  
		}else{
			extract($this->multipage->get_data_array([
			'first_name','last_name','member_id',
			'invoice_number','event_name','event_distance','event_DateTime'
			]));

            $birth_year=$this->input->post('birth_year');
            $birth_month=$this->input->post('birth_month');
            $birth_day=$this->input->post('birth_day');
                        
            $birth_data = [
            	'member_id' => $member_id,
				'year' => $birth_year,
				'month'  => $birth_month,
				'day'  => $birth_day 
			];

			$this->db->replace('member_dob', $birth_data);

            
            $dob_yyyymmdd=sprintf("%04d%02d%02d", $birth_year+0, $birth_month+0, $birth_day+0);
            
            
            // $this->die_error(__METHOD__, "Birthday OK,  $dob_yyyymmdd");		
		
            
            $designated_event = "'Event: $event_name $event_distance'";
            $event_date_str = $event_DateTime->format("'m d Y'");


            $order_token=$this->accounting_parando->make_order_token($invoice_number);  // NO! DONT USE THE SAME ORDER TOKEN AS WITH PAYPAL -- SECURITY!!
            $sw_parms=[
                'auto_tag' => $order_token,
                // 'wautofill_tag' => $token+1,
                'wautofill_firstname' => $first_name,
                'wautofill_lastname' => $last_name,
                'wautofill_dobyyyymmdd' => $dob_yyyymmdd,
                
                'customfield_bGxWQ4or3FMmqCXz5uFF3M' => $designated_event,
                'customfield_5HHCfZ22ijRkmmv3LbmrfB' => $event_date_str
                
//                'auto_anyoneelseneedtosign' => 0
            ];
            
// https://waiver.smartwaiver.com/w/61ec4a88504eb/kiosk/?auto_tag=234-1681922200-AB449E69&wautofill_firstname=Chris&wautofill_lastname=Nadovich&wautofill_dobyyyymmdd=19600503&customfield_bGxWQ4or3FMmqCXz5uFF3M='Event: Tscheschter Kaundi 200K Brevet'&customfield_5HHCfZ22ijRkmmv3LbmrfB='06 04 2023'
            
            $a=[];
            foreach($sw_parms as $k=>$v){
                $a[]="$k=$v";
            }
            $sw_put_parms=implode('&',$a);
            
            $sw_put = $this->smartwaiver_url . "/?" . $sw_put_parms;
            
//            $this->die_error(__METHOD__, "SW Put=$sw_put");
            
            redirect($sw_put); 
        }
	}	

	public function smartwaiver_complete($token,$smartwaiver_id){
	
        // Validate token 
        
        $result = $this->accounting_parando->check_order_token($token);
        if(is_array($result)){
            list($member_id, $token_time, $hash)=$result;
            $invoice_number="$member_id-$token_time";
        }else $this->die_error(__METHOD__,"Invalid token: '$token'");
        
        $purchase=$this->accounting_parando->get_purchase($invoice_number);
        if(empty($purchase)) $this->die_error(__METHOD__,"No such purchase: '$invoice_number'");
        extract($purchase);
        

        $total_fee=$purchase['amount_due'];

        // We should be sure of at least these
        $this->multipage->set_data_array(compact('invoice_number','total_fee','smartwaiver_id','member_id','event_id'));
        
        if($total_fee==0){  // free event
            $rc=$this->_fulfill_all($purchase);
            if($rc!='OK'){
                $deleted=$this->accounting_parando->delete_purchase($invoice_number); // don't want to accidentally make two POs!
                $this->die_error(__METHOD__, "Error fulfilling purchase order $invoice_number: $rc");
            }
            $ar=$this->put_waiver($smartwaiver_id,$member_id,$event_id);
            if($ar!=1) $this->die_error(__METHOD__,"Failed to put waiver id $smartwaiver_id, ar=$ar, m=$member_id, e=$event_id");
            $rc=$this->send_confirmation_email();		
            if($rc!==true){$this->die_error(__METHOD__, "Error sending confirmation emails:\n".$rc);}
            $this->multipage->set_current_page('register_done'); 
        }else{ // non free event
            // Online payment only -- no support for any other method
            $this->multipage->set_current_page('pay_online');
        }
        
        
        $this->multipage->view_current_page();

	}
	
	const saved_waiver_path="/home/tomr/public_html/assets/parando/waivers";
	const saved_waiver_url="https://parando.org/assets/parando/waivers";
	
 	public function save_waiver(){
         $this->die_error(__METHOD__, print_r($this->smartwaiver_save_pdf(234, 371,'gjkY6qjmTkArWGNhzY8fwJ'),true));
 	}
	
	private function put_waiver($waiver_id,$member_id,$event_id){
        if(empty($waiver_id)) $this->die_error(__METHOD__, "Missing waiver ID");

        $waiver_url=$this->smartwaiver_save_pdf($member_id,$event_id,$waiver_id);
        
        $this->multipage->set_data('waiver_url',$waiver_url);
        
		$sql="update roster set waiver_url='{$waiver_url}' WHERE event_id='{$event_id}' AND member_id='{$member_id}'";
		$q=$this->db->query($sql);
		return $this->db->affected_rows();
	}
	
	private function get_waiver($member_id,$event_id){
		$sql="select waiver_url from roster WHERE event_id='{$event_id}' AND member_id='{$member_id}'";
		$q=$this->db->query($sql);
			if($q->num_rows()==1){
                $a=$q->row_array();
				return $a['waiver_url'];
			}else{
				return null;
			}		
	}

	const SMARTWAIVER_RETRIES=8;
	
	private function smartwaiver_save_pdf($member_id, $event_id, $waiver_id){
	
        // The API Key for your account
        $apiKey = $this->smartwaiver_api_key;

        // The unique ID of the signed waiver to be retrieved
        $waiverId = $waiver_id;
        
        sleep(2); // lets not rush things....
        $sw = new Smartwaiver($apiKey);
        for($got_waiver=false, $i=0; !$got_waiver && $i<self::SMARTWAIVER_RETRIES; $i++){
            try{ 
                $waiver = $sw->getWaiver($waiverId, true); 
                $pdf_base64=$waiver->pdf;
                if(!empty($pdf_base64)){
                    $pdf=base64_decode($pdf_base64);
                    if($pdf!==false) $got_waiver=true;
                }
            }
            catch(Exception $e) { sleep(pow(2,$i+1)); }
        }
        if($got_waiver==false) $this->die_error(__METHOD__,"Smartwaiver failed. Grrr... Maybe try again later?");
        
        // return self::saved_waiver_url . "/unavailable/$waiver_id";  // best we can do
        
        $event=$this->model_parando->get_event($event_id);
        $member=$this->model_parando->get_member($member_id);
        
        if(empty($event)) $this->die_error(__METHOD__,"Empty event");
        if(empty($member)) $this->die_error(__METHOD__,"Empty member");
        
        $event_tagname=$this->model_parando->make_event_tagname($event);
        $last_name=strtoupper($this->string_clean($member['last_name']));
        
        $file_basename="SW-$last_name-$waiver_id.pdf";
        $directory=self::saved_waiver_path . "/$event_tagname";
        $base_url=self::saved_waiver_url . "/$event_tagname";
        
        $file="$directory/$file_basename";
        $url="$base_url/$file_basename";
        if(!file_exists($directory)) mkdir($directory);
        $fp = fopen($file, "w") or $this->die_error(__METHOD__, "Couldn't open $file for writing!");
        fwrite($fp, $pdf) or $this->die_error(__METHOD__, "Couldn't write values to file!"); 
        fclose($fp); 
        
        return $url;

	}
	
    private function string_clean($string) {
        $string = trim($string);
        $string = str_replace(' ', '-', $string); // Replaces all internal spaces with hyphens.
        return preg_replace('/[^A-Za-z0-9\-]/', '', $string); // Removes special chars.
    }
	
	public function _get_year_options(){
        $year_now = date("Y");
        $year_list=[];
        for ($y=$year_now; $year_now-$y<100; $y--){
            $year_list["$y"]="$y";
        }
        return $year_list;
	}

	public function _get_month_options(){
        $month_list=[];
        for ($m=1; $m<=12; $m++){
            $m_str=sprintf("%02d",$m);
            $month_list[$m_str]= date('F', mktime(0, 0, 0, $m));;
        }
        return $month_list;
	}

	public function _get_day_options(){
        $day_list=[];
        for ($d=1; $d<=31; $d++){
            $d_str=sprintf("%02d",$d);
            $day_list[$d_str]= $d_str;
        }
        return $day_list;
	}

	
	public function _birthday_check() {

            $birth_year=$this->input->post('birth_year');
            $birth_month=$this->input->post('birth_month');
            $birth_day=$this->input->post('birth_day');
            
            $dob_yyyy_mm_dd="$birth_year-$birth_month-$birth_day";
            
            
            try{
                if (empty($birth_year) || empty($birth_month) || empty($birth_day) ) throw new Exception("No birthday specified.  Please enter a valid birthday.");
                if (strtotime($dob_yyyy_mm_dd) === false) throw new Exception("Invalid birthday: $dob_yyyy_mm_dd.  Please enter a valid birthday.");
                $dob_datetime = new DateTime($dob_yyyy_mm_dd);
                $test=$dob_datetime->format('Y-m-d');
                if($test!=$dob_yyyy_mm_dd) throw new Exception("Birthday date does not exist.  Please enter a valid birthday.");
                $now_datetime = new DateTime("now");
                $day_diff = 0 + ($dob_datetime->diff($now_datetime)->format("%r%a"));
                $abs_day_diff=abs($day_diff);
                if($day_diff<0) throw new Exception("Birthday $abs_day_diff days in the future! Please enter a valid birthday.");
                if($day_diff<365) throw new Exception("Birthday too recent! Only $abs_day_diff days ago. Newborns can't ride brevets.  Please enter a valid birthday.");
            }
            
            catch(Exception $e) {
                $this->form_validation->set_message('_birthday_check', $e->getMessage());
                return FALSE;
            }
            
            return TRUE;
        
	}



	public function _pay_online(){  // fulfillment deferred till after payment
	
		if( $this->input->post('submit') == 'cancel' ){
					$this->clean_up_before_cancel();
					$this->multipage->set_current_page('register_cancel');
		}else{
			// PAYPAL Express Checkout Controller Setup

			// Clear cart from session userdata
			$this->session->unset_userdata('shopping_cart');

			extract($this->multipage->get_data_array(['contact_email','first_name','last_name','member_id','invoice_number']));
			$order_token=$this->accounting_parando->make_order_token($invoice_number);
			$purchase=$this->accounting_parando->get_purchase($invoice_number);
			
			if(empty($purchase)){$this->die_error(__METHOD__,"Purchase order not found: ($invoice_number)");}

			// Set cart data into session
			$cart=$this->accounting_parando->purchase_item_cart($purchase);
			$cart['member_first_name'] = $first_name;
			$cart['member_last_name'] = $last_name;
			$cart['contact_email'] = $contact_email;
			$cart['invoice_number'] = $invoice_number;
			$cart['order_token'] = $order_token;
			$cart['fulfillment_url'] = site_url(strtolower( get_class($this)) . "/credit_paypal_payment/$order_token");
			$cart['cancel_url'] = site_url(strtolower( get_class($this)) . "/paypal_cancel");

			$this->session->set_userdata('shopping_cart', $cart);

			redirect(site_url('express_checkout/SetExpressCheckout'));
        }
	}	

	// Fulfillment Controller Method
	// .. Try not to assume session is intact here
	
	public function credit_paypal_payment($token){

        $result = $this->accounting_parando->check_order_token($token);
        if(is_array($result)){
            list($member_id, $token_time, $hash)=$result;
            $invoice_number="$member_id-$token_time";
        }else $this->die_error(__METHOD__,"Invalid token: '$token'");

        $purchase=$this->accounting_parando->get_purchase($invoice_number);
        if(empty($purchase)) $this->die_error(__METHOD__,"No such purchase: '$invoice_number'");
        extract($purchase);

        $rc = $this->accounting_parando->credit_paypal_payment($token,[$this,'_fulfill_all'], __METHOD__);  

		if($rc!='OK' && $rc!='DUPLICATE'){$this->die_error(__METHOD__,$rc);}
		
		$smartwaiver_id=$this->multipage->get_data('smartwaiver_id'); // Need session for this
		
        $ar=$this->put_waiver($smartwaiver_id,$member_id,$event_id);
        if($ar!=1) $this->die_error(__METHOD__,"Failed to put waiver id $smartwaiver_id, ar=$ar, m=$member_id, e=$event_id");

		$email_rc=true;
		if($rc!='DUPLICATE') $email_rc=$this->send_confirmation_email();		

		if($email_rc!==true){$this->die_error(__METHOD__, "Error sending confirmation emails:\n".$rc);}
		$this->multipage->set_current_page('register_done'); 
		$this->multipage->view_current_page();
//		$this->member_session_init(); // no turning back

	}
	
	
	private function member_session_init(){
		$d=[
			'this_organization' => $this->model_parando->organization,
			'base_url' => base_url(),
			'contact_url' => $this->model_parando->contact_url,
			'webmaster_contact_url' => $this->model_parando->webmaster_contact_url,
			'admin_login_url' => $this->model_parando->admin_login_url
		];
		$this->multipage->init($d);
	}


	public function _fulfill_all($purchase){
		$purchase_id=$purchase['id'];
		$invoice_number=$purchase['invoice_number'];
		$items=$this->accounting_parando->get_purchase_items($purchase_id);
		if(!empty($items)){
			foreach($items as $i){
				$sku =  $i['sku'];
				$fulfillment_method = 'fulfill_' . $sku;
				if(method_exists($this,$fulfillment_method)){
					$rc = $this->{$fulfillment_method}($purchase, $i);
					if('OK'!=$rc){return "Item SKU=$sku: $rc";}
				}
			}
		}else{
			return "No items in invoice number $invoice_number";
		}
		return "OK";
	}

	// Cancellation Controller method

	public function paypal_cancel(){
		$this->clean_up_before_cancel();		// assumes good session with ['member_id','invoice_number','fleche_team_data']
		$this->multipage->set_current_page('register_cancel'); 
		$this->multipage->view_current_page();
	}
	
	private function clean_up_before_cancel(){
		$this->load->model('fleche_parando');
			extract($this->multipage->get_data_array(['member_id','invoice_number','fleche_team_data']));
			if(!empty($invoice_number)){
				$deleted=$this->accounting_parando->delete_purchase($invoice_number);
			}
			if(!empty($fleche_team_data) && !empty($member_id) && $member_id==$fleche_team_data['captain_id']){
				$deleted=$this->fleche_parando->delete_fleche_team_by_data($fleche_team_data); 
			}
	}
	
  // Fulfillment functions. Ideally these should derive all information from the PO and Item objects

	private function fulfill_current_renew($purchase, $item){return $this->fulfill_join_renew($purchase, $item);}
	private function fulfill_expired_renew($purchase, $item){return $this->fulfill_join_renew($purchase, $item);}
	private function fulfill_new_member_join($purchase, $item){return $this->fulfill_join_renew($purchase, $item);}

	private function fulfill_join_renew($purchase, $item){
	
		$value = $item['value'];
		$member_id = $purchase['member_id'];
		$member=$this->model_parando->get_member($member_id);
		$expires=$member['expires'];
		$xdate = new DateTime($expires);
			
		if(!preg_match("/(\d{4})-(\d{2})-(\d{2})/", $value,$a)){return "Misformatted item value: '$value'";}
		list($membership_new_expires, $xyear, $xmonth, $xday)=$a;				
		$membership_new_xdate = new DateTime($membership_new_expires);

		if($expires!="0000-00-00" && $xdate >= $membership_new_xdate){
			return "Your membership term ends $expires and cannot be extended at this time.";
		}
								
		$this->db->where('id',$member_id);
		$rc=$this->db->update('member',['expires'=>$membership_new_expires]);
		if(FALSE==$rc){return "Failed to update membership term expiry date for member_id=$member_id to $new_expires";}
		
		return 'OK';

	}

	private function fulfill_registration_fee($purchase, $item){
		$value = $item['value'];
		$member_id = $purchase['member_id'];
		$total_fee = $purchase['amount_due'];
		$event_id = $purchase['event_id'];
		if(empty($event_id)){return "Missing or null event";}
		$invoice_number = $purchase['invoice_number'];
		$data = compact('event_id','member_id','invoice_number','total_fee');
		$rc=$this->db->insert('roster', $data);
		if(FALSE==$rc){return "Failed to insert member_id=$member_id into roster for event_id=$event_id";}
		return 'OK';
	}


	private function fulfill_new_fleche_team($purchase, $item){
		$value = $item['value'];
		if(preg_match("/^ID=(\d+);/",$value,$m) && !empty($m[1])){
			$fleche_team_id=$m[1];
		}else{
			return "Malformed fleche_team_id Value Field: $value";
		}
		$event_id = $purchase['event_id'];
		if(empty($event_id)){return "Missing or null event_id";}
		$this->db->where('id', $fleche_team_id);
		$rc=$this->db->update('fleche_team', ['event_id'=>$event_id]);
    $n=$this->db->affected_rows();
		if(!$rc || $n!=1){return "Failed to link event_id=$event_id to fleche_team_id=$fleche_team_id; RC=$rc; NAR=$n;";}
		return 'OK';
	}

	private function fulfill_new_fleche_rider($purchase, $item){
		$value = $item['value'];
		if(preg_match("/^ID=(\d+);/",$value,$m) && !empty($m[1])){
			$fleche_team_id=$m[1];
		}else{
			return "Malformed fleche_team_id Value Field: $value";
		}
		$member_id = $purchase['member_id'];
		if(empty($member_id)){return "Missing or null member_id";}
		$rc=$this->db->insert('fleche_team_member',compact('fleche_team_id','member_id'));
		if(!$rc){return "Failed to insert member_id=$member_id into fleche_team_id=$fleche_team_id";}
		return 'OK'; 
	}

	private function fulfill_tandem_bike($purchase, $item){
		$value = $item['value'];
		if(preg_match("/Partner: (.+);/",$value,$m) && !empty($m[1])){
			$tandem_partner=$m[1];
		}else{
			return "Malformed tandem_partner Value Field: $value";
		}
		$member_id = $purchase['member_id'];
		if(empty($member_id)){return "Missing or null member_id";}
		$event_id = $purchase['event_id'];
		if(empty($event_id)){return "Missing or null event";}
		$this->db->where('member_id', $member_id);
		$this->db->where('event_id', $event_id);
		$rc=$this->db->update('roster',['tandem'=>$tandem_partner]);  // assumes roster already exists
		$n=$this->db->affected_rows();
		if(!$rc || $n!=1){return "Failed to mark member_id=$member_id as tandem rider in event_id=$event_id; RC=$rc; NAR=$n";}
		return 'OK'; 
	}



// FLECHE PAGES

	public function _get_fleche_teams_options(){
		$event_id=$this->multipage->get_data('event_id');
		$sql="SELECT fleche_team.id as team_id, fleche_team.name as team_name, first_name, last_name 
			FROM fleche_team,member where fleche_team.captain_id=member.id AND fleche_team.event_id={$event_id} order by fleche_team.name";
		$options=[];
		$q=$this->db->query($sql);
		foreach($q->result() as $row){
			$options[$row->team_id]=$row->team_name . ' (Capt: ' . $row->first_name . '&nbsp;' . $row->last_name . ')';
		}
		return $options;
	}
	
	public function _register_fleche_form_body(){

		$teams = $this->_get_fleche_teams_options();

		if(!empty($teams)){
			$join_team_radio_button=<<<EOT
			<input type='radio' name='fleche_reg_type' id='fleche_reg_type_join_fleche_team' value='join_fleche_team'>
			<label for='fleche_reg_type_join_fleche_team'>Join Existing Team</label>
EOT;
		}else{
			$join_team_radio_button=<<<EOT
<div class=note>There are no existing teams to join at this time.</div>
EOT;
		}

		$event_id=$this->multipage->get_data('event_id');
		$event = $this->model_parando->get_event($event_id);
		if(empty($event)){
			$this->die_error('Event Error','Event (' . $event_id . ') not found.');
		}
		$schedule=$this->model_parando->event_schedule($event);
		$fleche_captain_reg_cutoff_datetime=$schedule['fleche_captain_reg_cutoff_datetime'];
		if($this->datetime_now < $fleche_captain_reg_cutoff_datetime){
			$create_team_radio_button=<<<EOT
			<input type='radio' name='fleche_reg_type' id='fleche_reg_type_create_fleche_team' value='create_fleche_team'>
			<label for='fleche_reg_type_create_fleche_team'>Create a New Team</label></div><div class='image-radio'>
EOT;
		}else{
			$cutoff=$fleche_captain_reg_cutoff_datetime->format('G:i T, l j F Y');
			$create_team_radio_button=<<<EOT
<div class=note>Teams may not be created at this time. Team registration closed $cutoff.</div>
EOT;
		}

		$error=(!empty(form_error('fleche_reg_type')))?("<div class=verror>" . form_error('fleche_reg_type') . "</div>"):"";
	
		$body =<<<EOT
<div class=form-group>
	<label for="fleche_reg_type">Registration Type*</label>    
	<div class='indented'>$error
		<div class='image-radio'>
			$create_team_radio_button
			$join_team_radio_button
		</div>
	</div>
</div>
EOT;

		return $body;
	}
	
	// REGISTER_FLECHE -- branches either to CREATE_FLECHE_TEAM OR JOIN_FLECHE_TEAM

	public function _register_fleche(){
 		$this->form_validation->set_rules('fleche_reg_type','Fleche Team Option', 'trim|required', 
				array('required'=>'You must choose to create a new team, or join an existing one.') );
				$fleche_reg_type=$this->input->post('fleche_reg_type');
			if ($this->form_validation->run()){
				switch($fleche_reg_type){
					case 'create_fleche_team':
					case 'join_fleche_team':
						$this->multipage->set_current_page($fleche_reg_type);
						break;
				}
			}
	}

	public function _fleche_team_name_check($proposed_name){
		$this->load->model('fleche_parando');
		$event_id=$this->multipage->get_data('event_id');
		$names=$this->fleche_parando->get_fleche_team_names($event_id);
		$test_name=preg_replace('/\s+/', '', strtolower($proposed_name));
		$best_match=null;
		$match_dist=null;
		foreach($names as $name){
			$existing_name=preg_replace('/\s+/', '', strtolower($name));
			$ld=levenshtein($existing_name,$test_name);
			if(empty($best_match) || $ld<$match_dist){
				$best_match=$name;
				$match_dist=$ld;
			}
		}
		if($match_dist!==null && $match_dist<5){
			$this->form_validation->set_message('_fleche_team_name_check',"There is already a team, '$best_match', with a name similar to your proposed name, '$proposed_name'. Team
			names must be <em>very</em> unique. Sorry. If you really meant '$best_match' that team already exists. You can't create it again.");
			return false;
		}else{
			return true;
		}
	}

	public function _fleche_team_schedule_check(){

		$this->load->model('fleche_parando');
		$event_id=$this->multipage->get_data('event_id');

		$start_day=(integer)$this->input->post('fleche_team_start_day');
		$start_hour=(integer)$this->input->post('fleche_team_start_hour');
		$start_min=(integer)$this->input->post('fleche_team_start_minute') * 15;

		extract($this->fleche_parando->calc_fleche_team_schedule($event_id, [$start_day, $start_hour, $start_min]));

		if($start_datetime<$earliest_start_datetime){
			$this->form_validation->set_message('_fleche_team_schedule_check', 
				"Start time too early ($start_dow_time_str). Allowable start times are from noon Thursday through 10 AM on Saturday.");
			return false;
		}elseif($start_datetime>$latest_start_datetime){
			$this->form_validation->set_message('_fleche_team_schedule_check', 
				"Start time too late ($start_dow_time_str). Allowable start times are from noon Thursday through 10 AM on Saturday.");
			return false;
		}else{
			return true;
		}

	}
	
	// CREATE or JOIN FLECHE TEAM methods eventually branch to BIKE_STYLE

	public function _join_fleche_team(){
		$this->load->model('fleche_parando');
		extract($this->multipage->get_data_array(['event_id','member_id','items']));
		$fleche_team_id=$this->input->post('fleche_team_id');
		if(empty($fleche_team_id)){$this->die_error(__METHOD__, "Empty or missing team ID");}
		$fleche_team_data=$this->fleche_parando->get_fleche_team_by_id($fleche_team_id);
		if(empty($fleche_team_data)){$this->die_error(__METHOD__, "Could not get data for fleche team ID=$fleche_team_id");}
		$fleche_team_schedule=$this->fleche_parando->calc_fleche_team_schedule($event_id, $fleche_team_data['start_date'] . ' ' . $fleche_team_data['start_time']); 
		if(empty($fleche_team_schedule)){$this->die_error(__METHOD__, "Could not calculate schedule for fleche team ID=$fleche_team_id");}
		$team_value="ID=$fleche_team_id; " . $fleche_team_data['name'];
		$new_fleche_rider_item=$this->accounting_parando->get_product_item('new_fleche_rider');
		$new_fleche_rider_item['value']=$team_value;
		$items[]=$new_fleche_rider_item;
		$this->multipage->set_data_array(compact('fleche_team_data','fleche_team_schedule','items'));
	}


	public function _create_fleche_team(){

		$this->load->model('fleche_parando');
		extract($this->multipage->get_data_array(['event_id','member_id']));

		$start_day=(integer)$this->input->post('fleche_team_start_day');
		$start_hour=(integer)$this->input->post('fleche_team_start_hour');
		$start_min=(integer)$this->input->post('fleche_team_start_minute') * 15;

		$schedule=$this->fleche_parando->calc_fleche_team_schedule($event_id, [$start_day, $start_hour, $start_min]); // could've cached this from the schedule check, but heck -- fast computer

		$data=[
			'event_id'=>$event_id,
			'name'=>strip_tags($this->input->post('fleche_team_name')),
			'captain_id'=>$member_id,
			'start_date'=>$schedule['start_date_str'],
			'start_time'=>$schedule['start_time_str'],
			'start_city'=>strip_tags($this->input->post('fleche_team_start_city')),
			'start_state'=>$this->input->post('fleche_team_start_state'),
			'map'=>$this->input->post('fleche_team_map'),
			'description'=>strip_tags($this->input->post('fleche_team_description')),
			'approved'=>false
		];

		$this->multipage->set_data_array(['fleche_team_data'=>$data, 'fleche_team_schedule'=>$schedule]);

	}

	public function _create_fleche_team_confirm_body(){
		$body="";
		extract($this->multipage->get_data('fleche_team_data'));
		extract($this->multipage->get_data('fleche_team_schedule'));
		$captain=$this->model_parando->members_by_id[$captain_id];
		$captain_name=$captain['first_name'].'&nbsp;'.$captain['last_name'];
		$body=<<<EOT
<h2>New Team</h2>
<div class=fleche-team>
<TABLE ID=team-summary Width=100%>
<TR><TD>Team Name</TD><TD>$name</TD></TR>
<TR><TD>Team Start</TD><TD>$start_datetime_str</TD></TR>
<TR><TD>Start Location</TD><TD>$start_city, $start_state</TD></TR>
<TR><TD>Team Captain</TD><TD>$captain_name</TD></TR>
<TR><TD>Map URL</TD><TD><A HREF='$map'>$map</A></TD></TR>
<TR><TD>Description</TD><TD>$description</TD></TR>
</TABLE>
</DIV>
EOT;
		return $body;
	}

	public function _create_fleche_team_confirm(){
		extract($this->multipage->get_data_array(['fleche_team_data','items']));
		if( $this->input->post('submit') == 'yes' ){
			if(empty($fleche_team_data)){	
				$this->die_error(__METHOD__, "Missing fleche team data.");
			}else{
				$ftd=$fleche_team_data;
				$ftd['event_id']=-1; // orphaned till fulfillment
				$rc=$this->db->insert('fleche_team', $ftd); 
				if(!$rc){$this->die_error(__METHOD__,"Failed to insert fleche team data into database.");}
				$fleche_team_id=$this->db->insert_id();
				$fleche_team_data['id']=$fleche_team_id;

				$team_value="ID=$fleche_team_id; " . $fleche_team_data['name'];

				$new_fleche_team_item=$this->accounting_parando->get_product_item('new_fleche_team');
				$new_fleche_team_item['value']=$team_value;
				$items[]=$new_fleche_team_item;

				$new_fleche_rider_item=$this->accounting_parando->get_product_item('new_fleche_rider');
				$new_fleche_rider_item['value']=$team_value; 
				$items[]=$new_fleche_rider_item;

				$this->multipage->set_data_array(compact('items','fleche_team_data'));
			}
		}elseif( $this->input->post('submit') == 'no' ){
			$this->multipage->set_current_page('register_cancel'); // should be nothing to clean up
		}

	}
	
	// BIKE_STYLE is the final page in the FLECHE registration. 
	// For tandems, it branches to TANDEM_PARTNER, but in either case, there is a return_page() to go back to the call_page(), maybe in register_known()  

	public function _bike_style(){
		$bike_style=$this->input->post('bike_style');
		if($bike_style=='tandem'){
			$this->multipage->set_current_page('tandem_partner');		
		}else{
			$this->multipage->return_page();		
		}
	}

	public function _tandem_partner(){
		$tandem_partner=$this->input->post('tandem_partner',true);
		extract($this->multipage->get_data_array(['items']));
		$new_item=$this->accounting_parando->get_product_item('tandem_bike');
		$new_item['value']="Partner: $tandem_partner;";
		$items[]=$new_item;
		$this->multipage->set_data_array(compact('items'));
		$this->multipage->return_page();		
	}
	
	
	// LODGING

	public function _select_lodging(){
	
        $lo=$this->parse_lodging_options();
		$loi=$this->input->post('lodging');
		if(!array_key_exists($loi,$lo)) $this->die_error(__METHOD__, "Invalid lodging option ID number: $loi");
		$lodging_choice_description=$lo[$loi]['description'];
		$lodging_choice_price=$lo[$loi]['price'];
		extract($this->multipage->get_data_array(['items']));
        $lodging_fee_item=$this->accounting_parando->get_product_item('lodging_fee', $lodging_choice_description);
        $lodging_fee_item['unit_price']=$lodging_choice_price;
        $lodging_fee_item['value']=$lodging_choice_description;
        $items[]=$lodging_fee_item;
        $total_fee = $this->accounting_parando->items_total($items);
		$this->multipage->set_data_array(compact('lodging_choice_description','lodging_choice_price','items','total_fee'));
		
		$this->multipage->return_page();
		
	}
	
	private function parse_lodging_options(){
			extract($this->multipage->get_data_array(['event_lodging_options']));
			$lol=explode(',',$event_lodging_options);
			$options=[];
			$options[]=['description'=>'No lodging needed', 'price'=>null];
			foreach($lol as $lo){
			  $dp=explode('=',$lo);
			  if(count($dp)!=2 || !is_numeric($dp[1]) || empty($dp[0])) $this->die_error(__METHOD__, "Invalid or missing lodging options for event: $event_lodging_options");
			  list($d,$p)=$dp;
			  $options[]=['description'=>$d, 'price'=>$p];
            }
            return $options;
	}
	
	public function _get_lodging_options(){
			$lo=$this->parse_lodging_options();
			$options=[];
			foreach($lo as $o){
                $d=$o['description'];
                $p=$o['price'];
                if($p===null)
                    $price='';
                elseif($p==0)
                    $price=' (Free)';
                else
                    $price=" (\$$p)";
                $options[]="$d$price";
            }
            return $options;
	}
 	
	

// MEMBERSHIP RENEWAL

	public function _choose_join(){
		$this->choose_join_renew();
	}
	public function _choose_rejoin(){
		$this->choose_join_renew();
	}
	public function _choose_renew(){
		$this->choose_join_renew();
	}

	private function choose_join_renew(){
			
		extract($this->multipage->get_data_array(['items','membership','expires','membership_new_expires','event_member_price','event_waive_guest_fee','event_id','coupon_data']));
	
		$join_renew=$this->input->post('join_renew');

		$join_renew_value = "New expiration: $membership_new_expires";

		switch($join_renew){
			case 'join':
				$items[]=$this->accounting_parando->get_product_item('new_member_join', $join_renew_value);
				$guest=false;
				break;
			case 'rejoin':
				$items[]=$this->accounting_parando->get_product_item('expired_renew', $join_renew_value);
				$guest=false;
				break;
			case 'renew':
				$items[]=$this->accounting_parando->get_product_item('current_renew', $join_renew_value);
				$guest=false;
				break;
			default:
				$guest=($membership!="current");
				break;
		}
			
		if($guest){ 
			$guest_value=($membership=="terminated")?"Membership expired: $expires":"";
			if(!empty($coupon_data)) $this->load->model('coupon');
			if(!empty($coupon_data) && $this->coupon->is_valid_for_event($coupon_data, $event_id)){
				$coupon_id=$coupon_data['id'];
				$guest_fee_item=$this->accounting_parando->get_product_item('guest_fee',$guest_value);
				$guest_fee_item['unit_price']=0;
				$guest_fee_item['value']=" Valid Coupon ($coupon_id), fee waived";
				$this->multipage->set_data_array(['spend_coupon'=>$coupon_data['code'],'free_event_comment'=>"Valid Coupon ($coupon_id)."]);
			//$this->die_error(__METHOD__,"Has valid coupon:  $coupon_id");

			}else if($event_waive_guest_fee==0){
				if($event_member_price == 0){
					$items[]=$this->accounting_parando->get_product_item('guest_fee_special',$guest_value);
				}else{
					$items[]=$this->accounting_parando->get_product_item('guest_fee',$guest_value);
				}
			}
		}
		$total_fee = $this->accounting_parando->items_total($items);
		$this->multipage->set_data_array(compact('items','join_renew','total_fee'));
		$this->multipage->return_page();
	}

	private function fleche_team_text(){
		extract($this->multipage->get_data('fleche_team_data'));
		extract($this->multipage->get_data('fleche_team_schedule'));
		$captain=$this->model_parando->members_by_id[$captain_id];
		$captain_name=$captain['first_name'].' '.$captain['last_name'];
		$body=<<<EOT

Team Name:  $name  
Team Captain:  $captain_name  
Team Start:  $start_datetime_str  
Start Location:  $start_city, $start_state  
Map URL:  $map   
EOT;
		return $body;
	}



// CONFIRMATION EMAIL

// THIS FUNCTION NEEDS SOME WORK. Needs to be adapted to use the proper club info based on event, and less dependant on session varbs

	private function send_confirmation_email(){
		
		extract($this->multipage->get_data_all()); // might consider how this could be done without the session variables
		
		if(empty($event_type)) $this->die_error(__METHOD__, "Session data for event missing or empty! Cookie blocked?");

		$rba=$this->model_parando->get_rba();
		$rba_name=$rba['first_name'] . ' ' . $rba['last_name'];
		$rba_email=$rba['m_email'];

		$webmaster=$this->model_parando->get_webmaster(); extract($webmaster);

		$this_organization = $this->model_parando->default_club['name'];
		
    	$login_url=site_url("login");
		$this_controller=strtoupper(get_class($this));
		
		$cancel_code=$this->make_cancel_code($event_id, $member_id);
		$cancel_url=site_url(strtolower(get_class($this)) . "/cancel/$cancel_code");
		$profile_edit_url=$this->model_parando->make_profile_edit_url($member_id);
		$contact_text=$this->model_parando->get_member_contact_text($member_id);
		
		$this->multipage->set_data_array(compact('contact_text','cancel_url','profile_edit_url'));  // Better if these were not in the session
																						// Maybe pages need body_init() method
																						// In this case, the register_done page
																						// needs these variables


    // EVENT INFO
		if($event_type=='fleche'){
			if(empty($fleche_team_data)){$this->die_error(__METHOD__,"No fleche team data in session.");}
			$event_distance=strtr($event_distance,['&egrave;'=>'e']);
			$event_info="Event: $event_name $event_distance\n";
			if(!empty($new_fleche_team_created)){
				$event_info.="!!!!NEW FLECHE TEAM CREATED!!!!\n";
			}
			$event_info.=$this->fleche_team_text();
		}else{
			$event_info=<<<EOT
Event: $event_name $event_distance
Start: $event_start_city, $event_start_state
Date:  $event_date
EOT;
		}

    // RIDER INFO
		$rider_info=<<<EOT
Rider Name:  $first_name $last_name
RUSA Member: $rms
Club Status: $membership
Club Expires: $expires

Rider Mailing Address:
  {$contact_street}
  {$contact_city} {$contact_state} {$contact_zip}
  {$contact_country}

Rider Email: {$contact_email}
Cell Phone: {$contact_phone}
Alt Phone:  {$contact_phone_alt}

Emergency Contact: 
  {$contact_emergency_name}
  {$contact_emergency_phone}
  {$contact_emergency_phone_alt}
EOT;


    $smartwaiver_info=<<<EOT
Smartwaiver: 
  Waiver ID: {$smartwaiver_id}
  Waiver URL: {$waiver_url}
EOT;

    // PAYMENT INFO
		if($total_fee>0){
			$purchase=$this->accounting_parando->get_purchase($invoice_number);
			$purchase_items_text=$this->accounting_parando->purchase_item_text($purchase);
			$order_info=<<<EOT
A total of \${$total_fee} is due for this event. 

Invoice Number: $invoice_number

Itemized charges are as follows: 

{$purchase_items_text}

EOT;
		}else{
			$payment_method='free_event';
			$purchase_items_text="";
			$order_info="";
		}

		switch($payment_method){
			case 'free_event':
				$payment_info=<<<EOT
There was no fee charged for this event, so no payment is due.

EOT;
				break;
			case 'pay_event':
				$payment_info=<<<EOT
Payment of \${$total_fee} in cash or check at the start of the 
event is expected. This total includes \${$possible_late_fee} a late fee. 

EOT;
				break;
			case 'pay_mail':
				$organizer_text_address=strtr($organizer_address,["<br>"=>"\n  "]);
				$payment_info=<<<EOT
Payment of \${$total_fee} in check or money order by mail is expected. Payment
should be mailed to 

  {$organizer_name}
  {$this_organization}
  {$organizer_text_address}

Please send payment early, so it arrives before the registration 
cutoff ({$event_reg_cutoff}).  

EOT;
				break;
			case 'pay_online':
				$payment_info=<<<EOT
A payment of \$$amount_paid has been entered through PayPal. Payment credit
is contingent on receipt of funds through the payment service. 
A description of the order is as follows:

Payer Name:	  		{$payer}
Amount Paid:      \${$amount_paid}
Order ID:         $invoice_number
Transaction ID:   {$transaction_id}
Entered On:       {$entered_on}

EOT;
				break;
			default:
				$payment_info=<<<EOT
Unknown Payment

EOT;
				break;
		}

		// CANCELLATION TEXT
		$cancel_message=<<<EOT
Should you be unable to start the event (result will be DNS), 
you can use the following link to notify us:

  $cancel_url

No refunds are issued for pre-paid registration fees.

EOT;

		$contact_organizer=<<<EOT
If you have any questions, contact event organizer $organizer_name
at $organizer_email by simply replying to this email. 

EOT;

		$contact_rider=<<<EOT
You can contact the rider $first_name $last_name at $contact_email
by simply replying to this email. 

EOT;

    // MESSAGE TEXT

		$rider_message=<<<EOT
This email confirms your registration in an 
upcoming PA Randonneurs event.

{$event_info}

======================================================
======================================================

PLEASE VERIFY THIS PERSONAL INFORMATION ON RECORD
PLEASE VERIFY YOUR PERSONAL INFORMATION ON RECORD!

* * * * * * * *

{$rider_info}

* * * * * * * *

If any this information is not correct, please click
the following URL to log in to the PA Rando website
and correct your information.

$profile_edit_url

That URL will only work for a limited time, so please
use it to fix your records immediately. 

You can also log in to your $this_organization Member Page $login_url
using your password, or by contacting event organizer $organizer_name
at $organizer_email by simply replying to this email.

======================================================
======================================================
EOT;


		$organizer_message=<<<EOT
The following event registration has been received:

{$event_info}

{$rider_info}

{$smartwaiver_info}

EOT;


		$footer_line=<<<EOT
===================================================

EOT;

    // EMAIL 

		$config['wordwrap'] = false;
		$this->email->initialize($config);


    // CONSTRUCT AND SEND RIDER MAIL

		$this->email->clear();
		if($this->all_email_to_webmaster){
		  // $this->email->to([$contact_email]);
			$this->email->to([$webmaster_email]);
		}else{
			$this->email->to([$contact_email]);
		}
		$this->email->from($organizer_email, $organizer_name, $this->model_parando->postmaster_email);
		$this->email->subject("$event_name $event_distance Registration Confirmation");
		$this->email->message(	implode("\n",
													[
														$rider_message,
														$order_info,
														$payment_info,
														$footer_line,
														$cancel_message,
														$contact_organizer
													]));
    $rc_1=$this->email->send(FALSE);
		$debug_1=$this->email->print_debugger();
    //sleep(2);

    // CONSTRUCT AND SEND ORGANIZER MAIL
    
    // TEMPORARY HACK TO SEND MAIL TO DAVE
    // $rusa_email='insurance@rusa.org';

		$this->email->clear();
		
// 		if($this->all_email_to_webmaster){
// 			$this->email->to([$webmaster_email]);
// 		}else{
// 			$this->email->bcc([$rba_email]);
// 		}

        $this->email->to([$organizer_email]);
        $this->email->bcc([$rba_email]);

		
		$this->email->from($contact_email, "$first_name $last_name", $this->model_parando->postmaster_email);
		$this->email->subject("[$this_controller] $event_name $event_distance");
		$this->email->message(	implode("\n",
													[
														$organizer_message,
														$order_info,
														$payment_info,
														$footer_line,
														$contact_rider
													]));
    $rc_2=$this->email->send(FALSE);
		$debug_2=$this->email->print_debugger();

    return ($rc_1 && $rc_2)?true:($debug_1 . "\n" . $debug_2);
//    return ($debug_1 . "\n" . $debug_2);

	}

	// EVENT CANCELLATION CONTROLLER METHOD

	public function cancel($cc="0-0-00000000"){

		$this->event_reg_session_init();
		$result = 'Invalid cancellation code';
		$e=0;
		$a = explode("-", $cc);
		if(count($a)==3){
			list($e, $m, $hash)=$a;
			if($cc == $this->make_cancel_code($e,$m)){
				$member=$this->model_parando->get_member($m);
				$event=$this->model_parando->get_event($e);
				if(isset($member) && isset($event)){
					$t=$this->model_parando->event_schedule($event);
					$event['distance_str']=$this->model_parando->make_distance($event);
					if($t['start_datetime']<$this->datetime_now){
					   	$result='Error! Too late to cancel registration.';
					}else{

						$result='OK';
						$schedule=$this->model_parando->event_schedule($event);
						$organizer_data=$this->model_parando->get_organizer($event);	
						$memdata=$this->model_parando->compute_memdata($member);
						
						$event_data=array(
									'event_id'		=> $e,
									'event_contact_url'	=> site_url("info/contact/$e"),
									'event_start_city'	=> $event['start_city'],
									'event_start_state'	=> $event['start_state'],
									'event_name'	=> $event['name'],
									'event_distance'	=> $this->model_parando->make_distance($event),
									'event_date'	=> $schedule['start_date_str'],
									'cancel_event' => $e,
									'cancel_member' => $m
							);

						$this->multipage->set_data_array(array_merge($memdata,$organizer_data,$event_data));
						$this->multipage->set_current_page('cancel_confirm');
						$this->multipage->view_current_page();
					}
				}
			}
		}

		if($result != 'OK'){
			$this->die_error('Registration Cancellation Error', $result);
		}

	}

	public function _cancel_confirm(){

		$e=$this->multipage->get_data('cancel_event');
		$m=$this->multipage->get_data('cancel_member');
		if(empty($m) || empty($e)){
			$this->die_error('_do_cancel','Missing parameters.');
		}

		$contact_url=site_url("info/contact/$e"); 
		$rbacontact=" If you have any questions, <A HREF=$contact_url>contact the event organizer</a>.";

		$this->db->where('member_id', $m);
		$this->db->where('event_id', $e);
		$this->db->update('roster', array('result'=>'DNS'));
		//$this->db->delete('roster', array('event_id' => $e, 'member_id'=>$m));
		if (!$this->db->affected_rows()) {
			$this->die_error('Registration Error','Registration not found. Maybe it was canceled already?');
		} 
	}

	private function make_cancel_code($e,$m){
		$em_id="$e-$m";
		$secret=$this->model_parando->secret;
		$hash=hash('crc32',$em_id.$secret);
		return "$em_id-$hash";
	}
	

 


}


?>
