<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

 
class Register extends MY_Controller {

  // CONSTRUCTOR

	public $datetime_now;
	public $all_email_to_webmaster;

	public function __construct() {
		parent::__construct();

		$this->load->model('accounting_parando');
		$this->load->helper('form');
		$this->load->library('email');
		$this->load->library('form_validation');
		$this->load->library('multipage');
		$this->load->library('registerpages');
		
		$this->multipage->define_pages($this->registerpages->page);  // SLOW -- Library parameter?
		$this->multipage->set_view('form_twocolumn','form_header','form_footer',['form']);
		$this->multipage->set_controller(strtolower(get_class($this)));
		$tz=$this->model_parando->get_local_time_zone();
		$this->datetime_now = new DateTime("now",$tz);
		
		// set this true for debugging
		$debug=false;
		$this->session->set_userdata('paypal_sandbox', $debug);
		$this->all_email_to_webmaster=$debug;
	}
	
	private function event_reg_session_init(){
		$date_stamp=$this->datetime_now->format('j F Y');
		$this->session->unset_userdata('PayPalResult');
		$this->session->unset_userdata('shopping_cart'); 
		
        $club=$this->model_parando->default_club;
		
		$d=[
            'club'=>$club,
			'this_organization' => $club['name'],
			'this_region' => $club['region'],
			'admin_login_url' => $this->model_parando->admin_login_url,
			'base_url' => base_url(),
			'date_stamp' => $date_stamp
		];
		$this->multipage->init($d);
	}

	// PAGE LIST DEFINITION
	
	public function _get_country_options(){
		$country_fullname = $this->model_parando->get_countries();
		return $country_fullname;
	}
	
	public function _get_state_options(){ 
		$state_fullname = $this->model_parando->get_states();
		return $state_fullname;
	}

	public function _get_fleche_teams_options(){
		$event_id=$this->multipage->get_data('event_id');
		$sql="SELECT fleche_team.id as team_id, fleche_team.name as team_name, first_name, last_name 
			FROM fleche_team,member where fleche_team.captain_id=member.id AND fleche_team.event_id={$event_id} order by fleche_team.name";
		$options=[];
		$q=$this->db->query($sql);
		foreach($q->result() as $row){
			$options[$row->team_id]=$row->team_name . ' (Capt: ' . $row->first_name . '&nbsp;' . $row->last_name . ')';
		}
		return $options;
	}
	
	public $payment_method_description=[
		'pay_online'=>'Pay now online with PayPal or Credit Card (lowest cost)',
		'pay_mail'=>'Pay by mail prior to the event',
		'pay_event'=>'Pay at the start of the event'
		];

	public function _get_payment_method_options(){
		$event_id=$this->multipage->get_data('event_id');
		$sql="SELECT payment_methods FROM event where event.id={$event_id}";
		$q=$this->db->query($sql);
		$r=$q->row();
		$payment_methods=explode(',',$r->payment_methods);
		//$this->die_error(__METHOD__, print_r($payment_methods,true));
		$options=[];
		foreach($payment_methods as $m){
			$options[$m]=$this->payment_method_description[$m];
		}
		return $options;
	}


	
	// GENERIC PAGE PROCESSING

	public function index() {
		$this->multipage->process_page();
	}


// EVENT REGISTRATION CONTROLLER METHOD

//	public function event($event_id=-1,$dist=200,$please='no'){


	public function event($event_id=-1,$coupon=null){

		$this->event_reg_session_init();

		if($event_id==-1){
			$event_id=$this->model_parando->next_event_id();
		}
		
		//elseif(preg_match('/\d{4}-\d{1,2}-\d{1,2}/',$event_id)){
		//	$event_id = $this->model_parando->get_event_id_by_date_dist($event_id,$dist);
		//}
		
		$event = $this->model_parando->get_event($event_id);
		if(empty($event)){
			$this->die_error('Event Error','Event (' . $event_id . ') not found.');
		}
		$schedule=$this->model_parando->event_schedule($event);
		if($schedule['start_datetime']<$this->datetime_now){
			$this->die_error('Registration Closed',
			  'It is too late to register for this event. Registration closed '. $schedule['reg_cutoff_datetime_str2'].'.');
		}

		//if(preg_match('/\d{4}-\d{1,2}-\d{1,2}/',$event_id)){
		//	$event_id = $this->model_parando->get_event_id_by_date_dist($event_id,$dist);
		//}
		
		$organizer_data=$this->model_parando->get_organizer($event);	
		$event_data=$this->make_event_data($event,$schedule);
		$this->multipage->set_data_array(array_merge($organizer_data,$event_data));

		if($coupon!==null){
					$this->load->model('coupon');
					$coupon_data=$this->coupon->get($coupon);
					if($coupon_data!==false && $this->coupon->is_valid_for_event($coupon_data,$event_id))
							$this->multipage->set_data_array(compact('coupon','coupon_data'));
					else 
						if($this->coupon->valid_syntax($coupon)) $this->die_error(__METHOD__,"Invalid Coupon Code: $coupon");
		}
		
		$this->multipage->set_current_page('find_member');
		$this->multipage->view_current_page();
	}

	private function make_event_data($event,$schedule){

		$event_id=$event['event_id'];

		$lodging_offered=false; // !empty($event['lodging_1n_price']) || !empty($event['lodging_2n_price']);
		$volunteers_needed=$this->model_parando->event_volunteers_needed($event_id);

		$event_data=array(
			    'event_id'		=> $event_id,
			    'event_contact_url'	=> site_url("info/contact/$event_id"),
			    'event_start_city'	=> $event['start_city'],
			    'event_start_state'	=> $event['start_state'],
			    'event_region' => $this->model_parando->region,
			    'event_name'	=> $event['name'],
			    'event_type'	=> $event['type'],
			    'event_distance'	=> $this->model_parando->make_distance($event),
			    'event_date'	=> $schedule['start_date_str'],
			    'event_DateTime' => $schedule['start_datetime'],
			    'event_start_datetime' => $schedule['start_datetime_str'],
			    'event_cutoff_datetime' => $schedule['cutoff_datetime'],
			    'event_time_zone' => $schedule['start_time_zone'],
			    'event_reg_cutoff'	=> $schedule['reg_cutoff_datetime_str2'],
			    'event_included'	=> $event['included'],
			    'event_member_price'	=> $event['member_price'],
			    'event_waive_guest_fee'	=> $event['waive_guest_fee'],
//			    'event_payment_methods'	=> $event['payment_methods'],
				'event_volunteers_needed' => $volunteers_needed,
				'event_lodging_offered' => $lodging_offered  
//,
//			    'event_lodging_1n_price'	=> $event['lodging_1n_price'],
//			    'event_lodging_2n_price'	=> $event['lodging_2n_price']
			);
		return $event_data;
	}
	
//  PAYMENT METHODS

// 	public function _choose_payment_all(){$this->_choose_payment();}
// 	public function _choose_payment_prepay_only(){$this->_choose_payment();}

	public function _choose_payment_method(){
		
		extract($this->multipage->get_data_array(['member_id','items','event_id','invoice_number','spend_coupon']));
		$payment_method=$this->input->post('payment_method');
		$event_id=(isset($event_id))?$event_id:null;
		
		if($payment_method == 'pay_event'){	// apply late fee
			$items[]=$this->accounting_parando->get_product_item('late_fee');
		}

		// Finally there can be no further charges, commit the purchase order
		// Since we can't fulfill yet (need payment confirmation) we assume any pre-existing
		// PO was generated by a passive payment cancellation, so we delete it silently. 
		
		$deleted=$this->accounting_parando->delete_purchase_by_event($event_id,$member_id); // don't want to accidentally make two POs!

		$description = "Event Registration";
		$purchase = $this->accounting_parando->create_purchase_order($member_id, $items, get_class($this) . ':' . $payment_method, $event_id, $description);
		if(is_string($purchase)){$this->die_error(__METHOD__, "Error creating purchase order: $purchase");}
		$invoice_number=$purchase['invoice_number'];
		$total_fee=$purchase['amount_due'];
		
		if(!empty($spend_coupon)){
			$this->load->model('coupon');
			$rc=$this->coupon->spend($spend_coupon,$invoice_number,$member_id);		
			if($rc!==true) $this->die_error(__METHOD__,"Couldn't spend coupon: $rc");
		}		
		
		$this->multipage->set_data_array(compact('payment_method','items','invoice_number','total_fee'));
		
		if($payment_method=='pay_online'){
			$this->_pay_online(); // skip immediate confirmation page, will confirm after payment capture
		}else{
			$this->multipage->set_current_page($payment_method);		
		}
	}
	
	
	public function _free_event(){
		if( $this->input->post('submit') == 'cancel' ){
					$this->clean_up_before_cancel();
					$this->multipage->set_current_page('register_cancel');
		}else{

			extract($this->multipage->get_data_array(['member_id','items','event_id','spend_coupon']));
			$payment_method='free_event'; // Even for free events we'll need a PO listing items to be fulfilled
			$event_id=(isset($event_id))?$event_id:null;

			// Finally there can be no further charges, commit the purchase order
			// and since there's no charge, we can fulfill_all now as well. In this 
			// case an existing PO is assumed to be evidence of prior fulfillment.
				
			$existing_po=$this->accounting_parando->get_purchase_by_event($event_id,$member_id);
			if(empty($existing_po)){ // don't want to accidentally make two POs or fulfill twice! 
				$description = "Event Registration (free)";
				$purchase = $this->accounting_parando->create_purchase_order($member_id, $items, get_class($this) . ':' . $payment_method, $event_id, $description);
				if(is_string($purchase)){$this->die_error(__METHOD__, "Error creating purchase order: $purchase");}
				$invoice_number=$purchase['invoice_number'];
				$total_fee=$purchase['amount_due'];

		if(!empty($spend_coupon)){
			$this->load->model('coupon');
			$rc=$this->coupon->spend($spend_coupon,$invoice_number,$member_id);		
			if($rc!==true) $this->die_error(__METHOD__,"Couldn't spend coupon: $rc");
		}		


				$this->multipage->set_data_array(compact('payment_method','invoice_number','total_fee')); 
				$rc=$this->_fulfill_all($purchase);
				if($rc!='OK'){
					$deleted=$this->accounting_parando->delete_purchase($invoice_number); // don't want to accidentally make two POs!
					$this->die_error(__METHOD__, "Error fulfilling purchase order $invoice_number: $rc");
				}
				$rc=$this->send_confirmation_email();		
				if($rc!==true){$this->die_error(__METHOD__, "Error sending confirmation emails:\n".$rc);}
				$this->multipage->set_current_page('register_done'); 
			}else{
				$this->die_error(__METHOD__, "Already fulfilled this purchase -- did you hit submit twice?");
			}

		}
	}
	public function _pay_mail(){$this->fulfill_advance();} // fulfill these now in good faith
	public function _pay_event(){$this->fulfill_advance();}
	
	// The session should be intact when fulfill_advance is called, but good practice would be to 
	// not assume this since late fulfillment calling fulfill_all() can't assume this. 
	
	private function fulfill_advance(){

		$invoice_number = $this->multipage->get_data('invoice_number');  // at least the session should have this? 

		if( $this->input->post('submit') == 'cancel' ){
					$this->clean_up_before_cancel();
					$this->multipage->set_current_page('register_cancel'); 
		}else{
			$purchase = $this->accounting_parando->get_purchase($invoice_number);
			if(empty($purchase)){$this->die_error(__METHOD__, "Error retrieving purchase order $invoice_number");}
			$rc=$this->_fulfill_all($purchase);
			if($rc!='OK'){$this->die_error(__METHOD__, "Error fulfilling purchase order $invoice_number: $rc");}
			$rc=$this->send_confirmation_email();		
			if($rc!==true){$this->die_error(__METHOD__, "Error sending confirmation emails:\n".$rc);}
			$this->multipage->set_current_page('register_done'); 
		}
	}

	public function _pay_online(){  // fulfillment deferred till after payment
	
			// PAYPAL Express Checkout Controller Setup

			// Clear cart from session userdata
			$this->session->unset_userdata('shopping_cart');

			extract($this->multipage->get_data_array(['contact_email','first_name','last_name','member_id','invoice_number']));
			$order_token=$this->accounting_parando->make_order_token($invoice_number);
			$purchase=$this->accounting_parando->get_purchase($invoice_number);
			
			if(empty($purchase)){$this->die_error(__METHOD__,"Purchase order not found: ($invoice_number)");}

			// Set cart data into session
			$cart=$this->accounting_parando->purchase_item_cart($purchase);
			$cart['member_first_name'] = $first_name;
			$cart['member_last_name'] = $last_name;
			$cart['contact_email'] = $contact_email;
			$cart['invoice_number'] = $invoice_number;
			$cart['order_token'] = $order_token;
			$cart['fulfillment_url'] = site_url(strtolower( get_class($this)) . "/credit_paypal_payment/$order_token");
			$cart['cancel_url'] = site_url(strtolower( get_class($this)) . "/paypal_cancel");

			$this->session->set_userdata('shopping_cart', $cart);

			redirect(site_url('express_checkout/SetExpressCheckout'));

	}	

	// Fulfillment Controller Method
	// .. Try not to assume session is intact here
	
	public function credit_paypal_payment($token){

		$rc = $this->accounting_parando->credit_paypal_payment($token,[$this,'_fulfill_all'], __METHOD__);  

		if($rc!='OK' && $rc!='DUPLICATE'){$this->die_error(__METHOD__,$rc);}
		
		$email_rc=true;
		if($rc!='DUPLICATE') $email_rc=$this->send_confirmation_email();		

		if($email_rc!==true){$this->die_error(__METHOD__, "Error sending confirmation emails:\n".$rc);}
		$this->multipage->set_current_page('register_done'); 
		$this->multipage->view_current_page();

	}

	public function _fulfill_all($purchase){
		$purchase_id=$purchase['id'];
		$invoice_number=$purchase['invoice_number'];
		$items=$this->accounting_parando->get_purchase_items($purchase_id);
		if(!empty($items)){
			foreach($items as $i){
				$sku =  $i['sku'];
				$fulfillment_method = 'fulfill_' . $sku;
				if(method_exists($this,$fulfillment_method)){
					$rc = $this->{$fulfillment_method}($purchase, $i);
					if('OK'!=$rc){return "Item SKU=$sku: $rc";}
				}
			}
		}else{
			return "No items in invoice number $invoice_number";
		}
		return "OK";
	}

	// Cancellation Controller method

	public function paypal_cancel(){
		$this->clean_up_before_cancel();		// assumes good session with ['member_id','invoice_number','fleche_team_data']
		$this->multipage->set_current_page('register_cancel'); 
		$this->multipage->view_current_page();
	}
	
  // Fulfillment functions. Ideally these should derive all information from the PO and Item objects

	private function fulfill_current_renew($purchase, $item){return $this->fulfill_join_renew($purchase, $item);}
	private function fulfill_expired_renew($purchase, $item){return $this->fulfill_join_renew($purchase, $item);}
	private function fulfill_new_member_join($purchase, $item){return $this->fulfill_join_renew($purchase, $item);}

	private function fulfill_join_renew($purchase, $item){
	
		$value = $item['value'];
		$member_id = $purchase['member_id'];
		$member=$this->model_parando->get_member($member_id);
		$expires=$member['expires'];
		$xdate = new DateTime($expires);
			
		if(!preg_match("/(\d{4})-(\d{2})-(\d{2})/", $value,$a)){return "Misformatted item value: '$value'";}
		list($membership_new_expires, $xyear, $xmonth, $xday)=$a;				
		$membership_new_xdate = new DateTime($membership_new_expires);

		if($expires!="0000-00-00" && $xdate >= $membership_new_xdate){
			return "Your membership term ends $expires and cannot be extended at this time.";
		}
								
		$this->db->where('id',$member_id);
		$rc=$this->db->update('member',['expires'=>$membership_new_expires]);
		if(FALSE==$rc){return "Failed to update membership term expiry date for member_id=$member_id to $new_expires";}
		
		return 'OK';

	}

	private function fulfill_registration_fee($purchase, $item){
		$value = $item['value'];
		$member_id = $purchase['member_id'];
		$total_fee = $purchase['amount_due'];
		$event_id = $purchase['event_id'];
		if(empty($event_id)){return "Missing or null event";}
		$invoice_number = $purchase['invoice_number'];
		$data = compact('event_id','member_id','invoice_number','total_fee');

		$cancel_code=$this->make_cancel_code($event_id, $member_id);
		$cancel_url=site_url(strtolower(get_class($this)) . "/cancel/$cancel_code");
		$this->multipage->set_data_array(compact('cancel_code','cancel_url'));

		$rc=$this->db->insert('roster', $data);
		if(FALSE==$rc){return "Failed to insert member_id=$member_id into roster for event_id=$event_id";}

		return 'OK';
	}


	private function fulfill_new_fleche_team($purchase, $item){
		$value = $item['value'];
		if(preg_match("/^ID=(\d+);/",$value,$m) && !empty($m[1])){
			$fleche_team_id=$m[1];
		}else{
			return "Malformed fleche_team_id Value Field: $value";
		}
		$event_id = $purchase['event_id'];
		if(empty($event_id)){return "Missing or null event_id";}
		$this->db->where('id', $fleche_team_id);
		$rc=$this->db->update('fleche_team', ['event_id'=>$event_id]);
    $n=$this->db->affected_rows();
		if(!$rc || $n!=1){return "Failed to link event_id=$event_id to fleche_team_id=$fleche_team_id; RC=$rc; NAR=$n;";}
		return 'OK';
	}

	private function fulfill_new_fleche_rider($purchase, $item){
		$value = $item['value'];
		if(preg_match("/^ID=(\d+);/",$value,$m) && !empty($m[1])){
			$fleche_team_id=$m[1];
		}else{
			return "Malformed fleche_team_id Value Field: $value";
		}
		$member_id = $purchase['member_id'];
		if(empty($member_id)){return "Missing or null member_id";}
		$rc=$this->db->insert('fleche_team_member',compact('fleche_team_id','member_id'));
		if(!$rc){return "Failed to insert member_id=$member_id into fleche_team_id=$fleche_team_id";}
		return 'OK'; 
	}

	private function fulfill_tandem_bike($purchase, $item){
		$value = $item['value'];
		if(preg_match("/Partner: (.+);/",$value,$m) && !empty($m[1])){
			$tandem_partner=$m[1];
		}else{
			return "Malformed tandem_partner Value Field: $value";
		}
		$member_id = $purchase['member_id'];
		if(empty($member_id)){return "Missing or null member_id";}
		$event_id = $purchase['event_id'];
		if(empty($event_id)){return "Missing or null event";}
		$this->db->where('member_id', $member_id);
		$this->db->where('event_id', $event_id);
		$rc=$this->db->update('roster',['tandem'=>$tandem_partner]);  // assumes roster already exists
		$n=$this->db->affected_rows();
		if(!$rc || $n!=1){return "Failed to mark member_id=$member_id as tandem rider in event_id=$event_id; RC=$rc; NAR=$n";}
		return 'OK'; 
	}

// FIND_MEMBER PAGE

	public function _find_member(){

		// Processing method for the find_member page
		
		// Decide what sort of match the first/last/rusa id makes with the database. 
		// Calls the method register_known($member) for a full match, otherwise it branches to two 
		// possible forms register_unknown for no match, or register_pick_member for multi match. 
	  
		$contact_url=$this->multipage->get_data('event_contact_url');
		$cto="<A HREF=$contact_url TITLE='Contact the Organizer'>contact the organizer</a>";
		$error_comment=" Registration can not proceed till this is fixed. If you have questions about this error, $cto.";

		$rusa_id=$this->input->post('rusa_id');
		$first_name=$this->input->post('first_name');
		$last_name=$this->input->post('last_name');
		


			if($this->model_parando->require_rusa_membership){		// Check for RUSA Membership through event date

			  $this->load->library('rusa');
				$event_cutoff_datetime=$this->multipage->get_data('event_cutoff_datetime');
				$this->rusa->check_rusa_membership_at_date($rusa_id,$last_name,$event_cutoff_datetime);
		
			}

		$found=$this->model_parando->find_member($first_name,$last_name,$rusa_id);	
		
		switch($found['result']){
		  case 'full_match_rusa':
			$q=$found['query'];
			$member=$q->row_array();
			$this->register_known($member);
			break;
		  case 'duplicate_entry':
			$this->die_error('Database Error','Your info matches two entries in our database.'.$error_comment);
			break;
		  case 'unknown':
			$this->load_unknown($first_name,$last_name,$rusa_id);
			$this->multipage->set_current_page('register_unknown');
			break;
		  default:
			$this->load_unknown($first_name,$last_name,$rusa_id);
			$this->multipage->set_current_page('register_pick_member'); 
			$q=$found['query'];
			$found_multiple = $q->result_array();
			$this->multipage->set_data('found_multiple',$found_multiple);
			break;
		}
	}

// REGISTER_PICK_MEMBER PAGE

	public function _register_pick_member(){

		// Processing method for the register_pick_member page 
		// will branch to register_unknown if no existing member of the same name/rusa
		// or branch to register_known($member) if there's now a full member match

 		$this->form_validation->set_rules('member_id','Member Selection', 'trim|required', array('required'=>'You must either select a member above that matches you, or select below that none are you.') );

		$contact_url=$this->multipage->get_data('event_contact_url');
		$cto="<A HREF=$contact_url TITLE='Contact the Organizer'>contact the organizer</a>";
		$error_comment=" Registration can not proceed till this is fixed. If you have questions about this error, $cto.";

	  $member_id=$this->input->post('member_id');
		$rusa_id= $this->multipage->get_data('rusa_id');
		$first_name=$this->multipage->get_data('first_name');
		$last_name= $this->multipage->get_data('last_name');
		$given_id= $this->multipage->get_data('given_id');
	  
	  if($this->form_validation->run()!=FALSE){
			if($member_id == "not_me"){
				$check=$this->model_parando->check_unknown($first_name,$last_name,$rusa_id);
				switch($check){
					case "existing_first_last":
						$this->die_error('Existing Member',
							"There is already a rider in our database with exactly the same first and last name as you have entered, yet you say this is not you. 
							I'm afraid you cannot register with a duplicate name. Use a different name or $cto.");   
						break;
					case "existing_rusa":
						$this->die_error('Existing Member',
							"There is already a rider in our database with exactly the same RUSA ID as you have entered, yet you say this is not you. 
							I'm afraid you cannot register with a duplicate RUSA ID. Use a different ID or $cto.");   
						break;
					case "ok":
						$this->multipage->set_current_page('register_unknown');   
						break;
				}
			}else{
				$mem=$this->model_parando->get_member($member_id);
				if(empty($mem)){
					$this->die_error('Member not found','No such member ID.');
				}else{

					if(empty($mem['rusa_id'])) $this->die_error('Previously No RUSA ID',
						"There is already a rider in our database with your name, 
						but that rider is listed in our database as not a RUSA member. Have you recently joined? ". $error_comment);

					if($rusa_id!=$mem['rusa_id']) $this->die_error('RUSA ID Change',
						"There is already a rider in our database with your name, 
						but that rider is listed in our database with a different RUSA ID than you. 
						Did you leave your RUSA number blank? Did you enter your RUSA ID incorrectly now, or in the past? ". $error_comment);

					$this->register_known($mem);

				}
			}
		}
	}

  private function load_unknown($first_name,$last_name,$rusa_id){
			$session_data=array(
				'rusa_id'	=> $rusa_id,
				'first_name'	=> $first_name,
				'last_name'	=> $last_name,
				'given_id' => ['first_name'=>$first_name, 'last_name'=>$last_name, 'rusa_id'=>$rusa_id]
			);
			$this->multipage->set_data_array($session_data);
	}

	
	private function register_known($member){

			// Begins finalizing the registration of a known member

			if(empty($member['id'])) $this->die_error(__METHOD__, "Empty member ID");
			$member_id=$member['id'];

			// We should know this much for sure now. Put it in session
			$session_data=array(
				'member_id' => $member_id,
				'rusa_id'	=> $member['rusa_id'],
				'first_name'	=> $member['first_name'],
				'last_name'	=> $member['last_name']
			);
			$this->multipage->set_data_array($session_data);

			// Check that we have an event to register for
			extract($this->multipage->get_data_array(['event_id','event_type','organizer_id','event_volunteers_needed','event_cutoff_datetime']));
			if(empty($event_id)){
				$this->die_error(__METHOD__,'Event not yet defined in session. Can\'t register.');
			}
	
			// Is this rider already registered?
			$reg_status=$this->model_parando->is_registered($event_id,$member_id);			
			if(!empty($reg_status)){
				$contact_url=$this->multipage->get_data('event_contact_url');
				$cto="<A HREF=$contact_url TITLE='Contact the Organizer'>contact the organizer</a>";
				if('' . $reg_status == 'DNS'){
					$emessage = <<<EOT
Our records indicate you have been withdrawn from this event ($reg_status). You can't re-register
after withdrawing. Please $cto if you have any questions.
EOT;
				}else{
					$emessage = <<<EOT
You are already registered for this event. Use the link in your 
confirmation email, or $cto, if you wish to cancel your registration.
EOT;
				}
				$this->die_error('Already Registered',$emessage);
			}

			// Is this the organizer? 
			$this_is_event_organizer=($member_id==$organizer_id);
			$this->multipage->set_data('this_is_event_organizer',$this_is_event_organizer);


			// Check to see if some member data is missing
			if($this->has_full_member_record($member)){
				$this->load_known_compute_fees($member);
				if($event_type=='fleche'){
					$this->multipage->call_page('register_fleche','release');
				}else{
					$this->multipage->set_current_page('release');
				}
			}else{
					$this->multipage->set_current_page('get_new_member_data');
			}
			
	}

	private function has_full_member_record($mem){
		$missing_field =
				empty($mem['m_street']) || 
				empty($mem['m_city']) || 
				empty($mem['m_state']) || 
				empty($mem['m_zip']) || 
				empty($mem['m_country']) || 
				empty($mem['m_phone']) || 
				empty($mem['m_email']) || 
				empty($mem['emergency_name']) || 
				empty($mem['emergency_phone']);
		return !$missing_field;
	}
	

	
	private function load_known_compute_fees($mem){

		// Saves data into the session for a matched member and computes fees based on membership
		// Can't add things like lodging, membership, or late fee yet. Dunno about fleche teams yet. 
		
		$memdata=$this->model_parando->compute_memdata($mem);
		extract($memdata);	

			if(empty($rusa_id) && $this->model_parando->require_rusa_membership){
				$this->die_error(__METHOD__,'Not a RUSA member. Can\'t register.');
			}
		
		$items=[];		// start the list of items

		extract($this->multipage->get_data_array(['event_name','event_distance','event_id','event_member_price','event_waive_guest_fee',
			'this_is_event_organizer','coupon','coupon_data']));
		$event_distance=strtr($event_distance,['&egrave;'=>'e']);
		$event_value="$event_name $event_distance (Event: $event_id)";
		$registration_fee_item=$this->accounting_parando->get_product_item('registration_fee');
		if(!empty($coupon_data)) $this->load->model('coupon');
		if(!empty($coupon_data) && $this->coupon->is_valid_for_event($coupon_data, $event_id)){
			$coupon_id=$coupon_data['id'];
			$registration_fee_item['unit_price']=0;
			$registration_fee_item['value']=" Valid Coupon ($coupon_id), fee waived";
			$this->multipage->set_data_array(['spend_coupon'=>$coupon_data['code'],'free_event_comment'=>"Valid Coupon ($coupon_id)."]);
			//$this->die_error(__METHOD__,"Has valid coupon:  $coupon_id");
		}else if($mem['administrator']==1 || $mem['waive_fees']==1 || $this_is_event_organizer){
			$registration_fee_item['unit_price']=0;
			$registration_fee_item['value']=$event_value . " Volunteer";
			$this->multipage->set_data('free_event_comment',"Event Volunteer.");
		}else{
			$registration_fee_item['unit_price']=$event_member_price;
			$registration_fee_item['value']=$event_value;
		}
		$items[]=$registration_fee_item;
		


// DO WE NEED ALL OF THESE HERE NOW?

		$new_member_join=$this->accounting_parando->get_product_price('new_member_join');
		$current_renew=$this->accounting_parando->get_product_price('current_renew');
		$expired_renew=$this->accounting_parando->get_product_price('expired_renew');
		$possible_late_fee=$this->accounting_parando->get_product_price('late_fee');
		if($event_waive_guest_fee){
			$guest_fee_normal=0;
			$guest_fee_special=0;
		}else{
			$guest_fee_normal=$this->accounting_parando->get_product_price('guest_fee');
			$guest_fee_special=$this->accounting_parando->get_product_price('guest_fee_special');
		}

		$nmonth = $this->datetime_now->format('m');
		$nyear = $this->datetime_now->format('Y');
		$october = $this->model_parando->membership_grace_month;
		$membership_grace_period=($nmonth >= $october);
		$new_xyear = ($membership_grace_period) ? $nyear+1 : $nyear;
		$membership_new_expires = $new_xyear . '-12-31';

		$session_data=compact(
			'membership_grace_period',
			'membership_new_expires',
			'new_member_join',
			'expired_renew',
			'current_renew',
			'guest_fee_normal',
			'guest_fee_special',
			'items',
			'possible_late_fee'
		);

		$this->multipage->set_data_array(array_merge($memdata,$session_data));

	}



	public function _pick_member_form_body(){

		// Callback that generates a radio button form for picking 
		// from multiple matched members

		$found=$this->multipage->get_data('found_multiple');

		$num_rows=sizeof($found);
		if($num_rows>1){
			$entries="one or more entries";
			$the_entry_that="the entry that";
			$not_me="None of those are me";
		}else{
			$entries="one entry";
			$the_entry_that="it if it";
			$not_me="That is not me.";
		}

		$body = <<<EOT
<p>Your info partially matches $entries in our rider database. Please select $the_entry_that exactly matches you, or select <em>not me</em></p><br>
<div class=roster>
<table width=100%>

EOT;

		foreach ($found as $row)
		{
			$member_id=$row['id'];
			$rusa_id=(!empty($row['rusa_id']) && $row['rusa_id']>0)?("RUSA " . $row['rusa_id']):"";
			if($row['m_country']=='US'){
				$state = $row['m_state'];
			}else{
				$country_fullname=$this->model_parando->get_countries();
				$state = $country_fullname[$row['m_country']];
			}
	    $body .= "<tr>";
	    $body .= "<td>";
			$body .= "<div class=image-radio><input type='radio' name='member_id' id='member_id$member_id' value=$member_id>";
	    $body .= "<label for='member_id$member_id'>" . $row['first_name'] . '&nbsp;' . $row['last_name'] . "</label></div>";
	    $body .= "</td>";
	    $body .= "<td>" . $row['m_city'] . "</td>";
	    $body .= "<td>" . $state . "</td>";
	    $body .= "<td>" . $rusa_id . "</td>";
	    $body .= "</tr>";
		}

		$body .= "</table></div>";

	if(!empty(form_error('member_id'))){
	  $body .= "<div class=verror>" . form_error('member_id') . "</div>";
	}

	$body .= "<div class=image-radio><input type='radio' name='member_id' id='member_id_unknown' value='not_me'><label for='member_id_unknown'>$not_me</label></div>";


		return $body;
	}


// FLECHE PAGES

	public function _register_fleche_form_body(){

		$teams = $this->_get_fleche_teams_options();

		if(!empty($teams)){
			$join_team_radio_button=<<<EOT
			<input type='radio' name='fleche_reg_type' id='fleche_reg_type_join_fleche_team' value='join_fleche_team'>
			<label for='fleche_reg_type_join_fleche_team'>Join Existing Team</label>
EOT;
		}else{
			$join_team_radio_button=<<<EOT
<div class=note>There are no existing teams to join at this time.</div>
EOT;
		}

		$event_id=$this->multipage->get_data('event_id');
		$event = $this->model_parando->get_event($event_id);
		if(empty($event)){
			$this->die_error('Event Error','Event (' . $event_id . ') not found.');
		}
		$schedule=$this->model_parando->event_schedule($event);
		$fleche_captain_reg_cutoff_datetime=$schedule['fleche_captain_reg_cutoff_datetime'];
		if($this->datetime_now < $fleche_captain_reg_cutoff_datetime){
			$create_team_radio_button=<<<EOT
			<input type='radio' name='fleche_reg_type' id='fleche_reg_type_create_fleche_team' value='create_fleche_team'>
			<label for='fleche_reg_type_create_fleche_team'>Create a New Team</label></div><div class='image-radio'>
EOT;
		}else{
			$cutoff=$fleche_captain_reg_cutoff_datetime->format('G:i T, l j F Y');
			$create_team_radio_button=<<<EOT
<div class=note>Teams may not be created at this time. Team registration closed $cutoff.</div>
EOT;
		}

		$error=(!empty(form_error('fleche_reg_type')))?("<div class=verror>" . form_error('fleche_reg_type') . "</div>"):"";
	
		$body =<<<EOT
<div class=form-group>
	<label for="fleche_reg_type">Registration Type*</label>    
	<div class='indented'>$error
		<div class='image-radio'>
			$create_team_radio_button
			$join_team_radio_button
		</div>
	</div>
</div>
EOT;

		return $body;
	}

	public function _register_fleche(){
 		$this->form_validation->set_rules('fleche_reg_type','Fleche Team Option', 'trim|required', 
				array('required'=>'You must choose to create a new team, or join an existing one.') );
				$fleche_reg_type=$this->input->post('fleche_reg_type');
			if ($this->form_validation->run()){
				switch($fleche_reg_type){
					case 'create_fleche_team':
					case 'join_fleche_team':
						$this->multipage->set_current_page($fleche_reg_type);
						break;
				}
			}
	}

	public function _fleche_team_name_check($proposed_name){
		$this->load->model('fleche_parando');
		$event_id=$this->multipage->get_data('event_id');
		$names=$this->fleche_parando->get_fleche_team_names($event_id);
		$test_name=preg_replace('/\s+/', '', strtolower($proposed_name));
		$best_match=null;
		$match_dist=null;
		foreach($names as $name){
			$existing_name=preg_replace('/\s+/', '', strtolower($name));
			$ld=levenshtein($existing_name,$test_name);
			if(empty($best_match) || $ld<$match_dist){
				$best_match=$name;
				$match_dist=$ld;
			}
		}
		if($match_dist!==null && $match_dist<5){
			$this->form_validation->set_message('_fleche_team_name_check',"There is already a team, '$best_match', with a name similar to your proposed name, '$proposed_name'. Team
			names must be <em>very</em> unique. Sorry. If you really meant '$best_match' that team already exists. You can't create it again.");
			return false;
		}else{
			return true;
		}
	}

	public function _fleche_team_schedule_check(){

		$this->load->model('fleche_parando');
		$event_id=$this->multipage->get_data('event_id');

		$start_day=(integer)$this->input->post('fleche_team_start_day');
		$start_hour=(integer)$this->input->post('fleche_team_start_hour');
		$start_min=(integer)$this->input->post('fleche_team_start_minute') * 15;

		extract($this->fleche_parando->calc_fleche_team_schedule($event_id, [$start_day, $start_hour, $start_min]));

		if($start_datetime<$earliest_start_datetime){
			$this->form_validation->set_message('_fleche_team_schedule_check', 
				"Start time too early ($start_dow_time_str). Allowable start times are from noon Thursday through 10 AM on Saturday.");
			return false;
		}elseif($start_datetime>$latest_start_datetime){
			$this->form_validation->set_message('_fleche_team_schedule_check', 
				"Start time too late ($start_dow_time_str). Allowable start times are from noon Thursday through 10 AM on Saturday.");
			return false;
		}else{
			return true;
		}

	}

	public function _join_fleche_team(){
		$this->load->model('fleche_parando');
		extract($this->multipage->get_data_array(['event_id','member_id','items']));
		$fleche_team_id=$this->input->post('fleche_team_id');
		if(empty($fleche_team_id)){$this->die_error(__METHOD__, "Empty or missing team ID");}
		$fleche_team_data=$this->fleche_parando->get_fleche_team_by_id($fleche_team_id);
		if(empty($fleche_team_data)){$this->die_error(__METHOD__, "Could not get data for fleche team ID=$fleche_team_id");}
		$fleche_team_schedule=$this->fleche_parando->calc_fleche_team_schedule($event_id, $fleche_team_data['start_date'] . ' ' . $fleche_team_data['start_time']); 
		if(empty($fleche_team_schedule)){$this->die_error(__METHOD__, "Could not calculate schedule for fleche team ID=$fleche_team_id");}
		$team_value="ID=$fleche_team_id; " . $fleche_team_data['name'];
		$new_fleche_rider_item=$this->accounting_parando->get_product_item('new_fleche_rider');
		$new_fleche_rider_item['value']=$team_value;
		$items[]=$new_fleche_rider_item;
		$this->multipage->set_data_array(compact('fleche_team_data','fleche_team_schedule','items'));
	}


	public function _create_fleche_team(){

		$this->load->model('fleche_parando');
		extract($this->multipage->get_data_array(['event_id','member_id']));

		$start_day=(integer)$this->input->post('fleche_team_start_day');
		$start_hour=(integer)$this->input->post('fleche_team_start_hour');
		$start_min=(integer)$this->input->post('fleche_team_start_minute') * 15;

		$schedule=$this->fleche_parando->calc_fleche_team_schedule($event_id, [$start_day, $start_hour, $start_min]); // could've cached this from the schedule check, but heck -- fast computer

		$data=[
			'event_id'=>$event_id,
			'name'=>strip_tags($this->input->post('fleche_team_name')),
			'captain_id'=>$member_id,
			'start_date'=>$schedule['start_date_str'],
			'start_time'=>$schedule['start_time_str'],
			'start_city'=>strip_tags($this->input->post('fleche_team_start_city')),
			'start_state'=>$this->input->post('fleche_team_start_state'),
			'map'=>$this->input->post('fleche_team_map'),
			'description'=>strip_tags($this->input->post('fleche_team_description')),
			'approved'=>false
		];

		$this->multipage->set_data_array(['fleche_team_data'=>$data, 'fleche_team_schedule'=>$schedule]);

	}

	public function _create_fleche_team_confirm_body(){
		$body="";
		extract($this->multipage->get_data('fleche_team_data'));
		extract($this->multipage->get_data('fleche_team_schedule'));
		$captain=$this->model_parando->members_by_id[$captain_id];
		$captain_name=$captain['first_name'].'&nbsp;'.$captain['last_name'];
		$body=<<<EOT
<h2>New Team</h2>
<div class=fleche-team>
<TABLE ID=team-summary Width=100%>
<TR><TD>Team Name</TD><TD>$name</TD></TR>
<TR><TD>Team Start</TD><TD>$start_datetime_str</TD></TR>
<TR><TD>Start Location</TD><TD>$start_city, $start_state</TD></TR>
<TR><TD>Team Captain</TD><TD>$captain_name</TD></TR>
<TR><TD>Map URL</TD><TD><A HREF='$map'>$map</A></TD></TR>
<TR><TD>Description</TD><TD>$description</TD></TR>
</TABLE>
</DIV>
EOT;
		return $body;
	}

	public function _create_fleche_team_confirm(){
		extract($this->multipage->get_data_array(['fleche_team_data','items']));
		if( $this->input->post('submit') == 'yes' ){
			if(empty($fleche_team_data)){	
				$this->die_error(__METHOD__, "Missing fleche team data.");
			}else{
				$ftd=$fleche_team_data;
				$ftd['event_id']=-1; // orphaned till fulfillment
				$rc=$this->db->insert('fleche_team', $ftd); 
				if(!$rc){$this->die_error(__METHOD__,"Failed to insert fleche team data into database.");}
				$fleche_team_id=$this->db->insert_id();
				$fleche_team_data['id']=$fleche_team_id;

				$team_value="ID=$fleche_team_id; " . $fleche_team_data['name'];

				$new_fleche_team_item=$this->accounting_parando->get_product_item('new_fleche_team');
				$new_fleche_team_item['value']=$team_value;
				$items[]=$new_fleche_team_item;

				$new_fleche_rider_item=$this->accounting_parando->get_product_item('new_fleche_rider');
				$new_fleche_rider_item['value']=$team_value; 
				$items[]=$new_fleche_rider_item;

				$this->multipage->set_data_array(compact('items','fleche_team_data'));
			}
		}elseif( $this->input->post('submit') == 'no' ){
			$this->multipage->set_current_page('register_cancel'); // should be nothing to clean up
		}

	}

	public function _bike_style(){
		$bike_style=$this->input->post('bike_style');
		if($bike_style=='tandem'){
			$this->multipage->set_current_page('tandem_partner');		
		}else{
			$this->multipage->return_page();		
		}
	}

	public function _tandem_partner(){
		$tandem_partner=$this->input->post('tandem_partner',true);
		extract($this->multipage->get_data_array(['items']));
		$new_item=$this->accounting_parando->get_product_item('tandem_bike');
		$new_item['value']="Partner: $tandem_partner;";
		$items[]=$new_item;
		$this->multipage->set_data_array(compact('items'));
		$this->multipage->return_page();		
	}
	
	private function clean_up_before_cancel(){
		$this->load->model('fleche_parando');
			extract($this->multipage->get_data_array(['member_id','invoice_number','fleche_team_data']));
			if(!empty($invoice_number)){
				$deleted=$this->accounting_parando->delete_purchase($invoice_number);
			}
			if(!empty($fleche_team_data) && !empty($member_id) && $member_id==$fleche_team_data['captain_id']){
				$deleted=$this->fleche_parando->delete_fleche_team_by_data($fleche_team_data); 
			}
	}
	


// VOLUNTEERING PAGE -- DEPRECIATED

	public function _volunteer_form_body(){

		// Callback that generates a radio button form for picking 
		// from multiple volunteer jobs

		$evn = $this->multipage->get_data('event_volunteers_needed');

		if(empty($evn)){
			$this->die_error("This can't happen.", "Don't call this function before setting the volunteer list in the session.");
		}

		$body = <<<EOT
<h3>Volunteers Needed for This Event</h3>
<p>Consider volunteering for this event. Volunteers support your sport of randonneuring. If
you volunteer, your entry fee and other expenses are reimbursed and you are eligable for a 
pre-ride up to two weeks prior to the event date.</p>
<p>This event requires volunteers for the following duties. 
If you believe you can perform one of these duties, 
select an available job below. </p><br>
<div class=roster>
<table width=100%>
EOT;

		foreach ($evn as $row)
		{
			$description=$row['description'];
			$text=$row['text'];
			$job_id=$row['job_id'];
	    $body .= "<tr><td><div class=image-radio>";
	    $body .= "<input type='radio' name='job_id' id='job_id$job_id' value=$job_id>";
	    $body .= "<label for='job_id$job_id'>" . $row['description'] . "</label>";
	    $body .= "</div></td></tr>";
		}

		$body .= "</table></div>";
		$body .= "<div class=image-radio><input type='radio' checked id='job_id_none' name='job_id' value='none'><label for='job_id_none'>Just Ride</label></div>";



		return $body;
	}

  // Not used

	public function _register_volunteer(){

		// Processing method for the register_volunteer page 
		// will branch to release if no job selected


		extract($this->multipage->get_data_array(['event_id','member_id','event_volunteers_needed']));
	  $job_id=$this->input->post('job_id');

		if($job_id == "none" || empty($event_volunteers_needed)){
						$this->multipage->set_data_array(['vjd'=>'No', 'vjd_text'=>'No job', 'job_id'=>$job_id]);
						$this->multipage->return_page();   
		}else{			
						foreach ($event_volunteers_needed as $row)
						{
							$description=$row['description'];
							$text=$row['text'];
							if($job_id == $row['job_id']){$match=true; break;}
						}
						if(isset($match)){
							$rc=$this->model_parando->add_volunteer_offer($event_id,$member_id,$job_id);
							if(!$rc){$this->die_error(__METHOD__,"Could not add volunteer offer");}
							$this->multipage->set_data_array(['vjd'=>$description, 'vjd_text'=>$text, 'job_id'=>$job_id]);
							$this->multipage->set_current_page('thank_offer');   
						}
		}
	}

	
// GET_NEW_MEMBER_DATA PAGE


	public function _get_new_member_data(){
	
	// Processing method for get_new_member_data page
	// Stores aquired data into database and proceeds with known member
	
		// Stuff already in session
		$sdata = $this->multipage->get_data_array(['rusa_id','first_name','last_name']);

		if(empty($sdata['first_name']) || empty($sdata['last_name'])) $this->die_error(__METHOD__, "No member name in session!");
		
		$pw = $this->input->post('password');
		$pw_hash = (empty($pw))?"":password_hash($pw,PASSWORD_DEFAULT);
		
		// Stuff added by contact form
		$fdata = array(
			'm_street'		=> $this->input->post('contact_street'),
			'm_city'		=> $this->input->post('contact_city'),
			'm_state'		=> $this->input->post('contact_state'),
			'm_zip'			=> $this->input->post('contact_zip'),
			'm_country'			=> $this->input->post('contact_country'),
			'tracker'			=> $this->input->post('contact_tracker'),
			'm_phone'		=> $this->input->post('contact_phone'),
			'm_phone_alt'		=> $this->input->post('contact_phone_alt'),
			'm_email'		=> $this->input->post('contact_email'),
			'emergency_name'	=> $this->input->post('contact_emergency_name'),
			'emergency_phone'	=> $this->input->post('contact_emergency_phone'),
			'emergency_phone_alt'	=> $this->input->post('contact_emergency_phone_alt'),
			'password' => $pw_hash

		);

		$mdata=array_merge($sdata,$fdata);

		// Totally new member?
		$member = $this->model_parando->get_member_firstlast($sdata['first_name'],$sdata['last_name']);
		if(empty($member)){ // No existing record, do an insert creating a new member ID
			$rc=$this->db->insert('member', $mdata);	// Add new member to database
			if($rc!==true) $this->die_error(__METHOD__, "Could not insert member record: " . print_r($mdata,true));
		}else{ // Existing member updating information
			if(empty($member['id'])) $this->die_error(__METHOD__, "Empty member ID for member query: " . $sdata['first_name'] . ' ' . $sdata['last_name']);
			$member_id=$member['id'];
			$mdata['id']=$member_id;
			$rc=$this->db->replace('member',$mdata);
			if($rc!==true) $this->die_error(__METHOD__, "Could not replace member record: " . print_r($mdata,true));
		}


		$member = $this->model_parando->get_member_firstlast($sdata['first_name'],$sdata['last_name']);
		if(empty($member)){$this->die_error('get_new_member_data',"This can't happen -- failed member readback.");}

		$this->register_known($member);

	}


	public function _phone_check() {

		// Validation callback to verify that at least one phone number was entered

		if($this->input->post('contact_phone') || $this->input->post('contact_phone_alt')){
				return TRUE;
		}else{
				$this->form_validation->set_message('_phone_check', 'Please enter atleast one phone number.');
		return FALSE;
		}
	}
	
// AGREE TO RELEASE



	private function get_release_text($newline="\n",$sample=false){
			$release_text = "";
			$release_text.= $this->linewrap($this->multipage->interpolate_strings($this->model_parando->get_paragraph('release_header')), $newline);
			$release_text .= $newline . $newline;
			$release_text.= $this->linewrap($this->multipage->interpolate_strings($this->model_parando->get_paragraph('release_title')), $newline);
			$release_text .= $newline . $newline;
			$release_text .= $this->linewrap($this->multipage->interpolate_strings($this->model_parando->get_paragraph('release_preamble')), $newline);
			$release_text .= $newline . $newline;

			$clauses=$this->model_parando->get_paragraph_array('release',true);
		    $needed=count($clauses);
		    $found=0;
			for($i=1; $i<=$needed; $i++){
				$ai = 'agree' . $i;
				if( $sample || $this->input->post($ai) == 'yes' ){ 
					$found++; 
					$c="AGREED";
				}else{
					$c="DECLINED";
				}
				$release_text .= $this->linewrap("[$c] " . $this->multipage->interpolate_strings($clauses[$ai]), $newline);
				$release_text .= $newline . $newline;
			}
			$release_text.= $this->linewrap($this->multipage->interpolate_strings($this->model_parando->get_paragraph('release_footer')), $newline);
			return ['release_text'=>$release_text, 'needed'=>$needed, 'found'=>$found];
	}

 // strip HTML and smooth wrap lines to width 

	private function linewrap($string, $break="\n", $width=60, $cut=false) {
		$content = preg_replace( "/\s+/s", " ", $string );
		$content = preg_replace( "!<[ a-zA-Z/]+>!s", "", $content );
		$content = wordwrap($content, $width, $break, $cut);
		return $content;
	}


	public function _release(){

		if( $this->input->post('submit') == 'no' ){
				$this->clean_up_before_cancel(); 
				$this->multipage->set_current_page('register_cancel');  
		}else{
		
			// After the release is e-signed we consider/offer membership renewal -- IS THIS THE RIGHT PLACE? 
			// Maybe it would be better to have the release be the last thing before payment. 

			$newline="\n";
			$agree_timestamp=$this->datetime_now->format('j F Y H:i:s T');
			extract($this->multipage->get_data_array(['first_name','last_name','membership','expires','membership_new_expires','items']));
		
			extract($this->get_release_text($newline));
			$release_text .= $newline . $newline;
			$release_text .= "---------------------" . $newline . $newline;

			$age=(!empty($this->input->post('age')))?$this->input->post('age'):"unknown";
			$release_text .= "E-SIGNED ON $agree_timestamp BY: $first_name $last_name, AGE $age";

			$this->multipage->set_data_array(['release_needed'=>$needed, 'release_found'=>$found, 'release_agreed_on'=>$agree_timestamp, 'release_text'=>$release_text]);

			if($membership=="current"){
				$new_xdate= new DateTime($membership_new_expires);
				$old_xdate= new DateTime($expires);
			}

			$this->form_validation->set_rules('age','Rider Age','trim|is_natural_no_zero|callback__age_check');
			if ($this->form_validation->run() && $needed == $found){
				if($membership=="guest"){
					$next_page='choose_join';
				}elseif($membership=="terminated"){
					$next_page='choose_rejoin';
				}elseif($membership=="current" && $new_xdate > $old_xdate){  
					$next_page='choose_renew';
				}else{

					// NO MEMBERSHIP RENEWAL -- TENTATIVE TOTAL FEE  -- CHOOSE PAYMENT

					$total_fee = $this->accounting_parando->items_total($items);
					$this->multipage->set_data('total_fee',$total_fee);
					if($total_fee > 0){
						$next_page='choose_payment_method';
					}else{
						$next_page='free_event';
					}
				}
				$this->multipage->set_current_page($next_page);
			}
		}
	}
	
	public function XXsample_release(){
	
		$newline="\n";
		$release_text = $this->get_release_text($newline,true);

		$this->email->from("null@parando.org", "Test Name");
		$wm=$this->model_parando->get_webmaster();
		$this->email->to($wm['webmaster_email']);
		$this->email->subject("Sample Release");
		$this->email->message($release_text);
		$this->email->send();
		
		$this->die_error(__METHOD__,"Done");

		
	}

 
	public function _release_form_body(){
	
		$form_body="<div class=release-form>";

		extract($this->multipage->get_data_array(['release_needed', 'release_found','date_stamp','first_name','last_name']));

		if(isset($release_needed,$release_found) && $release_needed!=$release_found){
			$checkbox_style= "style='background-color: red;'";
			$form_body .= '<p style="color:red; font-size:1.2em; text-align:center;">';
			$form_body .= "Only checked $release_found of $release_needed paragraphs. Please signify your agreement with all paragraphs by checking the box at the beginning of each.";
			$form_body .= "</p>";
		}else{
			$checkbox_style= "style='background-color: #C0C0C0;'";
		}

		$form_body.= '<div class=release-header>' . $this->multipage->interpolate_strings($this->model_parando->get_paragraph('release_header')) . '</div>';
		$form_body.= '<div class=release-title>' . $this->multipage->interpolate_strings($this->model_parando->get_paragraph('release_title')) . '</div>';
		$form_body.= '<p>' . $this->multipage->interpolate_strings($this->model_parando->get_paragraph('release_preamble')) . '</p>';

		$clauses=$this->model_parando->get_paragraph_array('release',true);
		
		foreach($clauses as $d=>$c){
			//$form_body.="<DL><DT><div class='checkboxThree'>";
			$form_body.="<p><span $checkbox_style><input type='checkbox' id='$d' name='$d' value='yes'></span>";
			$form_body.="<label for='$d'>";
			//$form_body.="</div></DT> ";
			$form_body.=$this->multipage->interpolate_strings($c) . '</label></p>';
		}


		$form_body.=<<<EOT
<p>&nbsp;
<div class=event>
<p>Current Date: $date_stamp</p>
<p>Participant Name: $first_name $last_name</p>
<br><em>To acknowledge your agreement with the release above, please read it carefully, 
check the box next to each and every paragraph, enter your age,
and then electronically sign the agreement by pressing the "I Agree" button below. If you <b>do not</b> agree with this release, press "Cancel". </em>
</div>
EOT;
		$form_body.= '<div class=release-footer>' . $this->multipage->interpolate_strings($this->model_parando->get_paragraph('release_footer')) . '</div>';
		$form_body.=$this->multipage->form_group('age','Rider Age','Enter your age',false);

		$form_body.="</div>";
		
		return $form_body;

	}
	
	public function _age_check() {

		// Validation callback to verify age

		if(($this->input->post('age')+0) > 18){
			return TRUE;
		}else{
			if(empty($this->input->post('age'))){
				$this->form_validation->set_message('_age_check', 'You must enter your current age in years.');
			}else{
				$this->form_validation->set_message('_age_check', 'You must be at least 18 years of age to register online.');
			}
			return FALSE;
		}
	}
	
// MEMBERSHIP RENEWAL

	public function _choose_join(){
		$this->choose_join_renew();
	}
	public function _choose_rejoin(){
		$this->choose_join_renew();
	}
	public function _choose_renew(){
		$this->choose_join_renew();
	}

	private function choose_join_renew(){
			
		extract($this->multipage->get_data_array(['items','membership','expires','membership_new_expires','event_member_price','event_waive_guest_fee','event_id','coupon_data']));
	
		$join_renew=$this->input->post('join_renew');

		$join_renew_value = "New expiration: $membership_new_expires";

		switch($join_renew){
			case 'join':
				$items[]=$this->accounting_parando->get_product_item('new_member_join', $join_renew_value);
				$guest=false;
				break;
			case 'rejoin':
				$items[]=$this->accounting_parando->get_product_item('expired_renew', $join_renew_value);
				$guest=false;
				break;
			case 'renew':
				$items[]=$this->accounting_parando->get_product_item('current_renew', $join_renew_value);
				$guest=false;
				break;
			default:
				$guest=($membership!="current");
				break;
		}
			
		if($guest){ 
			$guest_value=($membership=="terminated")?"Membership expired: $expires":"";
			if(!empty($coupon_data)) $this->load->model('coupon');
			if(!empty($coupon_data) && $this->coupon->is_valid_for_event($coupon_data, $event_id)){
				$coupon_id=$coupon_data['id'];
				$guest_fee_item=$this->accounting_parando->get_product_item('guest_fee',$guest_value);
				$guest_fee_item['unit_price']=0;
				$guest_fee_item['value']=" Valid Coupon ($coupon_id), fee waived";
				$this->multipage->set_data_array(['spend_coupon'=>$coupon_data['code'],'free_event_comment'=>"Valid Coupon ($coupon_id)."]);
			//$this->die_error(__METHOD__,"Has valid coupon:  $coupon_id");

			}else if($event_waive_guest_fee==0){
				if($event_member_price == 0){
					$items[]=$this->accounting_parando->get_product_item('guest_fee_special',$guest_value);
				}else{
					$items[]=$this->accounting_parando->get_product_item('guest_fee',$guest_value);
				}
			}
		}

		// MEMBERSHIP RENEWAL ADDED -- TENTATIVE TOTAL FEE (MAY STILL ADD LATE FEE)  -- CHOOSE PAYMENT

		$total_fee = $this->accounting_parando->items_total($items);
		if($total_fee > 0){
			$next_page='choose_payment_method';
		}else{
			$next_page='free_event';
		}

		$this->multipage->set_data_array(compact('items','join_renew','total_fee'));
		$this->multipage->set_current_page($next_page);
		
	}

	private function fleche_team_text(){
		extract($this->multipage->get_data('fleche_team_data'));
		extract($this->multipage->get_data('fleche_team_schedule'));
		$captain=$this->model_parando->members_by_id[$captain_id];
		$captain_name=$captain['first_name'].' '.$captain['last_name'];
		$body=<<<EOT

Team Name:  $name  
Team Captain:  $captain_name  
Team Start:  $start_datetime_str  
Start Location:  $start_city, $start_state  
Map URL:  $map   
EOT;
		return $body;
	}



// CONFIRMATION EMAIL

	private function send_confirmation_email(){
		
		extract($this->multipage->get_data_all()); // might consider how this could be done without the session variables

		$rba=$this->model_parando->get_rba();
		$rba_name=$rba['first_name'] . ' ' . $rba['last_name'];
		$rba_email=$rba['m_email'];

		$webmaster=$this->model_parando->get_webmaster(); extract($webmaster);

		$this_organization = $this->model_parando->default_club['name'];
		
    $login_url=site_url("login");
		$this_controller=strtoupper(get_class($this));

    // EVENT INFO
		if($event_type=='fleche'){
			if(empty($fleche_team_data)){$this->die_error(__METHOD__,"No fleche team data in session.");}
			$event_distance=strtr($event_distance,['&egrave;'=>'e']);
			$event_info="Event: $event_name $event_distance\n";
			if(!empty($new_fleche_team_created)){
				$event_info.="!!!!NEW FLECHE TEAM CREATED!!!!\n";
			}
			$event_info.=$this->fleche_team_text();
		}else{
			$event_info=<<<EOT
Event: $event_name $event_distance
Start: $event_start_city, $event_start_state
Date:  $event_date
EOT;
		}

    // RIDER INFO
		$rider_info=<<<EOT
Rider Name:  $first_name $last_name
RUSA Member: $rms
Club Status: $membership
Club Expires: $expires

Rider Mailing Address:
  {$contact_street}
  {$contact_city} {$contact_state} {$contact_zip}
  {$contact_country}

Rider Email: {$contact_email}
Cell Phone: {$contact_phone}
Alt Phone:  {$contact_phone_alt}

Emergency Contact: 
  {$contact_emergency_name}
  {$contact_emergency_phone}
  {$contact_emergency_phone_alt}
EOT;

    // PAYMENT INFO
		if($total_fee>0){
			$purchase=$this->accounting_parando->get_purchase($invoice_number);
			$purchase_items_text=$this->accounting_parando->purchase_item_text($purchase);
			$order_info=<<<EOT
A total of \${$total_fee} is due for this event. 

Invoice Number: $invoice_number

Itemized charges are as follows: 

{$purchase_items_text}

EOT;
		}else{
			$payment_method='free_event';
			$purchase_items_text="";
			$order_info="";
		}

		switch($payment_method){
			case 'free_event':
				$payment_info=<<<EOT
There was no fee charged for this event, so no payment is due.

EOT;
				break;
			case 'pay_event':
				$payment_info=<<<EOT
Payment of \${$total_fee} in cash or check at the start of the 
event is expected. This total includes \${$possible_late_fee} a late fee. 

EOT;
				break;
			case 'pay_mail':
				$organizer_text_address=strtr($organizer_address,["<br>"=>"\n  "]);
				$payment_info=<<<EOT
Payment of \${$total_fee} in check or money order by mail is expected. Payment
should be mailed to 

  {$organizer_name}
  {$this_organization}
  {$organizer_text_address}

Please send payment early, so it arrives before the registration 
cutoff ({$event_reg_cutoff}).  

EOT;
				break;
			case 'pay_online':
				$payment_info=<<<EOT
A payment of \$$amount_paid has been entered through PayPal. Payment credit
is contingent on receipt of funds through the payment service. 
A description of the order is as follows:

Payer Name:	  		{$payer}
Amount Paid:      \${$amount_paid}
Order ID:         $invoice_number
Transaction ID:   {$transaction_id}
Entered On:       {$entered_on}

EOT;
				break;
			default:
				$payment_info=<<<EOT
Unknown Payment

EOT;
				break;
		}

		// CANCELLATION TEXT
		$cancel_message=<<<EOT
Should you be unable to start the event (result will be DNS), 
you can use the following link to notify us:

  $cancel_url

No refunds are issued for pre-paid registration fees.

EOT;

		$contact_organizer=<<<EOT
If you have any questions, contact event organizer $organizer_name
at $organizer_email by simply replying to this email. 

EOT;

		$contact_rider=<<<EOT
You can contact the rider $first_name $last_name at $contact_email
by simply replying to this email. 

EOT;


		// RELEASE TEXT

		if(!empty($release_needed) && $release_needed==$release_found){
			$release_message = <<<EOT
Participant $first_name $last_name agreed to the following Accident Waiver by 
clicking AGREE at $release_found of $release_needed paragraphs:

[RELEASE TEXT BEGINS]

$release_text

[RELEASE TEXT ENDS]

EOT;
}else{
	$release_message = <<<EOT
PARTICIPANT $first_name $last_name DID NOT COMPLETE THE ACCIDENT WAIVER!!!!!!
There were $release_found of $release_needed paragraphs clicked.

[RELEASE TEXT BEGINS]

$release_text

[RELEASE TEXT ENDS]

EOT;
		}

    // MESSAGE TEXT

		$rider_message=<<<EOT
This email confirms your registration in an 
upcoming PA Randonneurs event.

{$event_info}

======================================================
======================================================

Please verify your personal information on record
with $this_organization

* * * * * * * *

{$rider_info}

* * * * * * * *

If this information is not correct, you can correct it by logging 
in to your $this_organization Member Page $login_url
or by contacting event organizer $organizer_name
at $organizer_email by simply replying to this email.

======================================================
======================================================
EOT;


		$organizer_message=<<<EOT
The following event registration has been received:

{$event_info}

{$rider_info}
EOT;


		$footer_line=<<<EOT
===================================================

EOT;


    // EMAIL 

		$config['wordwrap'] = false;
		$this->email->initialize($config);


    // CONSTRUCT AND SEND RIDER MAIL

		$this->email->clear();
		if($this->all_email_to_webmaster){
		  // $this->email->to([$contact_email]);
			$this->email->to([$webmaster_email]);
		}else{
			$this->email->to([$contact_email]);
		}
		$this->email->from($organizer_email, $organizer_name, $this->model_parando->postmaster_email);
		$this->email->subject("$event_name $event_distance Registration Confirmation");
		$this->email->message(	implode("\n",
													[
														$rider_message,
														$order_info,
														$payment_info,
														$footer_line,
														$cancel_message,
														$contact_organizer
													]));
    $rc_1=$this->email->send(FALSE);
		$debug_1=$this->email->print_debugger();
    //sleep(2);

    // CONSTRUCT AND SEND ORGANIZER MAIL

		$this->email->clear();
		if($this->all_email_to_webmaster){
			$this->email->to([$webmaster_email]);
		}else{
			$this->email->to([$organizer_email]);
			$this->email->bcc([$rba_email]);
		}
		$this->email->from($contact_email, "$first_name $last_name", $this->model_parando->postmaster_email);
		$this->email->subject("[$this_controller] $event_name $event_distance");
		$this->email->message(	implode("\n",
													[
														$organizer_message,
														$order_info,
														$payment_info,
														$release_message,
														$footer_line,
														$contact_rider
													]));
    $rc_2=$this->email->send(FALSE);
		$debug_2=$this->email->print_debugger();

    return ($rc_1 && $rc_2)?true:($debug_1 . "\n" . $debug_2);
//    return ($debug_1 . "\n" . $debug_2);

	}

	// EVENT CANCELLATION CONTROLLER METHOD

	public function cancel($cc="0-0-00000000"){

		$this->event_reg_session_init();
		$result = 'Invalid cancellation code';
		$e=0;
		$a = explode("-", $cc);
		if(count($a)==3){
			list($e, $m, $hash)=$a;
			if($cc == $this->make_cancel_code($e,$m)){
				$member=$this->model_parando->get_member($m);
				$event=$this->model_parando->get_event($e);
				if(isset($member) && isset($event)){
					$t=$this->model_parando->event_schedule($event);
					$event['distance_str']=$this->model_parando->make_distance($event);
					if($t['start_datetime']<$this->datetime_now){
					   	$result='Error! Too late to cancel registration.';
					}else{

						$result='OK';
						$schedule=$this->model_parando->event_schedule($event);
						$lodging_offered=!empty($event['lodging_1n_price']) || !empty($event['lodging_2n_price']);
						$organizer_data=$this->model_parando->get_organizer($event);	
						$memdata=$this->model_parando->compute_memdata($member);
						$event_data=array(
									'event_id'		=> $e,
									'event_contact_url'	=> site_url("info/contact/$e"),
									'event_start_city'	=> $event['start_city'],
									'event_start_state'	=> $event['start_state'],
									'event_name'	=> $event['name'],
									'event_distance'	=> $this->model_parando->make_distance($event),
									'event_date'	=> $schedule['start_date_str'],
									'cancel_event' => $e,
									'cancel_member' => $m
							);

						$this->multipage->set_data_array(array_merge($memdata,$organizer_data,$event_data));
						$this->multipage->set_current_page('cancel_confirm');
						$this->multipage->view_current_page();
					}
				}
			}
		}

		if($result != 'OK'){
			$this->die_error('Registration Cancellation Error', $result);
		}

	}

	public function _cancel_confirm(){

		$e=$this->multipage->get_data('cancel_event');
		$m=$this->multipage->get_data('cancel_member');
		if(empty($m) || empty($e)){
			$this->die_error('_do_cancel','Missing parameters.');
		}

		$contact_url=site_url("info/contact/$e"); 
		$rbacontact=" If you have any questions, <A HREF=$contact_url>contact the event organizer</a>.";

		$this->db->where('member_id', $m);
		$this->db->where('event_id', $e);
		$this->db->update('roster', array('result'=>'DNS'));
		//$this->db->delete('roster', array('event_id' => $e, 'member_id'=>$m));
		if (!$this->db->affected_rows()) {
			$this->die_error('Registration Error','Registration not found. Maybe it was canceled already?');
		} 
	}

	private function make_cancel_code($e,$m){
		$em_id="$e-$m";
		$secret=$this->model_parando->secret;
		$hash=hash('crc32',$em_id.$secret);
		return "$em_id-$hash";
	}

}


?>
