<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
  
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.


class Results extends MY_Controller {

	private $cache_enable;
	private $cache_minutes;

	public function __construct() {
		parent::__construct();
		$this->load->model('results_parando');
		$this->cache_enable=(!empty($this->model_parando->cache_enable));
		$this->cache_minutes=$this->model_parando->cache_enable;
	}


	// MEMBER SEARCH
	public function search(){
		if('POST' == $this->input->method(TRUE)){
				$this->_search_action();
		}else{
			$this->_search_display();
		}
	}

	public function _search_action(){

		if($this->input->post('member_id')>0){
			$this->member($this->input->post('member_id'));
		}else{

		$this->load->library('form_validation');
		$this->load->helper('form');
		$this->form_validation->set_rules('rusa_id','RUSA ID', 'trim|is_natural_no_zero|less_than[50000]');
		$this->form_validation->set_rules('last_name','Last Name', 'trim|regex_match[/^([a-zA-Z0-9]+ ?)+?$/]|min_length[2]|max_length[50]', array('regex_match'=>'Last Name must contain only letters, dashes, and single blanks'));

		if($this->form_validation->run()==FALSE){
			//$this->die_error(__METHOD__, "Form error = " . validation_errors());
			$this->_search_display();
		}else{
			$last_name=$this->input->post('last_name',true);
			$rusa_id=$this->input->post('rusa_id',true);

			if(!empty($rusa_id)){
				$member=$this->model_parando->get_member_by_rusa_id($rusa_id);
				if(empty($member)) 
					$this->die_error(__METHOD__, "RUSA ID $rusa_id not found");
				else{
					$member_id=$member['id'];
					$this->member($member_id);
				}
			}else if(!empty($last_name)){
				$member_list=$this->model_parando->get_member_by_last_name($last_name);
				if(count($member_list)>1)
					$this->_search_display($member_list); // $this->die_error(__METHOD__, "Multiple riders with last name $last_name. Please give RUSA ID.");
				else if(count($member_list)<1) $this->die_error(__METHOD__, "Last name $last_name not found.");
				else {
					$member=array_shift($member_list);
					$member_id=$member['id'];
					$this->member($member_id);
				}
			}else $this->_search_display();
		}

		}
	}

	public function _search_display($multi=null){

		$this->load->library('form_validation');
		$this->load->helper('form');

		$org=$this_organization=$this->model_parando->organization;
		$title = "$org Results Search";
		$data=[	'title'=>$title,
				'this_organization'=>$org];

		$data['form_title']=$title;
		$data['form_subject']=$title;
 
		$controller=strtolower(get_class($this));
		$data['form_action']=site_url($controller . '/search');

		$data['form_body']="";

		if($multi===null){

		$data['form_body']= <<<EOT
			<div class=form-group>
				<label for="last_name">Last Name</label>    
				<input type='text' placeholder='Enter a last name' name='last_name' value=''>
			</div>
			<div class=form-group>
				<label for="rusa_id">RUSA ID</label>    
				<input type='text' placeholder='Enter a RUSA ID' name='rusa_id' value=''>
			</div>
EOT;
			if(!empty(validation_errors())){
			  $data['form_body'] .= "<div class=verror>" . validation_errors() . "</div>";
			}

		}else{ // MULTI
			$data['form_body'] .= "<div class=form-group><label for='member_id'>Multiple matches. Please choose one rider:</label><div class=narrower>";
			foreach($multi as $member){
				$member_id=$member['id'];
				$rusa_id=$member['rusa_id'];
				$member_info=$member['first_name'] .' '. $member['last_name'];
				if($rusa_id>0) $member_info .= " (RUSA $rusa_id)";
				$data['form_body'].="<div class='image-radio'><input type='radio' name='member_id' id='member$member_id' value='$member_id'>$member_info</div>";
			}
			$data['form_body'] .= "</div></div>";
		}

		$tag='submit'; $text='Get Results';
		$data['form_submit']="<input type='submit' name='submit' value='$text'>";

		$this->load->view('form_header',array_merge($data,['page_styles'=>['form']]));
		$this->load->view('form_simple',$data);
		$this->load->view('form_footer',$data);
	}

	// 

	public function orphan_results(){
		$orphan_id=[];
		foreach($this->results_parando->results_bymd as $member_id=>$ymd){
			if(!isset($this->model_parando->members_by_id[$member_id])) $orphan_id[]=$member_id;
		}

		$mid=[];
		$missing="";
		foreach($orphan_id as $oid){
			$sql="SELECT event_id, date, name, distance, type FROM roster,event where event.id=roster.event_id AND roster.member_id='$oid'";
			$q=$this->db->query($sql);
			$events=$q->result_array();
			foreach($events as $e)
				$missing.= "MemberID=$oid missing from Event ID=" . implode(', ', array_values($e)) ."<br>\n";
		}

		sort($orphan_id);
		$missing.="Missing IDs<br>" . implode('<br>', $orphan_id);
		$this->die_error(__METHOD__, $missing);
	}


	// LEADER BOARD

	private $lb_headings=[
					''=>['#',''], 
					'last_name'=>['Rider Name','asc'], 
					'events_finished'=>['Total PA Finishes','desc'], 
					'total_km'=>['Total PA KM','desc'], 
					'average_kph'=>['Average KPH','desc'], 
					'awards'=>['Awards','desc'] ]; 

	private function update_main_leaderboard(){

		$lb_order_by='total_km'; $lb_order='desc'; $lb_min_rows=5;

		$leader_sort=function($a,$b) use($lb_order_by, $lb_order) {
			$o=($lb_order=='asc')?1:-1;
			if($a[$lb_order_by]!=$b[$lb_order_by])
				return $o*(($a[$lb_order_by]<$b[$lb_order_by])? -1 : 1);
			else
				return strcasecmp($a['last_name'],$b['last_name']);
		};

		$r=$this->calc_leaderboard(null,false);

		foreach($r as &$a){ // drop awards
			unset($a['awards']);
			unset($a['average_kph']);
		}

		uasort($r, $leader_sort);

		$lb_rows=0;
		$start=null;
		foreach($r as $lbr){
			if($start!=null && $lbr[$lb_order_by]<$start && $lb_rows>$lb_min_rows) 
				break;
			else{
				$lb_rows++;
				$start=$lbr[$lb_order_by];
			}
		}

		$lbl=array_slice($r,0,$lb_rows);

		$heading_list=$this->lb_headings;
		array_shift($heading_list); // drop #
		unset($heading_list['awards']);
		unset($heading_list['average_kph']);
		$headings=[];
		foreach($heading_list as $h)
			$headings[]=$h[0];

		$leader_txt="<TR><TH id=firstcol>" . implode('</TH><TH>',$headings) . '</TH><TR>';
		$leader_txt.=$this->lb_table_body($lbl,1,false);

		$this->db->where('location','leaderboard');
		$rc=$this->db->update('paragraph',['text'=>$leader_txt]);
		if(FALSE==$rc){$this->die_error(__METHOD__,"Could not update leaderboard");}

		// $this->die_info(__METHOD__, "Leaderboard updated");

	}

	public function leaderboard($order_by='total_km',$order="desc",$year=null,$must_have=1){


		if($this->cache_enable) $this->output->cache($this->cache_minutes);


		$reverse_order=['desc'=>'asc','asc'=>'desc'];

		$order_by=strtolower($order_by);
		if(!isset($this->lb_headings[$order_by])) $this->die_error(__METHOD__, "Unknown sort column $order_by");

		$order=strtolower($order);
		if(!isset($reverse_order[$order])) $this->die_error(__METHOD__, "Unknown ordering $order");
		$sort_icon=($order=='desc')?"<i class='fas fa-sort-down'></i>":"<i class='fas fa-sort-up'></i>";

		$org=$this_organization=$this->model_parando->organization;
		$org_abbrev=$this->model_parando->org_abbrev;

		$y_now=$this->results_parando->y_now;

		if($year===null){
			$board="All Time Rider Leaderboard";
			// $date_sql="";
			$all_time=true;
			$full_leaderboard='';
			$year=$y_now;
		}else{
			if($year==$y_now) $this->update_main_leaderboard(); // Side effect
			$last_year=$year-1;
			$board="Rider Leaderboard for $year";
			$full_leaderboard=anchor(site_url("results/leaderboard"), 'All Time Leaderboard');
			// $date_sql="AND YEAR(date)=$year";
			$all_time=false;
		}

		$title = "$org $board";
		$data=[	'title'=>$title,
				'this_organization'=>$org];


		$heading_html="";
		$first=" class=firstcol";
		foreach($this->lb_headings as $ob=>$th_props){
			$th=array_shift($th_props);
			$order_default=array_shift($th_props);
			if($ob==$order_by){
				$th_text="$th $sort_icon";
				$o=$reverse_order[$order];
			}else{
				$th_text="$th";
				$o=$order_default;
			}
			$y=($all_time)?'':"/$year";
			$th_link=(empty($ob))?$th:anchor("results/leaderboard/$ob/$o$y","$th_text","title='Sort by $th'");
			$heading_html.="<TH$first>$th_link</TH>";
			$first="";
		}
		$num_cols=count($this->lb_headings);

		$data['body']=<<<EOT
    <div class='roster td-center th-center'>
			<h1>$org</h1>
			<h2>$board</h2>

      <TABLE WIDTH=100%>
			<TR>$heading_html</TR>
EOT;

		$r=$this->calc_leaderboard($year, $all_time); 

		$numeric_sort=function($a,$b) use($order_by, $order) {return (($order=='asc')?1:-1)*(($a[$order_by]<$b[$order_by])? -1 : 1);};
		$strcasecmp_sort=function($a,$b) use($order_by, $order) {return (($order=='asc')?1:-1)*strcasecmp($a[$order_by],$b[$order_by]);};
		$award_sort=function($a,$b) use($order) {
			$o=($order=='asc')?1:-1;
			if($a['award_total']!=$b['award_total'])
				return $o*(($a['award_total']<$b['award_total'])? -1 : 1);
			else if($a['events_finished']!=$b['events_finished'])
				return $o*(($a['events_finished']<$b['events_finished'])?-1:1);
			else if($a['total_km']!=$b['total_km'])
				return $o*(($a['total_km']<$b['total_km'])?-1:1);
			else
				return $o*strcasecmp($a['last_name'],$b['last_name']);
		};



		switch($order_by){
			case 'awards':
				uasort($r, $award_sort);
				break;
			case 'last_name':
				uasort($r, $strcasecmp_sort);
				break;
			default:
				uasort($r, $numeric_sort);
				break;
		}

		$data['body'].=$this->lb_table_body($r,$must_have);

		$mht=($must_have<=1)? $mht="" : "<div class='text-center fineprint italic'>Minimum $must_have $org events finished.</div>";

		$data['body'].=<<<EOT
</TABLE>
			<div class='text-right'>$full_leaderboard</div>

</DIV>
EOT;

		$this->load->view('simple_header',array_merge($data,['page_styles'=>['roster','schedule']]));
		$this->load->view('simple_body',$data);
		$this->load->view('simple_footer',$data);
	}

	// Convert the $r array into HTML table rows
	private function lb_table_body($r,$must_have=1,$count_col=true){
		$i=1;
		$body="";
		foreach($r as $row){
			extract($row);
			if($events_finished<$must_have) continue;
			$rider_name=$first_name .' '. $last_name;
			$member_results_url=site_url("results/member/$member_id");
			if($count_col) $body.="<TR><TD>$i</TD>";
			$body.="<TD><A TITLE='Results' HREF='$member_results_url'>$rider_name<A></TD>";			
			$body.="<TD>$events_finished</TD>";			
			$body.="<TD>$total_km</TD>";			
			if(isset($average_kph)) $body.=sprintf("<TD>%.1f</TD>",$average_kph);			
			if(isset($awards)) $body.="<TD>$awards</TD>";			
			$body.="</TR>\n";
			$i++;
		}
		return $body;
	}

	// Compute the $r array from results db

	private function calc_leaderboard($y=null,$all_time=true){

		$year=($y==null)?$this->results_parando->y_now:$y;

		$r=[];
		foreach($this->results_parando->results_bymd as $member_id=>$ymd){

			if(!isset($this->model_parando->members_by_id[$member_id])) 
				$this->die_error(__METHOD__, "Results for rider (ID=$member_id) not listed in member database. Tell the webmaster!"); 

			$events_finished=0; $total_km=0; $total_minutes=0;

			$start_year=($all_time)?$this->results_parando->results_epoch:$year;
			for($y=$start_year; $y<=$year; $y++)
				if(isset($ymd[$y])){
					foreach($ymd[$y] as $m=>$d){
						foreach($d as $dist=>$etl){
							$n=count($etl);
							$total_minutes+=array_sum($etl);
							$events_finished+=$n;
							$total_km+=$dist*$n;
						}
					}
				}
			if($events_finished>0) {
				$first_name=$this->model_parando->members_by_id[$member_id]['first_name'];
				$last_name=$this->model_parando->members_by_id[$member_id]['last_name'];

				$average_kph=60*$total_km/$total_minutes;

				$award_total=0;
				$award_list=[];
				foreach($this->results_parando->award_keys as $ak){
					if(isset($this->results_parando->member_has[$ak][$member_id])){
						$has_by_y=$this->results_parando->member_has[$ak][$member_id];
						if($all_time)
							$na=max($has_by_y);
						else if(isset($has_by_y[$year]))
							$na=$has_by_y[$year];
						else 
							$na=0;
						if($na>0) $award_list[]=($na>1)?"$ak ($na)":$ak;
						$award_total+=$na;
					}
				}
				$awards=implode(', ',$award_list);



				$r[]=compact('member_id','first_name','last_name','events_finished','total_km','average_kph','awards','award_total');
			}
		}

		return $r;
	}



  // MEMBER RESULTS SUMMARY

	public function member($id=null,$order='desc'){

		if($id+0 > 0){ // numeric ID
			$member=$this->model_parando->get_member($id);
		}else if(preg_match("/^([a-z.']+)[_-]([a-z.']+)$/",$id,$m)){ // text name shortcut (fails if first/last name has a blank)
			$member=$this->model_parando->get_member_firstlast($m[1],$m[2]);
		}else{
			$this->die_error(__METHOD__,"Invalid Member ($id)");
		}

		if(empty($member))
			$this->die_error(__METHOD__, "No member (ID=$id)");

		$first_last=$member['first_name'] . ' ' . $member['last_name'];

		$org=$this_organization=$this->model_parando->organization;
		$org_abbrev=$this->model_parando->org_abbrev;

		$country_fullname=$this->model_parando->get_countries();
		$state=($member['m_country'] != "US")?($member['m_state']. ' (' . $country_fullname[$member['m_country']] . ')'):$member['m_state'];

		$rusa_id=$member['rusa_id'];
		$rusa_results_link=($rusa_id > 0) ? 
			"RUSA $rusa_id <A title='Complete RUSA results for $first_last' HREF=https://rusa.org/cgi-bin/resultsearch_PF.pl?mid=$rusa_id><i class='fa fa-book'></i></a>":'';

		$title = "$org Results for $first_last";
		$subtitle = $member['m_city'] . ', ' . $state; // . ' ' . anchor("info/contact/member/$id",'<i class="fa fa-envelope"></i>',"TITLE='Contact $first_last'");
		$data=[	'title'=>$title,
				'this_organization'=>$org];
		
		$results=$this->results_parando->member_detail($member);


		$headings=['Date','Sanction','Distance','Event','Time','Award','Notes'];
		$heading_html="<TH style='text-align:center'>" . implode("</TH><TH style='text-align:center'>",$headings) . "</TH>";
		$num_cols=count($headings);

		$ls=(!empty($results['lifetime']))?$this->format_summary($results['lifetime']):null;
		$lifetime_summary=(empty($ls))?"":"$org Lifetime Totals: $ls";

		$data['body']=<<<EOT
    <div class=roster>
			<h2>$org</h2>
			<h3>Results for $first_last</h3>
			<h4>$rusa_results_link</h4>
			<div id=lifetime>$lifetime_summary</div>
      <TABLE WIDTH=100%>
			<TR>$heading_html</TR>
EOT;


		if(!empty($results['annual']))
			$results_by_annum=($order=='desc')?array_reverse($results['annual'],true):$results['annual'];
		else
			$results_by_annum=[];

		// Loop through each result by rider


		foreach($results_by_annum as $annum=>$r_list){

			$data['body'].="<TR><TD id='yearbreak' colspan=$num_cols>$annum</TD></TR>";
		
			$as=$this->format_summary($r_list);
			$data['body'].=(empty($as))?"":"<TR><TD id='endyear' colspan=7>$as</TD></TR>";

			foreach($r_list['results'] as $r){

				$data['body'].='<TR>'; // Starting a new row

				$dt=new DateTime($r['date']);
				$date=$dt->format('M j');

				$data['body'].='<TD>' . $date . '</TD>'; 
				$data['body'].='<TD style="text-align:center">' . $r['sanction'] . '</TD>';

				$dist_str=($r['type']=='fleche')?'Fl&egrave;che':$r['distance'] . "K";
				$data['body'].='<TD style="text-align:center">' . $dist_str . '</TD>';

				$event_id=$r['event_id'];
				$event=$this->model_parando->get_event($event_id);

				$info=(!empty($event['info']))?$event['info']:site_url("info/event/$event_id");
				$info_url=make_site_url($info);
				$info_link=textlink($info_url,$r['name'],'Event Details');

				$report_url=make_site_url($event['report']);
				$report_link=iconlink($report_url,'fas fa-book-open',"Ride Report");

				$results=(!empty($event['results']))?$event['results']:site_url("roster/results/$event_id");
				$results_url=make_site_url($results);
				$results_link=iconlink($results_url,'fas fa-users',"Results");

				$data['body'].="<TD>" . $info_link . "</TD>";

				list($hr,$min,$sec)=explode(':',$r['elapsed_time']); 
				$data['body'].='<TD style="text-align:center">' . sprintf('%02d:%02d',$hr,$min) . '</TD>'; 

				$award_str=(count($r['award_list'])>0)?("<div class=medal>".implode(', ',$r['award_list'])."</div>"):"";
				$data['body'].="<TD>$award_str</TD>";

				$r5p="";

				// foreach($r['r5000_progress'] as $k=>$d){
				// 	if(!empty($d)) $r5p.=" $k(" . count($d) . ")";
				// }
				$data['body'].="<TD>  $report_link $results_link " . $r['comment'] . '</TD'; // '<span style="font-size: 0.5em">' . $r5p . '</span>' . '</TD>';

				$data['body'].='</TR>'; // End of row
			}
		}


		$data['body'].='</TABLE></DIV>';


		$this->load->view('simple_header',array_merge($data,['page_styles'=>['roster','results']]));
		$this->load->view('simple_body',$data);
		$this->load->view('simple_footer',$data);
	}

	private function format_summary($r){
		if(isset($r['summary'])){ // && $r['won_award']){
			$summary=$r['summary'];

			$award_text="";
			if(count($summary['award_list'])>0){
				$award_text .= "; Awards: " . implode(', ',$summary['award_list']);
			}

			$summary_text= sprintf("Finishes %d;  Distance %s km; Avg Speed %.1f km/h%s", 
					$summary['finishes'],  number_format($summary['distance']), $summary['avg_speed'] , $award_text);
		}else $summary_text="";
		
		return $summary_text;

	}




	public function annual_summary($year=null,$output_format="web"){

		if($this->cache_enable) $this->output->cache($this->cache_minutes);

		$this_organization=$this->model_parando->organization;
		$results_epoch=$this->model_parando->results_epoch;
		$tz=$this->model_parando->get_local_time_zone();
		$d_now = $this->results_parando->d_now;
		$m_now = $this->results_parando->m_now;
		$y_now = $this->results_parando->y_now;

		if(empty($year)){$year=$y_now;}

		if($year<$results_epoch){$this->die_error(__METHOD__,"Year before $results_epoch not allowed.");}
		if($year>$y_now){$this->die_error(__METHOD__,"Future years not allowed ($year > $y_now).");}
		
		$title = "$this_organization Results Summary for $year";

		$this->results_parando->annual_summary($year,$output_format);

		$n_events = $this->results_parando->n_events;
		$total_finishers = $this->results_parando->total_finishers;
		$total_starters = $this->results_parando->total_starters;
		$total_percent = $this->results_parando->total_percent;
		$total_km = $this->results_parando->total_km;

		$summary=<<<EOT
<TABLE CLASS=summary_results>
  <TR><TD>Events Organized:</TD><TD>$n_events</TD></TR>
  <TR><TD>Total Finishes:</TD><TD>$total_finishers</TD></TR>
  <TR><TD>Total Finished Km:</TD><TD>$total_km</TD></TR>
</TABLE>
EOT;

		$header=[];
		$table_head ="";
		
		$table_rows=[];
		$table_body ="";
		
		$sticker_fields=['last_name','first_name','rusa_id','date','name','distance','type','result','cert'];
		$result_fields=['date','name','distance','type','result'];
		
		switch($output_format){
			case 'csv_with_address':
			case 'with_address':

				$params = ['tag' => 'login_session_data'];
				$this->load->library('randosession',$params);
				$this->randosession->admin_check();

				$download_url=site_url("results/annual_summary/$year/csv_with_address");
				$header_titles=['First','Last','Email','Street','City','State','Zip','Country','RUSA',"Finishes","KM",'R12 Run','SR','R12','KM Rank','Results'];
				$header[]=$header_titles;
				
				if($output_format=='csv_with_address'){
					$start_table='';
					$end_table='';
					$start_row=''; 
					$end_row='';
					$row_delimiter="\n";
					$col_delimiter=' ';
				}else{
					$start_table='<TABLE><TR>';
					$end_table='</TR></TABLE>';
					$start_row='<TD>'; 
					$end_row='</TD>';
					$row_delimiter='</TR><TR>';
					$col_delimiter='</TD><TD>';
				}

				foreach($this->results_parando->table as $row){
					$table_body.="<TR>";
					$tr=[];
					foreach($row['row'] as $col){
						if(is_array($col)){  // Will be a list of finish arrays
					    	$finish_list=[];
							foreach($col as $e){
								extract($e);
								$distance="$distance"."K";
								$type=ucfirst($type);
								if(!$finish) continue;
								$result="FINISH $result";
								$finish_list[]=$start_row.implode($col_delimiter,compact($result_fields)).$end_row;
							}
							$text=$start_table.implode($row_delimiter,$finish_list).$end_table;
						}else{
							$text=$col;
						}
						$table_body.="<TD>$text</TD>";
						$tr[]=$text;
					}
					$table_body.="</TR>";
					$table_rows[]=$tr;
				}


				break;
			default:
				$download_url=site_url("results/annual_summary/$year/csv");
				$header_titles=['First','Last','RUSA',"Finishes","KM",'R12 Run','SR','R12','KM Rank'];
				$header[]=array_fill(0,count($header_titles),"");
				$header[]=array_fill(0,count($header_titles),"");
				$header[]=$header_titles;
				foreach($this->results_parando->event_ids as $event_id){
					$d_event = new DateTime($this->results_parando->event_data[$event_id]['date']);
					if($d_now>$d_event && !empty($this->results_parando->n_started[$event_id])  && isset($this->results_parando->n_finished[$event_id])){
						$nfs=$this->results_parando->n_finished[$event_id]."/".$this->results_parando->n_started[$event_id];
					}else{
						$nfs="";
					}
					$d  = $d_event->format("m-d");
					$k  = (($this->results_parando->event_data[$event_id]['type']=="fleche")?"Fleche":($this->results_parando->event_data[$event_id]['distance'] . "K"));
					$header[0][]=$d;
					$header[1][]=$k;
					$header[2][]=$nfs;
				}

				foreach($this->results_parando->table as $row){
					$table_body.="<TR>";
					$tr=[];
					$member_id=$row['member_id'];
					foreach($row['row'] as $col){
						if(is_array($col)){ // will be a single finish array
							$text=$col['result'];
						}else{
							$text=$col;
						}
						$table_body.="<TD>$text</TD>";
						$tr[]=$text;
					}
					$table_body.="</TR>";
					$table_rows[]=$tr;
				}


				break;
		}
		

		// Format
		foreach($header as $header_row){
			$table_head.="<TR>";
			foreach($header_row as $f){$table_head .= "<TH>$f</TH>";}
			$table_head.="</TR>";
		}


		$content=<<<EOT
EOT;

		if($output_format=="csv" || $output_format=="csv_with_address"){
			$this->load->helper('csv');
			$table_array=array_merge($header,$table_rows);
			array_to_csv_download($table_array,"ResultsSummary$year.csv");
		}else{

		// Output
		$this->load->view('annual_summary',['title'=>$title,
											'this_organization'=>$this_organization,
											'table_head'=>$table_head,
											'table_body'=>$table_body,
											'summary'=>$summary,
											'download_url'=>$download_url]
											);
		}
	}


// These functions are from the Info Controller

	private function info_view($view='info_three_column',$page_styles=[],$data=[]){
		$this->load->view('info_header',array_merge($data,['page_styles'=>$page_styles]));
		$this->load->view($view,$data);
		$this->load->view('info_footer',$data);
	}

  private function topnav_home(){
		$topnav=
		"<a href=" . base_url('') . ">Home</a>" .
		"<a href=" . site_url('info/volunteer') . ">Volunteer!</a>" .
		"<a href=" . site_url('info/tracker') . ">Rider Tracking</a>" .
		"<a href=" . site_url('info/resources') . ">New Rider Info</a>" .
		"<a href=" . base_url('member/join') . ">Join/Renew</a>" .
		"<A HREF=" . site_url('info/contact') . ">Contact</a>" .
		'<span class="float-right">&nbsp;<a href=' . site_url("login") . ' class="fas fa-cogs"></a></span>';

		return $topnav;
	}

	private function check_year($year){
		$results_epoch=$this->model_parando->results_epoch;
		$y_now=$this->results_parando->y_now;
		if(empty($year)){$year=$this->results_parando->y_now;}
		if($year<$results_epoch){$this->die_error(__METHOD__,"Year before $results_epoch not allowed.");}
		if($year>$y_now){$this->die_error(__METHOD__,"Future years not allowed.");}
		return $year;
	}

	// This function has the side effect of updating the database with text paragraphs that are 
  // used for the the honor role on the main home page

	public function honor_roll_lifetime($year=null){
	
		if($this->cache_enable) $this->output->cache($this->cache_minutes);

		$org=$this_organization=$this->model_parando->organization;

		$year=$this->check_year($year);
		
		$title = "$this_organization Lifetime Honor Roll";
		$data=[	'title'=>$title,
				'this_organization'=>$org];
		
		$this->update_honor_roll_current(); // Side effect


		$data['r5000_txt']=implode('<br>', $this->results_parando->honorees($this->results_parando->member_has['R5000'],$year)); // member_has_r5000 unimplemented
		$data['sr_txt']=implode('<br>', $this->results_parando->honorees($this->results_parando->member_has['SR'],$year)); 
		$data['r12_txt']=implode('<br>', $this->results_parando->honorees($this->results_parando->member_has['R12'],$year));


		$data['header_title']=$org;
		$data['header_subtitle']=$this->model_parando->slogan;

		$data['topnav']=$this->topnav_home();
		$this->info_view('honor_roll_lifetime',['honor'],$data);

	}

	public function honor_roll_lifetime_csv($year=null){

		$year=$this->check_year($year);
		$this->load->helper('csv');
		$table_rows=[];

		foreach($this->results_parando->award_keys as $k){
			$a_list=$this->results_parando->get_award_list($this->results_parando->member_has[$k],$year);
			foreach ($a_list as $row){
				$rvals=array_values($row);
				array_unshift($rvals,$k);
				$table_rows[]=$rvals; 
			}
		}


		$table_rows=array_merge([['Award','First','Last','Number']],$table_rows);
		//$this->die_error(__METHOD__,print_r($table_rows,true));
		array_to_csv_download($table_rows,"LifetimeHonorRoll$year.csv");

	}


	// This function has the side effect of updating the database with text paragraphs that are 
  // used for the the honor role on the main home page

	public function honor_roll_current(){
	
		if($this->cache_enable) $this->output->cache($this->cache_minutes);

		$this_organization=$this->model_parando->organization;
		$y_now=$this->results_parando->y_now;

		$title = "$this_organization Honor Roll for $y_now";
		
		$award_txt=$this->update_honor_roll_current();

		$data=array_merge(compact('title','this_organization'), $award_txt);

		$data['header_title']=$this_organization;
		$data['header_subtitle']=$this->model_parando->slogan;

		$data['topnav']=$this->topnav_home();
		$this->info_view('honor_roll_current',['honor'],$data);

	}
	
	public function show_uhrc(){
		$data['body']=print_r($this->update_honor_roll_current(),true);
		$this->load->view('simple_header',array_merge($data,['page_styles'=>[]]));
		$this->load->view('simple_body',$data);
		$this->load->view('simple_footer',$data);
    }

	private function update_honor_roll_current(){

		$latest_sr_year=$this->results_parando->latest_award_year($this->results_parando->member_has['SR']);
		$latest_r12_year=$this->results_parando->latest_award_year($this->results_parando->member_has['R12']);

		$r5000_txt=""; // member_has_r5000 unimplemented

		$sr_txt="<h4>$latest_sr_year PA Super Randonneurs</h4><div class='newspaper'>" . 
			implode('<br>', $this->results_parando->honorees($this->results_parando->member_has['SR'],$latest_sr_year,'current')) . "</div>"; 
		$this->db->where('location','honor_roll_sr');
		$rc=$this->db->update('paragraph',['text'=>$sr_txt]);
		if(FALSE==$rc){$this->die_error(__METHOD__,"Could not update SR honor roll");}

		$r12_txt="<h4>$latest_r12_year PA R-12</h4><div>" . 
			implode('<br>', $this->results_parando->honorees($this->results_parando->member_has['R12'],$latest_r12_year,'current')) . "</div>";
		$this->db->where('location','honor_roll_r12');
		$rc=$this->db->update('paragraph',['text'=>$r12_txt]);
		if(FALSE==$rc){$this->die_error(__METHOD__,"Could not update R12 honor roll");}

		return compact('r5000_txt','r12_txt','sr_txt','latest_sr_year','latest_r12_year');

	}
	
	
	public function annual_certs($year=null,$output_format="web"){

		$params = ['tag' => 'login_session_data'];
		$this->load->library('randosession',$params);
		$this->randosession->admin_check();

		if($this->cache_enable) $this->output->cache($this->cache_minutes);

		$this_organization=$this->model_parando->organization;
		$results_epoch=$this->model_parando->results_epoch;
		$tz=$this->model_parando->get_local_time_zone();
		$d_now = $this->results_parando->d_now;
		$m_now = $this->results_parando->m_now;
		$y_now = $this->results_parando->y_now;

		if(empty($year)){$year=$y_now;}
		
		if(preg_match('!(\d{4})_(\d{1,2})_to_(\d{1,2})!', $year, $m)){
			$year=$m[1];
			$start_month=$m[2];
			$end_month=$m[3];
			$n_months=1 + $end_month-$start_month;
			$start_date=new DateTimeImmutable("$year-$start_month-01");
			$end_date=$start_date->add(new DateInterval("P" . $n_months ."M"));
		}else{
			$n_months=0;
		}

		if($year<$results_epoch){$this->die_error(__METHOD__,"Year before $results_epoch not allowed.");}
		if($year>$y_now){$this->die_error(__METHOD__,"Future years not allowed ($year > $y_now).");}
		
		$title = "$this_organization Certifications for $year";
		$download_url=site_url("results/annual_certs/$year/csv");
		$pdf_url=site_url("results/annual_certs/$year/pdf");

		$this->results_parando->event_calc($year);
		$this->results_parando->results_calc($year);

		$n_events = $this->results_parando->n_events;
		$total_finishers = $this->results_parando->total_finishers;
		$total_starters = $this->results_parando->total_starters;
		$total_percent = $this->results_parando->total_percent;
		$total_km = $this->results_parando->total_km;

		$summary=<<<EOT
<TABLE CLASS=summary_results>
  <TR><TD>Events Organized:</TD><TD>$n_events</TD></TR>
  <TR><TD>Total Finishes:</TD><TD>$total_finishers</TD></TR>
  <TR><TD>Total Finished Km:</TD><TD>$total_km</TD></TR>
</TABLE>
EOT;

		
		
		// Generate header
//		$header=[];
		$header_fields=['last_name','first_name','rusa_id','date','sanction','name','distance','type','result','cert'];

		// Construct list of cert results
		$table_rows =[];
		foreach($this->results_parando->results as $member_id=>$event_list){
			foreach($event_list as $event_id=>$result){
				extract($result);
				$event_date=new DateTime($date);
				if($finish && ($n_months==0 || ($event_date>=$start_date && $event_date<$end_date))){
					$row_fields=compact($header_fields);
					$table_rows[]=$row_fields;
				}
			}
		}

		// Sort
		usort($table_rows, [$this, 'by_date_last_first']);

		// Construct HTML table
		$table_body ="";
		foreach($table_rows as $r){
					$table_body.="<TR><TD>" . implode('</TD><TD>',$r) . "</TD></TR>";
		}


		switch($output_format){
			case "pdf":
				$params = ['this_organization'=>$this_organization, 'cert_table' => $table_rows];
				$this->load->library('Labels',$params);
					$this->labels->AddPage();
					$this->labels->render_sheet();
					$this->labels->Output("I", "CertLabels-$year.pdf");
				break;
			case "csv":
				$this->load->helper('csv');
				$table_array=array_merge([$header_fields],$table_rows);
				array_to_csv_download($table_array,"CertSummary$year.csv");
				break;
			default:
				$table_head = '<TR><TH>' . implode('</TH><TH>',$header_fields) . '</TH></TR>';
				$this->load->view('annual_summary',['title'=>$title,
												'this_organization'=>$this_organization,
												'table_head'=>$table_head,
												'table_body'=>$table_body,
												'summary'=>$summary,
												'download_url'=>$download_url,'pdf_url'=>$pdf_url]
												);
				break;
		}
	}
	
	function by_date_last_first($a,$b){
		$a_date=new DateTime($a['date']);
		$b_date=new DateTime($b['date']);

		$c=$this->spaceship($a_date,$b_date);		
		if($c==0) $c=$this->spaceship($a['last_name'],$b['last_name']);		
		if($c==0) $c=$this->spaceship($a['first_name'],$b['first_name']);		
		
		return $c;
	}
	
	function spaceship($a,$b){
		if($a==$b)
			return 0;
		else
			return ($a>$b)?1:-1;
	}


}
