<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

 
class Roster extends MY_Controller {

  // Produces various flavors of results pages, rosters, release sheets, csvs, and other paperwork support

	public function __construct() {
		parent::__construct();
		$params = ['tag' => 'login_session_data'];
		$this->load->library('randosession',$params);
				$this->load->library('controletimes');

	}

	private function admin_check(){
		if($this->randosession->get_data('login_is_admin')){
			return true;
		}else{
			$this->die_error('Access Denied',"This function is available only to administrators.");
		}

	}

	private function login_check(){
		if($this->randosession->get_data('login_logged_in')){
			return true;
		}else{
			$this->die_error('Access Denied',"This function is available only when logged in.");
		}

	}

	public function online($event_id){ $this->roster($event_id,"online"); }
	public function start($event_id){ $this->roster($event_id,"start_signin"); }
	public function start_html($event_id){ $this->roster($event_id,"start_signin_html"); }
	public function waiver($event_id){ $this->roster($event_id,"waiver"); }
	public function cwaiver($event_id){ $this->roster($event_id,"cwaiver"); }
	public function waiver_html($event_id){ $this->roster($event_id,"waiver_html"); }
	public function controle_html($event_id){ $this->roster($event_id,"controle_html"); }
	public function controle($event_id){ $this->roster($event_id,"controle"); }
	public function postcard_backs($event_id){ $this->roster($event_id,"postcard_backs"); }
	public function postcard_fronts($event_id){ $this->roster($event_id,"postcard_fronts"); }
//	public function qrcheckin($event_id){ $this->roster($event_id,"qrcheckin"); }
	public function results($event_id){ $this->roster($event_id,"results"); }
	public function comcsv($event_id){ $this->roster($event_id,"comcsv"); }
	public function rusacsv($event_id){ $this->roster($event_id,"rusacsv"); }
	public function fullcsv($event_id){ $this->roster($event_id,"fullcsv"); }
	public function setnullfinished($event_id){ $this->roster($event_id,"setnullfinished"); }
	public function rusa_member_check($event_id){ $this->roster($event_id,"rusa_member_check"); }
	public function email($event_id){ $this->roster($event_id,"email"); }
	public function volunteer($event_id){ $this->roster($event_id,"volunteer"); }
	

	private function interpolate_strings($s,$map){
			$vars=[];
			foreach ($map as $k => $v){
				if((is_string($v) || is_int($v) || is_float($v)) && !empty($k)){
					$tag = '{$' . $k . '}';
					$vars[$tag]=$v . '';
				}
			}
			$s_interp=strtr($s,$vars);
			//$this->die_error(__METHOD__, '<pre>'. print_r($vars,true) . '----\n' . print_r($s,true). '----\n' . print_r($s_interp,true) .'</pre>');
			return $s_interp; 
	}

	private function get_release_html($signature_line=false,$map=null){		
		$release_text = "<div class=release-form>";
		$release_text.= '<div class=release-header>' . $this->interpolate_strings($this->model_parando->get_paragraph('release_header'),$map) . '</div>';
		$release_text.= '<div class=release-title>' . $this->interpolate_strings($this->model_parando->get_paragraph('release_title'),$map) . '</div>';
		$release_text.= '<p>' . $this->interpolate_strings($this->model_parando->get_paragraph('release_preamble'),$map) . '</p>';

			$clauses=$this->model_parando->get_paragraph_array('release');
		  $needed=count($clauses);
		  $found=0;
			for($i=1; $i<=$needed; $i++){
				$ai = 'agree' . $i;
				$release_text .= "<p>" . $this->interpolate_strings($clauses[$ai], $map) . "</p>";
			}
			if($signature_line){
				$release_text .= '<div class=release-signature>';

				$release_text .= <<<EOT
<TABLE WIDTH=100%>
<TR><TH WIDTH=20%>DATE</TH><TH>NAME (PRINTED)</TH><TH>SIGNATURE (only if age 18 or over)</TH></TR>
<TR><TD></TD><TD></TD><TD></TD></TR>
</TABLE>
EOT;

				$release_text .= '</div>';
			}
			$release_text.= '<div class=release-footer>' . $this->interpolate_strings($this->model_parando->get_paragraph('release_footer'),$map)  . '</div>';
			$release_text .= "</div>";
			return $release_text;
	}

  private function interpolate_and_strip($d,$map){
		$out=strip_tags($this->interpolate_strings($d,$map));
		return $this->model_parando->he_decode($out);
	}
			

	private function get_release_array($map=null){		
		$release_text = [];
		$release_text['header'] = ($this->interpolate_and_strip($this->model_parando->get_paragraph('release_header'),$map));
		$release_text['title'] = ($this->interpolate_and_strip($this->model_parando->get_paragraph('release_title'),$map));
		$release_text['preamble'] = ($this->interpolate_and_strip($this->model_parando->get_paragraph('release_preamble'),$map));

		$clauses=$this->model_parando->get_paragraph_array('release');
	  $needed=count($clauses);
		$release_text['clause']=[];
		for($i=1; $i<=$needed; $i++){
			$ai = 'agree' . $i;
			$release_text['clause'][] = $this->interpolate_and_strip($clauses[$ai], $map);
		}
		$release_text ['signature'] =  <<<EOT
<TABLE WIDTH=100%>
<TR><TH WIDTH=20%>DATE</TH><TH>NAME (PRINTED)</TH><TH>SIGNATURE (only if age 18 or over)</TH></TR>
<TR><TD></TD><TD></TD><TD></TD></TR>
</TABLE>
EOT;
			$release_text['footer'] = ($this->interpolate_and_strip($this->model_parando->get_paragraph('release_footer'),$map));
			return $release_text;
	}
	
	
	public function rusa_status($rusa_id,$new=0){
					$this->login_check();
					$this->load->library('rusa');
					$now=new DateTime();
					
					if($new>0)
					  $result=$this->rusa->new_query_rusa_id($rusa_id);
					else
					  $result=$this->rusa->query_rusa_id($rusa_id);
					  
		if($result==false) $result="NOT FOUND";
		
		$data=['title'=>"RUSA Memeber Lookup", 'left_column'=>print_r($result,true), 'right_column'=>''];
		$this->simple_view('info_one_column',[],$data);
}


	public function roster($event_id=-1,$style="online"){
		if($event_id==-1){
			$event_id=$this->model_parando->next_event_id();
		}
		$event=$this->model_parando->get_event($event_id);
		if(isset($event)){
		
            $club=$event['club'];
            $club_name=$this_organization=$club['name'];
            $club_region=$club['region'];
				
			if($event['type']=='fleche'){
				$this->load->model('fleche_parando');
				$roster=$this->fleche_parando->get_fleche_roster($event_id);
			}else{
				$roster=$this->model_parando->get_roster($event_id);
			}
			$event['distance_str']=$this->model_parando->make_distance($event);
			$event['event_id']=$event_id;
			$schedule=$this->model_parando->event_schedule($event);
			$event=array_merge($event,$schedule);
			switch($style){
				case 'xonline':
					$this->load->view('roster_online',array('event'=>$event, 'roster'=>$roster));
					break;
				case 'online':
					$this->roster_online($event, $roster);
					break;
				case 'postcard_backs':	
					$this->login_check();
					$event_distance=$event['distance_str'];
					$event_name=$this->model_parando->he_decode($event['name']);
					$event_date=$event['start_daymonthyear_str'];
					$tagname=$this->model_parando->make_event_tagname($event);
					$params = [
						'event'=>$event, 
						'this_organization'=>$club_name, 
						'club'=>$this->model_parando->club,
						'tagname'=>$tagname,
						'name_dist'=>"$event_name $event_distance",
						'roster'=>$roster
					];
					$this->load->library('postcard',$params);
					$this->postcard->render_backs();
					$this->postcard->Output("I", $tagname . "-Postcard-Backs.pdf");
					break;
				case 'postcard_fronts':	
					$this->login_check();
					$event_distance=$event['distance_str'];
					$event_name=$this->model_parando->he_decode($event['name']);
					$event_date=$event['start_daymonthyear_str'];
					$tagname=$this->model_parando->make_event_tagname($event);
					$params = [
						'event'=>$event, 
						'this_organization'=>$club_name, 
						'club'=>$this->model_parando->club,
						'tagname'=>$tagname,
						'name_dist'=>"$event_name $event_distance",
						'roster'=>$roster
					];
					$this->load->library('postcard',$params);
					$this->postcard->render_fronts();
					$this->postcard->Output("I", $tagname . "-Postcard-Fronts.pdf");
					break;
				case 'start_signin':	
				case 'controle':	
					$this->login_check();
					$event_distance=$event['distance_str'];
					$cue_version = $event['cue_version'];
					$event_name=$this->model_parando->he_decode($event['name']);
					$event_date=$event['start_daymonthyear_str'];
					$tagname=$this->model_parando->make_event_tagname($event);
					$name_suffix=($style=='controle')?'Control':'Start';
					$controle_type=($style=='start_signin')?'start':$style;
					$roster_table = $this->make_roster_table_array($event,$roster,$controle_type);
					$params = [
						'event'=>$event, 
						'this_organization'=>$club_name, 
						'tagname'=>$tagname,
						'name_dist'=>"$event_name $event_distance",
						'controle_type'=>$controle_type,
						'roster'=>$roster, 
						'cue_version'=>$cue_version,
						'roster_table'=>$roster_table
					];
					$this->load->library('signin',$params);
					$this->signin->AddPage();
					$this->signin->render_sheet();
					$this->signin->Output("I", $tagname . "-{$name_suffix}SignIn.pdf");
					break;
				case 'waiver':	
				case 'cwaiver':	
					// $this->login_check();
					$event_region=$club_region;
					$event_distance=$event['distance_str'];
					$event_name=$this->model_parando->he_decode($event['name']);
					
					$event_name=str_replace('è','e',$event_name);
					
					$event_date=$event['start_daymonthyear_str'];
					$map=compact('event_date','event_region','event_name','event_distance');
					$release_data = $this->get_release_array($map);
					$release_title = strip_tags($this->model_parando->get_paragraph('release_title'));
					$tagname=$this->model_parando->make_event_tagname($event);
					$params = [
						'event'=>$event, 
						'tagname'=>$tagname,
						'region_name_dist'=>"$event_region $event_name $event_distance",
						'this_organization'=>$club_name, 
						'release_title'=>$release_title,
						'release_type'=>$style
					];
					$this->load->library('waiver',$params);
					$this->waiver->AddPage();
					$this->waiver->render_release($release_data);
					$this->waiver->Output("I", $tagname . "-Waiver.pdf");
					break;
				case 'results':
					if($event['type']=='fleche'){
						$preliminary=($this->model_parando->event_status_q($event,'locked'))?"Final":"Preliminary";
						$title=$preliminary . " Results for " . $event['name'] . ' ' . $event['distance_str'];
						$this->simple_view('roster_results_fleche', ['roster'],compact('title', 'preliminary', 'event', 'roster', 'this_organization'));
					}else{
						$this->roster_results($event, $roster);
						// $this->load->view('roster_results',array('event'=>$event, 'roster'=>$roster, 'this_organization'=>$this->model_parando->organization));
					}
					break;
// 				case 'qrcheckin':
// 						$this->roster_qrcheckin($event, $roster);
// 					break;
				case 'email':
					$this->login_check();
					$this->load->view('roster_email',array('event'=>$event, 'roster'=>$roster, 'this_organization'=>$club_name));
					break;
				case 'rusa_member_check':
					$this->login_check();
					$this->load->library('rusa');
					$event_cutoff_datetime=$event['cutoff_datetime'];
					foreach ($roster as &$row){
						$result=$this->rusa->rusa_status_at_date($row->rusa_id,$row->last_name,$event_cutoff_datetime);
						$row->rusa_status=$result;
					}

					$this->load->view('roster_rusa_check',array('event'=>$event, 'roster'=>$roster, 'this_organization'=>$club_name));
					break;
				case 'comcsv':
					$this->login_check();
					$filename='roster-'.$event['date'].'-'.str_replace(' ','-',$event['distance_str']).'.csv';
					$com_roster=$this->model_parando->get_com_roster($event_id);
					$this->csv($com_roster,$filename);
					$this->load->view('roster_online',array('event'=>$event, 'roster'=>$roster));
					break;
				case 'rusacsv_quoted':
					$this->login_check();
					$filename='results-'.$event['date'].'-'.str_replace(' ','-',$event['distance_str']).'.csv';
					$rusa_results=$this->model_parando->get_rusa_results($event_id);
					$this->csv($rusa_results,$filename);
					$this->load->view('roster_online',array('event'=>$event, 'roster'=>$roster));
					break;
				case 'rusacsv':
					$this->login_check();
					$filename='results-'.$event['date'].'-'.str_replace(' ','-',$event['distance_str']).'.csv';
					$rusa_results=$this->model_parando->get_rusa_results($event_id);
					$this->csv_noquote($rusa_results,$filename);
					$this->load->view('roster_online',array('event'=>$event, 'roster'=>$roster));
					break;
				case 'fullcsv':
					$this->login_check();
					$filename='full-results-'.$event['date'].'-'.str_replace(' ','-',$event['distance_str']).'.csv';
					$rusa_results=$this->model_parando->get_full_results($event_id);
					$this->csv($rusa_results,$filename);
					$this->load->view('roster_online',array('event'=>$event, 'roster'=>$roster));
					break;
				case 'setnullfinished':
					$this->login_check();
					$q=$this->model_parando->set_null_finished($event_id);
					if(!$q) $this->die_error(__METHOD__, 'Could not set null entries to FINISHED');
					$this->load->view('roster_online',array('event'=>$event, 'roster'=>$roster));
					break;
				default:
			$this->load->view('errors/html/error_general',array('heading'=>'Event Roster','message'=> 'Unknown style.'));
					break;
			}
		}else{
			$this->load->view('errors/html/error_general',array('heading'=>'Event Roster','message'=> 'Invalid event.'));
		}
	}

	private function roster_online($event, $roster){
		$this->show_roster($event, $roster, 'online');
	}

// 	private function roster_qrcheckin($event, $roster){
// 		$this->show_roster($event, $roster, 'qrcheckin');  // UNIMPLEMENTED
// 	}

	private function roster_results($event, $roster){
		$this->show_roster($event, $roster, 'results');
	}	

	private function show_roster($event, $roster, $view_type){

            $club=$event['club'];
            $org=$this_organization=$club['name'];
            $event_id=$event['id'];
			$event_tz=$this->model_parando->get_local_time_zone();
			$checkin_data_url=$this->model_parando->ebrevet_baseurl . "/checkin_status/$event_id";

		$country_fullname=$this->model_parando->get_countries();
		$max_rusa_digits=$this->model_parando->max_rusa_digits;

		extract($event);
		$num_riders=count($roster);
		$ss=($num_riders==1)?'':'s';

		$show_trackers=false;
		$show_results=false;
		$show_checkins=false;
		$ebrevet_results="";

		switch($view_type){
			case 'online':
				$super_title="Roster";
				$title="Registered Riders for $name $distance_str";
				$show_trackers=true;
				$show_results=false;
				$ebrevet_results="<div class='half-width large-print padded bordered text-center'>
					<A HREF='$checkin_data_url'>eBrevet Check In Results</a></div>";
				
				$show_checkins=$this->model_parando->event_checkins_q($event_id);
				$heading=['','Name','Address','RUSA'];
				if($show_checkins){
					$heading[]='Status';
							$this->load->library('rwgps');
							$this->load->library('route_event');
					$num_controls=$this->route_event->control_count($event_id);
				}
				break;
			case 'results':
				$super_title="Results";
				$preliminary=($this->model_parando->event_status_q($event,'locked'))?"Final":"Preliminary";
				$title=$preliminary . " Results for $name $distance_str";
				$heading=['','Name','Address','RUSA','Result','Cert','Elapsed Time','Notes'];
				$show_trackers=false;
				$show_results=true;
				$ebrevet_results="<div class='half-width large-print padded bordered text-center'>
					<A HREF='$checkin_data_url'>eBrevet Check In Results</a></div>";
				$tandem_img="<IMG TITLE='Tandem' SRC='" . base_url('images/tandem.png') . "'>";
				$tandem_ivs="<IMG TITLE='Tandem' STYLE='visibility: hidden;' SRC='" . base_url('images/tandem.png') . "'>";
				$i_started=0; $i_finished=0;
				$min_seconds=null;
				$first_finisher_time=null;
				foreach ($roster as $row)
				{
					if(!empty($row->elapsed_time)){
						list($h,$m,$s)=explode(':',$row->elapsed_time);
						$seconds=3600*$h+60*$m+$s;
						// $seconds = strtotime("1970-01-01 $time UTC");
						if($row->result == 'FINISH'){
							if($min_seconds===null || $seconds<=$min_seconds){
								$min_seconds=$seconds;
								$first_finisher_time=$row->elapsed_time;
							}
						}
					}
				}
				break;
			default:
				$this->die_error(__METHOD__, "Unkown roster view type $view_type");
				break;
		}

		if($show_trackers){
				$marker_personal='<i class="fas fa-map-marker-alt" style="color: #355681;"></i>';
				$marker_club='<i class="fas fa-map-marker-alt" style="color: #ff824c;"></i>';
				$club_tracker=$this->model_parando->get_club_trackers();
		}

		$heading_text="<TR><TH ID=firstcol>" . implode('</TH><TH>', $heading) . "</TH></TR>\n";

		$data=[	'title'=>$title,
				'this_organization'=>$org];
		$data['body']=<<<EOT
<div class=roster-container>
	<h2>$super_title</h2>
	<div class=event>
		<h3>$title</h3>
		Start Date: $start_date_str<br>
		Start Time: $start_time_str
  </div>
$ebrevet_results
	<h3>$num_riders Registered Rider$ss</h3>
	<div class=roster>
      <TABLE WIDTH=100%>
      $heading_text
EOT;


	$i=1;

	foreach ($roster as $row)
	{

		$grayed = ($row->result == "DNS")?"STYLE='color:#d0d0d0'":"";
		$rusa_id=$row->rusa_id;
    $first_last_name=$row->first_name . ' ' . $row->last_name;

		$state=($row->m_country != "US")?($row->m_state . ' (' . $country_fullname[$row->m_country] . ')'):$row->m_state;

		if($show_results){
			$tandem=(!empty($row->tandem))?("$tandem_img with " . $row->tandem):""; 
			if(in_array($row->result,['FINISH','DNF','DNQ'])){$i_started++;}
			if($row->result == "FINISH"){$i_finished++;}
		}

		if($show_trackers){
			$tracker_url=$row->tracker;
			$club_tracker_id=$row->tracker_id;
			if(!empty($club_tracker_id)){
				$url=$club_tracker['url'][$club_tracker_id];		   
				$tag=$club_tracker['tag'][$club_tracker_id];		   
				$tracker_link='<A HREF="' . $url . '" TITLE="Club Satellite Tracker ' . $tag . '">' . $marker_club . "</A>";
			}elseif(!empty($tracker_url)){		   
				$tracker_link="<A HREF=\"$tracker_url\" TITLE='Personal Satellite Tracker'>$marker_personal</A>";
			}else{
				$tracker_link="";
			}
		}else $tracker_link="";

		$rusa_member_search="https://rusa.org/cgi-bin/membersearch_PF.pl";
		$rusa_id_pad=str_repeat('&nbsp;',$max_rusa_digits-strlen("$rusa_id"));
		$rusa_result_link=($row->rusa_id > 0) ? 
			"<A $grayed title='Detailed RUSA results for $first_last_name' HREF=https://rusa.org/cgi-bin/resultsearch_PF.pl?mid=$rusa_id><i $grayed class='fa fa-book'></i></a>":'';
		$rusa_member_link=($row->rusa_id > 0) ? "$rusa_id_pad<A $grayed title='RUSA Membership Info for $first_last_name' HREF=$rusa_member_search?mid=$rusa_id>$rusa_id</a>":'&nbsp;&nbsp;N/A';

		$club_results_link = anchor("results/member/" . $row->member_id, $first_last_name, "title='Detailed $org results for $first_last_name' $grayed");
		

		$data['body'] .= "<TR $grayed>";
		$data['body'] .= '<TD ID=firstcol>' . $tracker_link . '</TD>';
		$data['body'] .= '<TD>' . $club_results_link . '</TD>';
		$data['body'] .= '<TD>' . $row->m_city . ', ' . $state . '</TD>';
		$data['body'] .= "<TD ID=tt>$rusa_member_link</TD>"; // . '&nbsp;' . $rusa_result_link . '</TD>';

		if($show_results){
			$data['body'] .=  '<TD style="text-align:center">' . $row->result . '</TD>';
			$data['body'] .=  '<TD style="text-align:center">' . $row->cert . '</TD>';
//			$data['body'] .=  '<TD style="text-align:center">' . $row->elapsed_time . '</TD>';

				if(!empty($row->elapsed_time)){
					$etl=explode(':',$row->elapsed_time);
					if(count($etl)>=2){
						list($hr,$min,$sec)=$etl; 
						$et_text=sprintf('%02d:%02d',$hr,$min);
					}else $et_text="";
				}else{
					if($row->result == "FINISH")
						$et_text="Under Review";
					else
						$et_text="";
				}

				$data['body'].='<TD style="text-align:center">' . $et_text . '</TD>'; 


			if(!empty($row->elapsed_time) && 
				$first_finisher_time == $row->elapsed_time && 
				(strpos(strtolower($row->comment), 'first finish') === false)){
												$ff_text="First Finisher. ";
			} else $ff_text='';
			
				$data['body'] .=  '<TD>' . $ff_text . $row->comment . ' ' . $tandem . '</TD>';
		}
		
		if($show_checkins){
			$checkin_data=$this->model_parando->get_last_checkin($event_id,$rusa_id);
			if(null===$checkin_data){
				$checkin_text='';
			}else{
				$control_number=$checkin_data['control_number'];
				if($control_number==1) 
					$control_text='START';
				elseif($control_number==$num_controls) 
					$control_text='FINISH';
				else
					$control_text="C$control_number";
				$checkin_datetime=$checkin_data['checkin_datetime'];
				$checkin_datetime->setTimezone($event_tz);
				$checkin_time_text=$checkin_datetime->format('H:i');
				$checkin_text="$control_text @ $checkin_time_text";
			}
			$data['body'] .= "<TD>$checkin_text</TD>"; 
		}


		$data['body'] .= "</TR>\n";
		$i++;
	}

  $data['body'] .= "</TABLE></div>";

	if($show_results){
      $s=(count($roster)==1)?'':'s';
      $data['body'] .= "<h5>$i_started started; $i_finished finished";
      if($i_started>0) $data['body'] .= " (" . round(100*$i_finished/$i_started) . "%)";
      $data['body'] .= "</h5><div class=bigskip></div>";
	}

  $data['body'] .= "</div>";


		$this->load->view('simple_header',array_merge($data,['page_styles'=>['roster']]));
		$this->load->view('simple_body',$data);
		$this->load->view('simple_footer',$data);
	}

	private function make_roster_table($roster){

		$this->load->model('accounting_parando');
		$i=1;
		$roster_table_body="";
		$club_tracker=$this->model_parando->get_club_trackers();
			foreach ($roster as $row)
			{
				$tracker_url=$row->tracker;
				$club_tracker_id=$row->tracker_id;
				if(!empty($club_tracker_id)){
					$tag=$club_tracker['tag'][$club_tracker_id];		   
					$tracker_link=$tag;
				}elseif(!empty($tracker_url)){		   
					$tracker_link="per";
				}else{
					$tracker_link="";
				}
				$full_name =  $row->first_name . ' ' . $row->last_name;
				$member_id = $row->member_id;
				$event_id = $row->event_id;
				// $lodging_nights=$row->lodging_nights;
				// $lodging=($lodging_nights!='none')?"<br>$lodging_nights":"";
				$expires=$row->membership . ' ' . (($row->expires != "0000-00-00")?$row->expires:"");
				$total_fee=$row->total_fee; // ($row->total_fee == 0)?"-":("$".$row->total_fee);
				$opd=$this->accounting_parando->event_total_order_paid($event_id,$member_id);
				$total_paid=$opd['total_paid'];
				$pay_by_mail = (false !== strpos($opd['point_of_sale'], 'pay_mail'));

				if($total_fee==0){
					$total_due="-";
				}elseif($total_fee == $total_paid){
					$total_due="Paid";
				}elseif($pay_by_mail){
					$total_due=('Mail Payment Overdue $' . (string)($total_fee-$total_paid));
				}else{
					$total_due=('$' . (string)($total_fee-$total_paid));
				}

				$dns=(!empty($row->result))?$row->result:"";
				if($dns=="FINISH")$dns="PRE RIDE";
				
				$roster_table_body .= '<TR>';
				$roster_table_body .= '<TD>' . $i . '</TD>';
				$roster_table_body .= '<TD>' . $tracker_link . '</TD>';
				$roster_table_body .= '<TD id="fullname">' . $full_name . $fleche_team . '</TD>';
				$roster_table_body .= '<TD>' . $total_due . '</TD>';
				$roster_table_body .= '<TD>' . $expires . '</TD>';
				$roster_table_body .= '<TD id="dns">' . $dns . '</TD>';
				$roster_table_body .= '<TD>' . "" . '</TD>';
				$roster_table_body .= '</TR>';
				$i++;
			}

		$roster_table=<<<EOT
<div class=roster>
	<TABLE BORDER WIDTH=100%>
		<TR>
			<TH WIDTH=3%>#</TH>
			<TH WIDTH=4%>TRK</TH>
			<TH WIDTH=25%>Name</TH>
			<TH WIDTH=4%>Fee</TH>
			<TH WIDTH=10%>Member</TH>
			<TH>Sign</TH>
			<TH WIDTH=8%>Date</TH>
		</TR>
		$roster_table_body
	</TABLE>
</div>
EOT;

		return $roster_table;

	}
	
	private function make_roster_table_array($event,$roster,$type='start'){

		$this->load->model('accounting_parando');
		
		$i=1;
		$roster_table_array=[];
		$lodger=[];

		$club_tracker=$this->model_parando->get_club_trackers();
			foreach ($roster as $row)
			{
				$tracker_url=$row->tracker;
				$club_tracker_id=$row->tracker_id;
				if(!empty($club_tracker_id)){
					$tag=$club_tracker['tag'][$club_tracker_id];		   
					$tracker_link=$tag;
				}elseif(!empty($tracker_url)){		   
					$tracker_link="per";
				}else{
					$tracker_link="";
				}
				$full_name =  $row->first_name . ' ' . $row->last_name;
				$member_id = $row->member_id;
				$event_id = $row->event_id;
				
				$rusa_id=$row->rusa_id;
				

				
				$start_code=$this->model_parando->make_start_code($event,$rusa_id);
				
				if(empty($lodger)){
                    $lfe=$this->model_parando->get_lodging_for_event($event_id);
                    foreach($lfe as $ld){
                        $lodger[$ld['member_id']]=$ld;
                    }
				}
				if(array_key_exists($member_id,$lodger)){
                    $l=$lodger[$member_id];
                    $accomodation=$l['value'];
                    if($accomodation=='No lodging needed') $accomodation="";
                    $price=$l['total_price'];
				}else{
                    $accomodation=$price="";
                }
				$expires=$row->membership; // . ' ' . (($row->expires != "0000-00-00")?$row->expires:"");
				$total_fee=$row->total_fee; // ($row->total_fee == 0)?"-":("$".$row->total_fee);
				$opd=$this->accounting_parando->event_total_order_paid($event_id,$member_id);
				$total_paid=$opd['total_paid'];
				$pay_by_mail = (false !== strpos($opd['point_of_sale'], 'pay_mail'));

				if($total_fee==0){
					$total_due="-";
				}elseif($total_fee == $total_paid){
					$total_due="Paid";
				}elseif($pay_by_mail){
					$total_due=('Mail Payment Overdue $' . (string)($total_fee-$total_paid));
				}elseif($total_fee>$total_paid){
					$total_due=('$' . (string)($total_fee-$total_paid));
				}else{
					$total_due=('$' . (string)($total_paid-$total_fee). 'CR');
				}

				$dns=(!empty($row->result))?$row->result:"";
				if($dns=="FINISH")$dns="FINISH: " . $row->elapsed_time;


				$phonelist=[$row->m_phone,$row->m_phone_alt];
				$ephonelist=[$row->emergency_phone,$row->emergency_phone_alt];
		

				$ename=$row->emergency_name;		
				$phone=$this->goodphones($phonelist);
				$ephone=$this->goodphones($ephonelist);

				if(!empty($row->name))
					$fleche_team=$row->name;
				else
					$fleche_team="";

		

				switch($type){
					case 'start':
						$roster_table_array[] = [
							[ 'text'=>"$i" ],
							[ 'text'=>strtoupper($tracker_link) ],
							[ 'text'=>"$full_name", 'align'=>'L', 'style'=>'fit' ],
							// [ 'text'=>"$total_due" ],
							[ 'text'=>"$start_code" ],
							[ 'text'=>"$accomodation", 'style'=>'fit' ],
							[ 'text'=>"$dns" ],
							[ 'text'=>"" ]
						];
						break;
					default:
						$roster_table_array[] = [
							[ 'text'=>"$i" ],
							[ 'text'=>strtoupper($tracker_link) ],
							[ 'text'=>"$full_name", 'align'=>'L', 'style'=>'fit' ],
							[ 'text'=>"" ],
							[ 'text'=>"$dns" ],
							[ 'text'=>"$phone", 'style'=>'multi' ],
							[ 'text'=>"$ename\n$ephone", 'style'=>'multi','fontsize'=>0.75 ],
							[ 'text'=>$fleche_team, 'style'=> 'hidden','fleche_team'=>true]
						];
						break;
					}
				$i++;
			}


		return $roster_table_array;

	}

	private function goodphones($inlist){
		$goodlist=[];
		foreach($inlist as $p){
			if(preg_match_all('/\d/',$p)>9) $goodlist[]=$p;
		}
		return implode("\n",$goodlist);
		
	}

	private function csv($query,$filename='data.csv')
	{
		$this->load->dbutil();
		$this->load->helper('file');
		$this->load->helper('download');
		$delimiter = ",";
		$newline = "\r\n";
		$data = $this->dbutil->csv_from_result($query, $delimiter, $newline);
		force_download($filename, $data);
	}

	private function csv_noquote($query,$filename='data.csv')
	{
		$this->load->dbutil();
		$this->load->helper('file');
		$this->load->helper('download');
		$delimiter = ",";
		$newline = "\r\n";
                $enclosure = "";
		$data = $this->dbutil->csv_from_result($query, $delimiter, $newline, $enclosure);
		force_download($filename, $data);
	}



	public function import_results($event_id,$filename){
		$data = @file($filename);
		if($data === false){
			$this->load->view('errors/html/error_general',array('heading'=>'Results Import','message'=> "Failed to open file: $filename"));
		}else{
			$csv = array_map('str_getcsv', $data);
			array_walk($csv, function(&$a) use ($csv) {
				$a = array_combine($csv[0], $a);
			});
			array_shift($csv); 

			// Find each rider
			$not_found="";
			$i=0;
			foreach($csv as $r){
				$mem=$this->model_parando->get_member_by_numname($r['#RUSA#'], $r['FirstName'], $r['LastName']);
				if($mem=="not_found"){
					$not_found .= $r['#RUSA#'].', '. $r['FirstName'].', '.  $r['LastName']."<br>";
				}else{
					$csv[$i] = array_merge($csv[$i],$mem);
				}
				$i++;
			}
			$nr=count($csv);

			if(!empty($not_found)){
				$this->load->view('errors/html/error_general',array('title'=>'Results Import', 'heading'=>'Results Import Failed','message'=> "Not found:<br> $not_found"));
			}else{
				$nr=0;
				foreach($csv as $r){
					$new_result=array(
						'event_id'=>$event_id, 
						'member_id'=>$r['id'], 
						'result'=>(($r['DNF'])?'DNF':'FINISH'),
						'elapsed_time'=>sprintf("%02d:%02d",$r['Hours'],$r['Minutes'])
					);
					$this->db->insert('roster', $new_result);
					$nr++;
				}
				$this->load->view('errors/html/info_general',array('heading'=>'Results Import','message'=> "Successfully imported $nr results"));
			}
			
		}
	}


}

?>
