<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.


class Route extends MY_Controller {

	private $controller_name;

	function __construct(){
		parent::__construct();

		$this->load->library('rwgps');
		$this->controller_name=strtolower(get_class($this));
	}

// PUBLIC CONTROLLER FUNCTIONS

private function get_event_and_route($event_id){
		// Get the event
    		$event=$this->model_parando->get_event($event_id);
		if(empty($event['map'])) $this->die_error(__METHOD__,"No event, or no map URL set for event $event_id");
		
		// Get the route
		$route_id=$this->rwgps->extract_route_id($event['map']);
		if($route_id===null) $this->die_error(__METHOD__,"Must have valid RWGPS route. Malformed map url: '$url'");
		$route=$this->rwgps->get_route($route_id);
		
		return [$event, $route];
}
	
  // Official PA Rando Event Cue Wizard
	public function event($event_id, $view=null){
		
		list($event, $route) = $this->get_event_and_route($event_id);

		// Combine the two
		list($controles, $controle_warnings) = $this->rwgps->extract_controles($route);
		list($cues, $cue_warnings) = $this->rwgps->extract_cues($route);
		list($route_event, $re_warnings) = $this->merge_route_event_data($route,$event);
		$warnings=array_merge($controle_warnings, $re_warnings,$cue_warnings);
		
		// Date/Time of event must be completely valid before computing control times
		
		if(empty($route_event['event_date']) || $route_event['event_date']==$this->default_event_date){
			$route_event['event_date']=$event_date=$this->default_event_date;
			$warnings[]="Event date not set. Use tag #date=YYYY-MM-DD to set the start date. Default is " . $this->default_event_date;
		}
		
		if(empty($route_event['event_time']) || $route_event['event_time']==$this->default_event_time){
			$route_event['event_time']=$event_time=$this->default_event_time;
			$warnings[]="Event time not set. Use tag #time=HH:MM to set the start time.";
		}
		
		if(empty($route_event['event_distance'])){ 
			$route_event['event_distance']=$event_distance=ceil($route_event['distance_km']);
			$warnings[]="Official distance not specified. Assuming $event_distance K";
		}

		$this->load->library('controletimes');
		$route_event['event_datetime'] = $event_datetime = $this->controletimes->make_event_datetime($route_event);
		
		$dtz= "D=" . $route_event['event_date'] . "; T=" . $route_event['event_time'] . "; Z=" . print_r($route_event['event_tz'],true);

		if(false == $event_datetime) $this->die_error(__METHOD__,"Invalid or missing date, time, or timezone. Please use the format YYYY-MM-DD for dates, and HH:MM:SS for time, and specify a timezone. DTZ=$dtz");
		
		$route_event['event_datetime_str'] = $event_datetime_str = $event_datetime->format($this->event_datetime_format);
		$route_event['event_datetime_str_verbose'] = $event_datetime_str_verbose = $event_datetime->format($this->event_datetime_format_verbose);

		// Now we can SAFELY compute the control times
		$this->controletimes->compute_open_close($controles,$route_event);

		// And view the result
		
		// SOME views require admin or organizer of THIS event   TODO: MAKE THIS MORE FINE GRAINED;  MULTIPLE ORGANIZERS
		$params = ['tag' => 'login_session_data'];
		$this->load->library('randosession',$params);
		$login_member_id=$this->randosession->login_check();  // only allowed after login
		if($event['organizer']!=$login_member_id) $this->randosession->admin_check();  // and only allowed if organizer

		if($view===null)$view=$this->input->post('view');

		if(empty($view)){
			$this->view_html($route_event, $controles, $warnings);
		}else{
			$this->{'view_'.$view}($route_event, $controles, $warnings);
		}
	}



  // General PA Rando Event Route Information page (linked from event page) 
  //
  // TODO!!! SHOULD USE or be refactored with $this->view_html()
  
	public function info($event_id){
		$event=$this->model_parando->get_event($event_id);
		if(empty($event)) $this->die_error(__METHOD__,"Invalid event. Can't get info.");
		$event_name_dist=$event['name'] . ' ' . $this->model_parando->make_distance($event);
		
		$map_url=(isset($event['map']))?$event['map']:'';
		$rusa_route=(isset($event['rusa_route']))?$event['rusa_route']:'';
		$route_id=$this->rwgps->extract_route_id($map_url);
		if($route_id!==null){
			$rwgps_url=$map_url;
			$has_rwgps_route=true;
		}else{
			$rwgps_url=null;
			$map_url=make_site_url($map_url);
			$has_rwgps_route=false;
		}


		//	$cue_version=(empty($event['cue_version']))?"None Published":$event['cue_version'];
		//	$cue_version_str="<TR><TD><b>Cuesheet Version</b></TD><TD>$cue_version</TD></TR>";

		if(!empty($event['cuesheet'])){
			$cuesheet_url=make_site_url($event['cuesheet']);
			$has_cuesheet=true; $cuesheet_location='url'; 
		}else	if($has_rwgps_route && isset($event['cue_version']) && $event['cue_version']>0){
			$has_cuesheet=true; $cuesheet_location='auto'; $cuesheet_url=null;
		}else{
			$has_cuesheet=false; $cuesheet_location=null; $cuesheet_url=null;
		}


		$left_column="<div class='narrower'><H2>GPS/Cuesheet Data</H2></DIV>";
		$buttons="";
		
		// General Info About Event
		$schedule=$this->model_parando->start_schedule($event);
		$event_datetime_str=$schedule['start_datetime_str'] . ' ' . $schedule['start_time_zone'];
		$event_type=$event['sanction'] . ' ' . $event['type'];
		$event_distance=$event['distance'];
		$left_column.=<<<EOT
<H3>$event_name_dist</H3>
<H4>GENERAL INFO</h4>
<div class='narrower ftable'><TABLE WIDTH=100%>
<TR><TD WIDTH=31.6%><b>Start Date/Time</b></TD><TD>$event_datetime_str</TD></TR>
<TR><TD WIDTH=31.6%><b>Event Type</b></TD><TD>$event_type</TD></TR>
<TR><TD WIDTH=31.6%><b>Official Distance</b></TD><TD>$event_distance km</TD></TR>
</TABLE></DIV>
EOT;

		// Disclaimer Paragraph
		$left_column.="<DIV CLASS='narrower bordered'><p>";
		$left_column.=$this->model_parando->get_paragraph("auto_cue");
		$left_column.="</p></div>";

		// Information about the route
		// RWGPS Routes
		if($has_rwgps_route){
			$route=$this->rwgps->get_route($route_id);
			$route_name=$route['name'];
			$event_tzname=$this->model_parando->tzname; // BUG! Will fail if PA Rando ever has events outside default TZ
			date_default_timezone_set($event_tzname);
			$last_update=date("Y-m-j H:i:s T",$route['updated_at']); 
			$last_download=date("Y-m-j H:i:s T",$route['downloaded_at']); 
			$distance_km=round($route['distance']/rwgps::m_per_km,1);
			$distance_mi=round($route['distance']/rwgps::m_per_km/rwgps::km_per_mi,1);
			$climbing_ft=round($route['elevation_gain']*rwgps::ft_per_m,1);
			if(!empty($route['description']))
				$ed=$this->rwgps->parse_description($route['description'],$this->rwgps->valid_event_description_keys);
			else 
				$ed=[];
				
			if(!empty($ed['pavement_type'])){
				$pavement_type=$ed['pavement_type'];
				$pt="<TR><TD WIDTH=31.6%><b>Pavement Type</b></TD><TD>$pavement_type</TD></TR>";
			}else{
				$pt="<TR><TD WIDTH=31.6%><b>Pavement Type</b></TD><TD>Unknown</TD></TR>";
				//$pt="";
			}
				

			$df_links=[];
			foreach($route['route_datafile'] as $ext=>$fn){
				$uc_ext=strtoupper($ext);
				$base_fn=basename($fn);
				$df_links[]="<A TITLE='$uc_ext file download' HREF='" . $route['saved_route_url'][$ext] . "'>$uc_ext</A>"; 
			}
			$data_file_row="<TR><TD><b>Route File Download</b></TD><TD>" . implode(', ', $df_links) . "</TD></TR>";
			
			$left_column.=<<<EOT
<H4>GPS ROUTE DATA</h4>
<div class='narrower ftable'><TABLE WIDTH=100%>
<TR><TD WIDTH=31.6%><b>Route Name</b></TD><TD>$route_name</TD></TR>
<TR><TD WIDTH=31.6%><b>RWGPS URL</b></TD><TD><A HREF=$rwgps_url>$rwgps_url</a></TD></TR>
$data_file_row
<TR><TD WIDTH=31.6%><b>Last Modified</b></TD><TD>$last_update</TD></TR>
<TR><TD WIDTH=31.6%><b>Distance</b></TD><TD>$distance_mi mi / $distance_km km</TD></TR>
<TR><TD WIDTH=31.6%><b>Official Distance</b></TD><TD>$event_distance km</TD></TR>
<TR><TD WIDTH=31.6%><b>Climbing</b></TD><TD>$climbing_ft ft</TD></TR>
$pt
</TABLE></DIV>
EOT;

//<TR><TD><b>Controles Found</b></TD><TD>$ncontroles</TD></TR>

		//	$buttons.="<A HREF=$rwgps_url>Open in RWGPS</A>";
		//	$buttons.="<A HREF='" . $route['saved_route_url']['gpx'] . "' TITLE='GPX File Download'>GPX File</A>";

		// Static Non-RWGPS Map Link
		}elseif(!empty($map_url)){
			$left_column.=<<<EOT
<H4>MAP/PROFILE</h4>
<p>A static (non-GPS) Map and Profile is available for this event.</P>
<div class='narrower ftable'><TABLE WIDTH=100%>
<TR><TD WIDTH=31.6%><b>Map Profile Link</b></TD><TD><A HREF=$map_url>$map_url</A></TD></TR>
</TABLE></DIV>
EOT;
			$map_button="<A HREF=$map_url>Download Map and Profile Info</A>";
		$left_column.="<div class='button-container text-center'>$map_button</DIV>";

		// No Map
		}else{
			$left_column.=<<<EOT
<H4>MAP/PROFILE</h4>
<p>At this time no map and profile information is available for this event.</p>
EOT;
		}
		
		// Course History
		$course_history=$this->model_parando->event_course_history($event);
  	if(!empty($course_history) && !empty($course_history['events'])){
		$events=$course_history['events'];
		$left_column.="<H4>COURSE HISTORY</H4>";

		$this->load->library('rusa');
		$rusa_route_url=$this->rusa->route_url($rusa_route);
		$left_column.=<<<EOT
<p>The following events have been run on this course (RUSA Route ID <A HREF='$rusa_route_url'>$rusa_route</a>).</p>
EOT;

		$left_column.="<div class=half-width><TABLE class='course-history' width=100%><TR><TH WIDTH=31.6%>Date</TH><TH>Finishers</TH><TR>";
		foreach($events as $e){
			$date=$e['date']; $nf=$e['num_finishers']; $eid=$e['id'];
			$this_date=($date==$event['date']);
			$highlight_style=($this_date)?"class=row-highlight":"";
			$results_link=anchor($this->model_parando->event_results_url . "/$eid", $nf, 'TITLE="Detailed Results"');
			// $mapcue_link=anchor($this->model_parando->route_info_url . "/$eid", "<i class='fas fa-map'></i>", 'TITLE="Map/Cuesheet"');
			$left_column.="<TR $highlight_style><TD>$date</TD><TD>$results_link</TD></TR>";
		}
		$left_column.="</TABLE>";
		$cr=$course_history['course_record'];
		$crd=$cr['date']; $crn=$cr['names']; $crt=$cr['elapsed_time']; $crs=$cr['speed'];
		$left_column.="<div class='fine-print'><b>Course Record</b> $crd: $crn ($crt, $crs kph)</div>";
		$left_column.="<DIV class=vspace></div></DIV>";
}

		// Information about CueSheet
		// No Cues
		if($has_cuesheet===false){
			$left_column.=<<<EOT
<H4>CURRENT CUE SHEET</h4>
<p>At this time no cue sheet is available for this event.</p>
EOT;
		// Cue Wizard Cuesheet
		}elseif($cuesheet_location=='auto'){
			$event_tagname=$this->model_parando->make_event_tagname($event);
			$cue_version=$event['cue_version'];
			$cue_basename="$event_tagname-CueSheetV$cue_version";
			$csv_filename=$this->model_parando->cuesheet_path . "/" . $cue_basename . ".csv";
		$event_tzname=$this->model_parando->tzname; // BUG! Will fail if PA Rando ever has events outside default TZ
		date_default_timezone_set($event_tzname);
			$cuesheet_gentime_str=date("Y-m-j H:i:s T",filectime($csv_filename));
			
			$cuesheet_url_P=$this->model_parando->cuesheet_baseurl . "/" . $cue_basename . "-P.pdf";
			$cuesheet_url_L=$this->model_parando->cuesheet_baseurl . "/" . $cue_basename . "-L.pdf";
			$cuesheet_url_C=$this->model_parando->cuesheet_baseurl . "/" . $cue_basename . ".csv";
			$left_column.=<<<EOT
<H4>CURRENT CUE SHEET</h4>
<div class='narrower ftable'><TABLE WIDTH=100%>
<TR><TD WIDTH=31.6%><b>Cuesheet Generated</b></TD><TD>$cuesheet_gentime_str</TD></TR>
<TR><TD WIDTH=31.6%><b>Cuesheet Version</b></TD><TD>$cue_version</TD></TR>
</TABLE></DIV>
EOT;
			$buttons.="<A HREF=$cuesheet_url_P>Cue Sheet (Portrait)</A>";
			$buttons.="<A HREF=$cuesheet_url_L>Cue Sheet (Landscape)</A>";
			$buttons.="<A HREF=$cuesheet_url_C>Cue Sheet (CSV)</A>";

		// Static cue link
		}else{
			$left_column.=<<<EOT
<H4>CURRENT CUE SHEET</h4>
<p>A legacy Cuesheet is available for this event.
<div class='narrower ftable'><TABLE WIDTH=100%>
<TR><TD WIDTH=31.6%><b>Cuesheet</b></TD><TD><A HREF=$cuesheet_url>$cuesheet_url</a></TD></TR>
</TABLE></DIV>
EOT;
			$buttons.="<A HREF=$cuesheet_url>Download Cue Sheet</A>";
		}

		// Cue Sheet Buttons

		$left_column.="<div class='button-container text-center'>$buttons</DIV>";

		if($has_rwgps_route){
			$map_divid='randomap';
			$graph_divid='eprofile';
		list($controles, $controle_warnings) = $this->rwgps->extract_controles($route);
			$map_script=$this->generate_map_script($route,$controles,$map_divid);
			$ep_script=$this->generate_ep_script($route, $controles, $graph_divid);
			$left_column.=<<<EOT
<H4>MAP</H4>
<div class='narrower text-center'>
<div id=$map_divid></div>
<A HREF=$rwgps_url>View in Ride With GPS</a>
<div class=bigskip></div>
<div id=$graph_divid style="margin-left:10%; margin-right:10%; width:80%; height:180px;"></div>
</div>
$map_script
$ep_script
<div class=bigskip></div>
EOT;

		}

		$data=['title'=>$event_name_dist, 'left_column'=>$left_column, 'right_column'=>''];
		$this->simple_view('info_one_column',['randomap','table','buttonlink'],$data);
		
	}





    // Re-copy route to local cache
	public function recache($route_id,$event_id){
		// $this->login_check(); $this->admin_check();
		if($this->rwgps->is_good_route_id($route_id)){
			$result=$this->rwgps->download_route_data($route_id);
			if($result!==true)
				$this->die_error(__METHOD__, $result);
			else
				redirect(site_url($this->controller_name . "/event/$event_id"));
//			if(is_numeric($event_id) && $event_id > 0){
//				redirect(site_url($this->controller_name . "/event/$event_id"));
//			}else{
//				redirect(site_url($this->controller_name . "/id/$route_id"));
//			}
		}else{
			$this->die_error(__METHOD__,"Invalid parameters.");
		}
	}

    // Publish cues and cards for PA Rando Event
	public function publish_cuesheet($event_id){

		$event=$this->model_parando->get_event($event_id);
		if(empty($event['map'])) $this->die_error(__METHOD__,"No event, or no map URL set for event $event_id");
		$url=$event['map'];
		$route_id=$this->rwgps->extract_route_id($url);
		if($route_id===null) $this->die_error(__METHOD__,"Malformed map url: '$url'");
		$route=$this->rwgps->get_route($route_id);
		
		// SOME views require admin or organizer of THIS event
		$params = ['tag' => 'login_session_data'];
		$this->load->library('randosession',$params);
		$login_member_id=$this->randosession->login_check();  // only allowed after login
		if($event['organizer']!=$login_member_id) $this->randosession->admin_check();  // and only allowed if organizer
		
		list($controles, $controle_warnings) = $this->rwgps->extract_controles($route);
		list($cues, $cue_warnings) = $this->rwgps->extract_cues($route);
		list($route_event, $re_warnings) = $this->merge_route_event_data($route,$event);
		$warnings=array_merge($controle_warnings, $re_warnings,$cue_warnings);
		$this->die_view_warnings($warnings);
		$this->load->library('controletimes');
		$route_event['event_datetime'] = $event_datetime = $this->controletimes->make_event_datetime($route_event);

		if(false == $event_datetime) $this->die_error(__METHOD__,"Invalid or missing event date, time or timezone.");
		
		$route_event['event_datetime_str'] = $event_datetime_str = $event_datetime->format($this->event_datetime_format);
		$route_event['event_datetime_str_verbose'] = $event_datetime_str_verbose = $event_datetime->format($this->event_datetime_format_verbose);
		$this->controletimes->compute_open_close($controles,$route_event);

		
		$size='letter';
		$orientation='P';
		$params=['event'=>$route_event, 'orientation'=>$orientation, 'size'=>$size];
		$this->load->library('cuesheet', $params);  

		$route_event['cue_version']=$cue_version=1+$route_event['cue_version'];	
		
		$orientation='P';
		$cue_data_P=$this->generate_pdf_cuesheet($route_event, $cues, $controles, $orientation, $size);

		$orientation='L';
		$cue_data_L=$this->generate_pdf_cuesheet($route_event, $cues, $controles, $orientation, $size);

		$cue_data_C=$this->generate_csv_cuesheet($route_event, $cues, $controles);

		
		$this->model_parando->set_event_cuesheet_version($event_id,$cue_version);

		$event_name_dist=$route_event['event_name_dist'];
		$cue_url_L=$cue_data_L['cue_url'];
		$cue_url_P=$cue_data_P['cue_url'];
		$cue_url_C=$cue_data_C['cue_url'];
		$route_url=site_url(strtolower(get_class($this))."/event/$event_id");
		$event_url=site_url("info/event/$event_id");
		
		$left_column=<<<EOT
<H3>Cue Sheet Published</H3>
<p>Cue Sheets (version $cue_version) successfully published to event database 
for $event_name_dist.</p>
<ul>
<li><A HREF="$cue_url_L">PDF File Landscape</A>
<li><A HREF="$cue_url_P">PDF File Portrait</A>
<li><A HREF="$cue_url_C">Unformatted CSV File</A>
</ul>
<div class="button-container text-center">
<A HREF=$route_url>Route</A>
<A HREF=$event_url>Event</A>
</div>
EOT;

		$data=['title'=>"Cue Sheet Published", 'left_column'=>$left_column, 'right_column'=>''];
		$this->simple_view('info_one_column',['buttonlink'],$data);

	}

	// Simple map view of event

	public function map($event_id,$provider='mapbox'){
			
		list($event, $route) = $this->get_event_and_route($event_id);

		$org=$this_organization=$this->model_parando->organization;
		$org_abbrev=$this->model_parando->org_abbrev;

		list($controles, $controle_warnings) = $this->rwgps->extract_controles($route);

		$name=$event['name']; $route_id=$route['id'];
		$title = "$name (RWGPS #$route_id)";
		$data=[	'title'=>$title,
				'this_organization'=>$org];

		$map_divid='randomap';
		$graph_divid='eprofile';
		$map_script=$this->generate_map_script($route, $controles, $map_divid,'randomapv',$provider);
		$ep_script=$this->generate_ep_script($route, $controles, $graph_divid);

		$data['body']=<<<EOT
<div class='narrower'>
	<h1>$title</h1>
	<div id=$map_divid></div>
	<div class=bigskip></div>
	<div id=$graph_divid style="margin-left:10%; margin-right:10%; width:80%; height:180px;"></div>
	<div class=bigskip></div>
</div>

$map_script
$ep_script
EOT;

		$this->load->view('simple_header',array_merge($data,['page_styles'=>['randomap']]));
		$this->load->view('simple_body',$data);
		$this->load->view('simple_footer',$data);
	}


  // Generic Non-Event Cue Wizard DEPRECIATED
	public function id($route_id=0, $view=null){
		
	$this->die_error(__METHOD__,"This route/id interface to cue wizard is currently disabled and it's future availability is uncertain. <A HREF='https://parando.org/info/contact/webmaster'>Contact the webmaster</a> for more info.");
	
//		if($route_id+0 == 0) $this->die_error(__METHOD__, "Missing or invalid RWGPS route ID");
//		$event=null;
//		$view=$this->input->post('view');
//		$this->process_route_event($route_id,$event,$view);

	}

// PRIVATE FUNCTIONS


	// MAP MAP MAP MAP MAP MAP MAP MAP MAP MAP MAP MAP MAP MAP MAP MAP MAP 

	// Elevation profile graph
	private function generate_ep_script($route, $controles, $graph_divid='eprofile', $epvar='epvar'){
		$csv_data=[];
		foreach($route['track_points'] as $p){
			$d=$p['d']; 
			$cd_mi=round($d/(rwgps::m_per_km*rwgps::km_per_mi),3);
			$ce_ft=$p['e']*(rwgps::ft_per_m);
			$csv_data[] = "[$cd_mi,$ce_ft]";
		}
		$csv_text='['. implode(',' , $csv_data) .']';

		$ep_an=[];
		foreach($controles as $i=>$c){
			$d=$c['d'];
			$cd_mi=round($d/(rwgps::m_per_km*rwgps::km_per_mi),3);
			$controle_num=$i+1;
			$pu_text=implode('; ', $this->make_pu_array($controle_num,$c));
			$ep_an[]="{series: \"Elev\", x:$cd_mi, shortText:\"$controle_num\", text:\"$pu_text\", height: 18, width: 18, cssClass: \"elevation-annotation\"}";
		}
		$ep_an_func = "$epvar.setAnnotations([" . implode(',',$ep_an) . "]);" ;

		$ep_script=<<<EOT
<script type="text/javascript"
  src="https://parando.org/assets/js/dygraph.min.js"></script>
<script type="text/javascript">
$epvar = new Dygraph(document.getElementById("$graph_divid"),$csv_text,
  {labels: [ "Dist", "Elev" ], ylabel: "Elevation (ft)", xlabel: "Distance (mi)", colors: ["#355681"]});
$epvar.ready(function() { $ep_an_func });
</script>
EOT;

		return $ep_script;

	}

	// Pop-Up information for controles available on map
	private function make_pu_array($controle_num,$c){
			$pu_arr=[];

			$cd_mi=round($c['d']/(rwgps::m_per_km*rwgps::km_per_mi),1);
			$cd_km=round($c['d']/(rwgps::m_per_km),1);
			$ce_ft=round($c['e']*(rwgps::ft_per_m),1);
			$is_sf=(isset($c['start']))?" [START]":((isset($c['finish']))?" [FINISH]":"");
			$pu_arr[] ="Controle $controle_num$is_sf";
			if(isset($c['attributes'])){
				$at=$c['attributes'];
				$pu_arr[] = isset($at['name'])?'Name: ' . $at['name']:'';
				$pu_arr[] = isset($at['style'])?'Style: ' . $at['style']:'';
			}
			$pu_arr[] ="Dist: $cd_mi mi ($cd_km km)";
			$pu_arr[] ="Elev: $ce_ft ft";
			return $pu_arr;
	}

	// Generates javascript to create Leaflet map in specified div id. 
	private function generate_map_script($route,$controles,$divid='randomap',$mapvar='randomapv',$provider='mapbox'){

		$markers="";
		foreach($controles as $i=>$c){
			$lat=$c['y']; $lng=$c['x']; 

			$controle_num=$i+1;
			$pos=isset($c['finish'])?'-28px':'0px';
			$controle_num_text="<div style=\"position:relative; top:$pos;\">$controle_num</div>";
			$markers.="var icon$i = L.divIcon({className: 'controle-icon', html: '$controle_num_text'});\n";

			$markers.="var marker$i = L.marker([$lat, $lng],{icon: icon$i, title: 'Controle $controle_num (click for info)'}).addTo($mapvar);\n";

			$pu_text=implode('<br>', $this->make_pu_array($controle_num,$c));
			$markers.="marker$i.bindPopup(\"$pu_text\");\n";
		}

		$track="";
		$comma=false;
		if(isset($route['track_points'])){
			$track="var polygon = L.polyline([";
			foreach($route['track_points'] as $p){
				if($comma) $track.=","; $comma=true;
				$lat=$p['y']; $lng=$p['x'];
				$track.="[$lat,$lng]";
			}
			$track.="],{fill: false}).addTo($mapvar);";
		}
		
		$ul=$route['bounding_box'][0];
		$ul_lat=$ul['lat'];
		$ul_lng=$ul['lng'];
		$lr=$route['bounding_box'][1];
		$lr_lat=$lr['lat'];
		$lr_lng=$lr['lng'];


		switch($provider){
			case 'mapbox':
				$script=<<<EOT
 <script src="https://parando.org/assets/js/leaflet.js"
   integrity="sha512-gZwIG9x3wUXg2hdXF6+rVkLF/0Vi9U8D2Ntg4Ga5I5BZpVkVxlJWbSQtXPSiUTtC0TjtGOmxa1AJPuV0CPthew=="
   crossorigin=""></script>

<script>
var $mapvar = L.map('$divid',{scrollWheelZoom: false}).fitBounds([[$ul_lat,$ul_lng], [$lr_lat,$lr_lng]]);
L.tileLayer('https://api.mapbox.com/styles/v1/{id}/tiles/{z}/{x}/{y}?access_token={accessToken}', {
    attribution: 'Map data &copy; <a href="https://www.openstreetmap.org/">OpenStreetMap</a> contributors, <a href="https://creativecommons.org/licenses/by-sa/2.0/">CC-BY-SA</a>, Imagery © <a href="https://www.mapbox.com/">Mapbox</a>',
    maxZoom: 18,
    id: 'mapbox/streets-v11',
    tileSize: 512,
    zoomOffset: -1,
    accessToken: 'pk.eyJ1IjoiY3RuYWRvdmljaCIsImEiOiJja2F5YXh3b2MwMGszMnlvbHZyazBvaXVnIn0.M9R153X9-76WlVKOwM-v5Q'
}).addTo($mapvar);
$markers
$track
</script>
EOT;
				break;

			case 'stamen-toner':
				if(!isset($layer_type)) $layer_type='toner';
			case 'stamen-terrain':
				if(!isset($layer_type)) $layer_type='terrain';
			case 'stamen':
				if(!isset($layer_type)) $layer_type='terrain';
				$file_type='png';
				$script=<<<EOT
<script src="https://parando.org/assets/js/leaflet.js"
   integrity="sha512-gZwIG9x3wUXg2hdXF6+rVkLF/0Vi9U8D2Ntg4Ga5I5BZpVkVxlJWbSQtXPSiUTtC0TjtGOmxa1AJPuV0CPthew=="
   crossorigin=""></script>
<script>
var $mapvar = L.map('$divid',{scrollWheelZoom: false}).fitBounds([[$ul_lat,$ul_lng], [$lr_lat,$lr_lng]]);
L.tileLayer('https://stamen-tiles-{s}.a.ssl.fastly.net/$layer_type/{z}/{x}/{y}.$file_type', {
    attribution: [
                'Map tiles by <a href="http://stamen.com/">Stamen Design</a>, ',
                'under <a href="http://creativecommons.org/licenses/by/3.0">CC BY 3.0</a>. ',
                'Data by <a href="http://openstreetmap.org/">OpenStreetMap</a>, ',
                'under <a href="http://creativecommons.org/licenses/by-sa/3.0">CC BY SA</a>.'
            ].join(""),
		type: '$file_type',
		minZoom: 1,
    maxZoom: 18,
		subdomains: "a b c d".split(" "),
    scheme: 'xyz'}).addTo($mapvar);
$markers
$track
</script>
EOT;
				break;
			default:
				$this->die_error(__METHOD__, "Uknown map tile provider: $provider");
				break;
		}
		return $script;
	}
	
	
	// CUESHEET GENERATION

	private function generate_csv_cuesheet($route_event, $cues, $controles){
		$event_tagname=$this->make_event_tagname($route_event);
		$cue_version=$route_event['cue_version'];
		$cue_basename="$event_tagname-CueSheetV$cue_version";
		$cue_filename=Cuesheet::cuesheet_path . "/" . $cue_basename . ".csv";
		$cue_url=Cuesheet::cuesheet_baseurl . "/" . $cue_basename . ".csv";
		$this->cuesheet->set_controle_date_format($route_event);
		$header_text=$this->cuesheet->header_text_array($route_event);
		$cue_text=$this->cuesheet->cue_text_array($cues, $controles);
		$this->emit_csv(array_merge($header_text,$cue_text), $cue_filename, true);
		return compact('cue_url','cue_filename','cue_version');
	}

	private function generate_pdf_cuesheet($route_event, $cues, $controles, $orientation, $size){

		$event_tagname=$this->make_event_tagname($route_event);
		$cue_version=$route_event['cue_version'];
		$cue_basename="$event_tagname-CueSheetV$cue_version";
		$params=['event'=>$route_event, 'orientation'=>$orientation, 'size'=>$size];
		$cuesheet=new Cuesheet($params);
		$cuesheet->set_controle_date_format($route_event);

//$this->die_error(__METHOD__, print_r($controles,true));

		$cuesheet->AddPage();
		$cuesheet->draw_cuesheet_pages($route_event, $cues, $controles);
		$cue_filename=Cuesheet::cuesheet_path . "/" . $cue_basename . "-$orientation.pdf";
		$cue_url=Cuesheet::cuesheet_baseurl . "/" . $cue_basename . "-$orientation.pdf";
		$cuesheet->Output("F", $cue_filename);
		$fstat=@stat($cue_filename);
		if($fstat===false || empty($fstat['size'])) $die->error(__METHOD__, "Failed to create cuesheet file '$cue_filename'");
		return compact('cue_url','cue_filename','cue_version');
	}
	
	

 
// In order to support either a standalone RWPGPS route description with embedded event details given
// as markup in the description, or a bare route paired with a PA Rando Event Database entry, we 
// need this function to merge and harmonize the two. It produces a route_event array, along with 
// a list of warnings. 


	private  $datetime_attributes=[
				['event_date', 'Event Start Date (YYYY-MM-DD)', ''],
				['event_time', 'Event Start Time (HH:MM)', '']
		];


 	private $event_datetime_format='Y-m-d H:i T';
	private $event_datetime_format_verbose='l j F Y, H:i T';

	private $default_event_date="2000-01-01";
	private $default_event_time="00:00";

	private $parando_logo_url="https://parando.org/images/parando3D.png";


	private $valid_event_types = ['acp'=>true, 'rusa'=>true, 'permanent'=>true];

	private 		$required_tags=[
//			'this_organization'=>'organization',
			'organizer_name'=>true,
			'organizer_phone'=>true,
			'event_distance'=>'distance',
			'event_type'=>'type',
			'event_location'=>'location',
			'event_distance'=>'distance',
			'event_name'=>'name'
		];

	private function merge_route_event_data($route, $event=null){		

		$warnings=[];
		$re=[];
		
		// Put all route strings and numbers into route_event
		foreach($route as $key=>$val){
			if(is_string($val) || is_numeric($val)) $re[$key]=$val;
		}

		$re['distance_km']=$distance_km=round($route['distance']/rwgps::m_per_km,1);
		$re['distance_mi']=$distance_mi=round($route['distance']/rwgps::m_per_km/rwgps::km_per_mi,1);
		$re['climbing_ft']=$climbing_ft=round($route['elevation_gain']*rwgps::ft_per_m,1);
		$re['route_id']=$route_id=$route['id'];
		
		// Some data about the RWGPS route itself in all cases
		$re['route_url']=$this->rwgps->make_route_url($route['id']);


		$re['route_filename']=$route_filename=$this->rwgps->make_route_filename($route['id']);
		$re['route_file_basename']=basename($route_filename);


		$re['download_note']=$route['download_note'];
		$re['route_name']=$route['name'];

		// If there's nothing to go on, complain and create some basic info

		if(empty($route['description']) && empty($event)){
			$warnings[]="No event specified in database, and no RWGPS route description. Please add a full description to the RWGPS route, or add this event to the database.";
			$route['description']='';
		}
		
//		$this->die_error(__METHOD__, print_r($route,true));

		// Put any route description attributes into route_event

		$ed=$this->rwgps->parse_description($route['description'],$this->rwgps->valid_event_description_keys);
		if(!empty($ed['unrecognized'])){
				$warnings[]="Unrecognized attributes in RWGPS route description: " . implode(', ',$ed['unrecognized']); 
				$warnings[]="Valid route description attribute keys are: " . implode(', ',$this->rwgps->valid_event_description_keys);
		}

		foreach($ed as $key=>$val){
			if(is_string($val) || is_numeric($val)) $re[$key]=$val;
		}

		$re['this_organization']=$this_organization=(isset($ed['organization']))?$ed['organization']:"";
		$re['event_name_dist']=(isset($ed['name'], $ed['distance']))?$event_name_dist=$ed['name'] . ' ' . $ed['distance'] . 'K':$route['name'];

		$re['organizer_name']=$organizer_name=(isset($ed['organizer_name']))?$ed['organizer_name']:null;
		$re['organizer_phone']=$organizer_phone=(isset($ed['organizer_phone']))?$ed['organizer_phone']:null;
		$re['organizer_rusa_id']=$organizer_rusa_id=(isset($ed['organizer_rusa_id']))?$ed['organizer_rusa_id']:null;
		$re['rusa_route_id']=$rusa_route_id=(isset($ed['rusa_route_id']))?$ed['rusa_route_id']:null;
		$re['event_distance']=$event_distance=(isset($ed['distance']))?$ed['distance']:null;
		$re['event_name']=$event_name=(isset($ed['name']))?$ed['name']:null;
		$re['event_type']=$event_type=(isset($ed['type']))?strtolower($ed['type']):null;
		$re['event_style']=$event_style=(isset($ed['style']))?strtolower($ed['style']):null;
		$re['event_date']=$event_date=(isset($ed['date']))?$ed['date']:null;
		$re['event_time']=$event_time=(isset($ed['time']))?$ed['time']:null;
		$re['event_location']=$event_location=(isset($ed['location']))?$ed['location']:null;
		$re['event_tzname']=$event_tzname=(isset($ed['tzname']))?$ed['tzname']:$this->model_parando->tzname;
		$re['event_tz']=$event_tz=new DateTimeZone( $event_tzname );
		
//$this->die_error(__METHOD__, "edtzn=" . $ed['tzname'] . "; etzn=$event_tzname; etz=" . print_r($event_tz, true));		

		$re['event_id']=$event_id=null;



		// Now look for official PA Rando event with $event database object
		// and override as required
		if(!empty($event)){  

			// store route_event data if present in local organizer event database 
			$re['this_organization']=$this_organization=$this->model_parando->organization;
			$re['logo_url']=$logo_url=$this->model_parando->logo_url;
			extract($this->model_parando->get_organizer($event));
			$re['organizer_name']=$organizer_name;
			$re['organizer_phone']=$organizer_phone;
			$re['event_id']=$event_id=$event['id'];			
			$re['route_id']=$route_id=$route['id'];

		   if(!empty($event['rusa_route'])) {$re['rusa_route_id']=$rusa_route_id=$event['rusa_route'];}
			
			$re['event_name_dist']=$event_name_dist=$event['name'] . ' ' . $this->model_parando->make_distance($event);
			$re['event_name']=$event_name=$event['name'];
			$re['event_distance']=$event_distance=$event['distance'];
			$re['event_type']=$event_type=strtolower($event['sanction']);
			$re['event_style']=$event_style=strtolower($event['type']);
			$re['event_date']=$event_date=$event['date'];
			$re['event_time']=$event_time=$event['start_time'];
//$this->die_error(__METHOD__, "Event date $event_date; Event time $event_time");
			$re['event_location']=$event_location=$event['start_city'] . ' ' . $event['start_state'];
			$re['event_tzname']=$event_tzname=$this->model_parando->tzname; // BUG! Will fail if PA Rando ever has events outside default TZ
			$re['event_tz']=$event_tz=new DateTimeZone( $event_tzname );
			$re['event_roster']=$event_roster=$this->model_parando->get_roster($event_id);

		}
		
		// Now that all info is in, some calculations are possible

		date_default_timezone_set($event_tzname);
		$re['last_update']=date("Y-m-j H:i:s T",$re['updated_at']); 
		$re['last_download']=date("Y-m-j H:i:s T",$re['downloaded_at']); 

		// Generate warnings/defaults for missing data
		
		$missing_tags=[];
		foreach($this->required_tags as $k=>$v){
    		if(!isset(${$k})){
					if(is_string($v))
						$missing_tags[]=$v;
					else
						$missing_tags[]=$k;
				}
		}

		foreach($missing_tags as $t) 
			$warnings[]="Please provide missing tag in event description: $t";
		
		if(empty($event_location)){
			$re['event_location']=$event_location=" ";
			$warnings[]="Event location not set. Set #location equal to 'City, State' of event start.";
		}

		if(empty($event_tzname)){
			$re['event_tzname']=$event_tzname=$this->model_parando->tzname;
			$re['event_tz']=$event_tz=new DateTimeZone( $event_tzname );
			$warnings[]="Time zone not set. Use tag #tzname to set time zone in route description. Main US time zones are <br>
<br>
Eastern ........... America/New_York <br>
Central ........... America/Chicago <br>
Mountain .......... America/Denver <br>
Mountain no DST ... America/Phoenix <br>
Pacific ........... America/Los_Angeles <br>
Alaska ............ America/Anchorage <br>
Hawaii ............ America/Adak <br>
Hawaii no DST ..... Pacific/Honolulu <br>
<br>
For complete list of valid timezones, see <A HREF=https://www.php.net/manual/en/timezones.php>https://www.php.net/manual/en/timezones.php</a>";
		}
		
		if($event_distance>$distance_km){
			$warnings[]="Route is too short. Route distance $distance_km km < Official distance $event_distance km.";
		}
		
		if(!empty($event_type) && !isset($this->valid_event_types[$event_type])) 
			$this->die_error(__METHOD__, "Unsupported event type: $event_type. Valid types are: " . 
				implode(', ', array_keys($this->valid_event_types)));

		if(empty($event_type)){
			$re['event_type']=$event_type='unknown';
			$warnings[]="Event type not set. Set #type equal to one of the following:" . implode(', ', array_keys($this->valid_event_types));
		}

		if($event_type=='permanent' && !isset($re['organizer_rusa_id'])) 
			$warnings[]="Event is permanent. Must set: #organizer_rusa_id";

		if($event_type=='permanent' && !isset($re['rusa_route_id'])) 
			$warnings[]="Event is permanent. Must set: #rusa_route_id";

		// Now
		$re['now']=$now=new DateTime('now',$event_tz);
		$re['now_str']=$now->format($this->event_datetime_format);
		$re['cue_version']=$cue_version=(empty($event['cue_version']))?0:($event['cue_version']);
		
		return [$re, $warnings];
	}

 	private function make_event_tagname($re){ // adapter for route_event rather than event
    $d=(empty($re['event_datetime']))?"":$re['event_datetime']->format('Y-m-d');
 		$event=[
 			'sanction'=>$re['event_type'], 
 			'type'=>$re['event_style'],
 			'distance'=>$re['event_distance'], 
 			'date'=>$d
 		];
		if(isset($re['rusa_route_id'])) $event['rusa_route_id']=$re['rusa_route_id'];
 		$tagname=$this->model_parando->make_event_tagname($event);
 		return $tagname; 
 	}

	
// VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS VIEWS

	private function die_view_warnings($warnings){ // for views that want to die if there are warnings
		
		if(!empty($warnings)){
			$cw=implode('</li><li>',$warnings);
			$left_column =<<<EOT
<p>The requested function is not available because of Errors in route data.</p>
<ul><li>$cw</li></ul>
<p>Fix these Errors by editing the RWGPS route, saving in RWGPS, and re-downloading.</p>
EOT;
			$this->die_error('Route Errors', $left_column);
		}
		
	}
	
	// Raw JSON View
	
	private function view_json($route_event, $controles, $warnings){
		$route=$this->rwgps->get_route($route_event['route_id']);
		header("Content-Type: application/json; charset=UTF-8");
		echo json_encode($route);
		exit();
	}

	// Cue processing debug log

	private function view_search_replace_changes($route_event, $controles, $warnings){

		$ncontroles=count($controles);
		extract($route_event);


		$route=$this->rwgps->get_route($route_id);
		list($cues, $cue_warnings)=$this->rwgps->extract_cues($route);
		$warnings=array_merge($warnings, $cue_warnings);
		$this->die_view_warnings($warnings);

		$params=['event'=>$route_event];
		$this->load->library('cuesheet', $params);
		$this->cuesheet->set_controle_date_format($route_event);
		// $header_text=$this->cuesheet->header_text_array($route_event);
		$cue_text=$this->cuesheet->cue_text_array($cues, $controles);
		
		$change_log=[];
		foreach($cue_text as $line){
			$controle_i=array_shift($line);
			if(empty($controle_i)){
				$line[$this->cuesheet->dist_cols+1]=$this->cuesheet->clean_string($line[$this->cuesheet->dist_cols+1]); // clean note
				$line_r=$this->cuesheet->replace_cues($line);
					$dist=$line[0];
					$dir=$line[$this->cuesheet->dist_cols+0];
					$dir_r=$line_r[$this->cuesheet->dist_cols+0];
					$note=$line[$this->cuesheet->dist_cols+1];
					$note_r=$line_r[$this->cuesheet->dist_cols+1];
				if($note_r != $note){
		      $change_log[]=[$dist, "$dir: $note", ">>>>>", "$dir_r: $note_r"];
				}
			}
		}



		if(!empty($warnings)){
		$cw=implode('</li><li>',$warnings);
		$warning_text=<<<EOT
<div class="bordered narrower red">
<div class="vspace"></div>
<div class="large-print sans">ERRORS:</div>
<ul><li>$cw</li></ul>
EOT;
		}else{$warning_text='';}

		$left_column=<<<EOT
<H3>$event_name_dist</H3>
$warning_text
<H4>Cue Search/Replace Changes</H4>
<div class='narrower ftable'><TABLE WIDTH=100%><TR><TD>
EOT;
		$func=function($a){
			return implode("</TD><TD>",$a);
		};
		$left_column .= implode("</TD></TR><TR><TD>", array_map($func ,$change_log));
		$left_column .=<<<EOT
</TD></TR></TABLE></DIV>

EOT;


		$data=['title'=>$event_name_dist, 'left_column'=>$left_column, 'right_column'=>''];
		$this->simple_view('info_one_column',['table'],$data);
	}

	// General view of route as HTML page
	
	private function view_html($re, $controles, $warnings){

		$this_organization=$this->model_parando->organization;
		$this_url=site_url(strtolower(get_class($this)));
		$download_url="$this_url/recache";
		
		$ncontroles=count($controles);
		extract($re);

		$route=$this->rwgps->get_route($route_id);
		$ed=$this->rwgps->parse_description($route['description']);		
		$ed_str=$this->format_attributes($ed);

		if(empty($event_id)){
			$method='id';
			$route_event_id=$id=$route_id;
			$cue_version_str="";
		}else{
			$method='event';
			$id=$event_id;
			$route_event_id="$route_id/$event_id";
			$cue_version=(empty($re['cue_version']))?"None Published":$re['cue_version'];
			$cue_version_str="<TR><TD><b>Cuesheet Version</b></TD><TD>$cue_version</TD></TR>";
		}

		$controles_table_body="";
		$controles_table_body.= "<TR><TH>Controle</TH><TH>Distance</TH><TH>Open/Close</TH><TH>Note</TH><TH>Description (attributes)</TH></TR>";
		for($i=0; $i<$ncontroles; $i++){
			$cd_mi=round($controles[$i]['d']/(rwgps::m_per_km*rwgps::km_per_mi),1);
			$cd_km=round($controles[$i]['d']/(rwgps::m_per_km),1);
			$is_start=(isset($controles[$i]['start']))?" [START]":"";
			$is_finish=(isset($controles[$i]['finish']))?" [FINISH]":"";
			$controle_num=$i+1;
			$open_str=$controles[$i]['open']->format('m-d H:i');
			$close_str=$controles[$i]['close']->format('m-d H:i');
			// $ca=print_r($controles[$i]['attributes'], true);
			$ca=$this->format_attributes($controles[$i]['attributes']); 
			$cn=(isset($controles[$i]['n']))?$controles[$i]['n']:"";
			$controles_table_body.= "<TR><TD>$controle_num$is_start$is_finish</TD><TD>$cd_mi mi<br>$cd_km km</TD><TD>$open_str<br>$close_str</TD><TD>$cn</TD><TD>$ca</TD></TR>";
		}
		
			$fineprint=<<<EOT
<hr width=61.8%>
<div class='text-center half-width'>Cue Wizard <A HREF=https://parando.org/cue_wizard.html>User Manual</A>
<div class='fine-print'>
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
<A HREF=https://parando.org/software_license.html>Click here</a> for access to source code and complete license details.
</div></div>
<div class=vspace></div>
EOT;

		if(!empty($rusa_route_id))
			$rusa_route_id_table_row="<TR><TD><b>RUSA Route ID</b></TD><TD>#$rusa_route_id</TD></TR>";
		else
			$rusa_route_id_table_row="";

		if(!empty($event_id)){
			$attrib_edit_controls="";
			$event_datetime_table_row="<TR><TD><b>Event Date/Time</b></TD><TD>$event_datetime_str</TD></TR>";
		}else{
			$attrib_edit_controls="<div class=vspace></div><p class=nopadding>The following event attributes can be changed to customize cue/card generation.</p><div>";
			$event_datetime_table_row="";
		  foreach($this->datetime_attributes as $a){
				$k=$a[0]; $v=$a[1];
				$value=${$k};
				//$value=set_value($k,${$k});

				//if(!empty(form_error($k))){
				//	$attrib_edit_controls .= "<div class=verror>" . form_error($k) . "</div>";
				//}
				$attrib_edit_controls.=<<<EOT
<div class=form-group>
	<label for="$k">$v</label>    
		<div class='half-width-indented'><input type='text' placeholder='Enter $v' name='$k' value='$value'></div>
</div>
EOT;
			}
			$attrib_edit_controls.=<<<EOT
<div class=form-group>
	<label for="roster_file">Roster CSV File</label>    
		<div class='half-width-indented'><input type='file' placeholder='No roster' name='roster_file'></div>
</div>
</div>
EOT;
		}

		if(!empty($ed_str))
			$rwgps_attributes=<<<EOT
<H4>ATTRIBUTES</H4>
<p class=nopadding>The following event attribute tags were set in the RWGPS route description:</p>
<div class='narrower bordered tt'>
$ed_str
</div>
EOT;
		else
			$rwgps_attributes="";

		if(!empty($event_id) && !empty($ed_str)) 
			$rwgps_attributes.=<<<EOT
<div class='narrower text-center'>
  <p class=nopadding><em>NB: Some of these event attribute tags in RWGPS route description have been supersceded 
  by attributes set in $this_organization event database for Event $event_id: $event_name_dist. </em></p>
</div>
EOT;

		if(!empty($warnings)){
			$cw=implode('</li><li>',$warnings);
			$generate_buttons =<<<EOT
<div class="bordered narrower red">
<div class="vspace"></div>
<div class="large-print sans">ERRORS:</div>
<ul><li>$cw</li></ul>
<p>Cue Sheet and Brevet Card Generation not available because of Errors in route.</p>
<p>Fix these Errors by editing the RWGPS route, saving in RWGPS, and re-downloading here.</p>
</div><div class="vspace"></div>
EOT;
		}else{
			if($method=='event'){
				$publish_cuesheet=<<<EOT
<div class=vspace></div><div class="bordered narrower text-center">Once you are happy with the cuesheets generated above, press the button below 
to publish this new version of the cuesheet to the $this_organization event info page.<br>
<div class="button-container">
<A HREF=$this_url/publish_cuesheet/$event_id>Publish Cuesheets to Event</A>
</div>
</div>
EOT;
			}else{
				$publish_cuesheet="";
			}
			$generate_buttons=<<<EOT
<div class="narrower button-stack">

<BUTTON TYPE=SUBMIT NAME='view' value='pdf_cue_portrait'>PDF Cuesheet (portrait)</BUTTON>
<BUTTON TYPE=SUBMIT NAME='view' value='pdf_cue_landscape'>PDF Cuesheet (landscape)</BUTTON>
<BUTTON TYPE=SUBMIT NAME='view' value='csv_cue'>CSV Cue Sheet</BUTTON>
<BUTTON TYPE=SUBMIT NAME='view' value='search_replace_changes'>Cue Sheet Search Replace Changes</BUTTON>
<BUTTON TYPE=SUBMIT NAME='view' value='card_outside_blank'>Brevet Card Outside Blank</BUTTON>
<BUTTON TYPE=SUBMIT NAME='view' value='card_outside_roster'>Brevet Card Outside Roster</BUTTON>
<BUTTON TYPE=SUBMIT NAME='view' value='card_inside'>Brevet Card Inside</BUTTON>

$publish_cuesheet
</div><div class="vspace"></div>
EOT;

$ignore=<<<EOT
<LI><A HREF=$this_url/$method/$id/search_replace_changes>Search Replace Changes</A>
<LI><A HREF=$this_url/$method/$id/card_outside_blank>Brevet Card Outside Blank</A>
<LI><A HREF=$this_url/$method/$id/card_outside_roster>Brevet Card Outside Roster</A>
<LI><A HREF=$this_url/$method/$id/card_inside>Brevet Card Inside</A>
<LI><A HREF=$this_url/$method/$id/pdf_cue_portrait>PDF Cuesheet (portrait)</A>
<LI><A HREF=$this_url/$method/$id/pdf_cue_landscape>PDF Cuesheet (landscape)</A>
<LI><A HREF=$this_url/$method/$id/csv_cue>CSV Cue Sheet</A>
EOT;

		}
		
		$left_column="";
		$right_column="";

//$this->die_error(__METHOD__,print_r($re,true));

if(!empty($pavement_type))
	$pt="<TR><TD><b>Pavement Type</b></TD><TD>$pavement_type</TD></TR>";
else
	$pt="";
		
		$map_divid='randomap'; // must match ID in randomap.css
		$graph_divid='epdivid'; // must match ID in randomap.css
		$map_script=$this->generate_map_script($route,$controles,$map_divid);
		$ep_script=$this->generate_ep_script($route, $controles, $graph_divid);


			$df_links=[];
			foreach($route['route_datafile'] as $ext=>$fn){
				$uc_ext=strtoupper($ext);
				$base_fn=basename($fn);
				$df_links[]="<A TITLE='$uc_ext file download' HREF='" . $route['saved_route_url'][$ext] . "'>$uc_ext</A>"; 
			}
			$data_file_row="<TR><TD><b>Route File Download</b></TD><TD>" . implode(', ', $df_links) . "</TD></TR>";


		$left_column.=<<<EOT
<H3>$event_name_dist</H3>
<div class="bordered narrower text-center">$download_note<br>
<div class="button-container"><A HREF=$download_url/$route_event_id>Re-download route from RWGPS</A>
</div></div>
<H4>ROUTE INFO</h4>
<div class='narrower ftable'><TABLE WIDTH=100%>
<TR><TD><b>Route Name</b></TD><TD>$route_name</TD></TR>
<TR><TD><b>Last Download</b></TD><TD>$last_download</TD></TR>
<TR><TD><b>Modified</b></TD><TD>$last_update</TD></TR>
<TR><TD><b>RWGPS URL</b></TD><TD><A HREF=$route_url>$route_url</a></TD></TR>
$data_file_row
<TR><TD><b>Distance</b></TD><TD>$distance_mi mi / $distance_km km</TD></TR>
<TR><TD><b>Climbing</b></TD><TD>$climbing_ft ft</TD></TR>
$pt
<TR><TD><b>Event Type</b></TD><TD>$event_type</TD></TR>
$rusa_route_id_table_row
<TR><TD><b>Official Distance</b></TD><TD>$event_distance km</TD></TR>
$event_datetime_table_row
<TR><TD><b>Controles Found</b></TD><TD>$ncontroles</TD></TR>
$cue_version_str
</TABLE></DIV>

<H4>MAP</H4>
<div class='narrower text-center'>
<div id=$map_divid></div>
<A HREF=$route_url>View in Ride With GPS</a>
<div class=bigskip></div>
<div id=$graph_divid style="margin-left:10%; margin-right:10%; width:80%; height:180px;"></div>
</div>
$map_script
$ep_script

<H4>CONTROLES</H4>
<div class='narrower ftable'><TABLE WIDTH=100%>
$controles_table_body
</TABLE></DIV>
$rwgps_attributes
<FORM ACTION=$this_url/$method/$id METHOD=POST  enctype="multipart/form-data">
$attrib_edit_controls
<div class=vspace></div>
<H4>GENERATE</H4>
$generate_buttons
</FORM>
$fineprint
EOT;
		$data=['title'=>$event_name_dist, 'left_column'=>$left_column, 'right_column'=>$right_column];
		$this->simple_view('info_one_column',['randomap','table','buttonlink','form'],$data);
	
	}
	
	private function route_info_table(){
	$rit=<<<EOF
<H4>ROUTE INFO</h4>
<div class='narrower ftable'><TABLE WIDTH=100%>
<TR><TD><b>Route Name</b></TD><TD>$route_name</TD></TR>
<TR><TD><b>Last Download</b></TD><TD>$last_download</TD></TR>
<TR><TD><b>Modified</b></TD><TD>$last_update</TD></TR>
<TR><TD><b>RWGPS URL</b></TD><TD><A HREF=$route_url>$route_url</a></TD></TR>
$data_file_row
<TR><TD><b>Distance</b></TD><TD>$distance_mi mi / $distance_km km</TD></TR>
<TR><TD><b>Climbing</b></TD><TD>$climbing_ft ft</TD></TR>
$pt
<TR><TD><b>Event Type</b></TD><TD>$event_type</TD></TR>
$rusa_route_id_table_row
<TR><TD><b>Official Distance</b></TD><TD>$event_distance km</TD></TR>
$event_datetime_table_row
<TR><TD><b>Controles Found</b></TD><TD>$ncontroles</TD></TR>
$cue_version_str
</TABLE></DIV>
EOF;
	}

	private function format_attributes($alist){
		$ca="";
		foreach($alist as $k=>$v){
				if(is_string($v))
					$ca.="<div class=attribute>#$k=$v</div>";
		}
		return $ca;
	}
	
	// PDF Rendered cue_sheet Views
	
	private function view_pdf_cue($route_event, $controles, $warnings){
		$this->_view_pdf_cue($route_event, $controles, $warnings, 'P');
	
	}
	private function view_pdf_cue_portrait($route_event, $controles, $warnings){
		$this->_view_pdf_cue($route_event, $controles, $warnings, 'P');
	
	}
	private function view_pdf_cue_landscape($route_event, $controles, $warnings){
		$this->_view_pdf_cue($route_event, $controles, $warnings, 'L');
	
	}

	private function _view_pdf_cue($route_event, $controles, $warnings, $orientation='P', $size='letter'){
	
		$route=$this->rwgps->get_route($route_event['route_id']);
		list($cues, $cue_warnings)=$this->rwgps->extract_cues($route);
		$warnings=array_merge($warnings, $cue_warnings);
		$this->die_view_warnings($warnings);
		$params=['event'=>$route_event, 'orientation'=>$orientation, 'size'=>$size];
		$this->load->library('cuesheet', $params);
		$this->cuesheet->AddPage();

		$this->cuesheet->set_controle_date_format($route_event);
		$this->cuesheet->draw_cuesheet_pages($route_event, $cues, $controles);
		$event_tagname=$this->make_event_tagname($route_event);
		$this->cuesheet->Output("I", "$event_tagname-CueSheet-$orientation.pdf");

	}

	private function view_csv_cue($route_event, $controles, $warnings){
	
		$route=$this->rwgps->get_route($route_event['route_id']);
		list($cues, $cue_warnings)=$this->rwgps->extract_cues($route);
		$warnings=array_merge($warnings, $cue_warnings);
		$this->die_view_warnings($warnings);

		$params=['event'=>$route_event];
		$this->load->library('cuesheet', $params);
		$event_tagname=$this->make_event_tagname($route_event);
		$csv_filename="$event_tagname-CueSheet.csv";

		$header_text=$this->cuesheet->header_text_array($route_event);
		$cue_text=$this->cuesheet->cue_text_array($cues, $controles);
		$this->emit_csv(array_merge($header_text,$cue_text), $csv_filename);
		exit();

	}
	
	private function emit_csv($array,$filename='data.csv',$write=false){
		if($write){
			$output = fopen($filename,'w') or die("Can't open $filename");
		}else{
			$output = fopen("php://output",'w') or die("Can't open php://output");
			header("Content-Type:application/csv"); 
			header("Content-Disposition:attachment;filename=$filename"); 
		}
		foreach($array as $row) {
			fputcsv($output, $row);
		}
		fclose($output) or die("Can't close output.");	
	}

		
	// PDF Rendered Brevet Card Views

	private function view_card_inside ($route_event, $controles, $warnings){
		$this->view_card_inside_withoptions($route_event, $controles, $warnings, ['validate_first'=>true]);
	}

	private function view_card_inside_novalidatefirst ($route_event, $controles, $warnings){
		$this->view_card_inside_withoptions($route_event, $controles, $warnings, []);
	}

	private function view_card_inside_withoptions ($route_event, $controles, $warnings, $opts=[] ){
	
		$this->die_view_warnings($warnings);

		$this->load->library('brevetcard', $route_event);

		while(count($controles)>($this->brevetcard->cpf*$this->brevetcard->n_folds)){ 
			$this->brevetcard->n_folds++;
		}

		$this->brevetcard->AddPage();
		$this->brevetcard->set_controle_date_format($route_event);

		// set false if you don't want the first controle validated when cards are prined
		$logo_url=(isset($opts['validate_first']) && isset($route_event['logo_url']))?$route_event['logo_url']:null;

		$this->brevetcard->draw_card_inside($controles,$logo_url);
		$event_tagname=$this->make_event_tagname($route_event);
		$this->brevetcard->Output("I", "$event_tagname-CardInside.pdf");
	
	}	

	private function view_card_outside_blank ($route_event, $controles, $warnings){
	
		$this->die_view_warnings($warnings);

		$this->load->library('brevetcard', $route_event);

		while(count($controles)>($this->brevetcard->cpf*$this->brevetcard->n_folds)){ 
			$this->brevetcard->n_folds++;
		}
		
		$this->brevetcard->AddPage();
		$this->brevetcard->draw_card_outside($route_event);
		$event_tagname=$this->make_event_tagname($route_event);
		$this->brevetcard->Output("I", "$event_tagname-CardOutsideBlank.pdf");
	
	}	

	private function view_card_outside_roster ($route_event, $controles, $warnings){
	
		$this->die_view_warnings($warnings);
		
		if(empty($route_event['event_roster'])) $this->die_error(__METHOD__, "No roster for event.");
		$roster=$route_event['event_roster']; // $route$this->model_parando->get_roster($route_event['event_id']);
		
		$this->load->library('brevetcard', $route_event);

		while(count($controles)>($this->brevetcard->cpf*$this->brevetcard->n_folds)){ 
			$this->brevetcard->n_folds++;
		}
			
		$n_riders=count($roster);
		$n_cards = $this->brevetcard->n_cards;
		
		for($i=0; $i<$n_riders; $i+=$n_cards){
			$r=array_slice($roster, $i, $n_cards);
			$this->brevetcard->AddPage();
			$this->brevetcard->draw_card_outside($route_event, $r);
		}
		
		$event_tagname=$this->make_event_tagname($route_event);
		$this->brevetcard->Output("I", "$event_tagname-CardOutsideRoster.pdf");
	
	}	

	
}
