<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
 
 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

class Store extends MY_Controller {

	
  // CONSTRUCTEUR

	public function __construct() {
		parent::__construct();
		$this->load->model('accounting_parando');
		$this->load->helper('form');
		$this->load->library('email');
		$this->load->library('form_validation');

		$multipage_params = ['tag' => 'store_session_data'];
		$this->load->library('multipage', $multipage_params);	
		$this->load->library('shoppages');
		
		$this->multipage->define_pages($this->shoppages->page);  // SLOW -- Library parameter?
		$this->multipage->set_view('form_simple','form_header','form_footer',['form','product_table','roster']);
		$this->multipage->set_controller(strtolower(get_class($this)));

		$this->session->set_userdata('paypal_sandbox', false);

	}
	
	public function die_error($a,$b){
		$this->member_session_init();
		parent::die_error($a,$b);
	}
	
	private $send_email=true;  // set true to allow email confirmations to be sent
	
	private function member_session_init(){
		$d=[
			'this_organization' => $this->model_parando->organization,
			'base_url' => base_url(),
			'contact_url' => $this->model_parando->contact_url,
			'webmaster_contact_url' => $this->model_parando->webmaster_contact_url,
			'admin_login_url' => $this->model_parando->admin_login_url
		];
		$this->multipage->init($d);
	}
	
	public function _get_country_options(){
		$country_fullname = $this->model_parando->get_countries();
		return $country_fullname;
	}
	
	public function _get_state_options(){ 
		$state_fullname = $this->model_parando->get_states();
		return $state_fullname;
	}

	public function _phone_check() {

		// Validation callback to verify that at least one phone number was entered

		if($this->input->post('contact_phone') || $this->input->post('contact_phone_alt')){
				return TRUE;
		}else{
				$this->form_validation->set_message('_phone_check', 'Please enter atleast one phone number.');
		return FALSE;
		}
	}


  // public $always_shipping_method='delivery'; 
  public $nocharge_shipping_method='pickup';
  public $known_shipping_methods=['pickup'=>'Pick up order at {$this_organization} event. (Free!)', 
				'delivery'=>'Ship order to my address of record ({$order_shipping_fee_text})'];

  public function _get_known_shipping_methods(){
		$options=[];
		foreach($this->known_shipping_methods as $k=>$v){
			$options[$k]=$this->multipage->interpolate_strings($v);
		}
		return $options;
	}


	
	// GENERIC PAGE PROCESSING

	public function index() {
		if($this->multipage->page_expired_q()){
			$this->shop();
		}else{
			$this->multipage->process_page();
		}
	}




// MAIN CONTROLLER METHOD

	public function shop(){
		$this->member_session_init();
		$this->multipage->set_current_page('shop');
		$this->multipage->view_current_page();
	}

	public function _shop_form_body(){

		$this_organization=$this->model_parando->organization;

		$body = <<<EOT
<p>The following items are available for purchase from {$this_organization}.
Please select "Buy" for the item you would like to purchase:
<div class=vspace></div>
<div class=product-table>
<table width=100%>
<TR>
<TH>Image</TH>
<TH>Description</TH>
<TH>Unit Price</TH>
<TH></TH>
</TR>
EOT;

		$for_sale=$this->accounting_parando->get_items_for_sale();
		foreach($for_sale as $row){
			$item_image=(empty($row['image_url']))?"":("<IMG HEIGHT=64 SRC='" . $row['image_url'] . "'>");
			$item_sku=$row['sku'];
	    $body .= "<tr>";
	    $body .= "<td>" . $item_image . "</td>";
	    $body .= "<td>" . $row['description'] . "</td>";
	    $body .= "<td>\$" . number_format($row['price'],2) . "</td>";
	    $body .= "<td><button type='submit' name='sku' value='$item_sku'>Buy</button></td>";
	    $body .= "</tr>";
		}

		$body .= "</table></div>";

		extract($this->get_cart_data());

		if($item_count>0)
			$body .= "<div class='text-center bordered'>Shopping Cart Items: $item_count, Total: $item_total_price_text</div>";

		return $body;
	}

	
	public function _shop(){
		
		$next_page=$this->input->post('submit');
		switch($next_page){
			case 'show_cart':
				$this->multipage->set_current_page($next_page);
				break;
			case 'checkout':
				extract($this->get_cart_data());
				if($item_count==0)
					$this->multipage->set_current_page('shop');
				else
					$this->multipage->set_current_page('checkout');
				break;
			default:
				$sku=$this->input->post('sku');
				$product=$this->accounting_parando->get_product($sku);
				if(empty($product)){
					$this->multipage->set_current_page('shop');
				} else {
					$product['options']=$this->parse_options($product['options']);
					$this->multipage->set_data('product',$product);
					$this->multipage->set_current_page('options');
				}
				break;
		}
	}


	public function _options_form_body(){

		extract($this->multipage->get_data_array(['product','options_error']));
		if(empty($product)) $this->die_error(__METHOD__, "Null product.");
		extract($product);

		$image_tag=(empty($image_url)) ? "" : "<IMG WIDTH=100% SRC='$image_url'>";
		
		if(empty($options)){
			$left_column = <<<EOT
<p><i>$long_description<i></p>
<div class=half-width>$image_tag</div>
EOT;
		}else{
			$left_column = <<<EOT
<p>Select options for: <i>$long_description</i></p>
<div>$image_tag</div>
EOT;
}

		$right_column="";
		foreach($options as $opt=>$optlist){
			$selections="";
			for($k=1; $k<=count($optlist); $k++){
				$choice = $optlist[$k-1];
				$selections .= "<div class='image-radio'><input type='radio' name='$opt' id='{$opt}_{$k}' value='$k'><label for='{$opt}'>$choice</label></div>";
			}
			$right_column .= $this->make_option_group($opt, $selections);
		}

		$opt='Quantity';
		$selections = "<input type='text' size=1 name='$opt' value=1>";
		$right_column .= $this->make_option_group($opt, $selections);


		if(empty($options_error)) $options_error="";
		$body = <<<EOT
<div class=narrower>
<div class="row">
<div class="twocolumn">   	
$left_column
</div>
<div class="twocolumn">   	
$right_column
</div>
</div>
$options_error
</div>
EOT;

		return $body;
	}
	
	public $max_quantity=99;
	
	public function _options(){
		$product=$this->multipage->get_data('product');
		if(empty($product)) $this->die_error(__METHOD__, "Null product.");
		extract($product);

		$quantity=$this->input->post('Quantity')+0;
		if($quantity<0) $this->die_error(__METHOD__, "Negative quantity.");
		if($quantity>$this->max_quantity) $this->die_error(__METHOD__, "Quantity too big.");
		if($quantity!=floor($quantity)) $this->die_error(__METHOD__, "Quantity not an integer.");

		if(empty($quantity)) {
			$this->multipage->set_current_page('shop');
		} else {
			
			$missing=[]; $options_selected=[];
			foreach($options as $opt=>$optlist){
				$opt_k=$this->input->post($opt);
				if(empty($opt_k)) 
					$missing[]=$opt;
				else{
					if(empty($optlist[$opt_k-1])) $this->die_error(__METHOD__, "Option $opt_k not in optlist.");
					$options_selected[$opt]=$optlist[$opt_k-1];
				}
			}
			if(!empty($missing)){
				$options_error="Please specify all options. Missing options: ". implode(", ", $missing);
				$this->set_options_error($options_error);
			}else{
				$this->clear_options_error();
				$items=$this->multipage->get_data('items');
				if(empty($items)) $items=[];
				$items[]=['product'=>$product, 'quantity'=>$quantity, 'value'=>$options_selected];  // ADD TO CART
				$this->multipage->set_data('items',$items);
			}
		}
	}
	
	private function set_options_error($options_error){
				$options_error="<div class='bordered red bigger text-center'>$options_error</div>";
				$this->multipage->set_data('options_error',$options_error);
				$this->multipage->set_current_page('options');
	}
	private function clear_options_error(){
				$this->multipage->set_data('options_error',"");
	}
	
	public function _show_cart_form_body(){
	
		$body = <<<EOT
<div class=product-table>
<table width=100%>
<TR>
<TH>Quantity</TH>
<TH>Description</TH>
<TH>Options</TH>
<TH>Unit Price</TH>
<TH>Ext Price</TH>
<TH></TH>
</TR>
EOT;
		$items=$this->multipage->get_data('items');
		//if(empty($items)) $this->die_error(__METHOD__, "Null items list.");

		$cart_row=$this->cart_row_array($items);
		if(count($cart_row)!=count($items)) $this->die_error(__METHOD__,"This can't happen.");
		for($i=1; $i<=count($items); $i++){
			extract($cart_row[$i-1]);
			$quantity_input="<input type=text value=$quantity name=quantity_$i>";
			$body .= <<<EOT
<TR>
<TD>$quantity_input</TD>
<TD>$description</TD>
<TD>$value_text</TD>
<TD>$unit_price</TD>
<TD>$ext_price</TD>
</TR>
EOT;

		}
		$body .= "</table></div>";
		
		extract($this->get_cart_data($items));

		$body .= "<div class='text-center bordered'>Items: $item_count, Total: $item_total_price_text</div>";

	if(!empty(form_error('sku'))){
	  $body .= "<div class=verror>" . form_error('sku') . "</div>";
	}

		return $body;
	}
	
	private function cart_row_array($items=null){
		if($items===null) $items=$this->multipage->get_data('items');
		if(empty($items)) $items=[];
		$row=[];
		foreach($items as $item){
			extract($item);
			if(empty($product)) $this->die_error(__METHOD__, "No product in this item. item=".print_r($item,true));
			extract($product);
			$unit_price=$this->dollar_format($price);
			if(!empty($unit)) $unit_price .= " ($unit)";
			$pq=$price*$quantity;
			$ext_price=$this->dollar_format($pq);
			$value_text=$this->print_aa($value);
			$row[]=compact('quantity','description','value_text','unit_price','ext_price');
		}
		return $row;
	}
	
	private function print_aa($value){
			$value_list=[];
			foreach($value as $k=>$v) $value_list[]="$k=>$v";
			$value_text=implode(", ", $value_list);
			return $value_text;
	}
	
	public function _show_cart(){
		$items=$this->multipage->get_data('items');
		//if(empty($items)) $this->die_error(__METHOD__, "Null items list.");

		$quantity_changed=false;
		for($i=1; $i<=count($items); $i++){
			$new_quantity=$this->input->post("quantity_$i");
			if($new_quantity<0 || $new_quantity>$this->max_quantity || $new_quantity!=floor($new_quantity)) continue;
			$item=$items[$i-1];
			$old_quantity=$item['quantity'];
			if($old_quantity != $new_quantity){
				if($new_quantity==0) 
					array_splice($items,$i-1,1);  // DELETE FROM CART
				else
					$items[$i-1]['quantity']=$new_quantity;  // CHANGE QUANTITY
				
				$quantity_changed=true;
			}
		}
		if($quantity_changed){
			$this->multipage->set_data('items',$items);
			$this->multipage->set_current_page('show_cart');
		}else{
			$next_page=$this->input->post('submit');
			switch($next_page){
				case 'checkout':
					if(count($items)==0)
						$this->multipage->set_current_page('shop');
					else{
						$order_shipping_fee=$this->get_order_shipping_fee($items);
						$this->multipage->set_data('order_shipping_fee_text',$this->dollar_format($order_shipping_fee));
						$this->multipage->set_current_page('checkout');
					}
					break;
				default:
					$this->multipage->set_current_page('shop');
					break;
			}
		}
	}
	
	private function dollar_format($amount){
		return '$' . number_format($amount,2); 
	}
	
	private function get_order_shipping_fee($items=null){
		if($items===null) $items=$this->multipage->get_data('items');
		$max_shipping_fee=0;
		foreach ($items as $item){
			$max_shipping_fee=max($max_shipping_fee,$item['product']['shipping_fee']);
		}
		return $max_shipping_fee;  // for want of a better algorithm for computing shipping
	}

	private function get_cart_data($items=null){
		if($items===null) $items=$this->multipage->get_data('items');

		$item_total_price=0;
		$item_count=0;
		
		for($i=1; $i<=count($items); $i++){
			$item=$items[$i-1];
			extract($item);
			if(empty($product)) $this->die_error(__METHOD__, "No product in this item. item=".print_r($item,true));
			extract($product);
			$pq=$price*$quantity;
			$item_total_price += $pq;
			$item_count += $quantity;
		}
		
		$item_total_price_text=$this->dollar_format($item_total_price);

		return compact(['item_total_price','item_count','item_total_price_text']);
	}


	private function parse_options($po){
		$option=[];
		$optopt=explode(';',$po);
		foreach($optopt as $oo){
			if(empty($oo)) continue;
			list($opt,$optlist)=explode('=',$oo);
			$opt=trim($opt); $optlist=trim($optlist);
			$optlist=explode(':',$optlist);
			if(empty($opt)) $this->die_error(__METHOD__, "Missing Option Tag: '$oo'");
			if(empty($optlist)) $this->die_error(__METHOD__, "Missing Option List: '$oo'");
			$option[$opt]=$optlist;
		}
		return $option;
	}

	private function make_option_group($opt, $selections){
	
	$text = <<<EOT
<div class=form-group>
	<label for="$opt">$opt</label>    
	<div class='indented'>
		<div class='image-radio'>
			$selections
		</div>
	</div>
</div>
EOT;
	
	return $text;
	}
	
	public function _checkout(){
		$shipping_method=$this->input->post('shipping_method');
		if(isset($this->known_shipping_methods[$shipping_method]))
			$this->multipage->set_data('shipping_method',$shipping_method);
		else
			$this->die_error(__METHOD__, "Unkown shipping method: $shipping_method");
	}
	

	public function _name_and_number(){
		$this->find_and_process_member('load_memdata_into_session', 'pay_for_item', 'new_member', 'join or renew membership');
	}

	public function _pay_for_item_form_body(){
		$body=<<<'EOT'
<div class='product-table'>
<p>You are ordering the following items:</p>
<br>
<TABLE WIDTH=100%>
<TR><TH>Quantity</TH><TH>Description</TH><TH>Option</TH><TH>Unit Price</TH><TH>Ext Price</TH></TR>
EOT;

		$items=$this->multipage->get_data('items');
		extract($this->get_cart_data($items));
		
		$shipping_method=$this->multipage->get_data('shipping_method');
		$ksm=$this->_get_known_shipping_methods();
		$order_shipping_method_description=$ksm[$shipping_method];
		if($shipping_method==$this->nocharge_shipping_method){
			$order_shipping_fee=0;
			$order_shipping_fee_text="Free!";
			$shipping_note=$order_shipping_method_description;
		}else{
			$order_shipping_fee=$this->get_order_shipping_fee($items);
			$order_shipping_fee_text=$this->dollar_format($order_shipping_fee);
			extract($this->multipage->get_data_array(['this_organization','first_name','last_name','contact_city', 'contact_street','contact_zip','contact_state','contact_country']));
			if($contact_state=='XX') 
			  $this->die_error(__METHOD__, "According to our records, your address country is not United States. Cannot ship to you. Re-enter order and choose pickup order at event.");
			$shipping_note="The items are to be shipped from $this_organization to your mailing address of record in $contact_city, $contact_state.";
			
		}		
		
		
		
		foreach($this->cart_row_array($items) as $cart_row){
			extract($cart_row);
			$body .= "<TR><TD>{$quantity}</TD><TD>{$description}</TD><TD>{$value_text}</TD><TD>{$unit_price}</TD><TD>{$ext_price}</TD></TR>";
		}

		$total_due_text=$this->dollar_format($order_shipping_fee+$item_total_price);
		$item_total_price_text=$this->dollar_format($item_total_price);
		$body .=<<<EOT
</TABLE>
</div>
<div class=vspace></div>
<div class='text-right'>Shipping Fee: {$order_shipping_fee_text}</div>
<div class='text-right bold'>Total Charge: {$total_due_text}</div>
<div class=vspace></div>
<div class='narrower bordered text-center'>$shipping_note</div>
<div class=vspace></div>
<img src="https://www.paypalobjects.com/webstatic/en_US/i/buttons/cc-badges-ppmcvdam.png"  class=center-block-half alt="Pay now with PayPal" />

EOT;

		$this->multipage->set_data_array(compact('order_shipping_fee','order_shipping_fee_text','item_total_price_text','total_due_text'));

		return $body;

	}

	public function _pay_for_item(){
		$submit=$this->input->post('submit');
		if( $submit == 'yes' ){
			$point_of_sale = __METHOD__;
			$items=$this->multipage->get_data('items');
			if(count($items)==0) $this->die_error(__METHOD__,"No items to pay for.");
		    extract($this->get_cart_data($items));
			extract($this->multipage->get_data_array(['this_organization','order_shipping_fee','shipping_method','contact_email','first_name','last_name','member_id']));
			
			$purchase_items=[];
			foreach($items as $item){
				extract($item);
				if(empty($product)) $this->die_error(__METHOD__, "No product in this item. item=".print_r($item,true));
				extract($product);
				$purchase_items[]=['sku'=>$sku, 'quantity'=>$quantity, 'value'=>$this->print_aa($value)];
			}

			if(((float)$order_shipping_fee)>0.0) $purchase_items[]=['sku'=>'other_fee', 'quantity'=>1, 'value'=>'Shipping', 'unit_price'=>$order_shipping_fee];

			$purchase = $this->accounting_parando->create_purchase_order($member_id, $purchase_items, $point_of_sale, null, "Order for $shipping_method", $shipping_method);
			if(is_string($purchase)){$this->die_error(__METHOD__, "Error creating purchase order: $purchase");}
			
			$invoice_number=$purchase['invoice_number'];
			$order_token=$this->accounting_parando->make_order_token($invoice_number);

			// PAYPAL Express Checkout Controller Setup

		// Clear cart from session userdata
		$this->session->unset_userdata('shopping_cart');

			// Set cart data into session
			$cart=$this->accounting_parando->purchase_item_cart($purchase);
			$cart['member_first_name'] = $first_name;
			$cart['member_last_name'] = $last_name;
			$cart['contact_email'] = $contact_email;
			$cart['invoice_number'] = $invoice_number;
			$cart['order_token'] = $order_token;
			$cart['fulfillment_url'] = site_url(strtolower( get_class($this)) . "/credit_paypal_payment/$order_token");

			$this->session->set_userdata('shopping_cart', $cart);
			
			
//$this->die_error(__METHOD__,"Purchase: " . print_r($purchase,true));
			
			redirect(site_url('express_checkout/SetExpressCheckout'));

		} else if( $submit == 'no' ){
			$this->member_session_init();
			redirect(base_url());
		}
	}	
	

	//////////////////////////////////////////////////////////////////////
	// Public controller generic credit/fulfillment
	
	
	
	public function credit_paypal_payment($token){
	
		$rc = $this->accounting_parando->credit_paypal_payment($token,[$this,'_fulfill_all'],__METHOD__);  

		if($rc!='OK' && $rc!='DUPLICATE'){$this->die_error(__METHOD__,$rc);}

			$result = $this->accounting_parando->check_order_token($token);
			if(is_array($result)){
				list($member_id, $token_time, $hash)=$result;
// 				$invoice_number="$member_id-$token_time";
// 				$member=$this->model_parando->get_member($member_id);
// 				$this->multipage->load_memdata_into_session($member);
			}else{
				$this->die_error(__METHOD__,"Bad token: $result");
			}
			
		$profile_edit_url=$this->model_parando->make_profile_edit_url($member_id);
		$contact_text=$this->model_parando->get_member_contact_text($member_id);
		
		$this->multipage->set_data_array(compact('contact_text','profile_edit_url'));  // Better if these were not in the session
																						// Maybe pages need body_init() method
																						// In this case, the register_done page
																						// needs these variables

		
		$email_rc=true;
		if($rc!='DUPLICATE') $email_rc=$this->send_confirmation_email();		

		if($email_rc!==true){$this->die_error(__METHOD__, "Error sending confirmation emails:\n".$rc);}



		
		$this->multipage->set_current_page('payment_credited'); 
		$this->multipage->view_current_page();
		$this->member_session_init(); // no turning back
		
	}

	public function _fulfill_all($purchase){
		$purchase_id=$purchase['id'];
		$items=$this->accounting_parando->get_purchase_items($purchase_id);
		if(!empty($items)){
			foreach($items as $i){
				$sku = $i['sku'];
				$product=$this->accounting_parando->get_product($sku);
				if(empty($product) || !isset($product['fulfill_as'])) return "Can't find fulfullment data for sku=$sku";				
				$method = $product['fulfill_as'];				
				$fulfillment_method = 'fulfill_' . $method;
				
				if(array_key_exists($method,$this->known_fulfillment_methods) && method_exists($this,$fulfillment_method)){
					$rc = $this->{$fulfillment_method}($purchase, $i);
					if('OK'!=$rc){return $rc;}
				}else{
					return "No known method to fulfill sku: $sku; did you set the 'fulfil_as' field? Known methods are: " . 
							implode(', ',array_keys($this->known_fulfillment_methods));
				}
			}
		}else{
			return "No items in purchase ID $purchase_id";
		}
		return "OK";
	}
	
	public $known_fulfillment_methods=['shipped_item'=>true,'nothing'=>true];
		
	private function fulfill_shipped_item($purchase, $item){
	
		$purchase_id=$purchase['id'];
		$catalog_id = $item['catalog_id'];
		$value = $item['value'];
		$quantity = $item['quantity'];
		$unit_price = $item['unit_price'];
		$member_id = $purchase['member_id'];
		$shipping_method = $purchase['shipping_method'];
			
		$rc=$this->db->insert('shipping_queue',compact('catalog_id','member_id','purchase_id','value','quantity','shipping_method'));
		if(FALSE==$rc){$this->die_error(__METHOD__,"Failed to enter purchase into log.");}
		
		return 'OK';

	}

	private function fulfill_nothing($purchase, $item){  // nothing to do for this
	
		return 'OK';

	}
	
    private function get_client_ip()  
    { 
       $ipaddress = ''; 
        if (getenv('HTTP_CLIENT_IP')) 
            $ipaddress = getenv('HTTP_CLIENT_IP'); 
        else if(getenv('HTTP_X_FORWARDED_FOR')) 
            $ipaddress = getenv('HTTP_X_FORWARDED_FOR'); 
        else if(getenv('HTTP_X_FORWARDED')) 
            $ipaddress = getenv('HTTP_X_FORWARDED'); 
        else if(getenv('HTTP_FORWARDED_FOR')) 
            $ipaddress = getenv('HTTP_FORWARDED_FOR'); 
        else if(getenv('HTTP_FORWARDED')) 
           $ipaddress = getenv('HTTP_FORWARDED'); 
        else if(getenv('REMOTE_ADDR')) 
            $ipaddress = getenv('REMOTE_ADDR'); 
        else 
            $ipaddress = 'UNKNOWN'; 
     
       return $ipaddress; 
    }  


	private function send_confirmation_email(){
		
		extract($this->multipage->get_data_all()); // might consider how this could be done without the session variables

		$rba=$this->model_parando->get_rba();
		$rba_name=$rba['first_name'] . ' ' . $rba['last_name'];
		$rba_email=$rba['m_email'];

		$webmaster=$this->model_parando->get_webmaster(); extract($webmaster);
		$noreply_email=$this->model_parando->noreply_email;
		
		$client_ip=$_SERVER["REMOTE_ADDR"];
		$client_browser=$_SERVER["HTTP_USER_AGENT"];
		

		$this_organization = $this->model_parando->organization;
    $login_url=site_url("login");
		$this_controller=strtoupper(get_class($this));

		$profile_edit_url=$this->model_parando->make_profile_edit_url($member_id);
		$contact_text=$this->model_parando->get_member_contact_text($member_id);
		
		$this->multipage->set_data_array(compact('contact_text','cancel_url','profile_edit_url'));  // Better if these were not in the session

//		$member=$this->model_parando->get_member($member_id);


		$shipping_message=<<<EOT
The items are to be shipped from $this_organization to 
the mailing address of record which is

  '{$first_name} {$last_name}'
  '{$contact_street}'
  '{$contact_city} {$contact_state} {$contact_zip}'
  '{$contact_country}'
EOT;
		$correct_address=<<<EOT
		
PLEASE CHECK THAT ADDRESS CAREFULLY.

If the above address is not correct, you can correct 
your information by editing your rider profile 
by following this URL:

  $profile_edit_url

That URL has a timeout. If it doesn't work, 
at any time you can edit your rider profile data by 
using  your password to log in to your 
$this_organization Member Page $login_url
or by emailing the RBA at $rba_email by simply replying to 
this email.
EOT;
		
		$order_info=<<<EOT
Name:	{$first_name} {$last_name}
Amount Paid:      \${$amount_paid}
Order ID:         $invoice_number
Shipping Method:  $shipping_method
Entered On:       {$entered_on}

Items: 

$purchase_items_text

Item Total:          $item_total_price_text
Shipping Fee:        $order_shipping_fee_text
Grand Total:         $total_due_text

EOT;

		$rider_message=<<<EOT
Your payment of \$$amount_paid has been credited. Thank You! Payment credit
is contingent on receipt of funds through the payment service. A description
of your order is as follows:

{$order_info}

{$shipping_message}

{$correct_address}

EOT;

		$rba_message=<<<EOT
A payment of \$$amount_paid has been credited. Payment credit
is contingent on receipt of funds through the payment service. 
A description of the order is as follows:

{$order_info}

{$shipping_message}

Client IP:           $client_ip
Client Browser:      $client_browser

EOT;


		$contact_rba=<<<EOT

If you have any questions or concerns, contact $this_organization
at $rba_email by simply replying to this email. 
EOT;

		$contact_rider=<<<EOT

You can contact the rider $first_name $last_name at $contact_email
by simply replying to this email. 
EOT;

		$rc=true;

		$this->email->from($noreply_email, $rba_name);
		$this->email->reply_to($rba_email, $rba_name);
		$this->email->to($contact_email);
		$this->email->subject($this_organization . " Order Confirmation");
		$this->email->message($rider_message.$contact_rba);
		if($this->send_email) $this->email->send();

		$this->email->from($noreply_email, "$first_name $last_name");
		$this->email->reply_to($contact_email, "$first_name $last_name");
//		$this->email->to($webmaster_email);
		$this->email->to([$rba_email, $webmaster_email]);
		$this->email->subject("[$this_controller] " . $this_organization . " Order received");
		$this->email->message($rba_message.$contact_rider);
		if($this->send_email) $rc = $this->email->send();

		return $rc;
	}
	


// GENERIC DECISION LOGIC FOR KNOWN/UNKNOWN MEMBER

	private function find_and_process_member($action_callback, $action_page, $unknown_page=null, $activity="continue"){
		$cto=$this->model_parando->contact_this_organization_link();
		$error_comment=" You cannot $activity until this is fixed. If you have questions about this error, $cto.";

		$rusa_id=$this->input->post('rusa_id');
		$first_name=$this->input->post('first_name');
		$last_name=$this->input->post('last_name');

		$this->load_unknown($first_name,$last_name,$rusa_id);

		$found=$this->model_parando->find_member($first_name,$last_name,$rusa_id);	
		
		switch($found['result']){
		  case 'full_match_rusa':
			$q=$found['query'];
			$member=$q->row_array();
			$rc = $this->{$action_callback}($member);
			if($rc){
				$this->multipage->set_current_page($action_page);
			}else{
				$this->die_error('Database Error', "Failed to $activity." . $error_comment);
			}
			break;
		  case 'duplicate_entry':
			$this->die_error('Database Error','Your info matches two entries in our database.'.$error_comment);
			break;
		  case 'unknown':
			if(!empty($unknown_page)){
				$this->multipage->set_current_page($unknown_page);
			}else{
				$this->die_error('Database Error','Your info is not in our database.'.$error_comment);
			}
			break;
		  default:
			$this->multipage->set_current_page('pick_member'); 
			$q=$found['query'];
			$found_multiple = $q->result_array();
			$this->multipage->set_data_array([
				'found_multiple' => $found_multiple,
				'find_and_process_parameters' => [
					'action_callback' => $action_callback,
					'action_page'=>$action_page,
					'unknown_page'=>$unknown_page,
					'activity'=>$activity 
				]
			]);
			break;
		}
	}
	
  private function load_unknown($first_name,$last_name,$rusa_id){

			// loads session with what we do know about an un-matched member.
			// Called prior to the member picker or unknown member data entry

			if(!empty($rusa_id)){
				$rms="Yes (". $rusa_id . ")";
			}else{
				$rms="No";
			}

			$session_data=array(
				'membership'	=> 'unknown',
				'rms'		=> $rms,
				'rusa_id'	=> $rusa_id,
				'first_name'	=> $first_name,
				'last_name'	=> $last_name,
				'expired'	=> null,
				'member_id'	=> null,
				'administrator'	=> null,
				'expires'	=> null
			);
			$this->multipage->set_data_array($session_data);
	  
	}


// PICK MEMBER 

	public function _pick_member(){

		// Processing method for the register_pick_member page 
		// Requires Find and Process parms in session
		// will branch to $unknown_page if no existing member of the same name/rusa
		// or do the action callback (and branch to action page) if there's now a full member match

		$fpp=$this->multipage->get_data('find_and_process_parameters');
		if(empty($fpp)){$this->die_error('_pick_member', 'Required parameters NOT in session.');}
		extract($fpp);

 		$this->form_validation->set_rules('member_id','Member Selection', 'trim|required', array('required'=>'You must either select a member above that matches you, or select below that none are you.') );

		$cto=$this->model_parando->contact_this_organization_link();
		$error_comment=" You cannot $activity untill this is fixed. If you have questions about this error, $cto.";
		
	  	$member_id=$this->input->post('member_id');
		$rusa_id= $this->multipage->get_data('rusa_id');
		$first_name=$this->multipage->get_data('first_name');
		$last_name= $this->multipage->get_data('last_name');
	  
	  if($this->form_validation->run()!=FALSE){
			if($member_id == "not_me"){
				$check=$this->model_parando->check_unknown($first_name,$last_name,$rusa_id);
				switch($check){
					case "existing_first_last":
						$this->die_error('Existing Member',
							"There is already a rider in our database with exactly the same first and last name as you have entered, yet you say this is not you. 
							$error_comment.");   
						break;
					case "existing_rusa":
						$this->die_error('Existing Member',
							"There is already a rider in our database with exactly the same RUSA ID as you have entered, yet you say this is not you. 
							$error_comment.");   
						break;
					case "ok":
						if(!empty($unknown_page)){
							$this->multipage->set_current_page($unknown_page);
						}else{
							$this->die_error('Database Error','Your info is not in our database.'.$error_comment);
						}
						break;
				}
			}else{
				$mem=$this->model_parando->get_member($member_id);
				if(empty($mem)){
					$this->die_error('_pick_member',"No such member ID ($member_id)");
				}elseif(!empty($action_callback) && method_exists($this,$action_callback)){
					$this->{$action_callback}($mem);
					$this->multipage->set_current_page($action_page);
				}elseif(!empty($action_callback)){
					$this->die_error('_pick_member',"No such action callback ($action_callback)");
				}
			}
		}
	}


	public function _pick_member_form_body(){

		// Callback that generates a radio button form for picking 
		// from multiple matched members

		$found=$this->multipage->get_data('found_multiple');

		$num_rows=sizeof($found);
		if($num_rows>1){
			$entries="one or more entries";
			$the_entry_that="the entry that";
			$not_me="None of those are me";
		}else{
			$entries="one entry";
			$the_entry_that="it if it";
			$not_me="That is not me.";
		}

		$body = <<<EOT
<p>Your info partially matches $entries in our rider database. Please select $the_entry_that exactly matches you, or select <em>not me</em></p><br>
<div class=roster>
<table width=100%>

EOT;

		foreach ($found as $row)
		{
			$member_id=$row['id'];
			$rusa_id=(!empty($row['rusa_id']) && $row['rusa_id']>0)?("RUSA " . $row['rusa_id']):"";
			if($row['m_country']=='US'){
				$state = $row['m_state'];
			}else{
				$country_fullname=$this->model_parando->get_countries();
				$state = $country_fullname[$row['m_country']];
			}
	    $body .= "<tr>";
	    $body .= "<td><input type='radio' name='member_id' value=$member_id></td>";
	    $body .= "<td>" . $row['first_name'] . "</td>";
	    $body .= "<td>" . $row['last_name'] . "</td>";
	    $body .= "<td>" . $row['m_city'] . "</td>";
	    $body .= "<td>" . $state . "</td>";
	    $body .= "<td>" . $rusa_id . "</td>";
	    $body .= "</tr>";
		}

		$body .= "</table></div>";

	if(!empty(form_error('member_id'))){
	  $body .= "<div class=verror>" . form_error('member_id') . "</div>";
	}

	$body .= "<p><input type='radio' name='member_id' value='not_me'>$not_me</p>";


		return $body;
	}

	
	public function _get_new_member_data(){
	
	// Processing method for get_new_member_data page
	// Stores aquired data into database and proceeds with known member
	
		// Stuff already in session
		$sdata = $this->multipage->get_data_array(['rusa_id','first_name','last_name']);
		
		$pw = $this->input->post('password');
		$pw_hash = (empty($pw))?"":password_hash($pw,PASSWORD_DEFAULT);
		
		// Stuff added by contact form
		$fdata = array(
			'm_street'		=> $this->input->post('contact_street'),
			'm_city'		=> $this->input->post('contact_city'),
			'm_state'		=> $this->input->post('contact_state'),
			'm_zip'			=> $this->input->post('contact_zip'),
			'm_country'			=> $this->input->post('contact_country'),
			'tracker'			=> $this->input->post('contact_tracker'),
			'm_phone'		=> $this->input->post('contact_phone'),
			'm_phone_alt'		=> $this->input->post('contact_phone_alt'),
			'm_email'		=> $this->input->post('contact_email'),
			'emergency_name'	=> $this->input->post('contact_emergency_name'),
			'emergency_phone'	=> $this->input->post('contact_emergency_phone'),
			'emergency_phone_alt'	=> $this->input->post('contact_emergency_phone_alt'),
			'password' => $pw_hash
		);
		$this->db->insert('member', array_merge($sdata,$fdata));	// Add new member to database

		$member = $this->model_parando->get_member_firstlast($sdata['first_name'],$sdata['last_name']);
		if(empty($member)){$this->die_error('get_new_member_data',"This can't happen -- failed member readback.");}

		$this->load_memdata_into_session($member);  

	}

	private function load_memdata_into_session($mem){

		$memdata=$this->model_parando->compute_memdata($mem);

		$this->multipage->set_data_array($memdata);
	
		return true;

	}


}


?>
