<?php defined('BASEPATH') OR exit('No direct script access allowed');

 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

// Brevet Card Rendering Class /////////////////////////////////////////////////////////////////////

require_once('Myfpdf.php');

class Controlqr extends Myfpdf {

	// distances	
	const km_per_mi = 1.609344;
	const ft_per_m = 3.2808398950131;
	const m_per_km = 1000.0;

	// paths
	const cuesheet_path="/home/tomr/public_html/assets/parando/cuesheets";
	const cuesheet_baseurl="https://parando.org/assets/parando/cuesheets";

	// Constructor parameters
	private $params;
	private $orientation='P'; // P - portrait; L - landscape
	private $unit='in'; // in, cm, mm, pt NOT SETTABLE
	private $size='letter'; // A3, A4, A5, letter, legal
  
	// Configuration variables
	public $controle_index_origin=1;  	// if set to 0 the first 
										// controle will be START
										// the first intermediate will be number 1
										// and the final FINISH
	public $logo_url="https://parando.org/images/parando3D.png";
	public $logo_width=0.75; // fraction of frame width
	public $logo_height=0.31; // fraction of frame height
	public $logo_center_y=0.27; // fraction of overall card height
	

	// layout constants
	public $vmargin=1.0;  // space between top/bottom page edge and cues
	public $hmargin=1.0;  // space between left/right page edge and cues
	public $vmargin_L=1.75;  // space between top/bottom page edge and cues
	public $hmargin_L=0.75;  // space between left/right page edge and cues
	public $vmargin_P=1.0;  // space between top/bottom page edge and cues
	public $hmargin_P=1.0;  // space between left/right page edge and cues
	public $baseline_skip = 0.15;  // Space between text lines
	public $qr_width = 0.75; // as fraction of page width
	
	public $qrs_per_page = 1; // Supported 1 or 2
	
    // Other class variables
	private $page_height;
	private $page_width;
	private $event;
	
		protected $CI;

		protected function die_error($heading,$message){
			$this->CI->load->view('errors/html/error_general',array('title'=>$heading, 'heading'=>$heading,'message'=> $message));
			$this->CI->output->_display();
			exit();
		}


	public function __construct($params=[]){
			$this->CI =& get_instance();			
		$this->params=$params;
		$this->orientation=(isset($params['orientation']))?$params['orientation']:'P';
		$this->size=(isset($params['size']))?$params['size']:'letter';
		parent::__construct($this->orientation, $this->unit, $this->size);

		$this->route_event=$params['route_event'];

		$this->page_height = $this->GetPageHeight();
		$this->page_width = $this->GetPageWidth();
		$this->SetMargins(0,0,0);
		switch($this->orientation){
			case 'L':
				$this->vmargin=$this->vmargin_L;
				$this->hmargin=$this->hmargin_L;
				break;
			case 'P':
				$this->vmargin=$this->vmargin_P;
				$this->hmargin=$this->hmargin_P;
				break;
		}
		$this->SetAutoPageBreak(false,0);
		$this->font_normal();
		$this->SetTitle($this->route_event['event_name_dist']);
		$this->SetSubject($this->route_event['event_name_dist']);
		$this->SetCreator('Cue Wizard rendering software');
		$this->SetAuthor($this->route_event['club_name']);
	}
	
	public function draw_qr_pages($controles){
		$x = $this->hmargin;
		$y = $this->vmargin;
		$w = $this->page_width - 2*$this->hmargin;
		$h = $this->page_height - 2*$this->vmargin;
		
		// Title Block
		$this->AddPage();
		$this->SetXY($x,$y);
		$this->font_bigprint();
		$this->font_italic();
		$this->Cell($w, 2*$this->baseline_skip, $this->route_event['club_name'], 0, 2, 'C');
		$this->font_bold();
		$this->MultiCell($w, 2*$this->baseline_skip, $this->route_event['event_name_dist'], 0, 'C');
		$this->font_italic();
		$this->font_size(1.2);
		$this->SetX($x);
		$this->Cell($w, 2*$this->baseline_skip, $this->route_event['event_datetime_str_verbose'], 0, 2, 'C');

        $this->Image($this->route_event['club_logo_url'], $x+($w-$w*$this->qr_width)/2, null, $w*$this->qr_width);


		$ncontroles=count($controles);

		for($i=0; $i<$ncontroles; $i++){
			$cd_mi=round($controles[$i]['d']/(rwgps::m_per_km*rwgps::km_per_mi),1);
			$cd_km=round($controles[$i]['d']/(rwgps::m_per_km),1);
			$is_start=(isset($controles[$i]['start']))?" [START]":"";
			$is_finish=(isset($controles[$i]['finish']))?" [FINISH]":"";
			$controle_num=$i+1;
			$open_str=$controles[$i]['open']->format('m-d H:i');
			$close_str=$controles[$i]['close']->format('m-d H:i');
			// $ca=print_r($controles[$i]['attributes'], true);
			// $ca=$this->format_attributes($controles[$i]['attributes']); 
			// $cn=(isset($controles[$i]['n']))?$controles[$i]['n']:"";
//         return compact('info_code','qrcode_dir','qrcode_file','qrcode_url','qrcode_checkin_url');
 			$info_code=$controles[$i]['info_code']; 
 			$qrcode_file=$controles[$i]['qrcode_file']; 
 			$control_name=(!empty($controles[$i]['attributes']['name']))?$controles[$i]['attributes']['name']:"";
// 			$qrcode_basefile="$info_code.png";
// 			$qrcode_file=$this->params['qrimg_path'] . '/' . $qrcode_basefile;
// 			$qrcode_link="<A HREF=" . $this->params['qrimg_url'] . "/$qrcode_basefile>$info_code</A>";
			$text=<<<EOT
CONTROL $controle_num$is_start$is_finish ($cd_mi mi)
$control_name
OPEN: $open_str  CLOSE: $close_str 
EOT;
			$this->AddPage();
			
			$y_inc=$w*$this->qr_width + 8*$this->baseline_skip;			
			$y_top=($this->qrs_per_page>1)?$y:$this->page_height/2 - $y_inc/2;
			
			for($j=0; $j<$this->qrs_per_page; $j++, $y_top+=$y_inc){
				$this->SetXY($x,$y_top);
				$this->font_normalprint();
				$this->font_plain();
				$this->MultiCell($w, 2*$this->baseline_skip, $text, 0, 'C');
				$this->font_bold();
				$this->font_size(3.16);
				$this->Image($qrcode_file, $x+($w-$w*$this->qr_width)/2, null, $w*$this->qr_width);
				$this->SetX($x);
				$this->MultiCell($w, 2*$this->baseline_skip, "$info_code", 0, 'C');
			}

// 			$this->SetXY($x,$this->page_height/2);
// 			$this->font_normalprint();
// 			$this->font_plain();
// 			$this->MultiCell($w, 2*$this->baseline_skip, $text, 0, 'C');
// 			$this->font_bold();
// 			$this->font_size(3.16);
// 			$this->Image($qrcode_file, $x+($w-$w*$this->qr_width)/2, null, $w*$this->qr_width);
// 			$this->SetX($x);
// 			$this->MultiCell($w, 2*$this->baseline_skip, "$info_code", 0, 'C');
			
		}


	}

//	const QRIMG_PATH="/home/tomr/public_html/assets/uploads/files";
	const QRIMG_PATH="/home/tomr/public_html/assets/parando/qrcodes";
//	const QRIMG_URL="https://parando.org/assets/uploads/files";
	const QRIMG_URL="https://parando.org/assets/parando/qrcodes";
	const QR_CHECKIN_URL="qr/ck";
	const QRIMG_EXT="png";

	public function make_qr_paths($route_event, $control_number=null){
		$event_tagname=$this->CI->model_parando->make_event_tagname($route_event); // Wasteful to recompute, but whatever...
		$qrcode_subdir="$event_tagname";
        $qrcode_dir=self::QRIMG_PATH . "/$qrcode_subdir";
        if($control_number!==null){
            $info_code=$this->CI->model_parando->info_code($route_event,$control_number);
            $qrcode_basefile="C$control_number-$info_code." . self::QRIMG_EXT;
            $qrcode_file=self::QRIMG_PATH . "/$qrcode_subdir/$qrcode_basefile";
            $qrcode_url=self::QRIMG_URL . "/$qrcode_subdir/$qrcode_basefile"; 
            $event_id=$route_event['event_id'];
            $qrcode_checkin_url=site_url(self::QR_CHECKIN_URL."/$event_id/$control_number/$info_code");
        }
        return compact('info_code','qrcode_dir','qrcode_file','qrcode_url','qrcode_checkin_url');
	}
	
	public function make_qr_images($route_event, $controles, $always_write_file=false){
	
		//extract($route_event);
		
		$ncontroles=count($controles);
 		
		for($i=0; $i<$ncontroles; $i++){
            $c=$controles[$i];
            $control_number=$i+1;
            extract($this->make_qr_paths($route_event,$control_number));
			if(!file_exists($qrcode_dir)) mkdir($qrcode_dir);
			$this->CI->load->library('ciqrcode');
			if($always_write_file || !file_exists($qrcode_file)){
				$params['data'] = $qrcode_checkin_url;
				$params['level'] = 'H';
				$params['size'] = 8;
				$params['savename'] =$qrcode_file;
				if(!$this->CI->ciqrcode->generate($params))
					$this->die_error(__METHOD__, "Could not generate QR code in $qrcode_file");
			}
		}	
	}
	
	public function compute_info_codes($route_event,&$controles){		
		for($i=0; $i<count($controles); $i++){ 
            $control_number=$i+1;
            $qrp=$this->make_qr_paths($route_event,$control_number);
			$controles[$i]['info_code']=$qrp['info_code'];
			$controles[$i]['qrcode_checkin_url']=$qrp['qrcode_checkin_url'];
			$controles[$i]['qrcode_url']=$qrp['qrcode_url'];
			$controles[$i]['qrcode_file']=$qrp['qrcode_file'];
		}
	}


	
}

