<?php defined('BASEPATH') OR exit('No direct script access allowed');
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

// Mailing Labels Rendering Class /////////////////////////////////////////////////////////////////////

require_once('application/libraries/Myfpdf.php');

class Labels extends Myfpdf {

	// Constructor parameters
	private $orientation='P'; // P - portrait; L - landscape
	private $unit='in'; // in, cm, mm, pt NOT SETTABLE
	private $size='letter'; // A3, A4, A5, letter, legal
	private $label_model='avery5160';
	private $organization='';
  
	// Configuration variables
	public $PAR_logo_url="https://parando.org/images/RUSA_cert_graphic.png";
	public $ACP_logo_url="https://parando.org/images/ACP_cert_graphic.png";
	public $logo_width=0.95; // fraction of frame width
	public $logo_height=0.95; // fraction of frame height
	public $logo_center_y=0.1; // fraction of overall page height
	public $logo_center_x=0.1; // fraction of overall page width
	
	
	// Define label sheet layouts
	private $label_data=[
		'avery5160'=>[
			'lwidth'=>2.625,
			'lheight'=>1.0,
			'hmargin'=>0.1,
			'vmargin'=>0.1,
			'rows'=>10,
			'cols'=>3,
			'h0'=>0.2,
			'v0'=>0.532,
			'vpitch'=>1.0,
			'hpitch'=>2.75
		]
	];


    // Other class variables
	private $page_height;
	private $page_width;
	private $params;
	private $title="Labels";

	public function __construct($params){
		if(empty($params)) trigger_error("Must specify parameters.",E_USER_ERROR);
		$this->params=$params;

		if(isset($params['orientation'])) $this->orientation=$params['orientation'];
		if(isset($params['size'])) $this->size=$params['size'];
		if(isset($params['label_model'])) $this->label_model=$params['label_model'];
		if(isset($params['this_organization'])) $this->organization=$params['this_organization'];

		parent::__construct($this->orientation, $this->unit, $this->size);
		$this->page_height = $this->GetPageHeight();
		$this->page_width = $this->GetPageWidth();
		$this->SetMargins(0,0,0);

		$this->SetAutoPageBreak(false,0);
		$this->font_normal();
		$this->SetTitle($this->title);
		$this->SetSubject($this->title);
		$this->SetCreator($this->organization . ' Mailing Label rendering software');
		$this->SetAuthor($this->organization);
		
		$this->lspec=$this->label_data[$this->label_model];
	}
	
	public function render_sheet(){
		extract($this->lspec);
		$x = $h0;
		$y = $v0;
		$r=$c=$p=0;
		foreach($this->params['cert_table'] as $cert){
			$this->render_cert_label($cert,$x,$y);
			$c = ($c+1)%$cols;
			$x = $h0 + $c*$hpitch;
			if($c==0){
				$r = ($r+1)%$rows;
				$y = $v0 + $r*$vpitch;
				if($r==0){
					$p++;
					$this->AddPage();
					$x = $h0;
					$y = $v0;
				}
			}
		}
	}
	
	private function render_cert_label($c,$x,$y){
		extract($this->lspec);
		extract($c);
		$x+=$hmargin;
		$y+=$vmargin;
		$w=$lwidth-2*$hmargin;
		$h=$lheight-2*$vmargin;
		$dist=($type=='fleche')?"Fleche":"{$distance}K";
		$txt="$first_name $last_name\nRUSA #$rusa_id\n$date $dist\nFINISH $result"; 
		
		switch($sanction){
			case 'ACP':
				$this->Image($this->ACP_logo_url,$x,$y,$w,$h);
				break;
			case 'RUSA':
				$this->Image($this->PAR_logo_url,$x,$y,$w,$h);
				break;
			default:
				trigger_error("Unknown sanction '$sanction'. Can't make label.", E_USER_ERROR);
				break;
		}
		//$txt=implode(' ',array_values($cert));
		
		$text_font_size=0.6;
		$cert_font_size=0.75;
		
		$this->Rect($x,$y,$w,$h);
		$this->SetXY($x+$w/2,$y+$this->baseline_skip);
		$this->font_size($text_font_size);
		$this->MultiCell($w/2,$text_font_size*$this->baseline_skip,$txt,0,'C');

		$this->SetXY($x+0.7*$w,$y+0.9*$h);
		$this->font_size($cert_font_size);
		$this->Cell(0,0,$cert);
	

	}

	

}

?>
