<?php defined('BASEPATH') OR exit('No direct script access allowed');

 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

// MULTI-PAGE FORMS SYSTEM

require_once('Randosession.php');

class Multipage extends Randosession {

	
	// CLASS VARIABLES

	protected $current_page_tag='current_page';
	protected $page_stack_tag='page_stack';
	protected $initial_page_tag='initial';

	protected $current_page;
	protected $page;
//	protected $subject='Undefined';
	
	public $controller;
	
	protected $view_header;
	protected $view_footer;
	protected $view_body;
	protected $view_page_styles=[];

  protected $debug=false;


	public function __construct($params=[]){
	
		if(empty($params)) $params = ['tag' => 'multipage_session_data'];
		parent::__construct($params);

		$d = $this->get_data_all();
		if(empty($d)){		
			$this->init();
		}

		$this->current_page = $this->get_data($this->current_page_tag);

	}
	
	// ERROR VIEW LOADER

  protected function debugtext($t,$c=""){
		return ($this->debug)?$t:$c;
  }

	private function Xdie_error($heading,$message){
		$this->CI->model_parando->die_error( '['. get_class($this). ']: ' . $heading, $message . 
			$this->debugtext('<br>Session Data: <br><pre>' . print_r($this->CI->session->userdata,true) . '</pre>'));
	}

	public function init($d=[]){
		$cp=[$this->current_page_tag => $this->initial_page_tag];
		$this->current_page = $this->initial_page_tag;
		parent::init(array_merge($cp,$d));
	}
	
	public function define_pages($p){
		$this->page = $p;
	}
	
//	public function set_subject($s){
//		$this->subject = $s;
//	}

	public function set_view($body,$header=null,$footer=null,$page_styles=[]){
		$this->view_header = $header;
		$this->view_footer = $footer;
		$this->view_body = $body;
		$this->view_page_styles = $page_styles;
	}

	
	public function set_controller($c){
		$this->controller = $c;
	}
	
	public function set_current_page($p){
		$this->current_page=$p;
		$this->set_data($this->current_page_tag,$p);
	}

	public function call_page($plist, $return=null){
        if(empty($plist)) $this->die_error(__METHOD__, "Empty called page or call list.");
		$page_stack=$this->get_data($this->page_stack_tag);
		if(empty($page_stack)){$page_stack=[];}
		if(is_array($plist)){
            $p=array_shift($plist);
            $page_stack=array_merge($page_stack,$plist);
		}else{
            $p=$plist;
        }
        $page_stack[]=(empty($return))?$this->current_page:$return;
		$this->set_data($this->page_stack_tag,$page_stack);
		$this->set_current_page($p);
		
		// $this->die_error(__METHOD__, "p=$p, return=$return, plist=" . print_r($plist,true) .", page_stack=" . print_r($page_stack,true));
	}

	public function return_page(){
		$page_stack=$this->get_data($this->page_stack_tag);
		$r=array_shift($page_stack);
		// $this->die_error(__METHOD__, "page_stack=" . print_r($page_stack,true));
		if(empty($r)){$this->die_error(__METHOD__,"Page stack empty");}
		$this->set_data($this->page_stack_tag,$page_stack);
		$this->set_current_page($r);		
	}

	public function page_expired_q(){	
		if(!isset($this->current_page)){
			$this->die_error('process_page', 'Current page property is not set in this object.');
		}
		$pt = $this->current_page;
		return ($pt == $this->initial_page_tag);
			
	}
	
	public function process_page(){
			
		if(!isset($this->current_page)){
			$this->die_error('process_page', 'Current page property is not set in this object.');
		}
		$pt = $this->current_page;
		if($pt == $this->initial_page_tag){
			$this->die_error('process_page', "Current page has expired.");
		}
		if(!isset($this->page[$pt])){
			$this->die_error('process_page', "Current page '$pt' not defined.");
		}
		$p=$this->page[$pt];
		
		if('POST' == $this->CI->input->method(TRUE) && 
				(empty($p['fields']) || 
                    $this->CI->input->post('submit') == 'cancel' || // bypass rule checks for magic 'cancel', need to handle this in explicit method
                    $this->validate_form($p['fields']))){

			
			if(!empty($p['next_page'])){
				if($p['next_page']=='RETURN'){
					$this->return_page();
				}elseif(preg_match('/FIELD:(\w+)/',$p['next_page'],$m)){
					$this->set_current_page($this->CI->input->post($m[1]));  // possible security issue
				}else{
					$this->set_current_page($p['next_page']);
				}
			}
			
			if(method_exists($this->CI,'_'.$pt)){  
				$this->CI->{'_'.$pt}();  // may reset current_page
			}

		}
		
		$this->view_current_page();
	  
	}


	private function validate_form($field_list){
		$rule_count=0;
		$rule_list=[];
		if(!empty($field_list)){
			foreach ($field_list as $f){
				if(isset($f[0], $f[1], $f[4])){
					$ff = [ $f[0], $f[1], $f[4] ];
					$rule_list[]=$ff;
					$this->CI->form_validation->set_rules(...$ff);
					$rule_count++;
				}
			}
		}
		
		
		if($rule_count>0){
            $v=$this->CI->form_validation->run();
           // if(!$v) $this->die_error(__METHOD__,print_r($rule_list,true));
            return $v;
        }else{
            return TRUE;
        }
	}


  // VIEW LOADER FUNCTIONS

	private function load_view($v,$data=null){
			$sd = $this->get_data_all();
			if(!empty($this->view_header)){
				$this->CI->load->view($this->view_header,array_merge($sd,$data,['page_styles'=>$this->view_page_styles]));
			}
			if(empty($data)){
				$this->CI->load->view($v,$sd);
			}else{
				$this->CI->load->view($v,array_merge($sd,$data));
			}
			if(!empty($this->view_footer)){
				$this->CI->load->view($this->view_footer,$sd);
			}
	}
	
	
	public function view_current_page(){
		
		if(!isset($this->current_page)){
			$this->die_error('view_current_page', 'The current page variable is not set in this object.');
		}
		$pt = $this->current_page;

		if(!isset($this->page[$this->current_page])){
			$this->die_error('process_page', "The current page tag '$pt' is not defined in the page definition array.");
		}
		$p=$this->page[$this->current_page];
		
		$buttons = (isset($p['submit'])) ? $p['submit'] : [['tag'=>'continue', 'text'=>'Continue']];

		if(!is_array($buttons)){
			$this->die_error('view_current_page','The "submit" property for this page (' . $this->current_page . ') must be an array.');
		}


		$form_submit = "";
		foreach ($buttons as $button){
			if(!array_key_exists('tag', $button) || !array_key_exists('text', $button)){
				$this->die_error('view_current_page','The "submit" property for this page (' . $this->current_page . ') must be an array of arrays with "tag" and "text" keys.');
			}
			$tag=$button['tag'];
			$text=$button['text'];
			$form_submit .=  "<button type='submit' name='submit' value='$tag'>$text</button>";
		}

		$title = (isset($p['title'])) ? $this->interpolate_strings($p['title']) : "Insert Title Here";
		$subject = (isset($p['subject'])) ? $p['subject'] : $title;
		$info_comment = (isset($p['info_comment'])) ? $p['info_comment'] : "";
		$comment = (isset($p['comment'])) ? ($this->interpolate_strings($p['comment'])) : "";
		$form_fineprint = (isset($p['form_fineprint'])) ? ($this->interpolate_strings($p['form_fineprint'])) : "";

		$form_show = (array_key_exists('show',$p)) ? $p['show'] : 'all';

		if(isset($p['fields'])){
			$fields =  $p['fields'];
			$form_body = $this->form_body($fields);
		}elseif(isset($p['form_body'])){
			if(is_string($p['form_body']) && method_exists($this->CI,'_'.$p['form_body'])){  
				$form_body = $this->CI->{'_'.$p['form_body']}();  
			}else{
				$form_body = $p['form_body'];
			}
		}else{
			$form_body = "";
		}


		$data = array(
			'info_comment'=>$info_comment,
		    'form_title'=>$title,
		    'form_subject'=>$subject,
			'form_comment'=>$comment,
			'form_action'=>site_url($this->controller . '/'),
			'form_body'=>$form_body,
			'form_submit'=>$form_submit,
			'form_fineprint'=>$form_fineprint,
			'form_show'=>$form_show,
			'form_debug'=>$this->debugtext(print_r($this->get_data_all(), true),"No Comment")
		);
		$view_body = (!empty($p['view_body']))?$p['view_body']:$this->view_body;
		$this->load_view($view_body,$data);
		
	}

	// Constructs a form group HTML paragraph from a record in the fields list. The order of list entries 
	// in the fields list must match the order of the parameters in this function. 
	// At least tag and label are required.
	
	public function form_group($tag, $label, $placeholder="", $required_q=true, $validation_rules="", $type="text", $options=[], $options_default=""){
		$text="";
		if(!empty(form_error($tag))){
		  $text .= "<div class=verror>" . form_error($tag) . "</div>";
		}
		$value=set_value($tag,$options_default);
		$required=($required_q)?"required":"";
		$asterisk=($required_q)?"*":"";
		if(is_string($options) && method_exists($this->CI,'_'.$options)){  
				$options = $this->CI->{'_'.$options}();  
		}
		switch($type){
			case 'select':
		if(!is_array($options)){
			$this->die_error(__METHOD__,'The "options" property for this page (' . $this->current_page . ') must be an array.<p> options=' .
print_r($options,true).'<p>');
		}
				$input ="<select name='$tag' value='$value' $required>";
				$ss=set_select($tag,'',true);
				$input.="<option value='' disabled selected>$placeholder</option>";
				//$input.="<!--".print_r($options,true)."-->";
				foreach ($options as $k => $v){
					$ss=set_select($tag,$k,(!empty($options_default) && $k==$options_default));
					$input.="<option value='$k' $ss>$v</option>";
				}
				$input.="</select>";
				break;
			case 'radio':
		if(!is_array($options)){
			$this->die_error(__METHOD__,'The "options" property for this page (' . $this->current_page . ') must be an array.<p> options=' .
print_r($options,true).'<p>');
		}
				$input ="<div class='indented'>";
				foreach ($options as $k => $v){
					$selected=($value==$k)?"checked":"";
					$input.="<div class='image-radio'><input type='radio' name='$tag' id='{$tag}_{$k}' value='$k' $selected><label for='{$tag}_{$k}'>$v</label></div>";
				}
				$input.="</div>";
				break;
			case 'password':
				$input="<input type='password' placeholder='$placeholder' name='$tag' value='$value' $required>";
				break;
			case 'textarea':
				$input="<textarea placeholder='$placeholder' name='$tag' maxlength=500 wrap=soft width=100% height=5 $required>$value</textarea>";
				break;
			default:
				$input="<input type='text' placeholder='$placeholder' name='$tag' value='$value' $required>";
				break;
		}
		$text .= <<<EOT
		<div class=form-group>
		  <label for="{$tag}">$label$asterisk</label>    
		  $input
		</div>
EOT;
		return $text;
	}

	
	// Makes a form group of every entry in the field_list 
  // and returns them all concatinated as the form body.
  // Each field list entry is assumed to be a list of 
  // parameters for the form_group function.

	private function form_body($field_list){
		$body='';
		foreach ($field_list as $f){
		  $body .= $this->form_group(...$f);
		}
		return $body;
	}
	
	
}
