<?php defined('BASEPATH') OR exit('No direct script access allowed');
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

// Post Card Rendering Class /////////////////////////////////////////////////////////////////////

require_once('fpdf/fpdf.php');

class Postcard extends FPDF {

	// colors
	const BLACK=[0,0,0];
	const LIGHT_GRAY=[192,192,192];
	const RED=[255,0,0];

	// Constructor parameters
	private $orientation='L'; // P - portrait; L - landscape
	private $unit='in'; // in, cm, mm, pt NOT SETTABLE
	private $size='letter'; // A3, A4, A5, letter, legal
  
	// Configuration variables
	public $logo_width=0.6; // fraction of frame width
	public $logo_height=0.6; // fraction of frame height
	

	// layout constants
	
	private $nrows=2; // Fixed 2x2 layout of postcards in landscape
	private $ncols=2; 
	
	public $baseline_skip = 0.15;  // Space between text lines
	public $thin_width=0.01; // Thin lines
	public $thick_width=0.03; // Thick lines
	public $font_normal='Helvetica';	// default font family
	public $font_fixed='Courier';	// fixed width font
	public $font_style=''; // 'B', 'I'
	public $font_current;
	public $em_points=9; // default font size
	public $font_size=1; // multiplier
	public $fine_print=0.75; // em size of fine text
	public $big_print=1.2; // em size of big text

    // Other class variables
	private $page_height;
	private $page_width;
	private $card_width;
	private $card_height;
	private $cards_per_page;
	private $params;
	private $title;
	
	private $club;
	private $event;
	protected $ci;

	public function __construct($params){
		if(empty($params)) trigger_error("Must specify parameters.",E_USER_ERROR);
		$this->params=$params;

		parent::__construct($this->orientation, $this->unit, $this->size);
		$this->CI =& get_instance();			

		$this->page_height = $this->GetPageHeight();
		$this->page_width = $this->GetPageWidth();
		$this->SetMargins(0,0,0);
		$this->SetAutoPageBreak(false,0);
		$this->font_normal();
		$this->SetTitle($this->title . " for " . $params['name_dist']);
		$this->SetSubject($this->title . " for " . $params['name_dist']);
		$this->SetCreator($this->params['this_organization'] . ' Postcard rendering software');
		$this->SetAuthor($this->params['this_organization']);
		
		$this->club = $this->params['club'];
		$this->event = $this->params['event'];
		
		$this->cards_per_page = $this->nrows * $this->ncols;
		$this->card_width = $this->page_width / $this->ncols;
		$this->card_height = $this->page_height / $this->nrows;
	}
	
	public function font_normal(){
		$this->font_current=$this->font_normal;
		$this->SetFont($this->font_normal,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_fixed(){
		$this->font_current=$this->font_fixed;
		$this->SetFont($this->font_fixed,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_style($s){
		$this->font_style=$s;
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_bold(){
		$this->font_style="B";
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_italic(){
		$this->font_style="I";
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_underline(){
		$this->font_style="U";
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_plain(){
		$this->font_style="";
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_size($x=1){
		$this->font_size = $x;
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_normalprint(){
		$this->font_size = 1;
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_fineprint(){
		$this->font_size=$this->fine_print;
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_bigprint(){
		$this->font_size=$this->big_print;
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}

	private function draw_line($x1,$y1,$x2,$y2,$width=null,$color=self::BLACK){
		if(empty($width)) $width=$this->thin_width;
		$this->SetLineWidth($width);
		$this->SetDrawColor(...$color);
		$this->Line($x1,$y1,$x2,$y2); 
	}

	private function draw_rect($x1,$y1,$w,$h,$width=null,$color=self::BLACK){
		if(empty($width)) $width=$this->thin_width;
		$this->SetLineWidth($width);
		$this->SetDrawColor(...$color);
		$this->Rect($x1,$y1,$w,$h); 
	}

	// Sign In Sheet Rendering
	
		private function put_string_centered($x,$y,$s){
			$w=$this->GetStringWidth($s);
		  $this->SetXY($x-$w/2,$y);
		  $this->Cell($w, $this->baseline_skip, $s, 0, 'C');
		}

	private function crlf($lines=1){
		$height=$lines*$this->baseline_skip;
		$x = $this->hmargin;
		$y = $this->GetY()+$height;
		$this->SetXY($x,$y);
	}

	private function vskip($height){
		$x = $this->hmargin;
		$y = $this->GetY()+$height;
		$this->SetXY($x,$y);
	}

	private function half_crlf(){
		$x = $this->hmargin;
		$y = $this->GetY()+$this->baseline_skip/2;
		$this->SetXY($x,$y);
	}
	public function render_fronts(){
	
        $this->AddPage();
        for($i=1; $i<$this->nrows; $i++)
            $this->draw_line(0 , $i*$this->card_height,$this->page_width, $i*$this->card_height,$this->thin_width,self::LIGHT_GRAY); 
        for($i=1; $i<$this->ncols; $i++)
            $this->draw_line($i*$this->card_width , 0,$i*$this->card_width, $this->page_height,$this->thin_width,self::LIGHT_GRAY); 

        for($row=0; $row<$this->nrows; $row++){
            for($col=0; $col<$this->ncols; $col++){
                $x=$col*$this->card_width;
                $y=$row*$this->card_height;
                $this->render_card_front($x,$y);
            }
        }
	}
	
	private $front_font_size=1.5;
	
    private function render_card_front($x,$y){
    
        extract($this->params);
        
        $logo_url=$club['logo_url'];
        $event_date=$event['start_daymonthyear_str'];

        $w=$this->logo_width*$this->card_width;
        $h=$this->logo_height*$this->card_height;
        $x_logo=$x+$this->card_width/2-$w/2;
        $y_logo=$y+$this->card_height/2-$h/2;
        $this->Image($logo_url,$x_logo,$y_logo,$w,$h);

        $this->font_size($this->front_font_size);
        $this->SetXY($x,$y+($this->card_height-$h)/4);
        $this->Cell($this->card_width, $this->front_font_size*$this->baseline_skip, $name_dist, 0, 1, 'C');
        $this->SetXY($x,$y+$this->card_height-($this->card_height-$h)/4 - $this->front_font_size*$this->baseline_skip);
        $this->Cell($this->card_width, $this->front_font_size*$this->baseline_skip, $event_date, 0, 1, 'C');
        

    }


	
	public function render_backs(){
	
        $irider=0;
        	
		foreach($this->params['roster'] as $r){
		
		
            $col=$irider % $this->ncols;
            $row=floor($irider / $this->ncols) % $this->nrows;

            $x=$col*$this->card_width;
            $y=$row*$this->card_height;
            
		    if(($irider % $this->cards_per_page) == 0){
                $this->AddPage();
                for($i=1; $i<$this->nrows; $i++)
                    $this->draw_line(0 , $y + $i*$this->card_height,$this->page_width, $y+$i*$this->card_height,$this->thin_width,self::LIGHT_GRAY); 
                for($i=1; $i<$this->ncols; $i++)
                    $this->draw_line($x + $i*$this->card_width , 0,$x + $i*$this->card_width, $this->page_height,$this->thin_width,self::LIGHT_GRAY); 
            }
            
            $irider++;
            $r->irider=$irider;
            
 			$this->render_card_back($x,$y,$r);
		}
	}
	
	private $padding=0.16;
	private $addr_top=0.55;

    private function render_card_back($x,$y,$r){
    
        extract(json_decode(json_encode($r), true)); // Convert class to array and extract
        $rider_name = "NAME: $first_name $last_name";
        $rider_id = "RUSA: $rusa_id";
        $club=$this->club;
        $event=$this->event;
        $tagname=$this->CI->model_parando->make_event_tagname($event);
        
        $club_name=$club['name'];
        $club_street=$club['street'];
        $club_csz=$club['city'] . ', ' . $club['state'] . ' ' . $club['zip'];
        $club_address = <<<EOF
$club_name
$club_street
$club_csz
EOF;

        $this->draw_line($x+$this->card_width/2,$y+$this->padding*$this->card_height,$x+$this->card_width/2,$y+(1-$this->padding)*$this->card_height,
            $this->thick_width,self::BLACK);
        $this->font_bigprint();
        $this->SetXY($x+$this->padding*$this->card_width/2,$y+$this->padding*$this->card_height);
        $this->Cell((1-$this->padding)*$this->card_width, $this->baseline_skip, $tagname);
        $this->SetXY($x+$this->padding*$this->card_width/2,$y+$this->padding*$this->card_height+2*$this->big_print*$this->baseline_skip);
        $this->Cell((1-$this->padding)*$this->card_width, $this->baseline_skip, $rider_name);
        $this->SetXY($x+$this->padding*$this->card_width/2,$y+$this->padding*$this->card_height+3*$this->big_print*$this->baseline_skip);
        $this->Cell((1-$this->padding)*$this->card_width, $this->baseline_skip, $rider_id);

        $this->SetXY($x+(1+$this->padding)*$this->card_width/2,$y+$this->addr_top*$this->card_height+$this->big_print*$this->baseline_skip);
        $this->MultiCell((1-$this->padding)*$this->card_width, $this->big_print*$this->baseline_skip, $club_address);


    }
			

}

?>
