<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');

 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

require_once('Randoclass.php');

class Route_event extends Randoclass{

    public function __construct(){
        parent::__construct();
    }

	const km_per_mi = 1.609344;
	const ft_per_m = 3.2808398950131;
	const m_per_km = 1000.0;

	// EVENT AND ROUTE FUNCTIONS
	// EVENT AND ROUTE FUNCTIONS
	// EVENT AND ROUTE FUNCTIONS
	// EVENT AND ROUTE FUNCTIONS
	
	
	// One function to rule them all
	
    public function compute_all_event_data($event_id, $from_scratch=false){

		list($event, $route) = $this->get_event_and_route($event_id);

		// Extract controls and cues from the route
		list($controles, $controle_warnings) = $this->CI->rwgps->extract_controles($route);
		list($cues, $cue_warnings) = $this->CI->rwgps->extract_cues($route);
		
		// Create a merged route+event
		list($route_event, $re_warnings) = $this->merge_route_event_data($route,$event);
		
		// Consolodate the warnings
		$warnings=array_merge($controle_warnings, $re_warnings,$cue_warnings);
		
		// Now we can compute the control times
		$this->CI->controletimes->compute_open_close($controles,$route_event);
		
		// And the info codes, in case these are needed (and SIDE EFFECT load the controlqr library)
		$params=['route_event'=>$route_event];
		$this->CI->load->library('controlqr', $params); 
		$this->CI->controlqr->compute_info_codes($route_event, $controles);

		// and make sure the QR files are all there
		$this->CI->controlqr->make_qr_images($route_event, $controles, $from_scratch);

        return compact('event','route','controles','warnings','cues','route_event');
    }
    
    // Maybe we just want to know the number of controls
    
    public function control_count($event_id){
		list($event, $route) = $this->get_event_and_route($event_id);
		list($controles, $controle_warnings) = $this->CI->rwgps->extract_controles($route);
		return count($controles);    	
    }

    // Get the basic event and route data -- not yet merged into a route event
	 
	public function get_event_and_route($event_id){

        // Get the event
        $event=$this->CI->model_parando->get_event($event_id);
		if(empty($event['map'])) $this->die_error(__METHOD__,"No event, or no map URL set for event $event_id");
				
		// Get the route
		$map_url=(isset($event['map']))?$event['map']:'';
		$route_id=$this->CI->rwgps->extract_route_id($map_url);

        if($route_id!==null){
            $route=$this->CI->rwgps->get_route($route_id);
			$event['has_rwgps_route']=true;
			$event['rwgps_url']=$this->CI->rwgps->make_route_url($route_id);
			$event['map_url']="";
		}else{
			$event['has_rwgps_route']=false;
			$event['map_url']=make_site_url($map_url);
			$event['rwgps_url']="";
			$route=null;
		}

		return [$event, $route];
	}
	
	// MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE MERGE
	//
    // Merge, validate, and harmonize route and event into a route_event. It produces a route_event array, along with 
    // a list of warnings. 

	public function merge_route_event_data($route, $event){		

		$warnings=[];
		$re=['is_route_event'=>true];
		
        // ROUTE ROUTE ROUTE ROUTE ROUTE ROUTE ROUTE ROUTE ROUTE ROUTE ROUTE ROUTE
		// This function assumes a valid $route with certain basic fields that 
		// should be guaranteed from the RWGPS data. 

		$required_route_keys=[
                                'id','distance','elevation_gain',
                                'name','description','download_note',
                                'updated_at','downloaded_at','route_datafile'
                            ];		
		
		// Sanity check of route
		if(empty($route)) $this->die_error(__METHOD__, "This can't happen: Empty route"); 
		foreach($required_route_keys as $k) 
			if(!array_key_exists($k, $route)) $this->die_error(__METHOD__,"Missing route field: $k");
		
		// Put all these route strings and numbers into route_event (NOT the array route itself)
		// The keys are exactly the same -- no 'route_' prefix when put in the $re array
		foreach($route as $key=>$val){
			if(is_string($val) || is_numeric($val)) $re[$key]=$val;
		}

		// In case the spelling isn't already there
		$re['route_id']=$route_id=$route['id'];

		// Calculate various additional route statistics in the usual units
		$re['distance_km']=$distance_km=round($route['distance']/rwgps::m_per_km,1);
		$re['distance_mi']=$distance_mi=round($route['distance']/rwgps::m_per_km/rwgps::km_per_mi,1);
		$re['climbing_ft']=$climbing_ft=round($route['elevation_gain']*rwgps::ft_per_m,1);
				
		// Other route strings
		$re['route_url']=$this->CI->rwgps->make_route_url($route_id);
		$re['route_filename']=$route_filename=$this->CI->rwgps->make_route_filename($route['id']);
		$re['route_file_basename']=basename($route_filename);
		$re['download_note']=$route['download_note']; // Did this route come from the cache
		$re['route_name']=$route['name'];
		
        // RWGPS Route Description tags
        
        if(!empty($route['description']))
            $re['route_tags']=$route_tags=$this->CI->rwgps->parse_description($route['description'],$this->CI->rwgps->valid_event_description_keys);
        else
            $route_tags=[];

        // Maybe someday this can be automatically computed from the RWGPS data? 
		$re['pavement_type']=(empty($route_tags['pavement_type']))?"Unspecified":$route_tags['pavement_type'];  

		
		// Data Files
		
        $df_links=[];
        foreach($route['route_datafile'] as $ext=>$fn){
            $uc_ext=strtoupper($ext);
            $base_fn=basename($fn);
            $df_links[]="<A TITLE='$uc_ext file download' HREF='" . $route['saved_route_url'][$ext] . "'>$uc_ext</A>"; 
        }
        $re['df_links_txt']=$df_links_txt=implode(', ', $df_links);

        // EVENT EVENT EVENT EVENT EVENT EVENT EVENT EVENT EVENT EVENT 
		// This function assumes a valid $event with certain basic fields that 
		// should be guaranteed from the database fetch

		$required_event_keys=['id','name','distance',
            'date','sanction','type','start_time','start_city',
            'start_state','organizer','club_id','rusa_route','cue_version'];		
		
		// Sanity check of event 
		if(empty($event)) $this->die_error(__METHOD__, "This can't happen: Empty event"); 
		foreach($required_event_keys as $k)
			if(!array_key_exists($k, $event) /*|| empty($event[$k]) */ ) {
                $this->die_error(__METHOD__,"Missing event field: $k<br>Event: " . print_r(array_keys($event),true));
            }
		
		// Copy into RE various event fields. These may require some calculation or respelling. Many (but not all) are given the event_ prefix
		// There is no indiscriminate copying like there was with the route object
		
		$re['event_id']=$event_id=$event['id'];	
		$re['event_name']=$event_name=$event['name'];	
		$re['event_distance']=$event_distance=$event['distance'];	
		$re['event_gravel_distance']=$event_gravel_distance=$event['gravel_distance'];	
		$re['event_date']=$event_date=$event['date'];	

		$re['has_rwgps_route']=$event['has_rwgps_route'];
		$re['rwgps_url']=$event['rwgps_url'];
		$re['map_url']=$event['map_url'];
		
        $re['rusa_route']=$rusa_route=$event['rusa_route'];	

		$od=$this->CI->model_parando->get_organizer($event);
		$re['organizer_name']=$organizer_name=$od['organizer_name'];
		$re['organizer_phone']=$organizer_phone=$od['organizer_phone'];
		
        // Fields in event that are a function of the club
		$re['club']=$club=$event['club'];
		$re['club_name']=$club_name=$club['name'];
		$re['this_organization']=$this_organization=$club['name'];
        $re['logo_url']=$club['logo_url'];
        $re['club_logo_url']=$club['logo_url'];
        $re['club_website_url']=$club['website_url'];
        $re['event_tzname']=$event_tzname=$club['tzname'];
        $re['club_acp_code']=$club_acp_code=$club['acp_code'];
		
		$re['event_name_dist']=$event_name_dist=$event['name'] . ' ' . $this->CI->model_parando->make_distance($event);
		$re['event_type']=$event_type=strtolower($event['sanction']);
		$re['event_style']=$event_style=strtolower($event['type']);
		$re['event_time']=$event_time=$event['start_time'];
		$re['event_location']=$event_location=$event['start_city'] . ' ' . $event['start_state'];
	
		$re['event_tz']=$event_tz=new DateTimeZone( $event_tzname );

		
		$re['event_roster']=$event_roster=$this->CI->model_parando->get_roster($event_id);
		
		
		
        $re['event_tagname']=$this->CI->model_parando->make_event_tagname($event);
		
		if(!empty($event['cuesheet'])){
			$re['cuesheet_url']=make_site_url($event['cuesheet']);
			$re['has_cuesheet']=true; $re['cuesheet_location']='url'; 
		}else	if($event['has_rwgps_route'] && isset($event['cue_version']) && $event['cue_version']>0){
			$re['has_cuesheet']=true; $re['cuesheet_location']='auto'; $re['cuesheet_url']=null;
		}else{
			$re['has_cuesheet']=false; $re['cuesheet_location']=null; $re['cuesheet_url']=null;
		}
		
		$re['cue_version']=$cue_version=(empty($event['cue_version']))?0:($event['cue_version']);
        $re['cue_version_str']=($cue_version>0)?"$cue_version":"None";
		$re['cue_next_version']=$cue_version+1;
		
		// Now that all info is in, do  some validation

		if(empty($event_location)){
			$re['event_location']=$event_location="UNKNOWN, XX";
			$warnings[]="Event location not set. Please set the 'City, State' of event start.";
		}
		
		if(empty($re['event_distance'])){ 
			$re['event_distance']=$event_distance=ceil($re['distance_km']);
			$warnings[]="Official distance not specified. Assuming $event_distance K";
		}

		if($event_distance>$distance_km){
			$warnings[]="Route is too short. Route distance $distance_km km < Official distance $event_distance km.";
		}
		
		$vet=$this->CI->controletimes->valid_event_types;
		if( !empty($event_type) && !array_key_exists($event_type,$vet)) {
			$this->die_error(__METHOD__, "Unsupported event type: $event_type. Valid types are: " . implode(', ', array_keys($vet)));
        }
		if(empty($event_type)){
			$re['event_type']=$event_type='unknown';
			$warnings[]="Event type not set. Set type equal to one of the following:" . implode(', ', array_keys($vet));
		}

		if($event_type=='permanent' && !isset($re['organizer_rusa_id'])) 
			$warnings[]="Event is permanent. Must set organizer_rusa_id";

		if($event_type=='permanent' && !isset($re['rusa_route_id'])) 
			$warnings[]="Event is permanent. Must set rusa_route_id";

		// TIME AND DATE

		date_default_timezone_set($event_tzname); 
		$re['last_update']=date("Y-m-j H:i:s T",$re['updated_at']); 
		$re['last_download']=date("Y-m-j H:i:s T",$re['downloaded_at']); 

		
		$re['now']=$now=new DateTime('now',$event_tz);
		$re['now_str']=$now->format($this->CI->controletimes->event_datetime_format);
		
		
        if(empty($re['event_date']) || $re['event_date']==$this->CI->controletimes->default_event_date){
			$re['event_date']=$event_date=$this->CI->controletimes->default_event_date;
			$warnings[]="Event date not set. Please set the start date. Default is " . $this->CI->controletimes->default_event_date;
		}
		
		if(empty($re['event_time']) || $re['event_time']==$this->CI->controletimes->default_event_time){
			$re['event_time']=$event_time=$this->CI->controletimes->default_event_time;
			$warnings[]="Event time not set. Please to set the start time. Default is " . $this->CI->controletimes->default_event_time;
		}
		

		// Attempt to make datetime object for start 
		$re['event_datetime'] = $event_datetime = $this->CI->controletimes->make_event_datetime($re);
		

		if(false == $event_datetime){ 
			$dtz= "D=" . $re['event_date'] . "; T=" . $re['event_time'] . "; Z=" . print_r($re['event_tz'],true);
			$this->die_error(__METHOD__,"Invalid or missing date, time, or timezone. DTZ=$dtz");
		}
		
		$re['event_datetime_str'] = $event_datetime_str = $event_datetime->format($this->CI->controletimes->event_datetime_format);
		$re['event_datetime_str_verbose'] = $event_datetime_str_verbose = $event_datetime->format($this->CI->controletimes->event_datetime_format_verbose);
		
		return [$re, $warnings];
	}

		public $disclaimer_html=<<<EOT
	<hr width=61.8%>
<div class='text-center half-width'>Cue Wizard <A HREF=https://parando.org/cue_wizard.html>User Manual</A>
<div class='fine-print'>
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
<A HREF=https://parando.org/software_license.html>Click here</a> for access to source code and complete license details.
</div></div>
<div class=vspace></div>
EOT;

	public function make_controles_table($controles,$style='info'){
        $controles_table= "";
        $ncontroles=count($controles);
		if($ncontroles>0){
            $controles_table ="<H4>CONTROLS</H4>";
            $controles_table.="<div class='narrower ftable'><TABLE WIDTH=100%>";
            
            switch($style){
                case 'wizard':
                    $controles_table.= "<TR><TH>Controle</TH><TH>Distance</TH><TH>Open/Close</TH><TH>Note</TH><TH>Description (attributes)</TH></TR>";
                    break;
                case 'info':
                    $controles_table.= "<TR><TH>Controle</TH><TH>Distance</TH><TH>Open/Close</TH><TH>Location</TH></TR>";
                    break;
                default:
                    $this->die_error(__METHOD__,'Unknown control table style.');
            }
            
            for($i=0; $i<$ncontroles; $i++){
                $cd_mi=round($controles[$i]['d']/(self::m_per_km*self::km_per_mi),1);
                $cd_km=round($controles[$i]['d']/(self::m_per_km),1);
                $is_start=(isset($controles[$i]['start']))?" [START]":"";
                $is_finish=(isset($controles[$i]['finish']))?" [FINISH]":"";
                $controle_num=$i+1;
                $open_str=$controles[$i]['open']->format('m-d H:i');
                $close_str=$controles[$i]['close']->format('m-d H:i');
                switch($style){
                    case 'wizard':
                        $ca=$this->format_attributes($controles[$i]['attributes']);
                        $qrcode_url=$controles[$i]['qrcode_url'];
                        $qrcode_checkin_url=$controles[$i]['qrcode_checkin_url'];
                        // $ca.="QR CODE: <A HREF=$qrcode_url>" . $controles[$i]['info_code'] . "</A>, Check in <A HREF=$qrcode_checkin_url>NOW</A><br/>";
                    $cn=(isset($controles[$i]['n']))?$controles[$i]['n']:"";
                        $controles_table.= "<TR><TD>$controle_num$is_start$is_finish</TD><TD>$cd_mi mi<br>$cd_km km</TD><TD>$open_str<br>$close_str</TD><TD>$cn</TD><TD>$ca</TD></TR>";
                        break;
                    case 'info':
                        $cd=$this->format_control_description_table($controles[$i]['attributes']); 
                        $controles_table.= "<TR><TD>$controle_num$is_start$is_finish</TD><TD>$cd_mi mi<br>$cd_km km</TD><TD>$open_str<br>$close_str</TD><TD>$cd</TD></TR>";
                        break;
                }
            }
            $controles_table.="</TABLE></DIV>";
        }
        return $controles_table;
	}

	public function format_attributes($alist){
		$ca="";
		foreach($alist as $k=>$v){
				if(is_string($v))
					$ca.="<div class=attribute>#$k=$v</div>";
		}
		return $ca;
	}

    public function format_control_description_table($alist){
 		$cdt="<div style='font-size: .7em; font-family: Arial, Helvetica, sans-serif'>";
        $cd_field=['name','address','style'];
        foreach($cd_field as $k){
            $v=(array_key_exists($k,$alist))?$alist[$k]:"";
            if($k=='style') $v=strtoupper($v);
            $n=ucfirst($k);
            $cdt.="$v<BR>";
        }
		$cdt.="</div>";
		return $cdt;
	}
	



}

?>
