<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');

 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

	require_once('Randoclass.php');

	class Rusa extends Randoclass{

		const rusa_query_url="https://rusa.org/cgi-bin/membersearch_PF.pl";
		const rusa_check_members_url="https://rusa.org/cgi-bin/check_members.pl";
		const rusa_membership_url="https://rusa.org/cgi-bin/memberlist_PF.pl?type=1";
		const saved_rusa_membership_path="/home/tomr/public_html/assets/parando/var/RUSA_members.csv";
		const rusa_route_query_url="https://rusa.org/cgi-bin/routesearch_PF.pl";
		const max_rusa_id=999999;
		const min_rusa_year=1990;
		const max_rusa_year=2099;

		public function __construct(){
			parent::__construct();
		}

		const rusa_csv_fields=[
										'rusa_id', 
										'expires',
										'last_name', 
										'first_name', 
										'city', 
										'state',
										'country',
										'club',
										'club_id' 
										]; 

		const rusa_webquery_fields=[
										'rusa_id', 
										'last_first',
										'city_state', 
										'club_name_code', 
										'expires'
										]; 

		public $rusa_error_msg="In order to ride this event, your RUSA membership must be in good standing and paid through the date of this event.";

		public function rusa_error($msg){
			$this->die_error("RUSA Membership Error", 
				"<p>" . $this->rusa_error_msg . "</p>" . "<p>" . $msg . "</p>");
		}

		public function curl_get_file_contents($URL){
			$c = curl_init();
			curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($c, CURLOPT_URL, $URL);
			$contents = curl_exec($c);
			curl_close($c);

			if ($contents) return $contents;
			    else return FALSE;
		}

		public function route_url($rtid){
			return self::rusa_route_query_url . "?rtid=$rtid";
		}
		
		
		public function query_rusa_id($rusa_id){
		
			$m=$this->check_member_cache($rusa_id);
			if($m!==false){
                $expires=$m['expires'];
                $tz=$this->CI->model_parando->get_local_time_zone();
                $exp_datetime=new DateTime($expires,$tz);
                $now_datetime=new DateTime('now',$tz);
                if($exp_datetime>$now_datetime) return $m;
            }
                
            return $this->check_member_api($rusa_id);
		
		}
		
		public function get_member($rusa_id){
                $rusa_id=$rusa_id+0;
                if($rusa_id < 1 || $rusa_id > self::max_rusa_id) return null;
				$this->CI->db->where('rusa_id',$rusa_id);
				$q=$this->CI->db->get('rusa');
				if($q->num_rows()>1){  // Multiple entries
					trigger_error("Duplicate entries in RUSA member cache for ID=$rusa_id",E_USER_ERROR);
				}elseif($q->num_rows()==0){
					return null; // not found
				}
				return $q->row_array();
		}

		public function get_member_by_first_last($first_name,$last_name){
                $rusa_id=$rusa_id+0;
                if($rusa_id < 1 || $rusa_id > self::max_rusa_id) return null;
				$this->CI->db->where('first_name',$first_name);
				$this->CI->db->where('last_name',$last_name);
				$q=$this->CI->db->get('rusa');
				if($q->num_rows()>1){  // Multiple entries
					trigger_error("Duplicate entries in RUSA member cache for ID=$rusa_id",E_USER_ERROR);
				}elseif($q->num_rows()==0){
					return null; // not found
				}
				return $q->row_array();
		}

		private function check_member_cache($rusa_id){
                $m=$this->get_member($rusa_id);
                if(null===$m) return false; // not found
				$m['from_cache']=true;
				$m['from_api']=false;
				return $m;
		}
		
// 		private function check_member_api($rusa_id){  //temporary wrapper before renaming
// 		  return $this->query_rusa_id($rusa_id);
// 		}
		
		public function check_member_api($rusa_id){
		
			$api_key=$this->CI->model_parando->rusa_member_api_key;
			$url=self::rusa_check_members_url . "?mid_list=$rusa_id";
			$data = @file_get_contents("$url" . "&apikey=$api_key");
			if(empty($data)) trigger_error("Failed to download any RUSA API data from $url",E_USER_ERROR);
			$member_record=json_decode($data,true);
			if(null===$member_record) trigger_error("Could not decode RUSA API response from $url<br>JSON data:<br>".print_r($data,true),E_USER_ERROR);
			if(count($member_record)!=1) trigger_error("Unexpected number of array elements in RUSA API response. Decoded data:<br>".print_r($member_record,true),E_USER_ERROR);
			$m = reset($member_record); // First element 
			
			if(!array_key_exists('name', $m)) return false;  // not found
			
			// Field remapping and validation for RUSA API
			$field_map=[
			'mid'=>'rusa_id',
			'exp_date'=>'expires',
			'name'=>'last_first'
			];
			foreach($field_map as $old=>$new){
				if(array_key_exists($old, $m)){
						if($old!=$new){
							$m[$new] = $m[$old];
							unset($m[$old]);
    					}
				}else trigger_error("Missing key '$old' in RUSA member query response: " . print_r($m,true), E_USER_ERROR);
				
			}
			
			// Split name on comma -- hope there are no names with commas!
			list($m['last_name'],$m['first_name'])=array_merge(explode(', ',$m['last_first'],2),['','']);
			
			// Convert YYYY/MM/DD date to ISO 8601 YYYY-MM-DD
			$m['expires']=str_replace('/','-',$m['expires']);

				$m['from_cache']=false;
				$m['from_api']=true;

			return $m;
		}


		public function old_query_rusa_id($rusa_id){
			$query_url=self::rusa_query_url . "?mid=$rusa_id";

			$c = curl_init();
			curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($c, CURLOPT_URL, $query_url);
			curl_setopt($c, CURLOPT_SSL_VERIFYPEER, false);
			// curl_setopt($c, CURLOPT_SSL_VERIFYSTATUS, false);
			$result = curl_exec($c);
			if(! $result)	$this->die_error(__METHOD__, "Could not fetch $query_url<br>" . curl_error($c));
        		curl_close($c);
//			$match_pattern='!1 Matches Found.*<thead>.*</thead>.*<TR>.*<TD[^>]*>(\d+)</TD>.*<TD[^>]*><B>([^<]+)</B></TD>.*<TD[^>]*>([^>]+)</TD>.*<TD[^>]*>([^>]+)</TD>.*<TD[^>]*>([^>]+)</TD>!s';
			$match_pattern='!1 Matches Found.*?<thead>.*?</thead>.*?<TR>.*?<TD.*?>\s*<A\s.*?>(\d+)</A>.*?</TD>.*<TD[^>]*><B>([^<]+)</B></TD>.*<TD[^>]*>([^>]+)</TD>.*<TD[^>]*>([^>]+)</TD>.*<TD[^>]*>([^>]+)</TD>!s';

			$nomatch_pattern='!there were no members that matched!s';

			$rc=preg_match($nomatch_pattern,$result,$m);
			if($rc===false) $this->die_error(__METHOD__,"Error matching RUSA membership response");
			if($rc===1){ 
				return false; 
			}else{
				$rc=preg_match($match_pattern,$result,$m);
				if($rc===false) $this->die_error(__METHOD__,"Error matching RUSA membership response");
				if($rc===0) $this->die_error(__METHOD__,"Unrecognized RUSA membership query response.");
				array_shift($m);

				$m=array_combine(self::rusa_webquery_fields,$m);

				list($m['last_name'],$m['first_name'])=array_merge(explode(', ',$m['last_first'],2),['','']);
				unset($m['last_first']);

				list($m['club'],$m['club_id'])=array_merge(explode(' / ',$m['club_name_code'],2),['','']);
				unset($m['club_name_code']);

				list($m['city'],$m['state'],$m['country'])=array_merge(explode(', ',$m['city_state'],3),['US','','']);
				unset($m['city_state']);

				$m['expires']=str_replace('/','-',$m['expires']);
				

				return $m;
			}
		}

		public function get_member_list($all_records=true){

				$url=self::rusa_membership_url;

				$rusa_file = @fopen($url, "r") or $this->die_error(__METHOD__, "Unable to access RUSA membership data with URL: $url");

				$n_fields=count(self::rusa_csv_fields);

				$member_list=[];
				$line=1;
        while (($row = fgetcsv($rusa_file)) !== FALSE){
					if(($row_map=array_combine(self::rusa_csv_fields,$row)) === FALSE) $this->die_error(__METHOD__, "CSV line $line does not have $n_fields fields"); 
					if(empty($row_map['rusa_id'])){
						// ignore line
					}else{
						if($row_map['rusa_id']<1 || $row_map['rusa_id']>self::max_rusa_id) $this->die_error(__METHOD__, "Malformed or out of range RUSA ID: " . $row_map['rusa_id']); 
						if(isset($member_list[$row_map['rusa_id']])) $this->die_error(__METHOD__, "CSV line $line Duplicate RUSA ID: " . $row_map['rusa_id']); 
						list($y,$m,$d)=explode('/',$row_map['expires'],3);
						if($y<self::min_rusa_year || $y>self::max_rusa_year) $this->die_error(__METHOD__, "CSV line $line has invalid year: " . $y); 
						if($m<1 || $m>12) $this->die_error(__METHOD__, "CSV line $line has invalid month: " . $m); 
						if($d<1 || $d>31) $this->die_error(__METHOD__, "CSV line $line has invalid day: " . $d); 
						if(0==strlen($row_map['first_name']) || 0==strlen($row_map['last_name']))  $this->die_error(__METHOD__, "CSV line $line has invalid name: ");

						$this->CI->db->select('code');
						$this->CI->db->where('fullname',$row_map['country']);
						$q=$this->CI->db->get('country');
						$r=$q->row();
						if(!isset($r->code)) $this->die_error(__METHOD__, "No code found for country: " . $row_map['country']);
						$code=$r->code;
						$row_map['country']=$code;
						
						$uexpire=mktime(23, 59, 59, $m, $d, $y);
						if($all_records || $uexpire>time()){
							$row_map['expires']=date('Y-m-d',$uexpire);
							$member_list[$row_map['rusa_id']]=$row_map;
						}
					}
          $line++;
        }
        fclose($rusa_file);

				return $member_list;
				// $this->die_error(__METHOD__, "Successfully parsed roster from CSV file: " . print_r($roster_data,true));

		}

		public function last_name_mismatchq($a,$b){
			$a=preg_replace('/[^a-z]/','',strtolower($a));
			$b=preg_replace('/[^a-z]/','',strtolower($b));
			//$this->die_error(__METHOD__,"'$a=$b'");
			return (empty($a) || empty($b) || levenshtein($a,$b)>1);
		}

		public function check_rusa_membership_at_date($rusa_id,$last_name,$event_cutoff_datetime){
			$r=$this->rusa_status_at_date($rusa_id,$last_name,$event_cutoff_datetime);
			if($r!='OK') $this->rusa_error($r);
		}
		public function rusa_status_at_date($rusa_id,$last_name,$event_cutoff_datetime){
				$result='OK';
				if(empty($rusa_id)){
					$result=("No RUSA ID");
				}else{
					$rusa_m=$this->query_rusa_id($rusa_id);
					if(false===$rusa_m){
						$result=("RUSA ID <b>$rusa_id</b> Not Found in RUSA Database");
					}else{
						if($this->last_name_mismatchq($rusa_m['last_name'],$last_name)){
										$a=preg_replace('/[^a-z]/','',strtolower($rusa_m['last_name']));
										$b=preg_replace('/[^a-z]/','',strtolower($last_name));

							$result=("Last name <b>" . strtoupper($last_name) . "</b> does not match last name for RUSA ID $rusa_id"); // ($a != $b)");
						}else{
							$rusa_exp=$rusa_m['expires']; $rusa_expires_datetime=new DateTime($rusa_exp); 
							$now_datetime=new DateTime('now');
							if($now_datetime>$rusa_expires_datetime){
								$result=("The membership for RUSA ID $rusa_id expired on $rusa_exp.");
							}else{			
								if($event_cutoff_datetime>$rusa_expires_datetime){
									$result=("Your RUSA membership expires $rusa_exp, which is before the date of this event.");
								}
							}
						}
					}
				}
				return $result;
		}

  }

?>

