<?php defined('BASEPATH') OR exit('No direct script access allowed');
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

// Brevet Card Rendering Class /////////////////////////////////////////////////////////////////////

require_once('fpdf/fpdf.php');

class Signin extends FPDF {

	// colors
	const BLACK=[0,0,0];
	const RED=[255,0,0];

	// Constructor parameters
	private $orientation='P'; // P - portrait; L - landscape
	private $unit='in'; // in, cm, mm, pt NOT SETTABLE
	private $size='letter'; // A3, A4, A5, letter, legal
  
	// Configuration variables
	public $logo_url="https://parando.org/images/parando3D.png";
	public $logo_width=0.12; // fraction of frame width
	public $logo_height=0.06; // fraction of frame height
	public $logo_center_y=0.04; // fraction of overall page height
	public $logo_center_x=0.8; // fraction of overall page width
	

	// layout constants
	public $vmargin=1.0;  // space between top/bottom page edge and cues
	public $hmargin=1.0;  // space between left/right page edge and cues
	public $vmargin_L=1.75;  // space between top/bottom page edge and cues
	public $hmargin_L=0.75;  // space between left/right page edge and cues
	public $vmargin_P=0.5;  // space between top/bottom page edge and cues
	public $hmargin_P=0.5;  // space between left/right page edge and cues
	public $baseline_skip = 0.15;  // Space between text lines
	public $row_padding = 0.05; // padding above and below cue text
	public $column_padding=0.05; // Horizontal padding in dist cols
	public $thin_width=0.01; // Thin lines
	public $thick_width=0.02; // Thick lines
	public $font_normal='Helvetica';	// default font family
	public $font_fixed='Courier';	// fixed width font
	public $font_style=''; // 'B', 'I'
	public $font_current;
	public $em_points=9; // default font size
	public $font_size=1; // multiplier
	public $fine_print=0.75; // em size of fine text
	public $big_print=2; // em size of big text

    // Other class variables
	private $page_height;
	private $page_width;
	private $params;
	private $title;

	public function __construct($params){
		if(empty($params)) trigger_error("Must specify parameters.",E_USER_ERROR);
		$this->params=$params;

		if(isset($params['orientation'])) $this->orientation=$params['orientation'];
		if(isset($params['size'])) $this->size=$params['size'];

		parent::__construct($this->orientation, $this->unit, $this->size);
		$this->page_height = $this->GetPageHeight();
		$this->page_width = $this->GetPageWidth();
		$this->SetMargins(0,0,0);
		switch($this->orientation){
			case 'L':
				$this->vmargin=$this->vmargin_L;
				$this->hmargin=$this->hmargin_L;
				break;
			case 'P':
				$this->vmargin=$this->vmargin_P;
				$this->hmargin=$this->hmargin_P;
				break;
		}

		if($params['controle_type']=='start'){
			$this->title=ucfirst("Start Contrôle Sign In");
		}else{
			$this->title=ucfirst("Contrôle Sign In");
		}
		$this->SetAutoPageBreak(false,0);
		$this->font_normal();
		$this->SetTitle($this->title . " for " . $params['name_dist']);
		$this->SetSubject($this->title . " for " . $params['name_dist']);
		$this->SetCreator($this->params['this_organization'] . ' Sign In Sheet rendering software');
		$this->SetAuthor($this->params['this_organization']);
	}
	
	public function font_normal(){
		$this->font_current=$this->font_normal;
		$this->SetFont($this->font_normal,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_fixed(){
		$this->font_current=$this->font_fixed;
		$this->SetFont($this->font_fixed,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_style($s){
		$this->font_style=$s;
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_bold(){
		$this->font_style="B";
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_italic(){
		$this->font_style="I";
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_underline(){
		$this->font_style="U";
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_plain(){
		$this->font_style="";
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_size($x=1){
		$this->font_size = $x;
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_normalprint(){
		$this->font_size = 1;
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_fineprint(){
		$this->font_size=$this->fine_print;
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}
	public function font_bigprint(){
		$this->font_size=$this->big_print;
		$this->SetFont($this->font_current,$this->font_style,$this->em_points*$this->font_size);
	}

	private function draw_line($x1,$y1,$x2,$y2,$width=null,$color=self::BLACK){
		if(empty($width)) $width=$this->thin_width;
		$this->SetLineWidth($width);
		$this->SetDrawColor(...$color);
		$this->Line($x1,$y1,$x2,$y2); 
	}

	private function draw_rect($x1,$y1,$w,$h,$width=null,$color=self::BLACK){
		if(empty($width)) $width=$this->thin_width;
		$this->SetLineWidth($width);
		$this->SetDrawColor(...$color);
		$this->Rect($x1,$y1,$w,$h); 
	}

	// Sign In Sheet Rendering
	
		private function put_string_centered($x,$y,$s){
			$w=$this->GetStringWidth($s);
		  $this->SetXY($x-$w/2,$y);
		  $this->Cell($w, $this->baseline_skip, $s, 0, 'C');
		}

	private function crlf($lines=1){
		$height=$lines*$this->baseline_skip;
		$x = $this->hmargin;
		$y = $this->GetY()+$height;
		$this->SetXY($x,$y);
	}

	private function vskip($height){
		$x = $this->hmargin;
		$y = $this->GetY()+$height;
		$this->SetXY($x,$y);
	}

	private function half_crlf(){
		$x = $this->hmargin;
		$y = $this->GetY()+$this->baseline_skip/2;
		$this->SetXY($x,$y);
	}
	
	public function render_sheet(){
	
		$x = $this->hmargin;
		$y = $this->vmargin;
		$full_width = $this->page_width - 2*$this->hmargin;
		$h = $this->page_height - 2*$this->vmargin;
		$w = $this->page_width - 2*$this->hmargin;

		$header_row_height=$this->baseline_skip;
		$table_row_height=2.5*$this->baseline_skip;

		$pw=$this->page_width;
		$ph=$this->page_height;

		$this->Image($this->logo_url,$pw*$this->logo_center_x,$ph*$this->logo_center_y,$pw*$this->logo_width,$ph*$this->logo_height);

		$name_dist = utf8_decode($this->params['name_dist']);
		$cue_version = $this->params['cue_version'];
		$this->SetXY($x,$y);
		$this->font_bigprint();
		$this->font_bold();
		$this->Cell($w, $this->baseline_skip,  utf8_decode($this->title), 0, 'L');
		$this->crlf(1.5);
		$this->font_normalprint();
		$this->Cell($w, $this->baseline_skip,  $name_dist, 0, 'L');
		$this->font_plain();
		$this->crlf();
		$this->Cell($w, $this->baseline_skip, 'Cue Version: ' . $cue_version, 0, 'L');
		$this->crlf(1.5);
		$this->Cell($w, $this->baseline_skip, 'Start Date: ' . $this->params['event']['start_date_str'], 0, 'L');
		$this->crlf();
		$this->Cell($w, $this->baseline_skip, 'Start Time: ' . $this->params['event']['start_time_str'], 0, 'L');
		
		$this->crlf(1.5);

		if($this->params['controle_type']!='start'){
			$this->crlf();
			$this->Cell($w, $this->baseline_skip, utf8_decode('Contrôle: _________________________________________'), 0, 'L');
			$this->crlf(2);
			$header_row = [
				['font'=>'bold', 'text'=>'#', 'align'=>'C', 'width'=>5],
				['font'=>'bold', 'text'=>'TRK', 'align'=>'C', 'width'=>5],
				['font'=>'bold', 'text'=>'Name', 'align'=>'C', 'width'=>20],
				['font'=>'bold', 'text'=>'Initials', 'align'=>'C', 'width'=>20],
				['font'=>'bold', 'text'=>'Time In', 'align'=>'C', 'width'=>15],
				['font'=>'bold', 'text'=>'Phone', 'align'=>'C', 'width'=>15],
				['font'=>'bold', 'text'=>'Emergency', 'align'=>'C', 'width'=>20]
			];
		}else{
			$header_row = [
				['font'=>'bold', 'text'=>'#', 'align'=>'C', 'width'=>5],
				['font'=>'bold', 'text'=>'TRK', 'align'=>'C', 'width'=>5],
				['font'=>'bold', 'text'=>'Name', 'align'=>'C', 'width'=>20],
				['font'=>'bold', 'text'=>'Code', 'align'=>'C', 'width'=>5],
				['font'=>'bold', 'text'=>'Lodging', 'align'=>'C', 'width'=>10],
				['font'=>'bold', 'text'=>'Sign', 'align'=>'C', 'width'=>45],
				['font'=>'bold', 'text'=>'Date', 'align'=>'C', 'width'=>10]
			];
		}



		$column_width=[];
		foreach($header_row as $r) $column_width[]=$full_width*$r['width']/100.0;


		$this->render_row($header_row,$column_width,$header_row_height);
		
		$fleche_team_name="";
		
		foreach($this->params['roster_table'] as $row){
		
			$le=end($row);
			if(isset($le['fleche_team'])){
				$ftn=$le['text'];
				if($ftn!=$fleche_team_name){
					$fleche_team_name=$ftn;
					$ftn_d=utf8_decode($ftn);
					
					$this->font_normalprint();
					$this->font_bold();
					$this->Cell($full_width,2*$this->baseline_skip,$ftn_d,1,2,'C');
					
				}
			}
		
		
			$y=$this->GetY();
			if($y+$table_row_height > ($this->page_height - $this->vmargin)){
				$this->AddPage();
				$x = $this->hmargin;
				$y = $this->vmargin;
				$this->SetXY($x,$y);
				$this->render_row($header_row,$column_width,$header_row_height);
			}
			$this->render_row($row,$column_width,$table_row_height);
		}
	}

  private function render_row($r,$column_width=null,$baseline_skip=null){

		$x = $this->hmargin;
		$y = $this->GetY();
		$full_width = $this->page_width - 2*$this->hmargin;
		$full_height = $this->page_height - 2*$this->vmargin;
		$w = $this->page_width - 2*$this->hmargin;

		$height=(!empty($baseline_skip))?$baseline_skip:$this->baseline_skip;

		$this->SetLineWidth($this->thin_width);
		$this->SetDrawColor(...self::BLACK);

		$nfield=count($r);

		for($i=0; $i<$nfield; $i++){

					$field=$r[$i];

					$style=(isset($field['style']))?array_fill_keys(explode(',',$field['style']),true):[];
					
					if(isset($style['hidden'])) continue;
					
					$font=(isset($field['font']))?explode(',',$field['font']):[];
					
					$this->font_normal();
					$this->font_plain();
					$this->font_normalprint();

					foreach($font as $method) $this->{'font_'.$method}();

					$text=(isset($field['text']))?utf8_decode($field['text']):''; //$text = "($x)$text";
					$fill=(isset($field['fill']))?explode(',',$field['fill']):[255,255,255];
					$text=(isset($style['toupper']))?strtoupper($text):$text;
					$indent=(isset($field['col']) && is_numeric($field['col']) && $field['col']>1)?str_repeat(' ',$field['col']-1):'';
					$border=(isset($style['noborder']))?0:1;
					$justify=(isset($field['align']))?$field['align']:'C';

					if(isset($field['fill'])) $this->SetFillColor(...$fill);
					if(isset($field['fontsize'])) $this->font_size($field['fontsize']);
					
					$string_width=$this->GetStringWidth($indent . $text);
					$dash_width=$this->GetStringWidth('-');

					if(isset($field['width'])){
						$width=$full_width*$field['width']/100.0;
					}elseif(isset($column_width[$i])){
						$width=$column_width[$i];
					}else{
						$width=$string_width +2*$dash_width;
					}

					if(isset($style['fit'])){
						$save_size=$this->font_size;
						$need_width=$string_width+2*$dash_width;
						if($need_width > $width){
						  $this->font_size($width/$need_width);
						}
					}

					if(isset($style['multi'])){ // multiline text cell
		$x = $this->GetX();
		$y = $this->GetY();
						$nlines=substr_count($text,"\n")+1;
						$this->MultiCell($width,$height/$nlines,$indent . $text,$border,$justify,isset($line['fill']));  // text
		$this->SetXY($x+$width,$y);
					}else{
						$this->Cell($width,$height,$indent . $text,$border,0,$justify);  // text
					}

					if(isset($style['fit'])){
						$this->font_size($save_size);
					}

		}

		$this->vskip($height);

	}
			

}

?>
