<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
 
 
//    PA Randonneurs Website Software
//    Copyright (C) 2019 Chris Nadovich https://parando.org/info/contact/webmaster
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Affero General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Affero General Public License for more details.
//
//    https://parando.org/assets/parando/src/LICENSE.txt
//
//    You should have received a copy of the GNU Affero General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

class Coupon extends CI_Model {
	
	public $prefix="";	
	
	public function __construct() {
		parent::__construct();
		$this->prefix=$this->model_parando->coupon_prefix;
	}
	
	
	public function mint($minter_id,$attributes){
		$mint_timestamp=time();
		$this->db->set('attributes',$this->encode_attributes($attributes));
		$this->db->set('mint_timestamp',$mint_timestamp);
		$this->db->set('minter_id',$minter_id);
		if(true!==$this->db->insert('coupon')) return false;
		$id=$this->db->insert_id();	
		$stuff="$id-$mint_timestamp-$minter_id";
		$hash=$this->model_parando->make_hash($stuff);
		$code=$this->prefix . "$id-$hash";
		$this->db->set('code',$code);
		$this->db->where('id',$id);
		if(true!==$this->db->update('coupon')) return false;
		return $code;
	}

	public function valid_syntax($c){
		$p=$this->prefix;
		$hhd=$this->model_parando->hash_hex_digits;
		if(preg_match("/$p(\d+)-([0-9A-F]{" . "$hhd,$hhd" . "})/", strtoupper($c), $m)) 
			return $m;
		else
			{
				//trigger_error("Bad syntax: ($c); p=$p; hhd=$hhd",E_USER_ERROR);
				return false;
			}
	}

   public function mint_free_event($issuer,$event_id,$authorized_spender=null){
   		$attributes=['free_event'=>$event_id];
   		if(null!==$authorized_spender)$attributes['authorized_spender']=$authorized_spender;
   		$c=$this->mint($issuer,$attributes);
   		if($c===false)trigger_error("Could not mint free event: i=$issuer, e=$event_id",E_USER_ERROR);
   		return $c;
   }
   
   public function encode_attributes($a){
   	return json_encode($a);
   }
   public function decode_attributes($ja){
   	return json_decode($ja, true);
   }
   
   public function is_valid_for_event($coupon_data,$event_id){
			return 
				empty($coupon_data['spend_timestamp']) &&
				!$coupon_data['void'] &&
				isset($coupon_data['attributes']['free_event']) && 
				$coupon_data['attributes']['free_event']==$event_id ;
   }

	public function get($c){
		$m=$this->valid_syntax($c);
		if($m!==false) {
			$given_id=$m[1];
			$given_hash=$m[2];
			$this->db->where('id',$given_id);
			$q=$this->db->get('coupon');
			if($q->num_rows()==1){
				$r=$q->row_array();
				extract($r);
				$stuff="$id-$mint_timestamp-$minter_id";
				$hash=$this->model_parando->make_hash($stuff);		
				if($hash==$given_hash){
					$r['code']=$c;
					$mdt=new DateTime('@'.$mint_timestamp);
					$localtz = new DateTimeZone($this->model_parando->tzname);
					$mdt->setTimezone($localtz);
					$r['mint_datetime']=$mdt; 
					$r['mint_datetime_str']=$mdt->format('Y-m-d H:i:s T');
					$r['attributes']=$this->decode_attributes($attributes);
					if($spend_timestamp>0){
						$r['spent']=true;
						$sdt=new DateTime('@'.$spend_timestamp);
						$sdt->setTimezone($localtz);
						$r['spend_datetime']=$sdt; 
						$r['spend_datetime_str']=$sdt->format('Y-m-d H:i:s T');
					}else{$r['spent']=false;}			
					return $r; 
				}
			}
		}
		return false;
	}
	
	public function is_spent($r){return $r['spent'];}
	public function is_unspent($r){return !$r['spent'];}
	
	public function spend($c,$invoice_number,$spender_id){
		$r=$this->get($c);
		if(false===$r)return "Invalid coupon";
		if($this->is_spent($r)) return "Already spent";
		$spend_timestamp=time();
		$sdata=compact('spend_timestamp','invoice_number','spender_id');
		$this->db->where('id',$r['id']);
		if(false === $this->db->update('coupon', $sdata)) return "Couldn't spend -- database error";
		return true;
	}
	
	
	
}

?>